@@ -26,7 +26,6 @@ import org.apache.hudi.cli.TableHeader;
 import org.apache.hudi.cli.commands.SparkMain.SparkCommand;
 import org.apache.hudi.cli.utils.InputStreamConsumer;
 import org.apache.hudi.cli.utils.SparkUtil;
-import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.HoodieLogFile;
 import org.apache.hudi.common.model.HoodieRecord;
 import org.apache.hudi.common.model.HoodieRecord.HoodieRecordType;
@@ -35,18 +34,21 @@ import org.apache.hudi.common.table.log.HoodieLogFormat;
 import org.apache.hudi.common.table.log.HoodieLogFormat.Reader;
 import org.apache.hudi.common.table.log.block.HoodieAvroDataBlock;
 import org.apache.hudi.common.table.timeline.HoodieTimeline;
-import org.apache.hudi.common.util.ClosableIterator;
 import org.apache.hudi.common.util.Option;
+import org.apache.hudi.common.util.collection.ClosableIterator;
+import org.apache.hudi.exception.HoodieException;
+import org.apache.hudi.storage.HoodieStorage;
+import org.apache.hudi.storage.StoragePathInfo;
+import org.apache.hudi.storage.StoragePath;
+import org.apache.hudi.storage.HoodieStorageUtils;
 
 import org.apache.avro.generic.GenericRecord;
 import org.apache.avro.generic.IndexedRecord;
 import org.apache.avro.specific.SpecificData;
-import org.apache.hadoop.fs.FileStatus;
-import org.apache.hadoop.fs.Path;
-import org.apache.logging.log4j.LogManager;
-import org.apache.logging.log4j.Logger;
 import org.apache.spark.launcher.SparkLauncher;
 import org.apache.spark.util.Utils;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.springframework.shell.standard.ShellComponent;
 import org.springframework.shell.standard.ShellMethod;
 import org.springframework.shell.standard.ShellOption;
@@ -62,7 +64,7 @@ import java.util.stream.Collectors;
  */
 @ShellComponent
 public class ArchivedCommitsCommand {
-  private static final Logger LOG = LogManager.getLogger(ArchivedCommitsCommand.class);
+  private static final Logger LOG = LoggerFactory.getLogger(ArchivedCommitsCommand.class);
   @ShellMethod(key = "trigger archival", value = "trigger archival")
   public String triggerArchival(
       @ShellOption(value = {"--minCommits"},
@@ -94,66 +96,65 @@ public class ArchivedCommitsCommand {
     return "Archival successfully triggered";
   }
 
-  @ShellMethod(key = "show archived commit stats", value = "Read commits from archived files and show details")
+  @ShellMethod(key = "show archived commit stats", value = "Read commits from archived files and show file group details")
   public String showArchivedCommits(
       @ShellOption(value = {"--archiveFolderPattern"}, help = "Archive Folder", defaultValue = "") String folder,
-      @ShellOption(value = {"--limit"}, help = "Limit commits", defaultValue = "-1") final Integer limit,
+      @ShellOption(value = {"--limit"}, help = "Limit commits", defaultValue = "10") final Integer limit,
       @ShellOption(value = {"--sortBy"}, help = "Sorting Field", defaultValue = "") final String sortByField,
       @ShellOption(value = {"--desc"}, help = "Ordering", defaultValue = "false") final boolean descending,
       @ShellOption(value = {"--headeronly"}, help = "Print Header Only",
               defaultValue = "false") final boolean headerOnly)
       throws IOException {
     System.out.println("===============> Showing only " + limit + " archived commits <===============");
-    String basePath = HoodieCLI.getTableMetaClient().getBasePath();
-    Path archivePath = new Path(HoodieCLI.getTableMetaClient().getArchivePath() + "/.commits_.archive*");
-    if (folder != null && !folder.isEmpty()) {
-      archivePath = new Path(basePath + "/.hoodie/" + folder);
-    }
-    FileStatus[] fsStatuses = FSUtils.getFs(basePath, HoodieCLI.conf).globStatus(archivePath);
+    HoodieTableMetaClient metaClient = HoodieCLI.getTableMetaClient();
+    StoragePath archivePath = folder != null && !folder.isEmpty()
+        ? new StoragePath(metaClient.getMetaPath(), folder)
+        : new StoragePath(metaClient.getArchivePath(), ".commits_.archive*");
+    HoodieStorage storage = metaClient.getStorage();
+    List<StoragePathInfo> pathInfoList = storage.globEntries(archivePath);
     List<Comparable[]> allStats = new ArrayList<>();
-    for (FileStatus fs : fsStatuses) {
+    for (StoragePathInfo pathInfo : pathInfoList) {
       // read the archived file
-      Reader reader = HoodieLogFormat.newReader(FSUtils.getFs(basePath, HoodieCLI.conf),
-          new HoodieLogFile(fs.getPath()), HoodieArchivedMetaEntry.getClassSchema());
-
-      List<IndexedRecord> readRecords = new ArrayList<>();
-      // read the avro blocks
-      while (reader.hasNext()) {
-        HoodieAvroDataBlock blk = (HoodieAvroDataBlock) reader.next();
-        blk.getRecordIterator(HoodieRecordType.AVRO).forEachRemaining(r -> readRecords.add((IndexedRecord) r.getData()));
+      try (Reader reader = HoodieLogFormat.newReader(storage, new HoodieLogFile(pathInfo.getPath()),
+          HoodieArchivedMetaEntry.getClassSchema())) {
+        List<IndexedRecord> readRecords = new ArrayList<>();
+        // read the avro blocks
+        while (reader.hasNext()) {
+          HoodieAvroDataBlock blk = (HoodieAvroDataBlock) reader.next();
+          blk.getRecordIterator(HoodieRecordType.AVRO).forEachRemaining(r -> readRecords.add((IndexedRecord) r.getData()));
+        }
+        List<Comparable[]> readCommits = readRecords.stream().map(r -> (GenericRecord) r)
+            .filter(r -> r.get("actionType").toString().equals(HoodieTimeline.COMMIT_ACTION)
+                || r.get("actionType").toString().equals(HoodieTimeline.DELTA_COMMIT_ACTION))
+            .flatMap(r -> {
+              HoodieCommitMetadata metadata = (HoodieCommitMetadata) SpecificData.get()
+                  .deepCopy(HoodieCommitMetadata.SCHEMA$, r.get("hoodieCommitMetadata"));
+              final String instantTime = r.get("commitTime").toString();
+              final String action = r.get("actionType").toString();
+              return metadata.getPartitionToWriteStats().values().stream().flatMap(hoodieWriteStats -> hoodieWriteStats.stream().map(hoodieWriteStat -> {
+                List<Comparable> row = new ArrayList<>();
+                row.add(action);
+                row.add(instantTime);
+                row.add(hoodieWriteStat.getPartitionPath());
+                row.add(hoodieWriteStat.getFileId());
+                row.add(hoodieWriteStat.getPrevCommit());
+                row.add(hoodieWriteStat.getNumWrites());
+                row.add(hoodieWriteStat.getNumInserts());
+                row.add(hoodieWriteStat.getNumDeletes());
+                row.add(hoodieWriteStat.getNumUpdateWrites());
+                row.add(hoodieWriteStat.getTotalLogFiles());
+                row.add(hoodieWriteStat.getTotalLogBlocks());
+                row.add(hoodieWriteStat.getTotalCorruptLogBlock());
+                row.add(hoodieWriteStat.getTotalRollbackBlocks());
+                row.add(hoodieWriteStat.getTotalLogRecords());
+                row.add(hoodieWriteStat.getTotalUpdatedRecordsCompacted());
+                row.add(hoodieWriteStat.getTotalWriteBytes());
+                row.add(hoodieWriteStat.getTotalWriteErrors());
+                return row;
+              })).map(rowList -> rowList.toArray(new Comparable[0]));
+            }).collect(Collectors.toList());
+        allStats.addAll(readCommits);
       }
-      List<Comparable[]> readCommits = readRecords.stream().map(r -> (GenericRecord) r)
-          .filter(r -> r.get("actionType").toString().equals(HoodieTimeline.COMMIT_ACTION)
-              || r.get("actionType").toString().equals(HoodieTimeline.DELTA_COMMIT_ACTION))
-          .flatMap(r -> {
-            HoodieCommitMetadata metadata = (HoodieCommitMetadata) SpecificData.get()
-                .deepCopy(HoodieCommitMetadata.SCHEMA$, r.get("hoodieCommitMetadata"));
-            final String instantTime = r.get("commitTime").toString();
-            final String action = r.get("actionType").toString();
-            return metadata.getPartitionToWriteStats().values().stream().flatMap(hoodieWriteStats -> hoodieWriteStats.stream().map(hoodieWriteStat -> {
-              List<Comparable> row = new ArrayList<>();
-              row.add(action);
-              row.add(instantTime);
-              row.add(hoodieWriteStat.getPartitionPath());
-              row.add(hoodieWriteStat.getFileId());
-              row.add(hoodieWriteStat.getPrevCommit());
-              row.add(hoodieWriteStat.getNumWrites());
-              row.add(hoodieWriteStat.getNumInserts());
-              row.add(hoodieWriteStat.getNumDeletes());
-              row.add(hoodieWriteStat.getNumUpdateWrites());
-              row.add(hoodieWriteStat.getTotalLogFiles());
-              row.add(hoodieWriteStat.getTotalLogBlocks());
-              row.add(hoodieWriteStat.getTotalCorruptLogBlock());
-              row.add(hoodieWriteStat.getTotalRollbackBlocks());
-              row.add(hoodieWriteStat.getTotalLogRecords());
-              row.add(hoodieWriteStat.getTotalUpdatedRecordsCompacted());
-              row.add(hoodieWriteStat.getTotalWriteBytes());
-              row.add(hoodieWriteStat.getTotalWriteErrors());
-              return row;
-            })).map(rowList -> rowList.toArray(new Comparable[0]));
-          }).collect(Collectors.toList());
-      allStats.addAll(readCommits);
-      reader.close();
     }
     TableHeader header = new TableHeader().addTableHeaderField("action").addTableHeaderField("instant")
         .addTableHeaderField("partition").addTableHeaderField("file_id").addTableHeaderField("prev_instant")
@@ -181,27 +182,27 @@ public class ArchivedCommitsCommand {
     System.out.println("===============> Showing only " + limit + " archived commits <===============");
     HoodieTableMetaClient metaClient = HoodieCLI.getTableMetaClient();
     String basePath = metaClient.getBasePath();
-    Path archivePath = new Path(metaClient.getArchivePath() + "/.commits_.archive*");
-    FileStatus[] fsStatuses =
-        FSUtils.getFs(basePath, HoodieCLI.conf).globStatus(archivePath);
+    StoragePath archivePath =
+        new StoragePath(metaClient.getArchivePath() + "/.commits_.archive*");
+    List<StoragePathInfo> pathInfoList =
+        HoodieStorageUtils.getStorage(basePath, HoodieCLI.conf).globEntries(archivePath);
     List<Comparable[]> allCommits = new ArrayList<>();
-    for (FileStatus fs : fsStatuses) {
+    for (StoragePathInfo pathInfo : pathInfoList) {
       // read the archived file
-      HoodieLogFormat.Reader reader = HoodieLogFormat.newReader(FSUtils.getFs(basePath, HoodieCLI.conf),
-          new HoodieLogFile(fs.getPath()), HoodieArchivedMetaEntry.getClassSchema());
-
-      List<IndexedRecord> readRecords = new ArrayList<>();
-      // read the avro blocks
-      while (reader.hasNext()) {
-        HoodieAvroDataBlock blk = (HoodieAvroDataBlock) reader.next();
-        try (ClosableIterator<HoodieRecord<IndexedRecord>> recordItr = blk.getRecordIterator(HoodieRecordType.AVRO)) {
-          recordItr.forEachRemaining(r -> readRecords.add(r.getData()));
+      try (HoodieLogFormat.Reader reader = HoodieLogFormat.newReader(HoodieStorageUtils.getStorage(basePath, HoodieCLI.conf),
+          new HoodieLogFile(pathInfo.getPath()), HoodieArchivedMetaEntry.getClassSchema())) {
+        List<IndexedRecord> readRecords = new ArrayList<>();
+        // read the avro blocks
+        while (reader.hasNext()) {
+          HoodieAvroDataBlock blk = (HoodieAvroDataBlock) reader.next();
+          try (ClosableIterator<HoodieRecord<IndexedRecord>> recordItr = blk.getRecordIterator(HoodieRecordType.AVRO)) {
+            recordItr.forEachRemaining(r -> readRecords.add(r.getData()));
+          }
         }
+        List<Comparable[]> readCommits = readRecords.stream().map(r -> (GenericRecord) r)
+            .map(r -> readCommit(r, skipMetadata)).collect(Collectors.toList());
+        allCommits.addAll(readCommits);
       }
-      List<Comparable[]> readCommits = readRecords.stream().map(r -> (GenericRecord) r)
-          .map(r -> readCommit(r, skipMetadata)).collect(Collectors.toList());
-      allCommits.addAll(readCommits);
-      reader.close();
     }
 
     TableHeader header = new TableHeader().addTableHeaderField("CommitTime").addTableHeaderField("CommitType");
@@ -213,8 +214,7 @@ public class ArchivedCommitsCommand {
     return HoodiePrintHelper.print(header, new HashMap<>(), sortByField, descending, limit, headerOnly, allCommits);
   }
 
-  private Comparable[] commitDetail(GenericRecord record, String metadataName,
-                                    boolean skipMetadata) {
+  private Comparable[] commitDetail(GenericRecord record, String metadataName, boolean skipMetadata) {
     List<Object> commitDetails = new ArrayList<>();
     commitDetails.add(record.get("commitTime"));
     commitDetails.add(record.get("actionType").toString());
@@ -225,26 +225,24 @@ public class ArchivedCommitsCommand {
   }
 
   private Comparable[] readCommit(GenericRecord record, boolean skipMetadata) {
-    try {
-      switch (record.get("actionType").toString()) {
-        case HoodieTimeline.CLEAN_ACTION:
-          return commitDetail(record, "hoodieCleanMetadata", skipMetadata);
-        case HoodieTimeline.COMMIT_ACTION:
-        case HoodieTimeline.DELTA_COMMIT_ACTION:
-          return commitDetail(record, "hoodieCommitMetadata", skipMetadata);
-        case HoodieTimeline.ROLLBACK_ACTION:
-          return commitDetail(record, "hoodieRollbackMetadata", skipMetadata);
-        case HoodieTimeline.SAVEPOINT_ACTION:
-          return commitDetail(record, "hoodieSavePointMetadata", skipMetadata);
-        case HoodieTimeline.COMPACTION_ACTION:
-          return commitDetail(record, "hoodieCompactionMetadata", skipMetadata);
-        default: {
-          return new Comparable[] {};
-        }
+    String actionType = record.get("actionType").toString();
+    switch (actionType) {
+      case HoodieTimeline.CLEAN_ACTION:
+        return commitDetail(record, "hoodieCleanMetadata", skipMetadata);
+      case HoodieTimeline.COMMIT_ACTION:
+      case HoodieTimeline.DELTA_COMMIT_ACTION:
+        return commitDetail(record, "hoodieCommitMetadata", skipMetadata);
+      case HoodieTimeline.ROLLBACK_ACTION:
+        return commitDetail(record, "hoodieRollbackMetadata", skipMetadata);
+      case HoodieTimeline.SAVEPOINT_ACTION:
+        return commitDetail(record, "hoodieSavePointMetadata", skipMetadata);
+      case HoodieTimeline.COMPACTION_ACTION:
+        return commitDetail(record, "hoodieCompactionMetadata", skipMetadata);
+      case HoodieTimeline.REPLACE_COMMIT_ACTION:
+        return commitDetail(record, "hoodieReplaceCommitMetadata", skipMetadata);
+      default: {
+        throw new HoodieException("Unexpected action type: " + actionType);
       }
-    } catch (Exception e) {
-      e.printStackTrace();
-      return new Comparable[] {};
     }
   }
 
