@@ -19,9 +19,6 @@
 
 package org.apache.hudi.cli.commands;
 
-import org.apache.hadoop.fs.FileStatus;
-import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.Path;
 import org.apache.hudi.avro.model.HoodieRollbackMetadata;
 import org.apache.hudi.avro.model.HoodieRollbackPlan;
 import org.apache.hudi.cli.HoodieCLI;
@@ -35,8 +32,12 @@ import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.common.table.timeline.TimelineMetadataUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.metadata.HoodieTableMetadata;
-import org.apache.logging.log4j.Logger;
-import org.apache.logging.log4j.LogManager;
+import org.apache.hudi.storage.StoragePathInfo;
+import org.apache.hudi.storage.StoragePath;
+import org.apache.hudi.storage.HoodieStorage;
+
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.springframework.shell.standard.ShellComponent;
 import org.springframework.shell.standard.ShellMethod;
 import org.springframework.shell.standard.ShellOption;
@@ -44,7 +45,6 @@ import org.springframework.shell.standard.ShellOption;
 import java.io.IOException;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Comparator;
 import java.util.Date;
 import java.util.HashMap;
@@ -61,7 +61,7 @@ import java.util.stream.Stream;
 @ShellComponent
 public class TimelineCommand {
 
-  private static final Logger LOG = LogManager.getLogger(TimelineCommand.class);
+  private static final Logger LOG = LoggerFactory.getLogger(TimelineCommand.class);
   private static final SimpleDateFormat DATE_FORMAT_DEFAULT = new SimpleDateFormat("MM-dd HH:mm");
   private static final SimpleDateFormat DATE_FORMAT_SECONDS = new SimpleDateFormat("MM-dd HH:mm:ss");
 
@@ -84,13 +84,13 @@ public class TimelineCommand {
         HoodieTableMetaClient mtMetaClient = getMetadataTableMetaClient(metaClient);
         return printTimelineInfoWithMetadataTable(
             metaClient.getActiveTimeline(), mtMetaClient.getActiveTimeline(),
-            getInstantInfoFromTimeline(metaClient.getFs(), metaClient.getMetaPath()),
-            getInstantInfoFromTimeline(mtMetaClient.getFs(), mtMetaClient.getMetaPath()),
+            getInstantInfoFromTimeline(metaClient.getStorage(), metaClient.getMetaPath()),
+            getInstantInfoFromTimeline(mtMetaClient.getStorage(), mtMetaClient.getMetaPath()),
             limit, sortByField, descending, headerOnly, true, showTimeSeconds, showRollbackInfo);
       }
       return printTimelineInfo(
           metaClient.getActiveTimeline(),
-          getInstantInfoFromTimeline(metaClient.getFs(), metaClient.getMetaPath()),
+          getInstantInfoFromTimeline(metaClient.getStorage(), metaClient.getMetaPath()),
           limit, sortByField, descending, headerOnly, true, showTimeSeconds, showRollbackInfo);
     } catch (IOException e) {
       e.printStackTrace();
@@ -113,7 +113,7 @@ public class TimelineCommand {
     try {
       return printTimelineInfo(
           metaClient.getActiveTimeline().filterInflightsAndRequested(),
-          getInstantInfoFromTimeline(metaClient.getFs(), metaClient.getMetaPath()),
+          getInstantInfoFromTimeline(metaClient.getStorage(), metaClient.getMetaPath()),
           limit, sortByField, descending, headerOnly, true, showTimeSeconds, showRollbackInfo);
     } catch (IOException e) {
       e.printStackTrace();
@@ -135,7 +135,7 @@ public class TimelineCommand {
     try {
       return printTimelineInfo(
           metaClient.getActiveTimeline(),
-          getInstantInfoFromTimeline(metaClient.getFs(), metaClient.getMetaPath()),
+          getInstantInfoFromTimeline(metaClient.getStorage(), metaClient.getMetaPath()),
           limit, sortByField, descending, headerOnly, true, showTimeSeconds, false);
     } catch (IOException e) {
       e.printStackTrace();
@@ -157,7 +157,7 @@ public class TimelineCommand {
     try {
       return printTimelineInfo(
           metaClient.getActiveTimeline().filterInflightsAndRequested(),
-          getInstantInfoFromTimeline(metaClient.getFs(), metaClient.getMetaPath()),
+          getInstantInfoFromTimeline(metaClient.getStorage(), metaClient.getMetaPath()),
           limit, sortByField, descending, headerOnly, true, showTimeSeconds, false);
     } catch (IOException e) {
       e.printStackTrace();
@@ -166,7 +166,7 @@ public class TimelineCommand {
   }
 
   private HoodieTableMetaClient getMetadataTableMetaClient(HoodieTableMetaClient metaClient) {
-    return HoodieTableMetaClient.builder().setConf(HoodieCLI.conf)
+    return HoodieTableMetaClient.builder().setConf(HoodieCLI.conf.newInstance())
         .setBasePath(HoodieTableMetadata.getMetadataTableBasePath(metaClient.getBasePath()))
         .setLoadActiveTimelineOnLoad(false)
         .setConsistencyGuardConfig(HoodieCLI.consistencyGuardConfig)
@@ -174,14 +174,14 @@ public class TimelineCommand {
   }
 
   private Map<String, Map<HoodieInstant.State, HoodieInstantWithModTime>> getInstantInfoFromTimeline(
-      FileSystem fs, String metaPath) throws IOException {
+      HoodieStorage storage, StoragePath metaPath) throws IOException {
     Map<String, Map<HoodieInstant.State, HoodieInstantWithModTime>> instantMap = new HashMap<>();
-    Stream<HoodieInstantWithModTime> instantStream = Arrays.stream(
-        HoodieTableMetaClient.scanFiles(fs, new Path(metaPath), path -> {
+    Stream<HoodieInstantWithModTime> instantStream =
+        HoodieTableMetaClient.scanFiles(storage, metaPath, path -> {
           // Include only the meta files with extensions that needs to be included
           String extension = HoodieInstant.getTimelineFileExtension(path.getName());
           return HoodieActiveTimeline.VALID_EXTENSIONS_IN_ACTIVE_TIMELINE.contains(extension);
-        })).map(HoodieInstantWithModTime::new);
+        }).stream().map(HoodieInstantWithModTime::new);
     instantStream.forEach(instant -> {
       instantMap.computeIfAbsent(instant.getTimestamp(), t -> new HashMap<>())
           .put(instant.getState(), instant);
@@ -207,41 +207,29 @@ public class TimelineCommand {
       Map<String, Map<HoodieInstant.State, HoodieInstantWithModTime>> instantInfoMap,
       Integer limit, String sortByField, boolean descending, boolean headerOnly, boolean withRowNo,
       boolean showTimeSeconds, boolean showRollbackInfo) {
-    Map<String, List<String>> rollbackInfo = getRolledBackInstantInfo(timeline);
+    Map<String, List<String>> rollbackInfoMap = getRolledBackInstantInfo(timeline);
     final List<Comparable[]> rows = timeline.getInstantsAsStream().map(instant -> {
-      int numColumns = showRollbackInfo ? 7 : 6;
-      Comparable[] row = new Comparable[numColumns];
+      Comparable[] row = new Comparable[6];
       String instantTimestamp = instant.getTimestamp();
+      String rollbackInfoString = showRollbackInfo
+          ? getRollbackInfoString(Option.of(instant), timeline, rollbackInfoMap) : "";
+
       row[0] = instantTimestamp;
-      row[1] = instant.getAction();
+      row[1] = instant.getAction() + rollbackInfoString;
       row[2] = instant.getState();
-      if (showRollbackInfo) {
-        if (HoodieTimeline.ROLLBACK_ACTION.equalsIgnoreCase(instant.getAction())) {
-          row[3] = "Rolls back\n" + getInstantToRollback(timeline, instant);
-        } else {
-          if (rollbackInfo.containsKey(instantTimestamp)) {
-            row[3] = "Rolled back by\n" + String.join(",\n", rollbackInfo.get(instantTimestamp));
-          } else {
-            row[3] = "-";
-          }
-        }
-      }
-      row[numColumns - 3] = getFormattedDate(
+      row[3] = getFormattedDate(
           instantTimestamp, HoodieInstant.State.REQUESTED, instantInfoMap, showTimeSeconds);
-      row[numColumns - 2] = getFormattedDate(
+      row[4] = getFormattedDate(
           instantTimestamp, HoodieInstant.State.INFLIGHT, instantInfoMap, showTimeSeconds);
-      row[numColumns - 1] = getFormattedDate(
+      row[5] = getFormattedDate(
           instantTimestamp, HoodieInstant.State.COMPLETED, instantInfoMap, showTimeSeconds);
       return row;
     }).collect(Collectors.toList());
     TableHeader header = new TableHeader()
         .addTableHeaderField(HoodieTableHeaderFields.HEADER_INSTANT)
         .addTableHeaderField(HoodieTableHeaderFields.HEADER_ACTION)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_STATE);
-    if (showRollbackInfo) {
-      header.addTableHeaderField(HoodieTableHeaderFields.HEADER_ROLLBACK_INFO);
-    }
-    header.addTableHeaderField(HoodieTableHeaderFields.HEADER_REQUESTED_TIME)
+        .addTableHeaderField(HoodieTableHeaderFields.HEADER_STATE)
+        .addTableHeaderField(HoodieTableHeaderFields.HEADER_REQUESTED_TIME)
         .addTableHeaderField(HoodieTableHeaderFields.HEADER_INFLIGHT_TIME)
         .addTableHeaderField(HoodieTableHeaderFields.HEADER_COMPLETED_TIME);
     return HoodiePrintHelper.print(
@@ -258,52 +246,42 @@ public class TimelineCommand {
     instantTimeSet.addAll(mtInstantInfoMap.keySet());
     List<String> instantTimeList = instantTimeSet.stream()
         .sorted(new HoodieInstantTimeComparator()).collect(Collectors.toList());
-    Map<String, List<String>> dtRollbackInfo = getRolledBackInstantInfo(dtTimeline);
+    Map<String, List<String>> dtRollbackInfoMap = getRolledBackInstantInfo(dtTimeline);
+    Map<String, List<String>> mtRollbackInfoMap = getRolledBackInstantInfo(mtTimeline);
 
     final List<Comparable[]> rows = instantTimeList.stream().map(instantTimestamp -> {
-      int numColumns = showRollbackInfo ? 12 : 11;
       Option<HoodieInstant> dtInstant = getInstant(dtTimeline, instantTimestamp);
       Option<HoodieInstant> mtInstant = getInstant(mtTimeline, instantTimestamp);
-      Comparable[] row = new Comparable[numColumns];
+      Comparable[] row = new Comparable[11];
       row[0] = instantTimestamp;
-      row[1] = dtInstant.isPresent() ? dtInstant.get().getAction() : "-";
+      String dtRollbackInfoString = showRollbackInfo
+          ? getRollbackInfoString(dtInstant, dtTimeline, dtRollbackInfoMap) : "";
+      row[1] = (dtInstant.isPresent() ? dtInstant.get().getAction() : "-") + dtRollbackInfoString;
       row[2] = dtInstant.isPresent() ? dtInstant.get().getState() : "-";
-      if (showRollbackInfo) {
-        if (dtInstant.isPresent()
-            && HoodieTimeline.ROLLBACK_ACTION.equalsIgnoreCase(dtInstant.get().getAction())) {
-          row[3] = "Rolls back\n" + getInstantToRollback(dtTimeline, dtInstant.get());
-        } else {
-          if (dtRollbackInfo.containsKey(instantTimestamp)) {
-            row[3] = "Rolled back by\n" + String.join(",\n", dtRollbackInfo.get(instantTimestamp));
-          } else {
-            row[3] = "-";
-          }
-        }
-      }
-      row[numColumns - 8] = getFormattedDate(
+      row[3] = getFormattedDate(
           instantTimestamp, HoodieInstant.State.REQUESTED, dtInstantInfoMap, showTimeSeconds);
-      row[numColumns - 7] = getFormattedDate(
+      row[4] = getFormattedDate(
           instantTimestamp, HoodieInstant.State.INFLIGHT, dtInstantInfoMap, showTimeSeconds);
-      row[numColumns - 6] = getFormattedDate(
+      row[5] = getFormattedDate(
           instantTimestamp, HoodieInstant.State.COMPLETED, dtInstantInfoMap, showTimeSeconds);
-      row[numColumns - 5] = mtInstant.isPresent() ? mtInstant.get().getAction() : "-";
-      row[numColumns - 4] = mtInstant.isPresent() ? mtInstant.get().getState() : "-";
-      row[numColumns - 3] = getFormattedDate(
+
+      String mtRollbackInfoString = showRollbackInfo
+          ? getRollbackInfoString(mtInstant, mtTimeline, mtRollbackInfoMap) : "";
+      row[6] = (mtInstant.isPresent() ? mtInstant.get().getAction() : "-") + mtRollbackInfoString;
+      row[7] = mtInstant.isPresent() ? mtInstant.get().getState() : "-";
+      row[8] = getFormattedDate(
           instantTimestamp, HoodieInstant.State.REQUESTED, mtInstantInfoMap, showTimeSeconds);
-      row[numColumns - 2] = getFormattedDate(
+      row[9] = getFormattedDate(
           instantTimestamp, HoodieInstant.State.INFLIGHT, mtInstantInfoMap, showTimeSeconds);
-      row[numColumns - 1] = getFormattedDate(
+      row[10] = getFormattedDate(
           instantTimestamp, HoodieInstant.State.COMPLETED, mtInstantInfoMap, showTimeSeconds);
       return row;
     }).collect(Collectors.toList());
     TableHeader header = new TableHeader()
         .addTableHeaderField(HoodieTableHeaderFields.HEADER_INSTANT)
         .addTableHeaderField(HoodieTableHeaderFields.HEADER_ACTION)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_STATE);
-    if (showRollbackInfo) {
-      header.addTableHeaderField(HoodieTableHeaderFields.HEADER_ROLLBACK_INFO);
-    }
-    header.addTableHeaderField(HoodieTableHeaderFields.HEADER_REQUESTED_TIME)
+        .addTableHeaderField(HoodieTableHeaderFields.HEADER_STATE)
+        .addTableHeaderField(HoodieTableHeaderFields.HEADER_REQUESTED_TIME)
         .addTableHeaderField(HoodieTableHeaderFields.HEADER_INFLIGHT_TIME)
         .addTableHeaderField(HoodieTableHeaderFields.HEADER_COMPLETED_TIME)
         .addTableHeaderField(HoodieTableHeaderFields.HEADER_MT_ACTION)
@@ -369,13 +347,30 @@ public class TimelineCommand {
     return rollbackInfoMap;
   }
 
+  private String getRollbackInfoString(Option<HoodieInstant> instant,
+                                       HoodieTimeline timeline,
+                                       Map<String, List<String>> rollbackInfoMap) {
+    String rollbackInfoString = "";
+    if (instant.isPresent()) {
+      if (HoodieTimeline.ROLLBACK_ACTION.equalsIgnoreCase(instant.get().getAction())) {
+        rollbackInfoString = "\nRolls back\n" + getInstantToRollback(timeline, instant.get());
+      } else {
+        String instantTimestamp = instant.get().getTimestamp();
+        if (rollbackInfoMap.containsKey(instantTimestamp)) {
+          rollbackInfoString = "\nRolled back by\n" + String.join(",\n", rollbackInfoMap.get(instantTimestamp));
+        }
+      }
+    }
+    return rollbackInfoString;
+  }
+
   static class HoodieInstantWithModTime extends HoodieInstant {
 
     private final long modificationTimeMs;
 
-    public HoodieInstantWithModTime(FileStatus fileStatus) {
-      super(fileStatus);
-      this.modificationTimeMs = fileStatus.getModificationTime();
+    public HoodieInstantWithModTime(StoragePathInfo pathInfo) {
+      super(pathInfo);
+      this.modificationTimeMs = pathInfo.getModificationTime();
     }
 
     public long getModificationTime() {
