@@ -27,7 +27,6 @@ import org.apache.hudi.cli.functional.CLIFunctionalTestHarness;
 import org.apache.hudi.cli.testutils.HoodieTestCommitMetadataGenerator;
 import org.apache.hudi.cli.testutils.ShellEvaluationResultUtil;
 import org.apache.hudi.common.config.HoodieCommonConfig;
-import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.HoodieAvroIndexedRecord;
 import org.apache.hudi.common.model.HoodieAvroRecord;
 import org.apache.hudi.common.model.HoodieLogFile;
@@ -44,13 +43,14 @@ import org.apache.hudi.common.util.FileIOUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.config.HoodieCompactionConfig;
 import org.apache.hudi.config.HoodieMemoryConfig;
+import org.apache.hudi.storage.HoodieStorage;
+import org.apache.hudi.storage.HoodieStorageUtils;
+import org.apache.hudi.storage.StoragePath;
 
 import com.fasterxml.jackson.core.JsonProcessingException;
 import com.fasterxml.jackson.databind.ObjectMapper;
 import org.apache.avro.Schema;
 import org.apache.avro.generic.IndexedRecord;
-import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.Path;
 import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Tag;
@@ -64,7 +64,6 @@ import java.net.URISyntaxException;
 import java.nio.file.Files;
 import java.nio.file.Paths;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
@@ -91,13 +90,13 @@ public class TestHoodieLogFileCommand extends CLIFunctionalTestHarness {
   private HoodieAvroDataBlock dataBlock;
   private HoodieCommandBlock commandBlock;
   private String tablePath;
-  private FileSystem fs;
+  private HoodieStorage storage;
 
   private static final String INSTANT_TIME = "100";
 
   @BeforeEach
   public void init() throws IOException, InterruptedException, URISyntaxException {
-    HoodieCLI.conf = hadoopConf();
+    HoodieCLI.conf = storageConf();
 
     // Create table and connect
     String tableName = tableName();
@@ -108,12 +107,12 @@ public class TestHoodieLogFileCommand extends CLIFunctionalTestHarness {
         "", TimelineLayoutVersion.VERSION_1, "org.apache.hudi.common.model.HoodieAvroPayload");
 
     Files.createDirectories(Paths.get(partitionPath));
-    fs = FSUtils.getFs(tablePath, hadoopConf());
+    storage = HoodieStorageUtils.getStorage(tablePath, storageConf());
 
     try (HoodieLogFormat.Writer writer = HoodieLogFormat.newWriterBuilder()
-        .onParentPath(new Path(partitionPath))
+        .onParentPath(new StoragePath(partitionPath))
         .withFileExtension(HoodieLogFile.DELTA_EXTENSION)
-        .withFileId("test-log-fileid1").overBaseCommit("100").withFs(fs)
+        .withFileId("test-log-fileid1").overBaseCommit("100").withStorage(storage)
         .withSizeThreshold(1).build()) {
 
       // write data to file
@@ -136,7 +135,7 @@ public class TestHoodieLogFileCommand extends CLIFunctionalTestHarness {
 
   @AfterEach
   public void cleanUp() throws IOException {
-    fs.close();
+    storage.close();
   }
 
   /**
@@ -201,16 +200,16 @@ public class TestHoodieLogFileCommand extends CLIFunctionalTestHarness {
 
     // write to path '2015/03/16'.
     Schema schema = HoodieAvroUtils.addMetadataFields(getSimpleSchema());
-    partitionPath = tablePath + Path.SEPARATOR + HoodieTestCommitMetadataGenerator.DEFAULT_SECOND_PARTITION_PATH;
+    partitionPath = tablePath + StoragePath.SEPARATOR + HoodieTestCommitMetadataGenerator.DEFAULT_SECOND_PARTITION_PATH;
     Files.createDirectories(Paths.get(partitionPath));
 
     HoodieLogFormat.Writer writer = null;
     try {
       // set little threshold to split file.
       writer =
-          HoodieLogFormat.newWriterBuilder().onParentPath(new Path(partitionPath))
+          HoodieLogFormat.newWriterBuilder().onParentPath(new StoragePath(partitionPath))
               .withFileExtension(HoodieLogFile.DELTA_EXTENSION)
-              .withFileId("test-log-fileid1").overBaseCommit(INSTANT_TIME).withFs(fs).withSizeThreshold(500).build();
+              .withFileId("test-log-fileid1").overBaseCommit(INSTANT_TIME).withStorage(storage).withSizeThreshold(500).build();
 
       SchemaTestUtil testUtil = new SchemaTestUtil();
       List<HoodieRecord> records1 = testUtil.generateHoodieTestRecords(0, 100).stream().map(HoodieAvroIndexedRecord::new).collect(Collectors.toList());
@@ -226,23 +225,21 @@ public class TestHoodieLogFileCommand extends CLIFunctionalTestHarness {
     }
 
     Object result = shell.evaluate(() -> "show logfile records --logFilePathPattern "
-            + partitionPath + "/* --mergeRecords true");
+        + partitionPath + "/* --mergeRecords true");
     assertTrue(ShellEvaluationResultUtil.isSuccess(result));
 
     // get expected result of 10 records.
-    List<String> logFilePaths = Arrays.stream(fs.globStatus(new Path(partitionPath + "/*")))
+    List<String> logFilePaths = storage.globEntries(new StoragePath(partitionPath + "/*"))
+        .stream()
         .map(status -> status.getPath().toString()).collect(Collectors.toList());
     HoodieMergedLogRecordScanner scanner = HoodieMergedLogRecordScanner.newBuilder()
-        .withFileSystem(fs)
+        .withStorage(storage)
         .withBasePath(tablePath)
         .withLogFilePaths(logFilePaths)
         .withReaderSchema(schema)
         .withLatestInstantTime(INSTANT_TIME)
         .withMaxMemorySizeInBytes(
             HoodieMemoryConfig.DEFAULT_MAX_MEMORY_FOR_SPILLABLE_MAP_IN_BYTES)
-        .withReadBlocksLazily(
-            Boolean.parseBoolean(
-                HoodieCompactionConfig.COMPACTION_LAZY_BLOCK_READ_ENABLE.defaultValue()))
         .withReverseReader(
             Boolean.parseBoolean(
                 HoodieCompactionConfig.COMPACTION_REVERSE_LOG_READ_ENABLE.defaultValue()))
