@@ -18,7 +18,6 @@
 
 package org.apache.hudi.cli.integ;
 
-import org.apache.hadoop.fs.Path;
 import org.apache.hudi.cli.HoodieCLI;
 import org.apache.hudi.cli.commands.TableCommand;
 import org.apache.hudi.cli.testutils.HoodieCLIIntegrationTestBase;
@@ -31,10 +30,12 @@ import org.apache.hudi.common.table.timeline.HoodieInstant;
 import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.common.table.timeline.versioning.TimelineLayoutVersion;
 import org.apache.hudi.common.testutils.HoodieTestDataGenerator;
-
 import org.apache.hudi.config.HoodieWriteConfig;
+import org.apache.hudi.hadoop.fs.HadoopFSUtils;
 import org.apache.hudi.metadata.HoodieTableMetadata;
 import org.apache.hudi.metadata.SparkHoodieBackedTableMetadataWriter;
+import org.apache.hudi.storage.StoragePath;
+
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Disabled;
 import org.junit.jupiter.api.Test;
@@ -65,7 +66,7 @@ public class ITTestSavepointsCommand extends HoodieCLIIntegrationTestBase {
   @BeforeEach
   public void init() throws IOException {
     String tableName = "test_table";
-    tablePath = basePath + Path.SEPARATOR + tableName;
+    tablePath = basePath + StoragePath.SEPARATOR + tableName;
 
     // Create table and connect
     new TableCommand().createTable(
@@ -81,7 +82,8 @@ public class ITTestSavepointsCommand extends HoodieCLIIntegrationTestBase {
     // generate four savepoints
     for (int i = 100; i < 104; i++) {
       String instantTime = String.valueOf(i);
-      HoodieTestDataGenerator.createCommitFile(tablePath, instantTime, jsc.hadoopConfiguration());
+      HoodieTestDataGenerator.createCommitFile(
+          tablePath, instantTime, HadoopFSUtils.getStorageConf(jsc.hadoopConfiguration()));
     }
 
     String savepoint = "102";
@@ -112,12 +114,14 @@ public class ITTestSavepointsCommand extends HoodieCLIIntegrationTestBase {
     // generate four commits
     for (int i = 100; i < 104; i++) {
       String instantTime = String.valueOf(i);
-      HoodieTestDataGenerator.createCommitFile(tablePath, instantTime, jsc.hadoopConfiguration());
+      HoodieTestDataGenerator.createCommitFile(
+          tablePath, instantTime, HadoopFSUtils.getStorageConf(jsc.hadoopConfiguration()));
     }
 
     // generate one savepoint
     String savepoint = "102";
-    HoodieTestDataGenerator.createSavepointFile(tablePath, savepoint, jsc.hadoopConfiguration());
+    HoodieTestDataGenerator.createSavepointFile(
+        tablePath, savepoint, HadoopFSUtils.getStorageConf(jsc.hadoopConfiguration()));
 
     result = shell.evaluate(() ->
             String.format("savepoint rollback --savepoint %s --sparkMaster %s", savepoint, "local"));
@@ -133,7 +137,7 @@ public class ITTestSavepointsCommand extends HoodieCLIIntegrationTestBase {
     assertEquals(1, timeline.getRestoreTimeline().countInstants());
 
     // 103 instant had rollback
-    assertFalse(timeline.getCommitTimeline().containsInstant(
+    assertFalse(timeline.getCommitAndReplaceTimeline().containsInstant(
         new HoodieInstant(HoodieInstant.State.COMPLETED, "commit", "103")));
   }
 
@@ -141,25 +145,28 @@ public class ITTestSavepointsCommand extends HoodieCLIIntegrationTestBase {
    * Test case of command 'savepoint rollback' with metadata table bootstrap.
    */
   @Disabled("HUDI-6571")
-  public void testRollbackToSavepointWithMetadataTableEnable() throws IOException {
+  public void testRollbackToSavepointWithMetadataTableEnable() throws Exception {
     // generate for savepoints
     for (int i = 101; i < 105; i++) {
       String instantTime = String.valueOf(i);
-      HoodieTestDataGenerator.createCommitFile(tablePath, instantTime, jsc.hadoopConfiguration());
+      HoodieTestDataGenerator.createCommitFile(
+          tablePath, instantTime, HadoopFSUtils.getStorageConf(jsc.hadoopConfiguration()));
     }
 
     // generate one savepoint at 102
     String savepoint = "102";
-    HoodieTestDataGenerator.createSavepointFile(tablePath, savepoint, jsc.hadoopConfiguration());
+    HoodieTestDataGenerator.createSavepointFile(
+        tablePath, savepoint, HadoopFSUtils.getStorageConf(jsc.hadoopConfiguration()));
 
     // re-bootstrap metadata table
-    Path metadataTableBasePath = new Path(HoodieTableMetadata.getMetadataTableBasePath(HoodieCLI.basePath));
+    StoragePath metadataTableBasePath =
+        new StoragePath(HoodieTableMetadata.getMetadataTableBasePath(HoodieCLI.basePath));
     // then bootstrap metadata table at instant 104
     HoodieWriteConfig writeConfig = HoodieWriteConfig.newBuilder().withPath(HoodieCLI.basePath)
         .withMetadataConfig(HoodieMetadataConfig.newBuilder().enable(true).build()).build();
-    SparkHoodieBackedTableMetadataWriter.create(HoodieCLI.conf, writeConfig, new HoodieSparkEngineContext(jsc));
+    SparkHoodieBackedTableMetadataWriter.create(HoodieCLI.conf, writeConfig, new HoodieSparkEngineContext(jsc)).close();
 
-    assertTrue(HoodieCLI.fs.exists(metadataTableBasePath));
+    assertTrue(HoodieCLI.storage.exists(metadataTableBasePath));
 
     // roll back to savepoint
     Object result = shell.evaluate(() ->
@@ -175,9 +182,9 @@ public class ITTestSavepointsCommand extends HoodieCLIIntegrationTestBase {
     assertEquals(1, timeline.getRestoreTimeline().countInstants());
 
     // 103 and 104 instant had rollback
-    assertFalse(timeline.getCommitTimeline().containsInstant(
+    assertFalse(timeline.getCommitAndReplaceTimeline().containsInstant(
         new HoodieInstant(HoodieInstant.State.COMPLETED, "commit", "103")));
-    assertFalse(timeline.getCommitTimeline().containsInstant(
+    assertFalse(timeline.getCommitAndReplaceTimeline().containsInstant(
         new HoodieInstant(HoodieInstant.State.COMPLETED, "commit", "104")));
   }
 
@@ -189,14 +196,17 @@ public class ITTestSavepointsCommand extends HoodieCLIIntegrationTestBase {
     // generate four savepoints
     for (int i = 100; i < 104; i++) {
       String instantTime = String.valueOf(i);
-      HoodieTestDataGenerator.createCommitFile(tablePath, instantTime, jsc.hadoopConfiguration());
+      HoodieTestDataGenerator.createCommitFile(
+          tablePath, instantTime, HadoopFSUtils.getStorageConf(jsc.hadoopConfiguration()));
     }
 
     // generate two savepoint
     String savepoint1 = "100";
     String savepoint2 = "102";
-    HoodieTestDataGenerator.createSavepointFile(tablePath, savepoint1, jsc.hadoopConfiguration());
-    HoodieTestDataGenerator.createSavepointFile(tablePath, savepoint2, jsc.hadoopConfiguration());
+    HoodieTestDataGenerator.createSavepointFile(
+        tablePath, savepoint1, HadoopFSUtils.getStorageConf(jsc.hadoopConfiguration()));
+    HoodieTestDataGenerator.createSavepointFile(
+        tablePath, savepoint2, HadoopFSUtils.getStorageConf(jsc.hadoopConfiguration()));
 
     HoodieActiveTimeline timeline = HoodieCLI.getTableMetaClient().getActiveTimeline();
     assertEquals(2, timeline.getSavePointTimeline().countInstants(), "There should 2 instants.");
