@@ -18,7 +18,6 @@
 
 package org.apache.hudi.client;
 
-import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.HoodieAvroIndexedRecord;
 import org.apache.hudi.common.model.HoodieAvroRecord;
 import org.apache.hudi.common.model.HoodieKey;
@@ -28,13 +27,15 @@ import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.common.table.view.FileSystemViewStorageConfig;
 import org.apache.hudi.common.testutils.HoodieTestUtils;
 import org.apache.hudi.common.testutils.RawTripTestPayload;
-import org.apache.hudi.common.util.BaseFileUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.exception.HoodieUpsertException;
+import org.apache.hudi.hadoop.fs.HadoopFSUtils;
 import org.apache.hudi.io.CreateHandleFactory;
 import org.apache.hudi.io.HoodieMergeHandle;
 import org.apache.hudi.io.HoodieWriteHandle;
+import org.apache.hudi.io.storage.HoodieIOFactory;
+import org.apache.hudi.storage.StoragePath;
 import org.apache.hudi.table.HoodieSparkTable;
 import org.apache.hudi.testutils.HoodieSparkClientTestHarness;
 
@@ -66,9 +67,9 @@ public class TestUpdateSchemaEvolution extends HoodieSparkClientTestHarness impl
   @BeforeEach
   public void setUp() throws Exception {
     initPath();
-    HoodieTestUtils.init(HoodieTestUtils.getDefaultHadoopConf(), basePath);
+    HoodieTestUtils.init(HoodieTestUtils.getDefaultStorageConf(), basePath);
     initSparkContexts("TestUpdateSchemaEvolution");
-    initFileSystem();
+    initHoodieStorage();
     initTimelineService();
   }
 
@@ -99,7 +100,7 @@ public class TestUpdateSchemaEvolution extends HoodieSparkClientTestHarness impl
     }).collect();
 
     final Path commitFile = new Path(config.getBasePath() + "/.hoodie/" + HoodieTimeline.makeCommitFileName("100"));
-    FSUtils.getFs(basePath, HoodieTestUtils.getDefaultHadoopConf()).create(commitFile);
+    HadoopFSUtils.getFs(basePath, HoodieTestUtils.getDefaultStorageConf()).create(commitFile);
     return statuses.get(0);
   }
 
@@ -131,9 +132,10 @@ public class TestUpdateSchemaEvolution extends HoodieSparkClientTestHarness impl
       Executable executable = () -> {
         HoodieMergeHandle mergeHandle = new HoodieMergeHandle(updateTable.getConfig(), "101", updateTable,
             updateRecords.iterator(), updateRecords.get(0).getPartitionPath(), insertResult.getFileId(), supplier, Option.empty());
-        List<GenericRecord> oldRecords = BaseFileUtils.getInstance(updateTable.getBaseFileFormat())
-            .readAvroRecords(updateTable.getHadoopConf(),
-                new Path(updateTable.getConfig().getBasePath() + "/" + insertResult.getStat().getPath()),
+        List<GenericRecord> oldRecords = HoodieIOFactory.getIOFactory(updateTable.getStorage())
+            .getFileFormatUtils(updateTable.getBaseFileFormat())
+            .readAvroRecords(updateTable.getStorage(),
+                new StoragePath(updateTable.getConfig().getBasePath() + "/" + insertResult.getStat().getPath()),
                 mergeHandle.getWriterSchemaWithMetaFields());
         for (GenericRecord rec : oldRecords) {
           // TODO create hoodie record with rec can getRecordKey
