@@ -18,15 +18,15 @@
 
 package org.apache.hudi.table;
 
-import org.apache.hudi.common.fs.ConsistencyGuard;
 import org.apache.hudi.common.fs.ConsistencyGuardConfig;
 import org.apache.hudi.common.fs.FailSafeConsistencyGuard;
 import org.apache.hudi.common.fs.OptimisticConsistencyGuard;
 import org.apache.hudi.common.table.HoodieTableConfig;
 import org.apache.hudi.common.testutils.FileCreateUtils;
-import org.apache.hudi.testutils.HoodieClientTestHarness;
+import org.apache.hudi.common.fs.ConsistencyGuard;
+import org.apache.hudi.storage.StoragePath;
+import org.apache.hudi.testutils.HoodieSparkClientTestHarness;
 
-import org.apache.hadoop.fs.Path;
 import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
@@ -43,7 +43,7 @@ import static org.junit.jupiter.api.Assertions.assertThrows;
 /**
  * Unit tests {@link ConsistencyGuard}s.
  */
-public class TestConsistencyGuard extends HoodieClientTestHarness {
+public class TestConsistencyGuard extends HoodieSparkClientTestHarness {
 
   private static final String BASE_FILE_EXTENSION = HoodieTableConfig.BASE_FILE_FORMAT.defaultValue().getFileExtension();
 
@@ -75,17 +75,24 @@ public class TestConsistencyGuard extends HoodieClientTestHarness {
 
     ConsistencyGuardConfig config = getConsistencyGuardConfig(1, 1000, 1000);
     ConsistencyGuard passing = consistencyGuardType.equals(FailSafeConsistencyGuard.class.getName())
-        ? new FailSafeConsistencyGuard(fs, config) : new OptimisticConsistencyGuard(fs, config);
-    passing.waitTillFileAppears(new Path(basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION));
-    passing.waitTillFileAppears(new Path(basePath + "/partition/path/f2_1-0-1_000" + BASE_FILE_EXTENSION));
+        ? new FailSafeConsistencyGuard(storage, config) :
+        new OptimisticConsistencyGuard(storage, config);
+    passing.waitTillFileAppears(
+        new StoragePath(basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION));
+    passing.waitTillFileAppears(
+        new StoragePath(basePath + "/partition/path/f2_1-0-1_000" + BASE_FILE_EXTENSION));
     passing.waitTillAllFilesAppear(basePath + "/partition/path", Arrays
         .asList(basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION,
             basePath + "/partition/path/f2_1-0-1_000" + BASE_FILE_EXTENSION));
 
-    fs.delete(new Path(basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION), false);
-    fs.delete(new Path(basePath + "/partition/path/f2_1-0-1_000" + BASE_FILE_EXTENSION), false);
-    passing.waitTillFileDisappears(new Path(basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION));
-    passing.waitTillFileDisappears(new Path(basePath + "/partition/path/f2_1-0-1_000" + BASE_FILE_EXTENSION));
+    storage.deleteFile(new StoragePath(
+        basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION));
+    storage.deleteFile(new StoragePath(
+        basePath + "/partition/path/f2_1-0-1_000" + BASE_FILE_EXTENSION));
+    passing.waitTillFileDisappears(
+        new StoragePath(basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION));
+    passing.waitTillFileDisappears(
+        new StoragePath(basePath + "/partition/path/f2_1-0-1_000" + BASE_FILE_EXTENSION));
     passing.waitTillAllFilesDisappear(basePath + "/partition/path", Arrays
         .asList(basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION,
             basePath + "/partition/path/f2_1-0-1_000" + BASE_FILE_EXTENSION));
@@ -94,7 +101,7 @@ public class TestConsistencyGuard extends HoodieClientTestHarness {
   @Test
   public void testCheckFailingAppearFailSafe() throws Exception {
     FileCreateUtils.createBaseFile(basePath, "partition/path", "000", "f1");
-    ConsistencyGuard passing = new FailSafeConsistencyGuard(fs, getConsistencyGuardConfig());
+    ConsistencyGuard passing = new FailSafeConsistencyGuard(storage, getConsistencyGuardConfig());
     assertThrows(TimeoutException.class, () -> {
       passing.waitTillAllFilesAppear(basePath + "/partition/path", Arrays
           .asList(basePath + "/partition/path/f1_1-0-2_000" + BASE_FILE_EXTENSION,
@@ -105,7 +112,7 @@ public class TestConsistencyGuard extends HoodieClientTestHarness {
   @Test
   public void testCheckFailingAppearTimedWait() throws Exception {
     FileCreateUtils.createBaseFile(basePath, "partition/path", "000", "f1");
-    ConsistencyGuard passing = new OptimisticConsistencyGuard(fs, getConsistencyGuardConfig());
+    ConsistencyGuard passing = new OptimisticConsistencyGuard(storage, getConsistencyGuardConfig());
     passing.waitTillAllFilesAppear(basePath + "/partition/path", Arrays
           .asList(basePath + "/partition/path/f1_1-0-2_000" + BASE_FILE_EXTENSION,
               basePath + "/partition/path/f2_1-0-2_000" + BASE_FILE_EXTENSION));
@@ -114,23 +121,25 @@ public class TestConsistencyGuard extends HoodieClientTestHarness {
   @Test
   public void testCheckFailingAppearsFailSafe() throws Exception {
     FileCreateUtils.createBaseFile(basePath, "partition/path", "000", "f1");
-    ConsistencyGuard passing = new FailSafeConsistencyGuard(fs, getConsistencyGuardConfig());
+    ConsistencyGuard passing = new FailSafeConsistencyGuard(storage, getConsistencyGuardConfig());
     assertThrows(TimeoutException.class, () -> {
-      passing.waitTillFileAppears(new Path(basePath + "/partition/path/f1_1-0-2_000" + BASE_FILE_EXTENSION));
+      passing.waitTillFileAppears(
+          new StoragePath(basePath + "/partition/path/f1_1-0-2_000" + BASE_FILE_EXTENSION));
     });
   }
 
   @Test
   public void testCheckFailingAppearsTimedWait() throws Exception {
     FileCreateUtils.createBaseFile(basePath, "partition/path", "000", "f1");
-    ConsistencyGuard passing = new OptimisticConsistencyGuard(fs, getConsistencyGuardConfig());
-    passing.waitTillFileAppears(new Path(basePath + "/partition/path/f1_1-0-2_000" + BASE_FILE_EXTENSION));
+    ConsistencyGuard passing = new OptimisticConsistencyGuard(storage, getConsistencyGuardConfig());
+    passing.waitTillFileAppears(
+        new StoragePath(basePath + "/partition/path/f1_1-0-2_000" + BASE_FILE_EXTENSION));
   }
 
   @Test
   public void testCheckFailingDisappearFailSafe() throws Exception {
     FileCreateUtils.createBaseFile(basePath, "partition/path", "000", "f1");
-    ConsistencyGuard passing = new FailSafeConsistencyGuard(fs, getConsistencyGuardConfig());
+    ConsistencyGuard passing = new FailSafeConsistencyGuard(storage, getConsistencyGuardConfig());
     assertThrows(TimeoutException.class, () -> {
       passing.waitTillAllFilesDisappear(basePath + "/partition/path", Arrays
           .asList(basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION,
@@ -141,7 +150,7 @@ public class TestConsistencyGuard extends HoodieClientTestHarness {
   @Test
   public void testCheckFailingDisappearTimedWait() throws Exception {
     FileCreateUtils.createBaseFile(basePath, "partition/path", "000", "f1");
-    ConsistencyGuard passing = new OptimisticConsistencyGuard(fs, getConsistencyGuardConfig());
+    ConsistencyGuard passing = new OptimisticConsistencyGuard(storage, getConsistencyGuardConfig());
     passing.waitTillAllFilesDisappear(basePath + "/partition/path", Arrays
           .asList(basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION,
               basePath + "/partition/path/f2_1-0-2_000" + BASE_FILE_EXTENSION));
@@ -151,9 +160,10 @@ public class TestConsistencyGuard extends HoodieClientTestHarness {
   public void testCheckFailingDisappearsFailSafe() throws Exception {
     FileCreateUtils.createBaseFile(basePath, "partition/path", "000", "f1");
     FileCreateUtils.createBaseFile(basePath, "partition/path", "000", "f1");
-    ConsistencyGuard passing = new FailSafeConsistencyGuard(fs, getConsistencyGuardConfig());
+    ConsistencyGuard passing = new FailSafeConsistencyGuard(storage, getConsistencyGuardConfig());
     assertThrows(TimeoutException.class, () -> {
-      passing.waitTillFileDisappears(new Path(basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION));
+      passing.waitTillFileDisappears(
+          new StoragePath(basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION));
     });
   }
 
@@ -161,8 +171,9 @@ public class TestConsistencyGuard extends HoodieClientTestHarness {
   public void testCheckFailingDisappearsTimedWait() throws Exception {
     FileCreateUtils.createBaseFile(basePath, "partition/path", "000", "f1");
     FileCreateUtils.createBaseFile(basePath, "partition/path", "000", "f1");
-    ConsistencyGuard passing = new OptimisticConsistencyGuard(fs, getConsistencyGuardConfig());
-    passing.waitTillFileDisappears(new Path(basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION));
+    ConsistencyGuard passing = new OptimisticConsistencyGuard(storage, getConsistencyGuardConfig());
+    passing.waitTillFileDisappears(
+        new StoragePath(basePath + "/partition/path/f1_1-0-1_000" + BASE_FILE_EXTENSION));
   }
 
   private ConsistencyGuardConfig getConsistencyGuardConfig() {
