@@ -51,6 +51,7 @@ import org.apache.hudi.common.util.collection.Pair;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.data.HoodieJavaRDD;
 import org.apache.hudi.exception.HoodieMetadataException;
+import org.apache.hudi.hadoop.fs.HadoopFSUtils;
 import org.apache.hudi.index.HoodieIndex;
 import org.apache.hudi.metadata.FileSystemBackedTableMetadata;
 import org.apache.hudi.metadata.HoodieBackedTableMetadataWriter;
@@ -58,6 +59,11 @@ import org.apache.hudi.metadata.HoodieTableMetadata;
 import org.apache.hudi.metadata.HoodieTableMetadataWriter;
 import org.apache.hudi.metadata.MetadataPartitionType;
 import org.apache.hudi.metadata.SparkHoodieBackedTableMetadataWriter;
+import org.apache.hudi.storage.HoodieStorage;
+import org.apache.hudi.storage.HoodieStorageUtils;
+import org.apache.hudi.storage.StorageConfiguration;
+import org.apache.hudi.storage.StoragePath;
+import org.apache.hudi.storage.StoragePathInfo;
 import org.apache.hudi.table.HoodieSparkTable;
 import org.apache.hudi.table.HoodieTable;
 import org.apache.hudi.table.WorkloadStat;
@@ -69,7 +75,8 @@ import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileStatus;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.LocalFileSystem;
-import org.apache.hadoop.fs.Path;
+import org.apache.spark.SparkConf;
+import org.apache.spark.SparkContext;
 import org.apache.spark.api.java.JavaRDD;
 import org.apache.spark.api.java.JavaSparkContext;
 import org.apache.spark.sql.SQLContext;
@@ -99,6 +106,7 @@ import java.util.stream.Collectors;
 
 import scala.Tuple2;
 
+import static org.apache.hudi.common.testutils.HoodieTestUtils.getDefaultStorageConf;
 import static org.apache.hudi.common.util.CleanerUtils.convertCleanMetadata;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
@@ -122,9 +130,9 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
   protected JavaSparkContext jsc;
   protected HoodieSparkEngineContext context;
   protected SparkSession sparkSession;
-  protected Configuration hadoopConf;
+  protected StorageConfiguration<Configuration> storageConf;
   protected SQLContext sqlContext;
-  protected FileSystem fs;
+  protected HoodieStorage storage;
   protected ExecutorService executorService;
   protected HoodieTableMetaClient metaClient;
   protected SparkRDDWriteClient writeClient;
@@ -152,7 +160,7 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
     initPath();
     initSparkContexts();
     initTestDataGenerator();
-    initFileSystem();
+    initHoodieStorage();
     initMetaClient();
     initTimelineService();
   }
@@ -192,11 +200,12 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
     }
 
     // Initialize a local spark env
-    jsc = new JavaSparkContext(HoodieClientTestUtils.getSparkConfForTest(appName + "#" + testMethodName));
+    SparkConf sc = HoodieClientTestUtils.getSparkConfForTest(appName + "#" + testMethodName);
+    SparkContext sparkContext = new SparkContext(sc);
+    HoodieClientTestUtils.overrideSparkHadoopConfiguration(sparkContext);
+    jsc = new JavaSparkContext(sparkContext);
     jsc.setLogLevel("ERROR");
-
-    hadoopConf = jsc.hadoopConfiguration();
-
+    storageConf = HadoopFSUtils.getStorageConf(jsc.hadoopConfiguration());
     sparkSession = SparkSession.builder()
         .withExtensions(JFunction.toScala(sparkSessionExtensions -> {
           sparkSessionExtensionsInjector.ifPresent(injector -> injector.accept(sparkSessionExtensions));
@@ -247,19 +256,19 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
   /**
    * Initializes a file system with the hadoop configuration of Spark context.
    */
-  protected void initFileSystem() {
+  protected void initHoodieStorage() {
     if (jsc == null) {
       throw new IllegalStateException("The Spark context has not been initialized.");
     }
 
-    initFileSystemWithConfiguration(hadoopConf);
+    initFileSystemWithConfiguration(storageConf);
   }
 
   /**
    * Initializes file system with a default empty configuration.
    */
   protected void initFileSystemWithDefaultConfiguration() {
-    initFileSystemWithConfiguration(new Configuration());
+    initFileSystemWithConfiguration(getDefaultStorageConf());
   }
 
   /**
@@ -268,10 +277,10 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
    * @throws IOException
    */
   protected void cleanupFileSystem() throws IOException {
-    if (fs != null) {
+    if (storage != null) {
       LOG.warn("Closing file-system instance used in previous test-run");
-      fs.close();
-      fs = null;
+      storage.close();
+      storage = null;
     }
   }
 
@@ -306,7 +315,7 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
     if (tableName != null && !tableName.isEmpty()) {
       properties.put(HoodieTableConfig.NAME.key(), tableName);
     }
-    metaClient = HoodieTestUtils.init(hadoopConf, basePath, tableType, properties);
+    metaClient = HoodieTestUtils.init(storageConf, basePath, tableType, properties);
   }
 
   /**
@@ -370,18 +379,18 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
     }
   }
 
-  private void initFileSystemWithConfiguration(Configuration configuration) {
+  private void initFileSystemWithConfiguration(StorageConfiguration<?> configuration) {
     if (basePath == null) {
       throw new IllegalStateException("The base path has not been initialized.");
     }
 
-    fs = FSUtils.getFs(basePath, configuration);
+    storage = HoodieStorageUtils.getStorage(basePath, configuration);
+    FileSystem fs = (FileSystem) storage.getFileSystem();
     if (fs instanceof LocalFileSystem) {
-      LocalFileSystem lfs = (LocalFileSystem) fs;
       // With LocalFileSystem, with checksum disabled, fs.open() returns an inputStream which is FSInputStream
       // This causes ClassCastExceptions in LogRecordScanner (and potentially other places) calling fs.open
       // So, for the tests, we enforce checksum verification to circumvent the problem
-      lfs.setVerifyChecksum(true);
+      ((LocalFileSystem) fs).setVerifyChecksum(true);
     }
   }
 
@@ -399,17 +408,18 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
     return writeClient;
   }
 
-  public HoodieTableMetaClient getHoodieMetaClient(Configuration conf, String basePath) {
-    metaClient = HoodieTableMetaClient.builder().setConf(conf).setBasePath(basePath).build();
+  public HoodieTableMetaClient getHoodieMetaClient(StorageConfiguration<?> conf, String basePath) {
+    metaClient = HoodieTestUtils.createMetaClient(conf, basePath);
     return metaClient;
   }
 
-  public HoodieTableFileSystemView getHoodieTableFileSystemView(HoodieTableMetaClient metaClient, HoodieTimeline visibleActiveTimeline,
-                                                                FileStatus[] fileStatuses) {
+  public HoodieTableFileSystemView getHoodieTableFileSystemView(HoodieTableMetaClient metaClient,
+                                                                HoodieTimeline visibleActiveTimeline,
+                                                                List<StoragePathInfo> pathInfoList) {
     if (tableView == null) {
-      tableView = new HoodieTableFileSystemView(metaClient, visibleActiveTimeline, fileStatuses);
+      tableView = new HoodieTableFileSystemView(metaClient, visibleActiveTimeline, pathInfoList);
     } else {
-      tableView.init(metaClient, visibleActiveTimeline, fileStatuses);
+      tableView.init(metaClient, visibleActiveTimeline, pathInfoList);
     }
     return tableView;
   }
@@ -502,13 +512,17 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
     metaClient = HoodieTableMetaClient.reload(metaClient);
     HoodieTable table = HoodieSparkTable.create(writeConfig, engineContext);
     TableFileSystemView tableView = table.getHoodieView();
-    List<String> fullPartitionPaths = fsPartitions.stream().map(partition -> basePath + "/" + partition).collect(Collectors.toList());
-    Map<String, FileStatus[]> partitionToFilesMap = tableMetadata.getAllFilesInPartitions(fullPartitionPaths);
+    List<String> fullPartitionPaths =
+        fsPartitions.stream().map(partition -> basePath + "/" + partition)
+            .collect(Collectors.toList());
+    Map<String, List<StoragePathInfo>> partitionToFilesMap =
+        tableMetadata.getAllFilesInPartitions(fullPartitionPaths);
     assertEquals(fsPartitions.size(), partitionToFilesMap.size());
 
     fsPartitions.forEach(partition -> {
       try {
-        validateFilesPerPartition(testTable, tableMetadata, tableView, partitionToFilesMap, partition);
+        validateFilesPerPartition(testTable, tableMetadata, tableView, partitionToFilesMap,
+            partition);
       } catch (IOException e) {
         fail("Exception should not be raised: " + e);
       }
@@ -521,11 +535,11 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
   }
 
   public void syncTableMetadata(HoodieWriteConfig writeConfig) {
-    if (!writeConfig.getMetadataConfig().enabled()) {
+    if (!writeConfig.getMetadataConfig().isEnabled()) {
       return;
     }
     // Open up the metadata table again, for syncing
-    try (HoodieTableMetadataWriter writer = SparkHoodieBackedTableMetadataWriter.create(hadoopConf, writeConfig, context)) {
+    try (HoodieTableMetadataWriter writer = SparkHoodieBackedTableMetadataWriter.create(storageConf, writeConfig, context)) {
       LOG.info("Successfully synced to metadata table");
     } catch (Exception e) {
       throw new HoodieMetadataException("Error syncing to metadata table.", e);
@@ -534,54 +548,65 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
 
   public HoodieBackedTableMetadataWriter metadataWriter(HoodieWriteConfig clientConfig) {
     return (HoodieBackedTableMetadataWriter) SparkHoodieBackedTableMetadataWriter
-        .create(hadoopConf, clientConfig, new HoodieSparkEngineContext(jsc));
+        .create(storageConf, clientConfig, new HoodieSparkEngineContext(jsc));
   }
 
-  public HoodieTableMetadata metadata(HoodieWriteConfig clientConfig, HoodieEngineContext hoodieEngineContext) {
-    return HoodieTableMetadata.create(hoodieEngineContext, clientConfig.getMetadataConfig(), clientConfig.getBasePath());
+  public HoodieTableMetadata metadata(HoodieWriteConfig clientConfig,
+                                      HoodieEngineContext hoodieEngineContext) {
+    return HoodieTableMetadata.create(
+        hoodieEngineContext, storage, clientConfig.getMetadataConfig(), clientConfig.getBasePath());
   }
 
-  protected void validateFilesPerPartition(HoodieTestTable testTable, HoodieTableMetadata tableMetadata, TableFileSystemView tableView,
-                                           Map<String, FileStatus[]> partitionToFilesMap, String partition) throws IOException {
-    Path partitionPath;
+  protected void validateFilesPerPartition(HoodieTestTable testTable,
+                                           HoodieTableMetadata tableMetadata,
+                                           TableFileSystemView tableView,
+                                           Map<String, List<StoragePathInfo>> partitionToFilesMap,
+                                           String partition) throws IOException {
+    StoragePath partitionPath;
     if (partition.equals("")) {
       // Should be the non-partitioned case
-      partitionPath = new Path(basePath);
+      partitionPath = new StoragePath(basePath);
     } else {
-      partitionPath = new Path(basePath, partition);
+      partitionPath = new StoragePath(basePath, partition);
     }
 
     FileStatus[] fsStatuses = testTable.listAllFilesInPartition(partition);
-    FileStatus[] metaStatuses = tableMetadata.getAllFilesInPartition(partitionPath);
+    List<StoragePathInfo> metaFilesList = tableMetadata.getAllFilesInPartition(partitionPath);
     List<String> fsFileNames = Arrays.stream(fsStatuses)
         .map(s -> s.getPath().getName()).collect(Collectors.toList());
-    List<String> metadataFilenames = Arrays.stream(metaStatuses)
+    List<String> metadataFilenames = metaFilesList.stream()
         .map(s -> s.getPath().getName()).collect(Collectors.toList());
     Collections.sort(fsFileNames);
     Collections.sort(metadataFilenames);
 
     assertLinesMatch(fsFileNames, metadataFilenames);
-    assertEquals(fsStatuses.length, partitionToFilesMap.get(partitionPath.toString()).length);
+    assertEquals(fsStatuses.length, partitionToFilesMap.get(partitionPath.toString()).size());
 
     // Block sizes should be valid
-    Arrays.stream(metaStatuses).forEach(s -> assertTrue(s.getBlockSize() > 0));
+    metaFilesList.forEach(s -> assertTrue(s.getBlockSize() > 0));
     List<Long> fsBlockSizes = Arrays.stream(fsStatuses).map(FileStatus::getBlockSize).sorted().collect(Collectors.toList());
-    List<Long> metadataBlockSizes = Arrays.stream(metaStatuses).map(FileStatus::getBlockSize).sorted().collect(Collectors.toList());
+    List<Long> metadataBlockSizes = metaFilesList.stream().map(StoragePathInfo::getBlockSize).sorted().collect(Collectors.toList());
     assertEquals(fsBlockSizes, metadataBlockSizes);
 
-    assertEquals(fsFileNames.size(), metadataFilenames.size(), "Files within partition " + partition + " should match");
-    assertEquals(fsFileNames, metadataFilenames, "Files within partition " + partition + " should match");
+    assertEquals(fsFileNames.size(), metadataFilenames.size(),
+        "Files within partition " + partition + " should match");
+    assertEquals(fsFileNames, metadataFilenames,
+        "Files within partition " + partition + " should match");
 
     // FileSystemView should expose the same data
-    List<HoodieFileGroup> fileGroups = tableView.getAllFileGroups(partition).collect(Collectors.toList());
+    List<HoodieFileGroup> fileGroups =
+        tableView.getAllFileGroups(partition).collect(Collectors.toList());
     fileGroups.addAll(tableView.getAllReplacedFileGroups(partition).collect(Collectors.toList()));
 
     fileGroups.forEach(g -> LoggerFactory.getLogger(getClass()).info(g.toString()));
-    fileGroups.forEach(g -> g.getAllBaseFiles().forEach(b -> LoggerFactory.getLogger(getClass()).info(b.toString())));
-    fileGroups.forEach(g -> g.getAllFileSlices().forEach(s -> LoggerFactory.getLogger(getClass()).info(s.toString())));
+    fileGroups.forEach(g -> g.getAllBaseFiles()
+        .forEach(b -> LoggerFactory.getLogger(getClass()).info(b.toString())));
+    fileGroups.forEach(g -> g.getAllFileSlices()
+        .forEach(s -> LoggerFactory.getLogger(getClass()).info(s.toString())));
 
     long numFiles = fileGroups.stream()
-        .mapToLong(g -> g.getAllBaseFiles().count() + g.getAllFileSlices().mapToLong(s -> s.getLogFiles().count()).sum())
+        .mapToLong(g -> g.getAllBaseFiles().count()
+            + g.getAllFileSlices().mapToLong(s -> s.getLogFiles().count()).sum())
         .sum();
     assertEquals(metadataFilenames.size(), numFiles);
   }
@@ -597,7 +622,7 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
     HoodieWriteConfig metadataWriteConfig = metadataWriter.getWriteConfig();
     assertFalse(metadataWriteConfig.isMetadataTableEnabled(), "No metadata table for metadata table");
 
-    HoodieTableMetaClient metadataMetaClient = HoodieTableMetaClient.builder().setConf(hadoopConf).setBasePath(metadataTableBasePath).build();
+    HoodieTableMetaClient metadataMetaClient = HoodieTestUtils.createMetaClient(storageConf, metadataTableBasePath);
 
     // Metadata table is MOR
     assertEquals(metadataMetaClient.getTableType(), HoodieTableType.MERGE_ON_READ, "Metadata Table should be MOR");
@@ -609,7 +634,7 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
     // Metadata table has a fixed number of partitions
     // Cannot use FSUtils.getAllFoldersWithPartitionMetaFile for this as that function filters all directory
     // in the .hoodie folder.
-    List<String> metadataTablePartitions = FSUtils.getAllPartitionPaths(engineContext, HoodieTableMetadata.getMetadataTableBasePath(basePath),
+    List<String> metadataTablePartitions = FSUtils.getAllPartitionPaths(engineContext, storage, HoodieTableMetadata.getMetadataTableBasePath(basePath),
         false, false);
 
     List<MetadataPartitionType> enabledPartitionTypes = metadataWriter.getEnabledPartitionTypes();
@@ -645,7 +670,7 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
 
   public HoodieInstant createCleanMetadata(String instantTime, boolean inflightOnly, boolean isEmptyForAll, boolean isEmptyCompleted) throws IOException {
     HoodieCleanerPlan cleanerPlan = new HoodieCleanerPlan(new HoodieActionInstant("", "", ""), "", "",
-            new HashMap<>(), CleanPlanV2MigrationHandler.VERSION, new HashMap<>(), new ArrayList<>());
+            new HashMap<>(), CleanPlanV2MigrationHandler.VERSION, new HashMap<>(), new ArrayList<>(), Collections.EMPTY_MAP);
     if (inflightOnly) {
       HoodieTestTable.of(metaClient).addInflightClean(instantTime, cleanerPlan);
     } else {
@@ -657,9 +682,21 @@ public abstract class HoodieSparkClientTestHarness extends HoodieWriterClientTes
               Collections.emptyList(),
               instantTime,
               "");
-      HoodieCleanMetadata cleanMetadata = convertCleanMetadata(instantTime, Option.of(0L), Collections.singletonList(cleanStats));
+      HoodieCleanMetadata cleanMetadata = convertCleanMetadata(instantTime, Option.of(0L), Collections.singletonList(cleanStats), Collections.EMPTY_MAP);
       HoodieTestTable.of(metaClient).addClean(instantTime, cleanerPlan, cleanMetadata, isEmptyForAll, isEmptyCompleted);
     }
     return new HoodieInstant(inflightOnly, "clean", instantTime);
   }
+
+  protected HoodieTableMetaClient createMetaClient(String basePath) {
+    return HoodieTestUtils.createMetaClient(storageConf, basePath);
+  }
+
+  protected HoodieTableMetaClient createMetaClient(SparkSession spark, String basePath) {
+    return HoodieClientTestUtils.createMetaClient(spark, basePath);
+  }
+
+  protected HoodieTableMetaClient createMetaClient(JavaSparkContext context, String basePath) {
+    return HoodieClientTestUtils.createMetaClient(context, basePath);
+  }
 }
