@@ -23,17 +23,19 @@ import org.apache.hudi.client.SparkTaskContextSupplier;
 import org.apache.hudi.common.bloom.BloomFilter;
 import org.apache.hudi.common.bloom.BloomFilterFactory;
 import org.apache.hudi.common.bloom.BloomFilterTypeCode;
+import org.apache.hudi.common.engine.HoodieEngineContext;
 import org.apache.hudi.common.model.HoodieLogFile;
 import org.apache.hudi.common.model.HoodieRecord;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
+import org.apache.hudi.common.util.Option;
 import org.apache.hudi.metadata.HoodieTableMetadataWriter;
+import org.apache.hudi.storage.StoragePath;
+import org.apache.hudi.storage.HoodieStorage;
 import org.apache.hudi.table.HoodieTable;
 
 import org.apache.avro.Schema;
-import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.Path;
-import org.apache.log4j.LogManager;
-import org.apache.log4j.Logger;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import java.util.Arrays;
 import java.util.List;
@@ -41,22 +43,44 @@ import java.util.Map;
 import java.util.UUID;
 
 public class HoodieSparkWriteableTestTable extends HoodieWriteableTestTable {
-  private static final Logger LOG = LogManager.getLogger(HoodieSparkWriteableTestTable.class);
+  private static final Logger LOG = LoggerFactory.getLogger(HoodieSparkWriteableTestTable.class);
 
-  private HoodieSparkWriteableTestTable(String basePath, FileSystem fs, HoodieTableMetaClient metaClient, Schema schema,
-                                        BloomFilter filter, HoodieTableMetadataWriter metadataWriter) {
-    super(basePath, fs, metaClient, schema, filter, metadataWriter);
+  private HoodieSparkWriteableTestTable(String basePath, HoodieStorage storage,
+                                        HoodieTableMetaClient metaClient, Schema schema,
+                                        BloomFilter filter,
+                                        HoodieTableMetadataWriter metadataWriter) {
+    this(basePath, storage, metaClient, schema, filter, metadataWriter, Option.empty());
   }
 
-  public static HoodieSparkWriteableTestTable of(HoodieTableMetaClient metaClient, Schema schema, BloomFilter filter) {
-    return new HoodieSparkWriteableTestTable(metaClient.getBasePath(), metaClient.getRawFs(),
-        metaClient, schema, filter, null);
+  private HoodieSparkWriteableTestTable(String basePath, HoodieStorage storage,
+                                        HoodieTableMetaClient metaClient, Schema schema,
+                                        BloomFilter filter,
+                                        HoodieTableMetadataWriter metadataWriter,
+                                        Option<HoodieEngineContext> context) {
+    super(basePath, storage, metaClient, schema, filter, metadataWriter, context);
+  }
+
+  public static HoodieSparkWriteableTestTable of(HoodieTableMetaClient metaClient, Schema schema,
+                                                 BloomFilter filter) {
+    return of(metaClient, schema, filter, Option.empty());
+  }
+
+  public static HoodieSparkWriteableTestTable of(HoodieTableMetaClient metaClient, Schema schema, BloomFilter filter, Option<HoodieEngineContext> context) {
+    return new HoodieSparkWriteableTestTable(metaClient.getBasePath(),
+        metaClient.getRawHoodieStorage(),
+        metaClient, schema, filter, null, context);
   }
 
   public static HoodieSparkWriteableTestTable of(HoodieTableMetaClient metaClient, Schema schema, BloomFilter filter,
                                                  HoodieTableMetadataWriter metadataWriter) {
-    return new HoodieSparkWriteableTestTable(metaClient.getBasePath(), metaClient.getRawFs(),
-        metaClient, schema, filter, metadataWriter);
+    return of(metaClient, schema, filter, metadataWriter, Option.empty());
+  }
+
+  public static HoodieSparkWriteableTestTable of(HoodieTableMetaClient metaClient, Schema schema, BloomFilter filter,
+                                                 HoodieTableMetadataWriter metadataWriter, Option<HoodieEngineContext> context) {
+    return new HoodieSparkWriteableTestTable(metaClient.getBasePath(),
+        metaClient.getRawHoodieStorage(),
+        metaClient, schema, filter, metadataWriter, context);
   }
 
   public static HoodieSparkWriteableTestTable of(HoodieTableMetaClient metaClient, Schema schema) {
@@ -67,9 +91,14 @@ public class HoodieSparkWriteableTestTable extends HoodieWriteableTestTable {
 
   public static HoodieSparkWriteableTestTable of(HoodieTableMetaClient metaClient, Schema schema,
                                                  HoodieTableMetadataWriter metadataWriter) {
+    return of(metaClient, schema, metadataWriter, Option.empty());
+  }
+
+  public static HoodieSparkWriteableTestTable of(HoodieTableMetaClient metaClient, Schema schema,
+                                                 HoodieTableMetadataWriter metadataWriter, Option<HoodieEngineContext> context) {
     BloomFilter filter = BloomFilterFactory
         .createBloomFilter(10000, 0.0000001, -1, BloomFilterTypeCode.DYNAMIC_V0.name());
-    return of(metaClient, schema, filter, metadataWriter);
+    return of(metaClient, schema, filter, metadataWriter, context);
   }
 
   public static HoodieSparkWriteableTestTable of(HoodieTable hoodieTable, Schema schema) {
@@ -115,7 +144,7 @@ public class HoodieSparkWriteableTestTable extends HoodieWriteableTestTable {
     return this;
   }
 
-  public Path withInserts(String partition, String fileId, List<HoodieRecord> records) throws Exception {
+  public StoragePath withInserts(String partition, String fileId, List<HoodieRecord> records) throws Exception {
     return super.withInserts(partition, fileId, records, new SparkTaskContextSupplier());
   }
 
