@@ -19,6 +19,8 @@
 package org.apache.hudi.common.config;
 
 import org.apache.hudi.common.engine.EngineType;
+import org.apache.hudi.common.table.view.FileSystemViewStorageConfig;
+import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.StringUtils;
 import org.apache.hudi.exception.HoodieNotSupportedException;
 
@@ -41,6 +43,15 @@ import java.util.Properties;
         + " to avoid overhead of accessing cloud storage, during queries.")
 public final class HoodieMetadataConfig extends HoodieConfig {
 
+  // Asynchronous cleaning for metadata table is disabled by default
+  public static final boolean DEFAULT_METADATA_ASYNC_CLEAN = false;
+  // Full scanning of log files while reading log records is enabled by default for metadata table
+  public static final boolean DEFAULT_METADATA_ENABLE_FULL_SCAN_LOG_FILES = true;
+  // Meta fields are not populated by default for metadata table
+  public static final boolean DEFAULT_METADATA_POPULATE_META_FIELDS = false;
+  // Default number of commits to retain, without cleaning, on metadata table
+  public static final int DEFAULT_METADATA_CLEANER_COMMITS_RETAINED = 20;
+
   public static final String METADATA_PREFIX = "hoodie.metadata";
   public static final String OPTIMIZED_LOG_BLOCKS_SCAN = ".optimized.log.blocks.scan.enable";
 
@@ -57,27 +68,15 @@ public final class HoodieMetadataConfig extends HoodieConfig {
   public static final ConfigProperty<Boolean> METRICS_ENABLE = ConfigProperty
       .key(METADATA_PREFIX + ".metrics.enable")
       .defaultValue(false)
+      .markAdvanced()
       .sinceVersion("0.7.0")
       .withDocumentation("Enable publishing of metrics around metadata table.");
 
-  // Parallelism for inserts
-  public static final ConfigProperty<Integer> INSERT_PARALLELISM_VALUE = ConfigProperty
-      .key(METADATA_PREFIX + ".insert.parallelism")
-      .defaultValue(1)
-      .sinceVersion("0.7.0")
-      .withDocumentation("Parallelism to use when inserting to the metadata table");
-
-  // Async clean
-  public static final ConfigProperty<Boolean> ASYNC_CLEAN_ENABLE = ConfigProperty
-      .key(METADATA_PREFIX + ".clean.async")
-      .defaultValue(false)
-      .sinceVersion("0.7.0")
-      .withDocumentation("Enable asynchronous cleaning for metadata table. This is an internal config and setting this will not overwrite the value actually used.");
-
   // Async index
   public static final ConfigProperty<Boolean> ASYNC_INDEX_ENABLE = ConfigProperty
       .key(METADATA_PREFIX + ".index.async")
       .defaultValue(false)
+      .markAdvanced()
       .sinceVersion("0.11.0")
       .withDocumentation("Enable asynchronous indexing of metadata table.");
 
@@ -85,44 +84,37 @@ public final class HoodieMetadataConfig extends HoodieConfig {
   public static final ConfigProperty<Integer> COMPACT_NUM_DELTA_COMMITS = ConfigProperty
       .key(METADATA_PREFIX + ".compact.max.delta.commits")
       .defaultValue(10)
+      .markAdvanced()
       .sinceVersion("0.7.0")
       .withDocumentation("Controls how often the metadata table is compacted.");
 
-  // Archival settings
-  public static final ConfigProperty<Integer> MIN_COMMITS_TO_KEEP = ConfigProperty
-      .key(METADATA_PREFIX + ".keep.min.commits")
-      .defaultValue(20)
-      .sinceVersion("0.7.0")
-      .withDocumentation("Archiving service moves older entries from metadata table’s timeline "
-          + "into an archived log after each write, to keep the overhead constant, even as the "
-          + "metadata table size grows.  This config controls the minimum number of instants "
-          + "to retain in the active timeline.");
+  public static final ConfigProperty<String> ENABLE_LOG_COMPACTION_ON_METADATA_TABLE = ConfigProperty
+      .key(METADATA_PREFIX + ".log.compaction.enable")
+      .defaultValue("false")
+      .markAdvanced()
+      .sinceVersion("0.14.0")
+      .withDocumentation("This configs enables logcompaction for the metadata table.");
 
-  public static final ConfigProperty<Integer> MAX_COMMITS_TO_KEEP = ConfigProperty
-      .key(METADATA_PREFIX + ".keep.max.commits")
-      .defaultValue(30)
-      .sinceVersion("0.7.0")
-      .withDocumentation("Similar to " + MIN_COMMITS_TO_KEEP.key() + ", this config controls "
-          + "the maximum number of instants to retain in the active timeline.");
-
-  // Cleaner commits retained
-  public static final ConfigProperty<Integer> CLEANER_COMMITS_RETAINED = ConfigProperty
-      .key(METADATA_PREFIX + ".cleaner.commits.retained")
-      .defaultValue(3)
-      .sinceVersion("0.7.0")
-      .withDocumentation("Number of commits to retain, without cleaning, on metadata table. "
-          + "This is an internal config and setting this will not overwrite the actual value used.");
+  // Log blocks threshold, after a file slice crosses this threshold log compact operation is scheduled.
+  public static final ConfigProperty<Integer> LOG_COMPACT_BLOCKS_THRESHOLD = ConfigProperty
+      .key(METADATA_PREFIX + ".log.compaction.blocks.threshold")
+      .defaultValue(5)
+      .markAdvanced()
+      .sinceVersion("0.14.0")
+      .withDocumentation("Controls the criteria to log compacted files groups in metadata table.");
 
   // Regex to filter out matching directories during bootstrap
   public static final ConfigProperty<String> DIR_FILTER_REGEX = ConfigProperty
       .key(METADATA_PREFIX + ".dir.filter.regex")
       .defaultValue("")
+      .markAdvanced()
       .sinceVersion("0.7.0")
       .withDocumentation("Directories matching this regex, will be filtered out when initializing metadata table from lake storage for the first time.");
 
   public static final ConfigProperty<String> ASSUME_DATE_PARTITIONING = ConfigProperty
       .key("hoodie.assume.date.partitioning")
       .defaultValue("false")
+      .markAdvanced()
       .sinceVersion("0.3.0")
       .withDocumentation("Should HoodieWriteClient assume the data is partitioned by dates, i.e three levels from base path. "
           + "This is a stop-gap to support tables created by versions < 0.3.1. Will be removed eventually");
@@ -130,16 +122,10 @@ public final class HoodieMetadataConfig extends HoodieConfig {
   public static final ConfigProperty<Integer> FILE_LISTING_PARALLELISM_VALUE = ConfigProperty
       .key("hoodie.file.listing.parallelism")
       .defaultValue(200)
+      .markAdvanced()
       .sinceVersion("0.7.0")
       .withDocumentation("Parallelism to use, when listing the table on lake storage.");
 
-  public static final ConfigProperty<Boolean> ENABLE_FULL_SCAN_LOG_FILES = ConfigProperty
-      .key(METADATA_PREFIX + ".enable.full.scan.log.files")
-      .defaultValue(true)
-      .sinceVersion("0.10.0")
-      .withDocumentation("Enable full scanning of log files while reading log records. If disabled, Hudi does look up of only interested entries. "
-          + "This is an internal config and setting this will not overwrite the actual value used.");
-
   public static final ConfigProperty<Boolean> ENABLE_METADATA_INDEX_BLOOM_FILTER = ConfigProperty
       .key(METADATA_PREFIX + ".index.bloom.filter.enable")
       .defaultValue(false)
@@ -151,6 +137,7 @@ public final class HoodieMetadataConfig extends HoodieConfig {
   public static final ConfigProperty<Integer> METADATA_INDEX_BLOOM_FILTER_FILE_GROUP_COUNT = ConfigProperty
       .key(METADATA_PREFIX + ".index.bloom.filter.file.group.count")
       .defaultValue(4)
+      .markAdvanced()
       .sinceVersion("0.11.0")
       .withDocumentation("Metadata bloom filter index partition file group count. This controls the size of the base and "
           + "log files and read parallelism in the bloom filter index partition. The recommendation is to size the "
@@ -159,6 +146,7 @@ public final class HoodieMetadataConfig extends HoodieConfig {
   public static final ConfigProperty<Integer> BLOOM_FILTER_INDEX_PARALLELISM = ConfigProperty
       .key(METADATA_PREFIX + ".index.bloom.filter.parallelism")
       .defaultValue(200)
+      .markAdvanced()
       .sinceVersion("0.11.0")
       .withDocumentation("Parallelism to use for generating bloom filter index in metadata table.");
 
@@ -173,20 +161,23 @@ public final class HoodieMetadataConfig extends HoodieConfig {
   public static final ConfigProperty<Integer> METADATA_INDEX_COLUMN_STATS_FILE_GROUP_COUNT = ConfigProperty
       .key(METADATA_PREFIX + ".index.column.stats.file.group.count")
       .defaultValue(2)
+      .markAdvanced()
       .sinceVersion("0.11.0")
       .withDocumentation("Metadata column stats partition file group count. This controls the size of the base and "
           + "log files and read parallelism in the column stats index partition. The recommendation is to size the "
           + "file group count such that the base files are under 1GB.");
 
   public static final ConfigProperty<Integer> COLUMN_STATS_INDEX_PARALLELISM = ConfigProperty
-          .key(METADATA_PREFIX + ".index.column.stats.parallelism")
-          .defaultValue(10)
-          .sinceVersion("0.11.0")
-          .withDocumentation("Parallelism to use, when generating column stats index.");
+      .key(METADATA_PREFIX + ".index.column.stats.parallelism")
+      .defaultValue(200)
+      .markAdvanced()
+      .sinceVersion("0.11.0")
+      .withDocumentation("Parallelism to use, when generating column stats index.");
 
   public static final ConfigProperty<String> COLUMN_STATS_INDEX_FOR_COLUMNS = ConfigProperty
       .key(METADATA_PREFIX + ".index.column.stats.column.list")
       .noDefaultValue()
+      .markAdvanced()
       .sinceVersion("0.11.0")
       .withDocumentation("Comma-separated list of columns for which column stats index will be built. If not set, all columns will be indexed");
 
@@ -197,6 +188,7 @@ public final class HoodieMetadataConfig extends HoodieConfig {
       .key(METADATA_PREFIX + ".index.column.stats.processing.mode.override")
       .noDefaultValue()
       .withValidValues(COLUMN_STATS_INDEX_PROCESSING_MODE_IN_MEMORY, COLUMN_STATS_INDEX_PROCESSING_MODE_ENGINE)
+      .markAdvanced()
       .sinceVersion("0.12.0")
       .withDocumentation("By default Column Stats Index is automatically determining whether it should be read and processed either"
           + "'in-memory' (w/in executing process) or using Spark (on a cluster), based on some factors like the size of the Index "
@@ -205,6 +197,7 @@ public final class HoodieMetadataConfig extends HoodieConfig {
   public static final ConfigProperty<Integer> COLUMN_STATS_INDEX_IN_MEMORY_PROJECTION_THRESHOLD = ConfigProperty
       .key(METADATA_PREFIX + ".index.column.stats.inMemory.projection.threshold")
       .defaultValue(100000)
+      .markAdvanced()
       .sinceVersion("0.12.0")
       .withDocumentation("When reading Column Stats Index, if the size of the expected resulting projection is below the in-memory"
           + " threshold (counted by the # of rows), it will be attempted to be loaded \"in-memory\" (ie not using the execution engine"
@@ -213,26 +206,22 @@ public final class HoodieMetadataConfig extends HoodieConfig {
   public static final ConfigProperty<String> BLOOM_FILTER_INDEX_FOR_COLUMNS = ConfigProperty
       .key(METADATA_PREFIX + ".index.bloom.filter.column.list")
       .noDefaultValue()
+      .markAdvanced()
       .sinceVersion("0.11.0")
       .withDocumentation("Comma-separated list of columns for which bloom filter index will be built. If not set, only record key will be indexed.");
 
   public static final ConfigProperty<Integer> METADATA_INDEX_CHECK_TIMEOUT_SECONDS = ConfigProperty
       .key(METADATA_PREFIX + ".index.check.timeout.seconds")
       .defaultValue(900)
+      .markAdvanced()
       .sinceVersion("0.11.0")
       .withDocumentation("After the async indexer has finished indexing upto the base instant, it will ensure that all inflight writers "
           + "reliably write index updates as well. If this timeout expires, then the indexer will abort itself safely.");
 
-  public static final ConfigProperty<Boolean> POPULATE_META_FIELDS = ConfigProperty
-      .key(METADATA_PREFIX + ".populate.meta.fields")
-      .defaultValue(false)
-      .sinceVersion("0.10.0")
-      .withDocumentation("When enabled, populates all meta fields. When disabled, no meta fields are populated. "
-          + "This is an internal config and setting this will not overwrite the actual value used.");
-
   public static final ConfigProperty<Boolean> IGNORE_SPURIOUS_DELETES = ConfigProperty
       .key("_" + METADATA_PREFIX + ".ignore.spurious.deletes")
       .defaultValue(true)
+      .markAdvanced()
       .sinceVersion("0.10.0")
       .withDocumentation("There are cases when extra files are requested to be deleted from "
           + "metadata table which are never added before. This config determines how to handle "
@@ -241,10 +230,104 @@ public final class HoodieMetadataConfig extends HoodieConfig {
   public static final ConfigProperty<Boolean> ENABLE_OPTIMIZED_LOG_BLOCKS_SCAN = ConfigProperty
       .key(METADATA_PREFIX + OPTIMIZED_LOG_BLOCKS_SCAN)
       .defaultValue(false)
+      .markAdvanced()
       .sinceVersion("0.13.0")
-      .withDocumentation("Optimized log blocks scanner that addresses all the multiwriter use-cases while appending to log files. "
+      .withDocumentation("Optimized log blocks scanner that addresses all the multi-writer use-cases while appending to log files. "
           + "It also differentiates original blocks written by ingestion writers and compacted blocks written by log compaction.");
 
+  public static final ConfigProperty<Integer> METADATA_MAX_NUM_DELTACOMMITS_WHEN_PENDING = ConfigProperty
+      .key(METADATA_PREFIX + ".max.deltacommits.when_pending")
+      .defaultValue(1000)
+      .markAdvanced()
+      .sinceVersion("0.14.0")
+      .withDocumentation("When there is a pending instant in data table, this config limits the allowed number of deltacommits in metadata table to "
+          + "prevent the metadata table's timeline from growing unboundedly as compaction won't be triggered due to the pending data table instant.");
+
+  public static final ConfigProperty<Boolean> RECORD_INDEX_ENABLE_PROP = ConfigProperty
+      .key(METADATA_PREFIX + ".record.index.enable")
+      .defaultValue(false)
+      .markAdvanced()
+      .sinceVersion("0.14.0")
+      .withDocumentation("Create the HUDI Record Index within the Metadata Table");
+
+  public static final ConfigProperty<Integer> RECORD_INDEX_MIN_FILE_GROUP_COUNT_PROP = ConfigProperty
+      .key(METADATA_PREFIX + ".record.index.min.filegroup.count")
+      .defaultValue(10)
+      .markAdvanced()
+      .sinceVersion("0.14.0")
+      .withDocumentation("Minimum number of file groups to use for Record Index.");
+
+  public static final ConfigProperty<Integer> RECORD_INDEX_MAX_FILE_GROUP_COUNT_PROP = ConfigProperty
+      .key(METADATA_PREFIX + ".record.index.max.filegroup.count")
+      .defaultValue(10000)
+      .markAdvanced()
+      .sinceVersion("0.14.0")
+      .withDocumentation("Maximum number of file groups to use for Record Index.");
+
+  public static final ConfigProperty<Integer> RECORD_INDEX_MAX_FILE_GROUP_SIZE_BYTES_PROP = ConfigProperty
+      .key(METADATA_PREFIX + ".record.index.max.filegroup.size")
+      .defaultValue(1024 * 1024 * 1024)
+      .markAdvanced()
+      .sinceVersion("0.14.0")
+      .withDocumentation("Maximum size in bytes of a single file group. Large file group takes longer to compact.");
+
+  public static final ConfigProperty<Float> RECORD_INDEX_GROWTH_FACTOR_PROP = ConfigProperty
+      .key(METADATA_PREFIX + ".record.index.growth.factor")
+      .defaultValue(2.0f)
+      .markAdvanced()
+      .sinceVersion("0.14.0")
+      .withDocumentation("The current number of records are multiplied by this number when estimating the number of "
+          + "file groups to create automatically. This helps account for growth in the number of records in the dataset.");
+
+  public static final ConfigProperty<Integer> RECORD_INDEX_MAX_PARALLELISM = ConfigProperty
+      .key(METADATA_PREFIX + ".max.init.parallelism")
+      .defaultValue(100000)
+      .markAdvanced()
+      .sinceVersion("0.14.0")
+      .withDocumentation("Maximum parallelism to use when initializing Record Index.");
+
+  public static final ConfigProperty<Long> MAX_READER_MEMORY_PROP = ConfigProperty
+      .key(METADATA_PREFIX + ".max.reader.memory")
+      .defaultValue(1024 * 1024 * 1024L)
+      .markAdvanced()
+      .sinceVersion("0.14.0")
+      .withDocumentation("Max memory to use for the reader to read from metadata");
+
+  public static final ConfigProperty<Integer> MAX_READER_BUFFER_SIZE_PROP = ConfigProperty
+      .key(METADATA_PREFIX + ".max.reader.buffer.size")
+      .defaultValue(10 * 1024 * 1024)
+      .markAdvanced()
+      .sinceVersion("0.14.0")
+      .withDocumentation("Max memory to use for the reader buffer while merging log blocks");
+
+  public static final ConfigProperty<String> SPILLABLE_MAP_DIR_PROP = ConfigProperty
+      .key(METADATA_PREFIX + ".spillable.map.path")
+      .noDefaultValue()
+      .withInferFunction(cfg -> Option.of(cfg.getStringOrDefault(FileSystemViewStorageConfig.SPILLABLE_DIR)))
+      .markAdvanced()
+      .sinceVersion("0.14.0")
+      .withDocumentation("Path on local storage to use, when keys read from metadata are held in a spillable map.");
+
+  public static final ConfigProperty<Long> MAX_LOG_FILE_SIZE_BYTES_PROP = ConfigProperty
+      .key(METADATA_PREFIX + ".max.logfile.size")
+      .defaultValue(2 * 1024 * 1024 * 1024L)  // 2GB
+      .markAdvanced()
+      .sinceVersion("0.14.0")
+      .withDocumentation("Maximum size in bytes of a single log file. Larger log files can contain larger log blocks "
+          + "thereby reducing the number of blocks to search for keys");
+
+  public static final ConfigProperty<Boolean> AUTO_INITIALIZE = ConfigProperty
+      .key(METADATA_PREFIX + ".auto.initialize")
+      .defaultValue(true)
+      .sinceVersion("0.14.0")
+      .markAdvanced()
+      .withDocumentation("Initializes the metadata table by reading from the file system when the table is first created. Enabled by default. "
+          + "Warning: This should only be disabled when manually constructing the metadata table outside of typical Hudi writer flows.");
+
+  public long getMaxLogFileSize() {
+    return getLong(MAX_LOG_FILE_SIZE_BYTES_PROP);
+  }
+
   private HoodieMetadataConfig() {
     super();
   }
@@ -261,7 +344,7 @@ public final class HoodieMetadataConfig extends HoodieConfig {
     return getBoolean(HoodieMetadataConfig.ASSUME_DATE_PARTITIONING);
   }
 
-  public boolean enabled() {
+  public boolean isEnabled() {
     return getBoolean(ENABLE);
   }
 
@@ -273,6 +356,10 @@ public final class HoodieMetadataConfig extends HoodieConfig {
     return getBooleanOrDefault(ENABLE_METADATA_INDEX_COLUMN_STATS);
   }
 
+  public boolean isRecordIndexEnabled() {
+    return isEnabled() && getBooleanOrDefault(RECORD_INDEX_ENABLE_PROP);
+  }
+
   public List<String> getColumnsEnabledForColumnStatsIndex() {
     return StringUtils.split(getString(COLUMN_STATS_INDEX_FOR_COLUMNS), CONFIG_VALUES_DELIMITER);
   }
@@ -309,7 +396,7 @@ public final class HoodieMetadataConfig extends HoodieConfig {
     return getIntOrDefault(METADATA_INDEX_CHECK_TIMEOUT_SECONDS);
   }
 
-  public boolean enableMetrics() {
+  public boolean isMetricsEnabled() {
     return getBoolean(METRICS_ENABLE);
   }
 
@@ -317,25 +404,54 @@ public final class HoodieMetadataConfig extends HoodieConfig {
     return getString(DIR_FILTER_REGEX);
   }
 
-  public boolean allowFullScan() {
-    return getBooleanOrDefault(ENABLE_FULL_SCAN_LOG_FILES);
-  }
-
-  public boolean populateMetaFields() {
-    return getBooleanOrDefault(HoodieMetadataConfig.POPULATE_META_FIELDS);
-  }
-
-  public boolean ignoreSpuriousDeletes() {
+  public boolean shouldIgnoreSpuriousDeletes() {
     return getBoolean(IGNORE_SPURIOUS_DELETES);
   }
 
-  public boolean doEnableOptimizedLogBlocksScan() {
+  public boolean isOptimizedLogBlocksScanEnabled() {
     return getBoolean(ENABLE_OPTIMIZED_LOG_BLOCKS_SCAN);
   }
 
-  /**
-   * Builder for {@link HoodieMetadataConfig}.
-   */
+  public int getMaxNumDeltacommitsWhenPending() {
+    return getIntOrDefault(METADATA_MAX_NUM_DELTACOMMITS_WHEN_PENDING);
+  }
+
+  public int getRecordIndexMinFileGroupCount() {
+    return getInt(RECORD_INDEX_MIN_FILE_GROUP_COUNT_PROP);
+  }
+
+  public int getRecordIndexMaxFileGroupCount() {
+    return getInt(RECORD_INDEX_MAX_FILE_GROUP_COUNT_PROP);
+  }
+
+  public float getRecordIndexGrowthFactor() {
+    return getFloat(RECORD_INDEX_GROWTH_FACTOR_PROP);
+  }
+
+  public int getRecordIndexMaxFileGroupSizeBytes() {
+    return getInt(RECORD_INDEX_MAX_FILE_GROUP_SIZE_BYTES_PROP);
+  }
+
+  public String getSplliableMapDir() {
+    return getString(SPILLABLE_MAP_DIR_PROP);
+  }
+
+  public long getMaxReaderMemory() {
+    return getLong(MAX_READER_MEMORY_PROP);
+  }
+
+  public int getMaxReaderBufferSize() {
+    return getInt(MAX_READER_BUFFER_SIZE_PROP);
+  }
+
+  public int getRecordIndexMaxParallelism() {
+    return getInt(RECORD_INDEX_MAX_PARALLELISM);
+  }
+
+  public boolean shouldAutoInitialize() {
+    return getBoolean(AUTO_INITIALIZE);
+  }
+
   public static class Builder {
 
     private EngineType engineType = EngineType.SPARK;
@@ -408,16 +524,6 @@ public final class HoodieMetadataConfig extends HoodieConfig {
       return this;
     }
 
-    public Builder withInsertParallelism(int parallelism) {
-      metadataConfig.setValue(INSERT_PARALLELISM_VALUE, String.valueOf(parallelism));
-      return this;
-    }
-
-    public Builder withAsyncClean(boolean asyncClean) {
-      metadataConfig.setValue(ASYNC_CLEAN_ENABLE, String.valueOf(asyncClean));
-      return this;
-    }
-
     public Builder withAsyncIndex(boolean asyncIndex) {
       metadataConfig.setValue(ASYNC_INDEX_ENABLE, String.valueOf(asyncIndex));
       return this;
@@ -428,19 +534,13 @@ public final class HoodieMetadataConfig extends HoodieConfig {
       return this;
     }
 
-    public Builder withPopulateMetaFields(boolean populateMetaFields) {
-      metadataConfig.setValue(POPULATE_META_FIELDS, Boolean.toString(populateMetaFields));
+    public Builder withLogCompactionEnabled(boolean enableLogCompaction) {
+      metadataConfig.setValue(ENABLE_LOG_COMPACTION_ON_METADATA_TABLE, Boolean.toString(enableLogCompaction));
       return this;
     }
 
-    public Builder archiveCommitsWith(int minToKeep, int maxToKeep) {
-      metadataConfig.setValue(MIN_COMMITS_TO_KEEP, String.valueOf(minToKeep));
-      metadataConfig.setValue(MAX_COMMITS_TO_KEEP, String.valueOf(maxToKeep));
-      return this;
-    }
-
-    public Builder retainCommits(int commitsRetained) {
-      metadataConfig.setValue(CLEANER_COMMITS_RETAINED, String.valueOf(commitsRetained));
+    public Builder withLogCompactBlocksThreshold(int logCompactBlocksThreshold) {
+      metadataConfig.setValue(LOG_COMPACT_BLOCKS_THRESHOLD, Integer.toString(logCompactBlocksThreshold));
       return this;
     }
 
@@ -449,6 +549,11 @@ public final class HoodieMetadataConfig extends HoodieConfig {
       return this;
     }
 
+    public Builder withRecordIndexMaxParallelism(int parallelism) {
+      metadataConfig.setValue(RECORD_INDEX_MAX_PARALLELISM, String.valueOf(parallelism));
+      return this;
+    }
+
     public Builder withAssumeDatePartitioning(boolean assumeDatePartitioning) {
       metadataConfig.setValue(ASSUME_DATE_PARTITIONING, String.valueOf(assumeDatePartitioning));
       return this;
@@ -459,11 +564,6 @@ public final class HoodieMetadataConfig extends HoodieConfig {
       return this;
     }
 
-    public Builder enableFullScan(boolean enableFullScan) {
-      metadataConfig.setValue(ENABLE_FULL_SCAN_LOG_FILES, String.valueOf(enableFullScan));
-      return this;
-    }
-
     public Builder ignoreSpuriousDeletes(boolean validateMetadataPayloadConsistency) {
       metadataConfig.setValue(IGNORE_SPURIOUS_DELETES, String.valueOf(validateMetadataPayloadConsistency));
       return this;
@@ -484,6 +584,52 @@ public final class HoodieMetadataConfig extends HoodieConfig {
       return this;
     }
 
+    public Builder withMaxNumDeltacommitsWhenPending(int maxNumDeltaCommitsWhenPending) {
+      metadataConfig.setValue(METADATA_MAX_NUM_DELTACOMMITS_WHEN_PENDING, String.valueOf(maxNumDeltaCommitsWhenPending));
+      return this;
+    }
+
+    public Builder withEnableRecordIndex(boolean enabled) {
+      metadataConfig.setValue(RECORD_INDEX_ENABLE_PROP, String.valueOf(enabled));
+      return this;
+    }
+
+    public Builder withRecordIndexFileGroupCount(int minCount, int maxCount) {
+      metadataConfig.setValue(RECORD_INDEX_MIN_FILE_GROUP_COUNT_PROP, String.valueOf(minCount));
+      metadataConfig.setValue(RECORD_INDEX_MAX_FILE_GROUP_COUNT_PROP, String.valueOf(maxCount));
+      return this;
+    }
+
+    public Builder withRecordIndexGrowthFactor(float factor) {
+      metadataConfig.setValue(RECORD_INDEX_GROWTH_FACTOR_PROP, String.valueOf(factor));
+      return this;
+    }
+
+    public Builder withRecordIndexMaxFileGroupSizeBytes(long sizeInBytes) {
+      metadataConfig.setValue(RECORD_INDEX_MAX_FILE_GROUP_SIZE_BYTES_PROP, String.valueOf(sizeInBytes));
+      return this;
+    }
+
+    public Builder withSpillableMapDir(String dir) {
+      metadataConfig.setValue(SPILLABLE_MAP_DIR_PROP, dir);
+      return this;
+    }
+
+    public Builder withMaxReaderMemory(long mem) {
+      metadataConfig.setValue(MAX_READER_MEMORY_PROP, String.valueOf(mem));
+      return this;
+    }
+
+    public Builder withMaxReaderBufferSize(long mem) {
+      metadataConfig.setValue(MAX_READER_BUFFER_SIZE_PROP, String.valueOf(mem));
+      return this;
+    }
+
+    public Builder withMaxLogFileSizeBytes(long sizeInBytes) {
+      metadataConfig.setValue(MAX_LOG_FILE_SIZE_BYTES_PROP, String.valueOf(sizeInBytes));
+      return this;
+    }
+
     public HoodieMetadataConfig build() {
       metadataConfig.setDefaultValue(ENABLE, getDefaultMetadataEnable(engineType));
       metadataConfig.setDefaults(HoodieMetadataConfig.class.getName());
@@ -525,28 +671,6 @@ public final class HoodieMetadataConfig extends HoodieConfig {
   @Deprecated
   public static final boolean DEFAULT_METADATA_METRICS_ENABLE = METRICS_ENABLE.defaultValue();
 
-  /**
-   * @deprecated Use {@link #INSERT_PARALLELISM_VALUE} and its methods.
-   */
-  @Deprecated
-  public static final String METADATA_INSERT_PARALLELISM_PROP = INSERT_PARALLELISM_VALUE.key();
-  /**
-   * @deprecated Use {@link #INSERT_PARALLELISM_VALUE} and its methods.
-   */
-  @Deprecated
-  public static final int DEFAULT_METADATA_INSERT_PARALLELISM = INSERT_PARALLELISM_VALUE.defaultValue();
-
-  /**
-   * @deprecated Use {@link #ASYNC_CLEAN_ENABLE} and its methods.
-   */
-  @Deprecated
-  public static final String METADATA_ASYNC_CLEAN_PROP = ASYNC_CLEAN_ENABLE.key();
-  /**
-   * @deprecated Use {@link #ASYNC_CLEAN_ENABLE} and its methods.
-   */
-  @Deprecated
-  public static final boolean DEFAULT_METADATA_ASYNC_CLEAN = ASYNC_CLEAN_ENABLE.defaultValue();
-
   /**
    * @deprecated Use {@link #COMPACT_NUM_DELTA_COMMITS} and its methods.
    */
@@ -558,36 +682,6 @@ public final class HoodieMetadataConfig extends HoodieConfig {
   @Deprecated
   public static final int DEFAULT_METADATA_COMPACT_NUM_DELTA_COMMITS = COMPACT_NUM_DELTA_COMMITS.defaultValue();
 
-  /**
-   * @deprecated Use {@link #MIN_COMMITS_TO_KEEP} and its methods.
-   */
-  @Deprecated
-  public static final String MIN_COMMITS_TO_KEEP_PROP = MIN_COMMITS_TO_KEEP.key();
-  /**
-   * @deprecated Use {@link #MIN_COMMITS_TO_KEEP} and its methods.
-   */
-  @Deprecated
-  public static final int DEFAULT_MIN_COMMITS_TO_KEEP = MIN_COMMITS_TO_KEEP.defaultValue();
-  /**
-   * @deprecated Use {@link #MAX_COMMITS_TO_KEEP} and its methods.
-   */
-  @Deprecated
-  public static final String MAX_COMMITS_TO_KEEP_PROP = MAX_COMMITS_TO_KEEP.key();
-  /**
-   * @deprecated Use {@link #MAX_COMMITS_TO_KEEP} and its methods.
-   */
-  @Deprecated
-  public static final int DEFAULT_MAX_COMMITS_TO_KEEP = MAX_COMMITS_TO_KEEP.defaultValue();
-  /**
-   * @deprecated Use {@link #CLEANER_COMMITS_RETAINED} and its methods.
-   */
-  @Deprecated
-  public static final String CLEANER_COMMITS_RETAINED_PROP = CLEANER_COMMITS_RETAINED.key();
-  /**
-   * @deprecated Use {@link #CLEANER_COMMITS_RETAINED} and its methods.
-   */
-  @Deprecated
-  public static final int DEFAULT_CLEANER_COMMITS_RETAINED = CLEANER_COMMITS_RETAINED.defaultValue();
   /**
    * @deprecated No longer takes any effect.
    */
