@@ -18,8 +18,9 @@
 
 package org.apache.hudi.common.fs;
 
-import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.Path;
+import org.apache.hudi.storage.StoragePath;
+import org.apache.hudi.storage.HoodieStorage;
+
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -52,30 +53,33 @@ public class OptimisticConsistencyGuard extends FailSafeConsistencyGuard {
 
   private static final Logger LOG = LoggerFactory.getLogger(OptimisticConsistencyGuard.class);
 
-  public OptimisticConsistencyGuard(FileSystem fs, ConsistencyGuardConfig consistencyGuardConfig) {
-    super(fs, consistencyGuardConfig);
+  public OptimisticConsistencyGuard(HoodieStorage storage,
+                                    ConsistencyGuardConfig consistencyGuardConfig) {
+    super(storage, consistencyGuardConfig);
   }
 
   @Override
-  public void waitTillFileAppears(Path filePath) throws TimeoutException {
+  public void waitTillFileAppears(StoragePath filePath) throws TimeoutException {
     try {
       if (!checkFileVisibility(filePath, FileVisibility.APPEAR)) {
         Thread.sleep(consistencyGuardConfig.getOptimisticConsistencyGuardSleepTimeMs());
       }
     } catch (IOException | InterruptedException ioe) {
-      LOG.warn("Got IOException or InterruptedException waiting for file visibility. Ignoring", ioe);
+      LOG.warn("Got IOException or InterruptedException waiting for file visibility. Ignoring",
+          ioe);
     }
   }
 
   @Override
-  public void waitTillFileDisappears(Path filePath) throws TimeoutException {
+  public void waitTillFileDisappears(StoragePath filePath) throws TimeoutException {
     // no op
   }
 
   @Override
   public void waitTillAllFilesAppear(String dirPath, List<String> files) throws TimeoutException {
     try {
-      if (!checkFilesVisibility(1, new Path(dirPath), getFilesWithoutSchemeAndAuthority(files), FileVisibility.APPEAR)) {
+      if (!checkFilesVisibility(1, new StoragePath(dirPath),
+          getFilesWithoutSchemeAndAuthority(files), FileVisibility.APPEAR)) {
         Thread.sleep(consistencyGuardConfig.getOptimisticConsistencyGuardSleepTimeMs());
       }
     } catch (InterruptedException ie) {
