@@ -18,8 +18,10 @@
 
 package org.apache.hudi.common.table;
 
-import org.apache.hudi.common.bootstrap.index.HFileBootstrapIndex;
 import org.apache.hudi.common.bootstrap.index.NoOpBootstrapIndex;
+import org.apache.hudi.common.bootstrap.index.hfile.HFileBootstrapIndex;
+import org.apache.hudi.common.config.ConfigClassProperty;
+import org.apache.hudi.common.config.ConfigGroups;
 import org.apache.hudi.common.config.ConfigProperty;
 import org.apache.hudi.common.config.HoodieConfig;
 import org.apache.hudi.common.config.OrderedProperties;
@@ -41,16 +43,18 @@ import org.apache.hudi.common.util.ValidationUtils;
 import org.apache.hudi.exception.HoodieIOException;
 import org.apache.hudi.keygen.constant.KeyGeneratorOptions;
 import org.apache.hudi.metadata.MetadataPartitionType;
+import org.apache.hudi.storage.HoodieStorage;
+import org.apache.hudi.storage.StoragePath;
 
 import org.apache.avro.Schema;
-import org.apache.hadoop.fs.FSDataInputStream;
-import org.apache.hadoop.fs.FSDataOutputStream;
-import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.Path;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import javax.annotation.concurrent.Immutable;
+
 import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
 import java.time.Instant;
 import java.util.Arrays;
 import java.util.HashSet;
@@ -61,7 +65,6 @@ import java.util.Set;
 import java.util.function.BiConsumer;
 import java.util.stream.Collectors;
 
-import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.DATE_TIME_PARSER;
 import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.INPUT_TIME_UNIT;
 import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.TIMESTAMP_INPUT_DATE_FORMAT;
@@ -70,14 +73,14 @@ import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.TIMESTAM
 import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.TIMESTAMP_OUTPUT_DATE_FORMAT;
 import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.TIMESTAMP_OUTPUT_TIMEZONE_FORMAT;
 import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.TIMESTAMP_TIMEZONE_FORMAT;
+import static org.apache.hudi.common.util.StringUtils.getUTF8Bytes;
 
-/**
- * Configurations on the Hoodie Table like type of ingestion, storage formats, hive table name etc Configurations are loaded from hoodie.properties, these properties are usually set during
- * initializing a path as hoodie base path and never changes during the lifetime of a hoodie table.
- *
- * @see HoodieTableMetaClient
- * @since 0.3.0
- */
+@Immutable
+@ConfigClassProperty(name = "Hudi Table Basic Configs",
+    groupName = ConfigGroups.Names.TABLE_CONFIG,
+    description = "Configurations of the Hudi Table like type of ingestion, storage formats, hive table name etc."
+        + " Configurations are loaded from hoodie.properties, these properties are usually set during"
+        + " initializing a path as hoodie base path and never changes during the lifetime of a hoodie table.")
 public class HoodieTableConfig extends HoodieConfig {
 
   private static final Logger LOG = LoggerFactory.getLogger(HoodieTableConfig.class);
@@ -270,12 +273,12 @@ public class HoodieTableConfig extends HoodieConfig {
   // Delay between retries while reading the properties file
   private static final int READ_RETRY_DELAY_MSEC = 1000;
 
-  public HoodieTableConfig(FileSystem fs, String metaPath, String payloadClassName, String recordMergerStrategyId) {
+  public HoodieTableConfig(HoodieStorage storage, StoragePath metaPath, String payloadClassName, String recordMergerStrategyId) {
     super();
-    Path propertyPath = new Path(metaPath, HOODIE_PROPERTIES_FILE);
+    StoragePath propertyPath = new StoragePath(metaPath, HOODIE_PROPERTIES_FILE);
     LOG.info("Loading table properties from " + propertyPath);
     try {
-      this.props = fetchConfigs(fs, metaPath);
+      this.props = fetchConfigs(storage, metaPath.toString());
       boolean needStore = false;
       if (contains(PAYLOAD_CLASS_NAME) && payloadClassName != null
           && !getString(PAYLOAD_CLASS_NAME).equals(payloadClassName)) {
@@ -289,7 +292,7 @@ public class HoodieTableConfig extends HoodieConfig {
       }
       if (needStore) {
         // FIXME(vc): wonder if this can be removed. Need to look into history.
-        try (FSDataOutputStream outputStream = fs.create(propertyPath)) {
+        try (OutputStream outputStream = storage.create(propertyPath)) {
           storeProperties(props, outputStream);
         }
       }
@@ -312,7 +315,7 @@ public class HoodieTableConfig extends HoodieConfig {
    * @return return the table checksum
    * @throws IOException
    */
-  private static String storeProperties(Properties props, FSDataOutputStream outputStream) throws IOException {
+  private static String storeProperties(Properties props, OutputStream outputStream) throws IOException {
     final String checksum;
     if (isValidChecksum(props)) {
       checksum = props.getProperty(TABLE_CHECKSUM.key());
@@ -337,21 +340,23 @@ public class HoodieTableConfig extends HoodieConfig {
     super();
   }
 
-  private static TypedProperties fetchConfigs(FileSystem fs, String metaPath) throws IOException {
-    Path cfgPath = new Path(metaPath, HOODIE_PROPERTIES_FILE);
-    Path backupCfgPath = new Path(metaPath, HOODIE_PROPERTIES_FILE_BACKUP);
+  public static TypedProperties fetchConfigs(HoodieStorage storage, String metaPath) throws IOException {
+    StoragePath cfgPath = new StoragePath(metaPath, HOODIE_PROPERTIES_FILE);
+    StoragePath backupCfgPath = new StoragePath(metaPath, HOODIE_PROPERTIES_FILE_BACKUP);
     int readRetryCount = 0;
     boolean found = false;
 
     TypedProperties props = new TypedProperties();
     while (readRetryCount++ < MAX_READ_RETRIES) {
-      for (Path path : Arrays.asList(cfgPath, backupCfgPath)) {
+      for (StoragePath path : Arrays.asList(cfgPath, backupCfgPath)) {
         // Read the properties and validate that it is a valid file
-        try (FSDataInputStream is = fs.open(path)) {
+        try (InputStream is = storage.open(path)) {
           props.clear();
           props.load(is);
           found = true;
-          ValidationUtils.checkArgument(validateChecksum(props));
+          if (props.containsKey(TABLE_CHECKSUM.key())) {
+            ValidationUtils.checkArgument(HoodieTableConfig.validateChecksum(props));
+          }
           return props;
         } catch (IOException e) {
           LOG.warn(String.format("Could not read properties from %s: %s", path, e));
@@ -376,22 +381,22 @@ public class HoodieTableConfig extends HoodieConfig {
     }
   }
 
-  public static void recover(FileSystem fs, Path metadataFolder) throws IOException {
-    Path cfgPath = new Path(metadataFolder, HOODIE_PROPERTIES_FILE);
-    Path backupCfgPath = new Path(metadataFolder, HOODIE_PROPERTIES_FILE_BACKUP);
+  public static void recover(HoodieStorage fs, StoragePath metadataFolder) throws IOException {
+    StoragePath cfgPath = new StoragePath(metadataFolder, HOODIE_PROPERTIES_FILE);
+    StoragePath backupCfgPath = new StoragePath(metadataFolder, HOODIE_PROPERTIES_FILE_BACKUP);
     recoverIfNeeded(fs, cfgPath, backupCfgPath);
   }
 
-  static void recoverIfNeeded(FileSystem fs, Path cfgPath, Path backupCfgPath) throws IOException {
-    if (!fs.exists(cfgPath)) {
+  static void recoverIfNeeded(HoodieStorage storage, StoragePath cfgPath, StoragePath backupCfgPath) throws IOException {
+    if (!storage.exists(cfgPath)) {
       // copy over from backup
-      try (FSDataInputStream in = fs.open(backupCfgPath);
-           FSDataOutputStream out = fs.create(cfgPath, false)) {
+      try (InputStream in = storage.open(backupCfgPath);
+           OutputStream out = storage.create(cfgPath, false)) {
         FileIOUtils.copy(in, out);
       }
     }
     // regardless, we don't need the backup anymore.
-    fs.delete(backupCfgPath, false);
+    storage.deleteFile(backupCfgPath);
   }
 
   private static void upsertProperties(Properties current, Properties updated) {
@@ -402,45 +407,45 @@ public class HoodieTableConfig extends HoodieConfig {
     deleted.forEach((k, v) -> current.remove(k.toString()));
   }
 
-  private static void modify(FileSystem fs, Path metadataFolder, Properties modifyProps, BiConsumer<Properties, Properties> modifyFn) {
-    Path cfgPath = new Path(metadataFolder, HOODIE_PROPERTIES_FILE);
-    Path backupCfgPath = new Path(metadataFolder, HOODIE_PROPERTIES_FILE_BACKUP);
+  private static void modify(HoodieStorage storage, StoragePath metadataFolder, Properties modifyProps, BiConsumer<Properties, Properties> modifyFn) {
+    StoragePath cfgPath = new StoragePath(metadataFolder, HOODIE_PROPERTIES_FILE);
+    StoragePath backupCfgPath = new StoragePath(metadataFolder, HOODIE_PROPERTIES_FILE_BACKUP);
     try {
       // 0. do any recovery from prior attempts.
-      recoverIfNeeded(fs, cfgPath, backupCfgPath);
+      recoverIfNeeded(storage, cfgPath, backupCfgPath);
 
       // 1. Read the existing config
-      TypedProperties props = fetchConfigs(fs, metadataFolder.toString());
+      TypedProperties props = fetchConfigs(storage, metadataFolder.toString());
 
       // 2. backup the existing properties.
-      try (FSDataOutputStream out = fs.create(backupCfgPath, false)) {
+      try (OutputStream out = storage.create(backupCfgPath, false)) {
         storeProperties(props, out);
       }
 
       // 3. delete the properties file, reads will go to the backup, until we are done.
-      fs.delete(cfgPath, false);
+      storage.deleteFile(cfgPath);
 
       // 4. Upsert and save back.
       String checksum;
-      try (FSDataOutputStream out = fs.create(cfgPath, true)) {
+      try (OutputStream out = storage.create(cfgPath, true)) {
         modifyFn.accept(props, modifyProps);
         checksum = storeProperties(props, out);
       }
 
       // 4. verify and remove backup.
-      try (FSDataInputStream in = fs.open(cfgPath)) {
+      try (InputStream in = storage.open(cfgPath)) {
         props.clear();
         props.load(in);
         if (!props.containsKey(TABLE_CHECKSUM.key()) || !props.getProperty(TABLE_CHECKSUM.key()).equals(checksum)) {
           // delete the properties file and throw exception indicating update failure
           // subsequent writes will recover and update, reads will go to the backup until then
-          fs.delete(cfgPath, false);
+          storage.deleteFile(cfgPath);
           throw new HoodieIOException("Checksum property missing or does not match.");
         }
       }
 
       // 5. delete the backup properties file
-      fs.delete(backupCfgPath, false);
+      storage.deleteFile(backupCfgPath);
     } catch (IOException e) {
       throw new HoodieIOException("Error updating table configs.", e);
     }
@@ -450,27 +455,27 @@ public class HoodieTableConfig extends HoodieConfig {
    * Upserts the table config with the set of properties passed in. We implement a fail-safe backup protocol
    * here for safely updating with recovery and also ensuring the table config continues to be readable.
    */
-  public static void update(FileSystem fs, Path metadataFolder, Properties updatedProps) {
-    modify(fs, metadataFolder, updatedProps, HoodieTableConfig::upsertProperties);
+  public static void update(HoodieStorage storage, StoragePath metadataFolder, Properties updatedProps) {
+    modify(storage, metadataFolder, updatedProps, HoodieTableConfig::upsertProperties);
   }
 
-  public static void delete(FileSystem fs, Path metadataFolder, Set<String> deletedProps) {
+  public static void delete(HoodieStorage storage, StoragePath metadataFolder, Set<String> deletedProps) {
     Properties props = new Properties();
     deletedProps.forEach(p -> props.setProperty(p, ""));
-    modify(fs, metadataFolder, props, HoodieTableConfig::deleteProperties);
+    modify(storage, metadataFolder, props, HoodieTableConfig::deleteProperties);
   }
 
   /**
    * Initialize the hoodie meta directory and any necessary files inside the meta (including the hoodie.properties).
    */
-  public static void create(FileSystem fs, Path metadataFolder, Properties properties)
+  public static void create(HoodieStorage storage, StoragePath metadataFolder, Properties properties)
       throws IOException {
-    if (!fs.exists(metadataFolder)) {
-      fs.mkdirs(metadataFolder);
+    if (!storage.exists(metadataFolder)) {
+      storage.createDirectory(metadataFolder);
     }
     HoodieConfig hoodieConfig = new HoodieConfig(properties);
-    Path propertyPath = new Path(metadataFolder, HOODIE_PROPERTIES_FILE);
-    try (FSDataOutputStream outputStream = fs.create(propertyPath)) {
+    StoragePath propertyPath = new StoragePath(metadataFolder, HOODIE_PROPERTIES_FILE);
+    try (OutputStream outputStream = storage.create(propertyPath)) {
       if (!hoodieConfig.contains(NAME)) {
         throw new IllegalArgumentException(NAME.key() + " property needs to be specified");
       }
@@ -503,7 +508,7 @@ public class HoodieTableConfig extends HoodieConfig {
     }
     String table = props.getProperty(NAME.key());
     String database = props.getProperty(DATABASE_NAME.key(), "");
-    return BinaryUtil.generateChecksum(String.format(TABLE_CHECKSUM_FORMAT, database, table).getBytes(UTF_8));
+    return BinaryUtil.generateChecksum(getUTF8Bytes(String.format(TABLE_CHECKSUM_FORMAT, database, table)));
   }
 
   public static boolean validateChecksum(Properties props) {
@@ -777,7 +782,7 @@ public class HoodieTableConfig extends HoodieConfig {
     }
     setValue(TABLE_METADATA_PARTITIONS, partitions.stream().sorted().collect(Collectors.joining(CONFIG_VALUES_DELIMITER)));
     setValue(TABLE_METADATA_PARTITIONS_INFLIGHT, partitionsInflight.stream().sorted().collect(Collectors.joining(CONFIG_VALUES_DELIMITER)));
-    update(metaClient.getFs(), new Path(metaClient.getMetaPath()), getProps());
+    update(metaClient.getStorage(), metaClient.getMetaPath(), getProps());
     LOG.info(String.format("MDT %s partition %s has been %s", metaClient.getBasePathV2(), partitionType.name(), enabled ? "enabled" : "disabled"));
   }
 
@@ -795,7 +800,7 @@ public class HoodieTableConfig extends HoodieConfig {
     });
 
     setValue(TABLE_METADATA_PARTITIONS_INFLIGHT, partitionsInflight.stream().sorted().collect(Collectors.joining(CONFIG_VALUES_DELIMITER)));
-    update(metaClient.getFs(), new Path(metaClient.getMetaPath()), getProps());
+    update(metaClient.getStorage(), metaClient.getMetaPath(), getProps());
     LOG.info(String.format("MDT %s partitions %s have been set to inflight", metaClient.getBasePathV2(), partitionTypes));
   }
 
