@@ -18,18 +18,18 @@
 
 package org.apache.hudi.common.table.log.block;
 
+import org.apache.hudi.avro.HoodieAvroUtils;
 import org.apache.hudi.common.fs.SizeAwareDataInputStream;
 import org.apache.hudi.common.model.HoodieAvroIndexedRecord;
 import org.apache.hudi.common.model.HoodieRecord;
 import org.apache.hudi.common.model.HoodieRecord.HoodieRecordType;
-import org.apache.hudi.common.table.log.block.HoodieLogBlock.HeaderMetadataType;
-import org.apache.hudi.common.table.log.block.HoodieLogBlock.HoodieLogBlockContentLocation;
-import org.apache.hudi.common.table.log.block.HoodieLogBlock.HoodieLogBlockType;
-import org.apache.hudi.common.util.ClosableIterator;
-import org.apache.hudi.common.util.MappingIterator;
 import org.apache.hudi.common.util.Option;
+import org.apache.hudi.common.util.collection.ClosableIterator;
+import org.apache.hudi.common.util.collection.CloseableMappingIterator;
 import org.apache.hudi.exception.HoodieIOException;
 import org.apache.hudi.internal.schema.InternalSchema;
+import org.apache.hudi.io.SeekableDataInputStream;
+import org.apache.hudi.storage.HoodieStorage;
 
 import org.apache.avro.Schema;
 import org.apache.avro.generic.GenericDatumReader;
@@ -41,7 +41,6 @@ import org.apache.avro.io.Decoder;
 import org.apache.avro.io.DecoderFactory;
 import org.apache.avro.io.Encoder;
 import org.apache.avro.io.EncoderFactory;
-import org.apache.hadoop.fs.FSDataInputStream;
 
 import javax.annotation.Nonnull;
 
@@ -52,7 +51,6 @@ import java.io.DataOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
-import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
@@ -60,9 +58,13 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
+import java.util.function.Supplier;
 import java.util.zip.DeflaterOutputStream;
 import java.util.zip.InflaterInputStream;
 
+import static org.apache.hudi.avro.HoodieAvroUtils.recordNeedsRewriteForExtendedAvroTypePromotion;
+import static org.apache.hudi.common.util.StringUtils.fromUTF8Bytes;
+import static org.apache.hudi.common.util.StringUtils.getUTF8Bytes;
 import static org.apache.hudi.common.util.ValidationUtils.checkArgument;
 import static org.apache.hudi.common.util.ValidationUtils.checkState;
 
@@ -73,7 +75,7 @@ public class HoodieAvroDataBlock extends HoodieDataBlock {
 
   private final ThreadLocal<BinaryEncoder> encoderCache = new ThreadLocal<>();
 
-  public HoodieAvroDataBlock(FSDataInputStream inputStream,
+  public HoodieAvroDataBlock(Supplier<SeekableDataInputStream> inputStreamSupplier,
                              Option<byte[]> content,
                              boolean readBlockLazily,
                              HoodieLogBlockContentLocation logBlockContentLocation,
@@ -81,12 +83,12 @@ public class HoodieAvroDataBlock extends HoodieDataBlock {
                              Map<HeaderMetadataType, String> header,
                              Map<HeaderMetadataType, String> footer,
                              String keyField) {
-    super(content, inputStream, readBlockLazily, Option.of(logBlockContentLocation), readerSchema, header, footer, keyField, false);
+    super(content, inputStreamSupplier, readBlockLazily, Option.of(logBlockContentLocation), readerSchema, header, footer, keyField, false);
   }
 
   public HoodieAvroDataBlock(@Nonnull List<HoodieRecord> records,
-      @Nonnull Map<HeaderMetadataType, String> header,
-      @Nonnull String keyField
+                             @Nonnull Map<HeaderMetadataType, String> header,
+                             @Nonnull String keyField
   ) {
     super(records, header, new HashMap<>(), keyField);
   }
@@ -97,42 +99,41 @@ public class HoodieAvroDataBlock extends HoodieDataBlock {
   }
 
   @Override
-  protected byte[] serializeRecords(List<HoodieRecord> records) throws IOException {
+  protected byte[] serializeRecords(List<HoodieRecord> records, HoodieStorage storage) throws IOException {
     Schema schema = new Schema.Parser().parse(super.getLogBlockHeader().get(HeaderMetadataType.SCHEMA));
     GenericDatumWriter<IndexedRecord> writer = new GenericDatumWriter<>(schema);
     ByteArrayOutputStream baos = new ByteArrayOutputStream();
-    DataOutputStream output = new DataOutputStream(baos);
+    try (DataOutputStream output = new DataOutputStream(baos)) {
+      // 1. Write out the log block version
+      output.writeInt(HoodieLogBlock.version);
 
-    // 1. Write out the log block version
-    output.writeInt(HoodieLogBlock.version);
+      // 2. Write total number of records
+      output.writeInt(records.size());
 
-    // 2. Write total number of records
-    output.writeInt(records.size());
+      // 3. Write the records
+      for (HoodieRecord<?> s : records) {
+        ByteArrayOutputStream temp = new ByteArrayOutputStream();
+        BinaryEncoder encoder = EncoderFactory.get().binaryEncoder(temp, encoderCache.get());
+        encoderCache.set(encoder);
+        try {
+          // Encode the record into bytes
+          // Spark Record not support write avro log
+          IndexedRecord data = s.toIndexedRecord(schema, new Properties()).get().getData();
+          writer.write(data, encoder);
+          encoder.flush();
 
-    // 3. Write the records
-    for (HoodieRecord<?> s : records) {
-      ByteArrayOutputStream temp = new ByteArrayOutputStream();
-      BinaryEncoder encoder = EncoderFactory.get().binaryEncoder(temp, encoderCache.get());
-      encoderCache.set(encoder);
-      try {
-        // Encode the record into bytes
-        // Spark Record not support write avro log
-        IndexedRecord data = s.toIndexedRecord(schema, new Properties()).get().getData();
-        writer.write(data, encoder);
-        encoder.flush();
-
-        // Get the size of the bytes
-        int size = temp.toByteArray().length;
-        // Write the record size
-        output.writeInt(size);
-        // Write the content
-        output.write(temp.toByteArray());
-      } catch (IOException e) {
-        throw new HoodieIOException("IOException converting HoodieAvroDataBlock to bytes", e);
+          // Get the size of the bytes
+          int size = temp.toByteArray().length;
+          // Write the record size
+          output.writeInt(size);
+          // Write the content
+          output.write(temp.toByteArray());
+        } catch (IOException e) {
+          throw new HoodieIOException("IOException converting HoodieAvroDataBlock to bytes", e);
+        }
       }
+      encoderCache.remove();
     }
-    encoderCache.remove();
-    output.close();
     return baos.toByteArray();
   }
 
@@ -143,7 +144,7 @@ public class HoodieAvroDataBlock extends HoodieDataBlock {
     checkArgument(type != HoodieRecordType.SPARK, "Not support read avro to spark record");
     // TODO AvroSparkReader need
     RecordIterator iterator = RecordIterator.getInstance(this, content);
-    return new MappingIterator<>(iterator, data -> (HoodieRecord<T>) new HoodieAvroIndexedRecord(data));
+    return new CloseableMappingIterator<>(iterator, data -> (HoodieRecord<T>) new HoodieAvroIndexedRecord(data));
   }
 
   private static class RecordIterator implements ClosableIterator<IndexedRecord> {
@@ -151,7 +152,7 @@ public class HoodieAvroDataBlock extends HoodieDataBlock {
     private final SizeAwareDataInputStream dis;
     private final GenericDatumReader<IndexedRecord> reader;
     private final ThreadLocal<BinaryDecoder> decoderCache = new ThreadLocal<>();
-
+    private Option<Schema> promotedSchema = Option.empty();
     private int totalRecords = 0;
     private int readRecords = 0;
 
@@ -166,7 +167,12 @@ public class HoodieAvroDataBlock extends HoodieDataBlock {
         this.totalRecords = this.dis.readInt();
       }
 
-      this.reader = new GenericDatumReader<>(writerSchema, readerSchema);
+      if (recordNeedsRewriteForExtendedAvroTypePromotion(writerSchema, readerSchema)) {
+        this.reader = new GenericDatumReader<>(writerSchema, writerSchema);
+        this.promotedSchema = Option.of(readerSchema);
+      } else {
+        this.reader = new GenericDatumReader<>(writerSchema, readerSchema);
+      }
     }
 
     public static RecordIterator getInstance(HoodieAvroDataBlock dataBlock, byte[] content) throws IOException {
@@ -199,6 +205,9 @@ public class HoodieAvroDataBlock extends HoodieDataBlock {
         IndexedRecord record = this.reader.read(null, decoder);
         this.dis.skipBytes(recordLength);
         this.readRecords++;
+        if (this.promotedSchema.isPresent()) {
+          return  HoodieAvroUtils.rewriteRecordWithNewSchema(record, this.promotedSchema.get());
+        }
         return record;
       } catch (IOException e) {
         throw new HoodieIOException("Unable to convert bytes to record.", e);
@@ -269,9 +278,9 @@ public class HoodieAvroDataBlock extends HoodieDataBlock {
   private static byte[] compress(String text) {
     ByteArrayOutputStream baos = new ByteArrayOutputStream();
     try {
-      OutputStream out = new DeflaterOutputStream(baos);
-      out.write(text.getBytes(StandardCharsets.UTF_8));
-      out.close();
+      try (OutputStream out = new DeflaterOutputStream(baos)) {
+        out.write(getUTF8Bytes(text));
+      }
     } catch (IOException e) {
       throw new HoodieIOException("IOException while compressing text " + text, e);
     }
@@ -287,7 +296,7 @@ public class HoodieAvroDataBlock extends HoodieDataBlock {
       while ((len = in.read(buffer)) > 0) {
         baos.write(buffer, 0, len);
       }
-      return new String(baos.toByteArray(), StandardCharsets.UTF_8);
+      return fromUTF8Bytes(baos.toByteArray());
     } catch (IOException e) {
       throw new HoodieIOException("IOException while decompressing text", e);
     }
@@ -298,45 +307,43 @@ public class HoodieAvroDataBlock extends HoodieDataBlock {
 
     GenericDatumWriter<IndexedRecord> writer = new GenericDatumWriter<>(schema);
     ByteArrayOutputStream baos = new ByteArrayOutputStream();
-    DataOutputStream output = new DataOutputStream(baos);
+    try (DataOutputStream output = new DataOutputStream(baos)) {
+      // 1. Compress and Write schema out
+      byte[] schemaContent = compress(schema.toString());
+      output.writeInt(schemaContent.length);
+      output.write(schemaContent);
 
-    // 1. Compress and Write schema out
-    byte[] schemaContent = compress(schema.toString());
-    output.writeInt(schemaContent.length);
-    output.write(schemaContent);
+      List<HoodieRecord<?>> records = new ArrayList<>();
+      try (ClosableIterator<HoodieRecord<Object>> recordItr = getRecordIterator(HoodieRecordType.AVRO)) {
+        recordItr.forEachRemaining(records::add);
+      }
 
-    List<HoodieRecord<?>> records = new ArrayList<>();
-    try (ClosableIterator<HoodieRecord<Object>> recordItr = getRecordIterator(HoodieRecordType.AVRO)) {
-      recordItr.forEachRemaining(records::add);
-    }
+      // 2. Write total number of records
+      output.writeInt(records.size());
 
-    // 2. Write total number of records
-    output.writeInt(records.size());
+      // 3. Write the records
+      Iterator<HoodieRecord<?>> itr = records.iterator();
+      while (itr.hasNext()) {
+        IndexedRecord s = itr.next().toIndexedRecord(schema, new Properties()).get().getData();
+        ByteArrayOutputStream temp = new ByteArrayOutputStream();
+        Encoder encoder = EncoderFactory.get().binaryEncoder(temp, null);
+        try {
+          // Encode the record into bytes
+          writer.write(s, encoder);
+          encoder.flush();
 
-    // 3. Write the records
-    Iterator<HoodieRecord<?>> itr = records.iterator();
-    while (itr.hasNext()) {
-      IndexedRecord s = itr.next().toIndexedRecord(schema, new Properties()).get().getData();
-      ByteArrayOutputStream temp = new ByteArrayOutputStream();
-      Encoder encoder = EncoderFactory.get().binaryEncoder(temp, null);
-      try {
-        // Encode the record into bytes
-        writer.write(s, encoder);
-        encoder.flush();
-
-        // Get the size of the bytes
-        int size = temp.toByteArray().length;
-        // Write the record size
-        output.writeInt(size);
-        // Write the content
-        output.write(temp.toByteArray());
-        itr.remove();
-      } catch (IOException e) {
-        throw new HoodieIOException("IOException converting HoodieAvroDataBlock to bytes", e);
+          // Get the size of the bytes
+          int size = temp.toByteArray().length;
+          // Write the record size
+          output.writeInt(size);
+          // Write the content
+          output.write(temp.toByteArray());
+          itr.remove();
+        } catch (IOException e) {
+          throw new HoodieIOException("IOException converting HoodieAvroDataBlock to bytes", e);
+        }
       }
     }
-
-    output.close();
     return baos.toByteArray();
   }
 }
