@@ -395,6 +395,28 @@ public interface HoodieTimeline extends Serializable {
    */
   Option<HoodieInstant> getFirstNonSavepointCommit();
 
+  /**
+   * get the most recent cluster commit if present
+   */
+  public Option<HoodieInstant> getLastClusteringInstant();
+
+  /**
+   * get the most recent pending cluster commit if present
+   *
+   */
+  public Option<HoodieInstant> getLastPendingClusterInstant();
+
+
+  /**
+   * get the least recent pending cluster commit if present
+   */
+  public Option<HoodieInstant> getFirstPendingClusterInstant();
+
+  /**
+   * return true if instant is a pending clustering commit, otherwise false
+   */
+  public boolean isPendingClusterInstant(String instantTime);
+
   /**
    * Read the completed instant details.
    */
