@@ -24,15 +24,18 @@ import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
 import org.apache.hudi.common.table.timeline.versioning.AbstractMigratorBase;
 import org.apache.hudi.common.util.collection.Pair;
-
-import org.apache.hadoop.fs.Path;
+import org.apache.hudi.storage.StoragePath;
 
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.stream.Collectors;
 
+/**
+ * Migration handler for clean plan in version 2.
+ */
 public class CleanPlanV2MigrationHandler extends AbstractMigratorBase<HoodieCleanerPlan> {
 
   public static final Integer VERSION = 2;
@@ -50,16 +53,16 @@ public class CleanPlanV2MigrationHandler extends AbstractMigratorBase<HoodieClea
   public HoodieCleanerPlan upgradeFrom(HoodieCleanerPlan plan) {
     Map<String, List<HoodieCleanFileInfo>> filePathsPerPartition =
         plan.getFilesToBeDeletedPerPartition().entrySet().stream().map(e -> Pair.of(e.getKey(), e.getValue().stream()
-          .map(v -> new HoodieCleanFileInfo(
-            new Path(FSUtils.getPartitionPath(metaClient.getBasePath(), e.getKey()), v).toString(), false))
-          .collect(Collectors.toList()))).collect(Collectors.toMap(Pair::getKey, Pair::getValue));
+            .map(v -> new HoodieCleanFileInfo(
+                new StoragePath(FSUtils.constructAbsolutePath(metaClient.getBasePathV2(), e.getKey()), v).toString(), false))
+            .collect(Collectors.toList()))).collect(Collectors.toMap(Pair::getKey, Pair::getValue));
     return new HoodieCleanerPlan(plan.getEarliestInstantToRetain(), plan.getLastCompletedCommitTimestamp(),
-        plan.getPolicy(), new HashMap<>(), VERSION, filePathsPerPartition, new ArrayList<>());
+        plan.getPolicy(), new HashMap<>(), VERSION, filePathsPerPartition, new ArrayList<>(), Collections.emptyMap());
   }
 
   @Override
   public HoodieCleanerPlan downgradeFrom(HoodieCleanerPlan input) {
     throw new IllegalArgumentException(
-      "This is the current highest version. Plan cannot be any higher version");
+        "This is the current highest version. Plan cannot be any higher version");
   }
 }
