@@ -42,9 +42,9 @@ import org.apache.hudi.common.util.CompactionUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.collection.Pair;
 import org.apache.hudi.exception.HoodieException;
+import org.apache.hudi.storage.StoragePath;
+import org.apache.hudi.storage.StoragePathInfo;
 
-import org.apache.hadoop.fs.FileStatus;
-import org.apache.hadoop.fs.Path;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -264,13 +264,13 @@ public abstract class IncrementalTimelineSyncFileSystemView extends AbstractTabl
       String partition = entry.getKey();
       if (isPartitionAvailableInStore(partition)) {
         LOG.info("Syncing partition (" + partition + ") of instant (" + instant + ")");
-        FileStatus[] statuses = entry.getValue().stream().map(p -> {
-          FileStatus status = new FileStatus(p.getFileSizeInBytes(), false, 0, 0, 0, 0, null, null, null,
-              new Path(String.format("%s/%s", metaClient.getBasePath(), p.getPath())));
-          return status;
-        }).toArray(FileStatus[]::new);
+        List<StoragePathInfo> pathInfoList = entry.getValue().stream()
+            .map(p -> new StoragePathInfo(
+                new StoragePath(String.format("%s/%s", metaClient.getBasePath(), p.getPath())),
+                p.getFileSizeInBytes(), false, (short) 0, 0, 0))
+            .collect(Collectors.toList());
         List<HoodieFileGroup> fileGroups =
-            buildFileGroups(statuses, timeline.filterCompletedAndCompactionInstants(), false);
+            buildFileGroups(partition, pathInfoList, timeline.filterCompletedAndCompactionInstants(), false);
         applyDeltaFileSlicesToPartitionView(partition, fileGroups, DeltaApplyMode.ADD);
       } else {
         LOG.warn("Skipping partition (" + partition + ") when syncing instant (" + instant + ") as it is not loaded");
@@ -363,8 +363,8 @@ public abstract class IncrementalTimelineSyncFileSystemView extends AbstractTabl
       final String basePath = metaClient.getBasePath();
       final String partitionPath = entry.getValue().getPartitionPath();
       List<String> fullPathList = entry.getValue().getSuccessDeleteFiles()
-          .stream().map(fileName -> new Path(FSUtils
-              .getPartitionPath(basePath, partitionPath), fileName).toString())
+          .stream().map(fileName -> new StoragePath(FSUtils
+              .constructAbsolutePath(basePath, partitionPath), fileName).toString())
           .collect(Collectors.toList());
       removeFileSlicesForPartition(timeline, instant, entry.getKey(), fullPathList);
     });
@@ -375,13 +375,11 @@ public abstract class IncrementalTimelineSyncFileSystemView extends AbstractTabl
       List<String> paths) {
     if (isPartitionAvailableInStore(partition)) {
       LOG.info("Removing file slices for partition (" + partition + ") for instant (" + instant + ")");
-      FileStatus[] statuses = paths.stream().map(p -> {
-        FileStatus status = new FileStatus();
-        status.setPath(new Path(p));
-        return status;
-      }).toArray(FileStatus[]::new);
+      List<StoragePathInfo> pathInfoList = paths.stream()
+          .map(p -> new StoragePathInfo(new StoragePath(p), 0, false, (short) 0, 0, 0))
+          .collect(Collectors.toList());
       List<HoodieFileGroup> fileGroups =
-          buildFileGroups(statuses, timeline.filterCompletedAndCompactionInstants(), false);
+          buildFileGroups(partition, pathInfoList, timeline.filterCompletedAndCompactionInstants(), false);
       applyDeltaFileSlicesToPartitionView(partition, fileGroups, DeltaApplyMode.REMOVE);
     } else {
       LOG.warn("Skipping partition (" + partition + ") when syncing instant (" + instant + ") as it is not loaded");
@@ -418,21 +416,21 @@ public abstract class IncrementalTimelineSyncFileSystemView extends AbstractTabl
      */
     Map<String, HoodieBaseFile> viewDataFiles = fileGroups.stream().flatMap(HoodieFileGroup::getAllRawFileSlices)
         .map(FileSlice::getBaseFile).filter(Option::isPresent).map(Option::get)
-        .map(df -> Pair.of(Path.getPathWithoutSchemeAndAuthority(new Path(df.getPath())).toString(), df))
+        .map(df -> Pair.of(FSUtils.getPathWithoutSchemeAndAuthority(new StoragePath(df.getPath())).toString(), df))
         .collect(Collectors.toMap(Pair::getKey, Pair::getValue));
     // Note: Delta Log Files and Data Files can be empty when adding/removing pending compactions
     Map<String, HoodieBaseFile> deltaDataFiles = deltaFileGroups.stream().flatMap(HoodieFileGroup::getAllRawFileSlices)
         .map(FileSlice::getBaseFile).filter(Option::isPresent).map(Option::get)
-        .map(df -> Pair.of(Path.getPathWithoutSchemeAndAuthority(new Path(df.getPath())).toString(), df))
+        .map(df -> Pair.of(FSUtils.getPathWithoutSchemeAndAuthority(new StoragePath(df.getPath())).toString(), df))
         .collect(Collectors.toMap(Pair::getKey, Pair::getValue));
 
     Map<String, HoodieLogFile> viewLogFiles =
         fileGroups.stream().flatMap(HoodieFileGroup::getAllRawFileSlices).flatMap(FileSlice::getLogFiles)
-            .map(lf -> Pair.of(Path.getPathWithoutSchemeAndAuthority(lf.getPath()).toString(), lf))
+            .map(lf -> Pair.of(FSUtils.getPathWithoutSchemeAndAuthority(lf.getPath()).toString(), lf))
             .collect(Collectors.toMap(Pair::getKey, Pair::getValue));
     Map<String, HoodieLogFile> deltaLogFiles =
         deltaFileGroups.stream().flatMap(HoodieFileGroup::getAllRawFileSlices).flatMap(FileSlice::getLogFiles)
-            .map(lf -> Pair.of(Path.getPathWithoutSchemeAndAuthority(lf.getPath()).toString(), lf))
+            .map(lf -> Pair.of(FSUtils.getPathWithoutSchemeAndAuthority(lf.getPath()).toString(), lf))
             .collect(Collectors.toMap(Pair::getKey, Pair::getValue));
 
     switch (mode) {
@@ -450,7 +448,7 @@ public abstract class IncrementalTimelineSyncFileSystemView extends AbstractTabl
 
     HoodieTimeline timeline = deltaFileGroups.stream().map(df -> df.getTimeline()).findAny().get();
     List<HoodieFileGroup> fgs =
-        buildFileGroups(viewDataFiles.values().stream(), viewLogFiles.values().stream(), timeline, true);
+        buildFileGroups(partition, viewDataFiles.values().stream(), viewLogFiles.values().stream(), timeline, true);
     storePartitionView(partition, fgs);
   }
 
