@@ -20,11 +20,9 @@ package org.apache.hudi.metadata;
 
 import org.apache.hudi.avro.model.HoodieMetadataColumnStats;
 import org.apache.hudi.common.bloom.BloomFilter;
-import org.apache.hudi.common.config.SerializableConfiguration;
 import org.apache.hudi.common.data.HoodieData;
 import org.apache.hudi.common.engine.HoodieEngineContext;
 import org.apache.hudi.common.fs.FSUtils;
-import org.apache.hudi.common.fs.HoodieSerializableFileStatus;
 import org.apache.hudi.common.model.HoodiePartitionMetadata;
 import org.apache.hudi.common.model.HoodieRecord;
 import org.apache.hudi.common.model.HoodieRecordGlobalLocation;
@@ -41,14 +39,13 @@ import org.apache.hudi.expression.Expression;
 import org.apache.hudi.expression.PartialBindVisitor;
 import org.apache.hudi.expression.Predicates;
 import org.apache.hudi.internal.schema.Types;
-
-import org.apache.hadoop.fs.FileStatus;
-import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.Path;
+import org.apache.hudi.storage.HoodieStorage;
+import org.apache.hudi.storage.HoodieStorageUtils;
+import org.apache.hudi.storage.StoragePath;
+import org.apache.hudi.storage.StoragePathInfo;
 
 import java.io.IOException;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
@@ -69,9 +66,9 @@ public class FileSystemBackedTableMetadata extends AbstractHoodieTableMetadata {
   private final boolean urlEncodePartitioningEnabled;
 
   public FileSystemBackedTableMetadata(HoodieEngineContext engineContext, HoodieTableConfig tableConfig,
-                                       SerializableConfiguration conf, String datasetBasePath,
+                                       HoodieStorage storage, String datasetBasePath,
                                        boolean assumeDatePartitioning) {
-    super(engineContext, conf, datasetBasePath);
+    super(engineContext, storage, datasetBasePath);
 
     this.hiveStylePartitioningEnabled = Boolean.parseBoolean(tableConfig.getHiveStylePartitioningEnable());
     this.urlEncodePartitioningEnabled = Boolean.parseBoolean(tableConfig.getUrlEncodePartitioning());
@@ -79,30 +76,37 @@ public class FileSystemBackedTableMetadata extends AbstractHoodieTableMetadata {
   }
 
   public FileSystemBackedTableMetadata(HoodieEngineContext engineContext,
-                                       SerializableConfiguration conf, String datasetBasePath,
+                                       HoodieStorage storage, String datasetBasePath,
                                        boolean assumeDatePartitioning) {
-    super(engineContext, conf, datasetBasePath);
+    super(engineContext, storage, datasetBasePath);
 
-    FileSystem fs = FSUtils.getFs(dataBasePath.get(), conf.get());
-    Path metaPath = new Path(dataBasePath.get(), HoodieTableMetaClient.METAFOLDER_NAME);
-    TableNotFoundException.checkTableValidity(fs, this.dataBasePath.get(), metaPath);
-    HoodieTableConfig tableConfig = new HoodieTableConfig(fs, metaPath.toString(), null, null);
-    this.hiveStylePartitioningEnabled = Boolean.parseBoolean(tableConfig.getHiveStylePartitioningEnable());
-    this.urlEncodePartitioningEnabled = Boolean.parseBoolean(tableConfig.getUrlEncodePartitioning());
+    StoragePath metaPath =
+        new StoragePath(dataBasePath, HoodieTableMetaClient.METAFOLDER_NAME);
+    TableNotFoundException.checkTableValidity(storage, this.dataBasePath, metaPath);
+    HoodieTableConfig tableConfig = new HoodieTableConfig(storage, metaPath, null, null);
+    this.hiveStylePartitioningEnabled =
+        Boolean.parseBoolean(tableConfig.getHiveStylePartitioningEnable());
+    this.urlEncodePartitioningEnabled =
+        Boolean.parseBoolean(tableConfig.getUrlEncodePartitioning());
     this.assumeDatePartitioning = assumeDatePartitioning;
   }
 
+  public HoodieStorage getStorage() {
+    if (storage == null) {
+      storage = HoodieStorageUtils.getStorage(dataBasePath, storageConf);
+    }
+    return storage;
+  }
+
   @Override
-  public FileStatus[] getAllFilesInPartition(Path partitionPath) throws IOException {
-    FileSystem fs = partitionPath.getFileSystem(hadoopConf.get());
-    return FSUtils.getAllDataFilesInPartition(fs, partitionPath);
+  public List<StoragePathInfo> getAllFilesInPartition(StoragePath partitionPath) throws IOException {
+    return FSUtils.getAllDataFilesInPartition(getStorage(), partitionPath);
   }
 
   @Override
   public List<String> getAllPartitionPaths() throws IOException {
-    Path basePath = dataBasePath.get();
     if (assumeDatePartitioning) {
-      FileSystem fs = basePath.getFileSystem(hadoopConf.get());
+      HoodieStorage fs = HoodieStorageUtils.getStorage(dataBasePath, storageConf);
       return FSUtils.getAllPartitionFoldersThreeLevelsDown(fs, dataBasePath.toString());
     }
 
@@ -141,9 +145,9 @@ public class FileSystemBackedTableMetadata extends AbstractHoodieTableMetadata {
   private List<String> getPartitionPathWithPathPrefixUsingFilterExpression(String relativePathPrefix,
                                                                            Types.RecordType partitionFields,
                                                                            Expression pushedExpr) throws IOException {
-    List<Path> pathsToList = new CopyOnWriteArrayList<>();
+    List<StoragePath> pathsToList = new CopyOnWriteArrayList<>();
     pathsToList.add(StringUtils.isNullOrEmpty(relativePathPrefix)
-        ? dataBasePath.get() : new Path(dataBasePath.get(), relativePathPrefix));
+        ? dataBasePath : new StoragePath(dataBasePath, relativePathPrefix));
     List<String> partitionPaths = new CopyOnWriteArrayList<>();
 
     int currentPartitionLevel = -1;
@@ -169,11 +173,11 @@ public class FileSystemBackedTableMetadata extends AbstractHoodieTableMetadata {
       int listingParallelism = Math.min(DEFAULT_LISTING_PARALLELISM, pathsToList.size());
 
       // List all directories in parallel
-      engineContext.setJobStatus(this.getClass().getSimpleName(), "Listing all partitions with prefix " + relativePathPrefix);
+      engineContext.setJobStatus(this.getClass().getSimpleName(),
+          "Listing all partitions with prefix " + relativePathPrefix);
       // Need to use serializable file status here, see HUDI-5936
-      List<HoodieSerializableFileStatus> dirToFileListing = engineContext.flatMap(pathsToList, path -> {
-        FileSystem fileSystem = path.getFileSystem(hadoopConf.get());
-        return Arrays.stream(HoodieSerializableFileStatus.fromFileStatuses(fileSystem.listStatus(path)));
+      List<StoragePathInfo> dirToFileListing = engineContext.flatMap(pathsToList, path -> {
+        return getStorage().listDirectEntries(path).stream();
       }, listingParallelism);
       pathsToList.clear();
 
@@ -184,27 +188,35 @@ public class FileSystemBackedTableMetadata extends AbstractHoodieTableMetadata {
         // result below holds a list of pair. first entry in the pair optionally holds the deduced list of partitions.
         // and second entry holds optionally a directory path to be processed further.
         engineContext.setJobStatus(this.getClass().getSimpleName(), "Processing listed partitions");
-        List<Pair<Option<String>, Option<Path>>> result = engineContext.map(dirToFileListing, fileStatus -> {
-          Path path = fileStatus.getPath();
-          FileSystem fileSystem = path.getFileSystem(hadoopConf.get());
-          if (fileStatus.isDirectory()) {
-            if (HoodiePartitionMetadata.hasPartitionMetadata(fileSystem, path)) {
-              return Pair.of(Option.of(FSUtils.getRelativePartitionPath(dataBasePath.get(), path)), Option.empty());
-            } else if (!path.getName().equals(HoodieTableMetaClient.METAFOLDER_NAME)) {
-              return Pair.of(Option.empty(), Option.of(path));
-            }
-          } else if (path.getName().startsWith(HoodiePartitionMetadata.HOODIE_PARTITION_METAFILE_PREFIX)) {
-            String partitionName = FSUtils.getRelativePartitionPath(dataBasePath.get(), path.getParent());
-            return Pair.of(Option.of(partitionName), Option.empty());
-          }
-          return Pair.of(Option.empty(), Option.empty());
-        }, fileListingParallelism);
+        List<Pair<Option<String>, Option<StoragePath>>> result =
+            engineContext.map(dirToFileListing,
+                fileInfo -> {
+                  StoragePath path = fileInfo.getPath();
+                  if (fileInfo.isDirectory()) {
+                    if (HoodiePartitionMetadata.hasPartitionMetadata(getStorage(), path)) {
+                      return Pair.of(
+                          Option.of(FSUtils.getRelativePartitionPath(dataBasePath,
+                              path)),
+                          Option.empty());
+                    } else if (!path.getName().equals(HoodieTableMetaClient.METAFOLDER_NAME)) {
+                      return Pair.of(Option.empty(), Option.of(path));
+                    }
+                  } else if (path.getName()
+                      .startsWith(HoodiePartitionMetadata.HOODIE_PARTITION_METAFILE_PREFIX)) {
+                    String partitionName =
+                        FSUtils.getRelativePartitionPath(dataBasePath,
+                            path.getParent());
+                    return Pair.of(Option.of(partitionName), Option.empty());
+                  }
+                  return Pair.of(Option.empty(), Option.empty());
+                }, fileListingParallelism);
 
         partitionPaths.addAll(result.stream().filter(entry -> entry.getKey().isPresent())
             .map(entry -> entry.getKey().get())
             .filter(relativePartitionPath -> fullBoundExpr instanceof Predicates.TrueExpression
                 || (Boolean) fullBoundExpr.eval(
-                extractPartitionValues(partitionFields, relativePartitionPath, urlEncodePartitioningEnabled)))
+                extractPartitionValues(partitionFields, relativePartitionPath,
+                    urlEncodePartitioningEnabled)))
             .collect(Collectors.toList()));
 
         Expression partialBoundExpr;
@@ -227,7 +239,7 @@ public class FileSystemBackedTableMetadata extends AbstractHoodieTableMetadata {
         pathsToList.addAll(result.stream().filter(entry -> entry.getValue().isPresent()).map(entry -> entry.getValue().get())
             .filter(path -> partialBoundExpr instanceof Predicates.TrueExpression
                 || (Boolean) partialBoundExpr.eval(
-                    extractPartitionValues(partitionFields, FSUtils.getRelativePartitionPath(dataBasePath.get(), path), urlEncodePartitioningEnabled)))
+                extractPartitionValues(partitionFields, FSUtils.getRelativePartitionPath(dataBasePath, path), urlEncodePartitioningEnabled)))
             .collect(Collectors.toList()));
       }
     }
@@ -235,7 +247,7 @@ public class FileSystemBackedTableMetadata extends AbstractHoodieTableMetadata {
   }
 
   @Override
-  public Map<String, FileStatus[]> getAllFilesInPartitions(Collection<String> partitionPaths)
+  public Map<String, List<StoragePathInfo>> getAllFilesInPartitions(Collection<String> partitionPaths)
       throws IOException {
     if (partitionPaths == null || partitionPaths.isEmpty()) {
       return Collections.emptyMap();
@@ -243,15 +255,19 @@ public class FileSystemBackedTableMetadata extends AbstractHoodieTableMetadata {
 
     int parallelism = Math.min(DEFAULT_LISTING_PARALLELISM, partitionPaths.size());
 
-    engineContext.setJobStatus(this.getClass().getSimpleName(), "Listing all files in " + partitionPaths.size() + " partitions");
+    engineContext.setJobStatus(this.getClass().getSimpleName(),
+        "Listing all files in " + partitionPaths.size() + " partitions");
     // Need to use serializable file status here, see HUDI-5936
-    List<Pair<String, HoodieSerializableFileStatus[]>> partitionToFiles = engineContext.map(new ArrayList<>(partitionPaths), partitionPathStr -> {
-      Path partitionPath = new Path(partitionPathStr);
-      FileSystem fs = partitionPath.getFileSystem(hadoopConf.get());
-      return Pair.of(partitionPathStr, HoodieSerializableFileStatus.fromFileStatuses(FSUtils.getAllDataFilesInPartition(fs, partitionPath)));
-    }, parallelism);
+    List<Pair<String, List<StoragePathInfo>>> partitionToFiles =
+        engineContext.map(new ArrayList<>(partitionPaths),
+            partitionPathStr -> {
+              StoragePath partitionPath = new StoragePath(partitionPathStr);
+              return Pair.of(partitionPathStr,
+                  FSUtils.getAllDataFilesInPartition(getStorage(), partitionPath));
+            }, parallelism);
 
-    return partitionToFiles.stream().collect(Collectors.toMap(Pair::getLeft, pair -> HoodieSerializableFileStatus.toFileStatuses(pair.getRight())));
+    return partitionToFiles.stream().collect(Collectors.toMap(pair -> pair.getLeft(),
+        pair -> pair.getRight()));
   }
 
   @Override
