@@ -19,9 +19,10 @@
 package org.apache.hudi.common.model;
 
 import org.apache.hudi.common.fs.FSUtils;
+import org.apache.hudi.common.table.HoodieTableConfig;
 import org.apache.hudi.common.table.timeline.HoodieActiveTimeline;
+import org.apache.hudi.storage.StoragePath;
 
-import org.apache.hadoop.fs.Path;
 import org.junit.jupiter.api.Test;
 
 import java.util.Date;
@@ -43,18 +44,19 @@ public class TestHoodieWriteStat {
     String fileName = UUID.randomUUID().toString();
     String writeToken = "1-0-1";
 
-    Path basePath = new Path(basePathString);
-    Path partitionPath = new Path(basePath, partitionPathString);
+    StoragePath basePath = new StoragePath(basePathString);
+    StoragePath partitionPath = new StoragePath(basePath, partitionPathString);
 
-    Path finalizeFilePath = new Path(partitionPath, FSUtils.makeBaseFileName(instantTime, writeToken, fileName));
+    StoragePath finalizeFilePath = new StoragePath(partitionPath, FSUtils.makeBaseFileName(instantTime, writeToken, fileName,
+        HoodieTableConfig.BASE_FILE_FORMAT.defaultValue().getFileExtension()));
     HoodieWriteStat writeStat = new HoodieWriteStat();
     writeStat.setPath(basePath, finalizeFilePath);
-    assertEquals(finalizeFilePath, new Path(basePath, writeStat.getPath()));
+    assertEquals(finalizeFilePath, new StoragePath(basePath, writeStat.getPath()));
 
     // test for null tempFilePath
     writeStat = new HoodieWriteStat();
     writeStat.setPath(basePath, finalizeFilePath);
-    assertEquals(finalizeFilePath, new Path(basePath, writeStat.getPath()));
+    assertEquals(finalizeFilePath, new StoragePath(basePath, writeStat.getPath()));
     assertNull(writeStat.getTempPath());
   }
 }
