@@ -36,6 +36,7 @@ import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.core.LogEvent;
 import org.apache.logging.log4j.core.Logger;
 import org.apache.logging.log4j.core.appender.AbstractAppender;
+import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.extension.ExtendWith;
@@ -52,11 +53,17 @@ import java.util.stream.Stream;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.mockito.ArgumentMatchers.any;
+import static org.mockito.Mockito.doThrow;
+import static org.mockito.Mockito.never;
 import static org.mockito.Mockito.reset;
 import static org.mockito.Mockito.times;
 import static org.mockito.Mockito.verify;
 import static org.mockito.Mockito.when;
 
+/**
+ * Tests {@link PriorityBasedFileSystemView}.
+ */
 @ExtendWith(MockitoExtension.class)
 public class TestPriorityBasedFileSystemView {
 
@@ -80,6 +87,13 @@ public class TestPriorityBasedFileSystemView {
         "file0001" + HoodieTableConfig.BASE_FILE_FORMAT.defaultValue().getFileExtension()));
   }
 
+  @AfterEach
+  public void tearDown() throws Exception {
+    testFileSliceStream.close();
+    testBaseFileStream.close();
+    fsView.close();
+  }
+
   private void resetMocks() {
     reset(primary, secondary);
   }
@@ -529,7 +543,7 @@ public class TestPriorityBasedFileSystemView {
   public void testGetPendingCompactionOperations() {
     Stream<Pair<String, CompactionOperation>> actual;
     Stream<Pair<String, CompactionOperation>> expected = Collections.singleton(
-        (Pair<String, CompactionOperation>) new ImmutablePair<>("test", new CompactionOperation()))
+            (Pair<String, CompactionOperation>) new ImmutablePair<>("test", new CompactionOperation()))
         .stream();
 
     when(primary.getPendingCompactionOperations()).thenReturn(expected);
@@ -554,6 +568,35 @@ public class TestPriorityBasedFileSystemView {
     });
   }
 
+  @Test
+  public void testGetPendingLogCompactionOperations() {
+    Stream<Pair<String, CompactionOperation>> actual;
+    Stream<Pair<String, CompactionOperation>> expected = Collections.singleton(
+            (Pair<String, CompactionOperation>) new ImmutablePair<>("test", new CompactionOperation()))
+        .stream();
+
+    when(primary.getPendingLogCompactionOperations()).thenReturn(expected);
+    actual = fsView.getPendingLogCompactionOperations();
+    assertEquals(expected, actual);
+
+    resetMocks();
+    when(primary.getPendingLogCompactionOperations()).thenThrow(new RuntimeException());
+    when(secondary.getPendingLogCompactionOperations()).thenReturn(expected);
+    actual = fsView.getPendingLogCompactionOperations();
+    assertEquals(expected, actual);
+
+    resetMocks();
+    when(secondary.getPendingLogCompactionOperations()).thenReturn(expected);
+    actual = fsView.getPendingLogCompactionOperations();
+    assertEquals(expected, actual);
+
+    resetMocks();
+    when(secondary.getPendingLogCompactionOperations()).thenThrow(new RuntimeException());
+    assertThrows(RuntimeException.class, () -> {
+      fsView.getPendingLogCompactionOperations();
+    });
+  }
+
   @Test
   public void testClose() {
     fsView.close();
@@ -658,6 +701,27 @@ public class TestPriorityBasedFileSystemView {
     });
   }
 
+  @Test
+  public void testLoadPartitions() {
+    String partitionPath = "/table2";
+
+    fsView.loadPartitions(Collections.singletonList(partitionPath));
+    verify(primary, times(1)).loadPartitions(Collections.singletonList(partitionPath));
+    verify(secondary, never()).loadPartitions(any());
+
+    resetMocks();
+    doThrow(new RuntimeException()).when(primary).loadPartitions(Collections.singletonList(partitionPath));
+    fsView.loadPartitions(Collections.singletonList(partitionPath));
+    verify(primary, times(1)).loadPartitions(Collections.singletonList(partitionPath));
+    verify(secondary, times(1)).loadPartitions(Collections.singletonList(partitionPath));
+
+    resetMocks();
+    doThrow(new RuntimeException()).when(secondary).loadPartitions(Collections.singletonList(partitionPath));
+    assertThrows(RuntimeException.class, () -> {
+      fsView.loadPartitions(Collections.singletonList(partitionPath));
+    });
+  }
+
   @Test
   public void testGetPreferredView() {
     assertEquals(primary, fsView.getPreferredView());
@@ -677,7 +741,7 @@ public class TestPriorityBasedFileSystemView {
 
     @Override
     public void append(LogEvent event) {
-      log.add(event);
+      log.add(event.toImmutable());
     }
 
     public List<LogEvent> getLog() {
