@@ -22,11 +22,12 @@ import org.apache.hudi.internal.schema.InternalSchema;
 import org.apache.hudi.internal.schema.Types;
 import org.apache.hudi.internal.schema.utils.SchemaChangeUtils;
 
-import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.Test;
 
 import java.util.Arrays;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+
 /**
  * Tests {@link InternalSchemaMerger}.
  */
@@ -34,11 +35,11 @@ public class TestMergeSchema {
 
   @Test
   public void testPrimitiveMerge() {
-    Types.RecordType record = Types.RecordType.get(Arrays.asList(new Types.Field[] {
+    Types.RecordType record = Types.RecordType.get(Arrays.asList(
         Types.Field.get(0, "col1", Types.BooleanType.get()),
         Types.Field.get(1, "col2", Types.IntType.get()),
         Types.Field.get(2, "col3", Types.LongType.get()),
-        Types.Field.get(3, "col4", Types.FloatType.get())}));
+        Types.Field.get(3, "col4", Types.FloatType.get())));
 
     InternalSchema oldSchema = new InternalSchema(record);
     // add c1 after 'col1', and c2 before 'col3'
@@ -52,6 +53,7 @@ public class TestMergeSchema {
     deleteChange.deleteColumn("col1");
     deleteChange.deleteColumn("col3");
     InternalSchema newDeleteSchema = SchemaChangeUtils.applyTableChanges2Schema(newAddSchema, deleteChange);
+    assertEquals(newAddSchema.getMaxColumnId(), newDeleteSchema.getMaxColumnId());
 
     TableChanges.ColumnUpdateChange updateChange = TableChanges.ColumnUpdateChange.get(newDeleteSchema);
     updateChange.updateColumnType("col2", Types.LongType.get())
@@ -67,25 +69,23 @@ public class TestMergeSchema {
     // merge schema by using columnType from query schema
     InternalSchema mergeSchema = new InternalSchemaMerger(oldSchema, finalSchema, true, false).mergeSchema();
 
-    InternalSchema checkedSchema = new InternalSchema(Types.RecordType.get(Arrays.asList(new Types.Field[] {
-            Types.Field.get(4, true, "c1", Types.BooleanType.get(), "add c1 after col1"),
-            Types.Field.get(5, true, "c2", Types.IntType.get(), "add c2 before col3"),
-            Types.Field.get(3, true, "col4", Types.FloatType.get()),
-            Types.Field.get(1, true, "col2", Types.LongType.get(), "alter col2 comments"),
-            Types.Field.get(6, true, "col1suffix", Types.BooleanType.get(), "add new col1")
-        })));
-    Assertions.assertEquals(mergeSchema, checkedSchema);
+    InternalSchema checkedSchema = new InternalSchema(Types.RecordType.get(Arrays.asList(
+        Types.Field.get(4, true, "c1", Types.BooleanType.get(), "add c1 after col1"),
+        Types.Field.get(5, true, "c2", Types.IntType.get(), "add c2 before col3"),
+        Types.Field.get(3, true, "col4", Types.FloatType.get()),
+        Types.Field.get(1, true, "col2", Types.LongType.get(), "alter col2 comments"),
+        Types.Field.get(6, true, "col1suffix", Types.BooleanType.get(), "add new col1"))));
+    assertEquals(mergeSchema, checkedSchema);
 
     // merge schema by using columnType from file schema
     InternalSchema mergeSchema1 = new InternalSchemaMerger(oldSchema, finalSchema, true, true).mergeSchema();
-    InternalSchema checkedSchema1 = new InternalSchema(Types.RecordType.get(Arrays.asList(new Types.Field[] {
-            Types.Field.get(4, true, "c1", Types.BooleanType.get(), "add c1 after col1"),
-            Types.Field.get(5, true, "c2", Types.IntType.get(), "add c2 before col3"),
-            Types.Field.get(3, true, "col4", Types.FloatType.get()),
-            Types.Field.get(1, true, "col2", Types.IntType.get(), "alter col2 comments"),
-            Types.Field.get(6, true, "col1suffix", Types.BooleanType.get(), "add new col1")
-        })));
-    Assertions.assertEquals(mergeSchema1, checkedSchema1);
+    InternalSchema checkedSchema1 = new InternalSchema(Types.RecordType.get(Arrays.asList(
+        Types.Field.get(4, true, "c1", Types.BooleanType.get(), "add c1 after col1"),
+        Types.Field.get(5, true, "c2", Types.IntType.get(), "add c2 before col3"),
+        Types.Field.get(3, true, "col4", Types.FloatType.get()),
+        Types.Field.get(1, true, "col2", Types.IntType.get(), "alter col2 comments"),
+        Types.Field.get(6, true, "col1suffix", Types.BooleanType.get(), "add new col1"))));
+    assertEquals(mergeSchema1, checkedSchema1);
   }
 }
 
