@@ -19,7 +19,7 @@
 package org.apache.hudi.examples.spark
 
 import org.apache.hudi.DataSourceReadOptions.{BEGIN_INSTANTTIME, END_INSTANTTIME, QUERY_TYPE, QUERY_TYPE_INCREMENTAL_OPT_VAL}
-import org.apache.hudi.DataSourceWriteOptions.{PARTITIONPATH_FIELD, PRECOMBINE_FIELD, RECORDKEY_FIELD, PARTITIONS_TO_DELETE, OPERATION, DELETE_PARTITION_OPERATION_OPT_VAL, DELETE_OPERATION_OPT_VAL}
+import org.apache.hudi.DataSourceWriteOptions.{DELETE_OPERATION_OPT_VAL, DELETE_PARTITION_OPERATION_OPT_VAL, OPERATION, PARTITIONPATH_FIELD, PARTITIONS_TO_DELETE, PRECOMBINE_FIELD, RECORDKEY_FIELD}
 import org.apache.hudi.QuickstartUtils.getQuickstartWriteConfigs
 import org.apache.hudi.common.model.HoodieAvroPayload
 import org.apache.hudi.config.HoodieWriteConfig.TBL_NAME
@@ -27,7 +27,7 @@ import org.apache.hudi.examples.common.{HoodieExampleDataGenerator, HoodieExampl
 import org.apache.spark.sql.SaveMode.{Append, Overwrite}
 import org.apache.spark.sql.SparkSession
 
-import scala.collection.JavaConversions._
+import scala.collection.JavaConverters._
 
 /**
  * Simple examples of [[org.apache.hudi.DefaultSource]]
@@ -73,7 +73,7 @@ object HoodieDataSourceExample {
   def insertData(spark: SparkSession, tablePath: String, tableName: String, dataGen: HoodieExampleDataGenerator[HoodieAvroPayload]): Unit = {
 
     val commitTime: String = System.currentTimeMillis().toString
-    val inserts = dataGen.convertToStringList(dataGen.generateInserts(commitTime, 20))
+    val inserts = dataGen.convertToStringList(dataGen.generateInserts(commitTime, 20)).asScala.toSeq
     val df = spark.read.json(spark.sparkContext.parallelize(inserts, 1))
     df.write.format("hudi").
       options(getQuickstartWriteConfigs).
@@ -118,7 +118,7 @@ object HoodieDataSourceExample {
   def updateData(spark: SparkSession, tablePath: String, tableName: String, dataGen: HoodieExampleDataGenerator[HoodieAvroPayload]): Unit = {
 
     val commitTime: String = System.currentTimeMillis().toString
-    val updates = dataGen.convertToStringList(dataGen.generateUpdates(commitTime, 10))
+    val updates = dataGen.convertToStringList(dataGen.generateUpdates(commitTime, 10)).asScala.toSeq
     val df = spark.read.json(spark.sparkContext.parallelize(updates, 1))
     df.write.format("hudi").
       options(getQuickstartWriteConfigs).
@@ -131,7 +131,7 @@ object HoodieDataSourceExample {
   }
 
   /**
-   * Deleta data based in data information.
+   * Delete data based in data information.
    */
   def delete(spark: SparkSession, tablePath: String, tableName: String): Unit = {
 
