@@ -27,6 +27,7 @@ import org.apache.hudi.common.model.HoodieRecordMerger;
 import org.apache.hudi.common.model.HoodieRecordLocation;
 import org.apache.hudi.common.model.HoodieRecordPayload;
 import org.apache.hudi.common.model.WriteOperationType;
+import org.apache.hudi.common.util.HoodieRecordUtils;
 import org.apache.hudi.common.util.ObjectSizeCalculator;
 import org.apache.hudi.common.util.ValidationUtils;
 import org.apache.hudi.configuration.FlinkOptions;
@@ -102,7 +103,7 @@ public class StreamWriteFunction<I> extends AbstractStreamWriteFunction<I> {
    */
   private transient Map<String, DataBucket> buckets;
 
-  private transient BiFunction<List<HoodieRecord>, String, List<WriteStatus>> writeFunction;
+  protected transient BiFunction<List<HoodieRecord>, String, List<WriteStatus>> writeFunction;
 
   private transient HoodieRecordMerger recordMerger;
 
@@ -186,6 +187,8 @@ public class StreamWriteFunction<I> extends AbstractStreamWriteFunction<I> {
         this.writeFunction = (records, instantTime) -> this.writeClient.insert(records, instantTime);
         break;
       case UPSERT:
+      case DELETE: // shares the code path with UPSERT
+      case DELETE_PREPPED:
         this.writeFunction = (records, instantTime) -> this.writeClient.upsert(records, instantTime);
         break;
       case INSERT_OVERWRITE:
@@ -200,7 +203,7 @@ public class StreamWriteFunction<I> extends AbstractStreamWriteFunction<I> {
   }
 
   private void initMergeClass() {
-    recordMerger = writeClient.getConfig().getRecordMerger();
+    recordMerger = HoodieRecordUtils.mergerToPreCombineMode(writeClient.getConfig().getRecordMerger());
     LOG.info("init hoodie merge with class [{}]", recordMerger.getClass().getName());
   }
 
@@ -244,7 +247,7 @@ public class StreamWriteFunction<I> extends AbstractStreamWriteFunction<I> {
   /**
    * Data bucket.
    */
-  private static class DataBucket {
+  protected static class DataBucket {
     private final List<DataItem> records;
     private final BufferSizeDetector detector;
     private final String partitionPath;
@@ -428,12 +431,8 @@ public class StreamWriteFunction<I> extends AbstractStreamWriteFunction<I> {
 
     List<HoodieRecord> records = bucket.writeBuffer();
     ValidationUtils.checkState(records.size() > 0, "Data bucket to flush has no buffering records");
-    if (config.getBoolean(FlinkOptions.PRE_COMBINE)) {
-      records = (List<HoodieRecord>) FlinkWriteHelper.newInstance()
-          .deduplicateRecords(records, null, -1, this.writeClient.getConfig().getSchema(), this.writeClient.getConfig().getProps(), recordMerger);
-    }
-    bucket.preWrite(records);
-    final List<WriteStatus> writeStatus = new ArrayList<>(writeFunction.apply(records, instant));
+    records = deduplicateRecordsIfNeeded(records);
+    final List<WriteStatus> writeStatus = writeBucket(instant, bucket, records);
     records.clear();
     final WriteMetadataEvent event = WriteMetadataEvent.builder()
         .taskID(taskID)
@@ -464,12 +463,8 @@ public class StreamWriteFunction<I> extends AbstractStreamWriteFunction<I> {
           .forEach(bucket -> {
             List<HoodieRecord> records = bucket.writeBuffer();
             if (records.size() > 0) {
-              if (config.getBoolean(FlinkOptions.PRE_COMBINE)) {
-                records = (List<HoodieRecord>) FlinkWriteHelper.newInstance()
-                    .deduplicateRecords(records, null, -1, this.writeClient.getConfig().getSchema(), this.writeClient.getConfig().getProps(), recordMerger);
-              }
-              bucket.preWrite(records);
-              writeStatus.addAll(writeFunction.apply(records, currentInstant));
+              records = deduplicateRecordsIfNeeded(records);
+              writeStatus.addAll(writeBucket(currentInstant, bucket, records));
               records.clear();
               bucket.reset();
             }
@@ -494,4 +489,18 @@ public class StreamWriteFunction<I> extends AbstractStreamWriteFunction<I> {
     // blocks flushing until the coordinator starts a new instant
     this.confirming = true;
   }
+
+  protected List<WriteStatus> writeBucket(String instant, DataBucket bucket, List<HoodieRecord> records) {
+    bucket.preWrite(records);
+    return writeFunction.apply(records, instant);
+  }
+
+  private List<HoodieRecord> deduplicateRecordsIfNeeded(List<HoodieRecord> records) {
+    if (config.getBoolean(FlinkOptions.PRE_COMBINE)) {
+      return FlinkWriteHelper.newInstance()
+          .deduplicateRecords(records, null, -1, this.writeClient.getConfig().getSchema(), this.writeClient.getConfig().getProps(), recordMerger);
+    } else {
+      return records;
+    }
+  }
 }
