@@ -26,6 +26,7 @@ import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.common.util.ClusteringUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.collection.Pair;
+import org.apache.hudi.metrics.FlinkClusteringMetrics;
 import org.apache.hudi.table.HoodieFlinkTable;
 import org.apache.hudi.util.ClusteringUtil;
 import org.apache.hudi.util.FlinkTables;
@@ -33,10 +34,15 @@ import org.apache.hudi.util.FlinkWriteClients;
 
 import org.apache.flink.annotation.VisibleForTesting;
 import org.apache.flink.configuration.Configuration;
+import org.apache.flink.metrics.MetricGroup;
 import org.apache.flink.streaming.api.operators.AbstractStreamOperator;
 import org.apache.flink.streaming.api.operators.OneInputStreamOperator;
 import org.apache.flink.streaming.api.operators.Output;
 import org.apache.flink.streaming.runtime.streamrecord.StreamRecord;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import java.util.List;
 
 /**
  * Operator that generates the clustering plan with pluggable strategies on finished checkpoints.
@@ -45,6 +51,7 @@ import org.apache.flink.streaming.runtime.streamrecord.StreamRecord;
  */
 public class ClusteringPlanOperator extends AbstractStreamOperator<ClusteringPlanEvent>
     implements OneInputStreamOperator<Object, ClusteringPlanEvent> {
+  private static final Logger LOG = LoggerFactory.getLogger(ClusteringPlanOperator.class);
 
   /**
    * Config options.
@@ -57,6 +64,8 @@ public class ClusteringPlanOperator extends AbstractStreamOperator<ClusteringPla
   @SuppressWarnings("rawtypes")
   private transient HoodieFlinkTable table;
 
+  private transient FlinkClusteringMetrics clusteringMetrics;
+
   public ClusteringPlanOperator(Configuration conf) {
     this.conf = conf;
   }
@@ -65,6 +74,7 @@ public class ClusteringPlanOperator extends AbstractStreamOperator<ClusteringPla
   public void open() throws Exception {
     super.open();
     this.table = FlinkTables.createTable(conf, getRuntimeContext());
+    registerMetrics();
     // when starting up, rolls back all the inflight clustering instants if there exists,
     // these instants are in priority for scheduling task because the clustering instants are
     // scheduled from earliest(FIFO sequence).
@@ -88,10 +98,17 @@ public class ClusteringPlanOperator extends AbstractStreamOperator<ClusteringPla
   }
 
   private void scheduleClustering(HoodieFlinkTable<?> table, long checkpointId) {
+    List<HoodieInstant> pendingClusteringInstantTimes =
+        ClusteringUtils.getPendingClusteringInstantTimes(table.getMetaClient());
     // the first instant takes the highest priority.
     Option<HoodieInstant> firstRequested = Option.fromJavaOptional(
-        ClusteringUtils.getPendingClusteringInstantTimes(table.getMetaClient()).stream()
+        pendingClusteringInstantTimes.stream()
             .filter(instant -> instant.getState() == HoodieInstant.State.REQUESTED).findFirst());
+
+    // record metrics
+    clusteringMetrics.setFirstPendingClusteringInstant(firstRequested);
+    clusteringMetrics.setPendingClusteringCount(pendingClusteringInstantTimes.size());
+
     if (!firstRequested.isPresent()) {
       // do nothing.
       LOG.info("No clustering plan for checkpoint " + checkpointId);
@@ -136,4 +153,10 @@ public class ClusteringPlanOperator extends AbstractStreamOperator<ClusteringPla
   public void setOutput(Output<StreamRecord<ClusteringPlanEvent>> output) {
     this.output = output;
   }
+
+  private void registerMetrics() {
+    MetricGroup metrics = getRuntimeContext().getMetricGroup();
+    clusteringMetrics = new FlinkClusteringMetrics(metrics);
+    clusteringMetrics.registerMetrics();
+  }
 }
