@@ -38,6 +38,8 @@ import org.apache.flink.streaming.api.operators.BoundedOneInput;
 import org.apache.flink.streaming.api.operators.OneInputStreamOperator;
 import org.apache.flink.streaming.api.operators.Output;
 import org.apache.flink.streaming.runtime.streamrecord.StreamRecord;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
 import java.util.List;
@@ -51,6 +53,7 @@ import static java.util.stream.Collectors.toList;
  */
 public class CompactionPlanOperator extends AbstractStreamOperator<CompactionPlanEvent>
     implements OneInputStreamOperator<Object, CompactionPlanEvent>, BoundedOneInput {
+  private static final Logger LOG = LoggerFactory.getLogger(CompactionPlanOperator.class);
 
   /**
    * Config options.
