@@ -20,7 +20,6 @@ package org.apache.hudi.sink.partitioner;
 
 import org.apache.hudi.client.FlinkTaskContextSupplier;
 import org.apache.hudi.client.common.HoodieFlinkEngineContext;
-import org.apache.hudi.common.config.SerializableConfiguration;
 import org.apache.hudi.common.model.BaseAvroPayload;
 import org.apache.hudi.common.model.HoodieAvroRecord;
 import org.apache.hudi.common.model.HoodieKey;
@@ -32,6 +31,7 @@ import org.apache.hudi.common.model.WriteOperationType;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.configuration.FlinkOptions;
 import org.apache.hudi.configuration.HadoopConfigurations;
+import org.apache.hudi.hadoop.fs.HadoopFSUtils;
 import org.apache.hudi.sink.bootstrap.IndexRecord;
 import org.apache.hudi.sink.utils.PayloadCreation;
 import org.apache.hudi.table.action.commit.BucketInfo;
@@ -117,7 +117,7 @@ public class BucketAssignFunction<K, I, O extends HoodieRecord<?>>
     super.open(parameters);
     HoodieWriteConfig writeConfig = FlinkWriteClients.getHoodieClientConfig(this.conf, true);
     HoodieFlinkEngineContext context = new HoodieFlinkEngineContext(
-        new SerializableConfiguration(HadoopConfigurations.getHadoopConf(this.conf)),
+        HadoopFSUtils.getStorageConfWithCopy(HadoopConfigurations.getHadoopConf(this.conf)),
         new FlinkTaskContextSupplier(getRuntimeContext()));
     this.bucketAssigner = BucketAssigners.create(
         getRuntimeContext().getIndexOfThisSubtask(),
@@ -173,36 +173,38 @@ public class BucketAssignFunction<K, I, O extends HoodieRecord<?>>
     final String partitionPath = hoodieKey.getPartitionPath();
     final HoodieRecordLocation location;
 
-    // Only changing records need looking up the index for the location,
-    // append only records are always recognized as INSERT.
-    HoodieRecordGlobalLocation oldLoc = indexState.value();
-    if (isChangingRecords && oldLoc != null) {
-      // Set up the instant time as "U" to mark the bucket as an update bucket.
-      if (!Objects.equals(oldLoc.getPartitionPath(), partitionPath)) {
-        if (globalIndex) {
-          // if partition path changes, emit a delete record for old partition path,
-          // then update the index state using location with new partition path.
-          HoodieRecord<?> deleteRecord = new HoodieAvroRecord<>(new HoodieKey(recordKey, oldLoc.getPartitionPath()),
-              payloadCreation.createDeletePayload((BaseAvroPayload) record.getData()));
+    if (isChangingRecords) {
+      // Only changing records need looking up the index for the location,
+      // append only records are always recognized as INSERT.
+      HoodieRecordGlobalLocation oldLoc = indexState.value();
+      if (oldLoc != null) {
+        // Set up the instant time as "U" to mark the bucket as an update bucket.
+        if (!Objects.equals(oldLoc.getPartitionPath(), partitionPath)) {
+          if (globalIndex) {
+            // if partition path changes, emit a delete record for old partition path,
+            // then update the index state using location with new partition path.
+            HoodieRecord<?> deleteRecord = new HoodieAvroRecord<>(new HoodieKey(recordKey, oldLoc.getPartitionPath()),
+                payloadCreation.createDeletePayload((BaseAvroPayload) record.getData()));
 
-          deleteRecord.unseal();
-          deleteRecord.setCurrentLocation(oldLoc.toLocal("U"));
-          deleteRecord.seal();
+            deleteRecord.unseal();
+            deleteRecord.setCurrentLocation(oldLoc.toLocal("U"));
+            deleteRecord.seal();
 
-          out.collect((O) deleteRecord);
+            out.collect((O) deleteRecord);
+          }
+          location = getNewRecordLocation(partitionPath);
+        } else {
+          location = oldLoc.toLocal("U");
+          this.bucketAssigner.addUpdate(partitionPath, location.getFileId());
         }
-        location = getNewRecordLocation(partitionPath);
       } else {
-        location = oldLoc.toLocal("U");
-        this.bucketAssigner.addUpdate(partitionPath, location.getFileId());
+        location = getNewRecordLocation(partitionPath);
       }
+      // always refresh the index
+      updateIndexState(partitionPath, location);
     } else {
       location = getNewRecordLocation(partitionPath);
     }
-    // always refresh the index
-    if (isChangingRecords) {
-      updateIndexState(partitionPath, location);
-    }
 
     record.unseal();
     record.setCurrentLocation(location);
