@@ -18,7 +18,6 @@
 
 package org.apache.hudi.source;
 
-import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.BaseFile;
 import org.apache.hudi.common.model.FileSlice;
 import org.apache.hudi.common.model.HoodieCommitMetadata;
@@ -36,8 +35,10 @@ import org.apache.hudi.common.table.view.HoodieTableFileSystemView;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.configuration.FlinkOptions;
 import org.apache.hudi.configuration.OptionsResolver;
-import org.apache.hudi.hadoop.utils.HoodieInputFormatUtils;
+import org.apache.hudi.metadata.HoodieTableMetadataUtil;
 import org.apache.hudi.sink.partitioner.profile.WriteProfiles;
+import org.apache.hudi.source.prune.PartitionPruners;
+import org.apache.hudi.storage.StoragePathInfo;
 import org.apache.hudi.table.format.cdc.CdcInputSplit;
 import org.apache.hudi.table.format.mor.MergeOnReadInputSplit;
 import org.apache.hudi.util.ClusteringUtil;
@@ -47,8 +48,6 @@ import org.apache.flink.annotation.VisibleForTesting;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.core.fs.Path;
 import org.apache.flink.table.types.logical.RowType;
-import org.apache.hadoop.fs.FileStatus;
-import org.apache.hadoop.fs.FileSystem;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -56,7 +55,6 @@ import javax.annotation.Nullable;
 
 import java.io.Serializable;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
@@ -70,6 +68,7 @@ import java.util.stream.Stream;
 
 import static org.apache.hudi.common.table.timeline.HoodieTimeline.GREATER_THAN;
 import static org.apache.hudi.common.table.timeline.HoodieTimeline.GREATER_THAN_OR_EQUALS;
+import static org.apache.hudi.common.table.timeline.HoodieTimeline.LESSER_THAN;
 import static org.apache.hudi.common.table.timeline.HoodieTimeline.LESSER_THAN_OR_EQUALS;
 
 /**
@@ -94,7 +93,7 @@ public class IncrementalInputSplits implements Serializable {
   private final RowType rowType;
   private final long maxCompactionMemoryInBytes;
   // for partition pruning
-  private final Set<String> requiredPartitions;
+  private final PartitionPruners.PartitionPruner partitionPruner;
   // skip compaction
   private final boolean skipCompaction;
   // skip clustering
@@ -105,14 +104,14 @@ public class IncrementalInputSplits implements Serializable {
       Path path,
       RowType rowType,
       long maxCompactionMemoryInBytes,
-      @Nullable Set<String> requiredPartitions,
+      @Nullable PartitionPruners.PartitionPruner partitionPruner,
       boolean skipCompaction,
       boolean skipClustering) {
     this.conf = conf;
     this.path = path;
     this.rowType = rowType;
     this.maxCompactionMemoryInBytes = maxCompactionMemoryInBytes;
-    this.requiredPartitions = requiredPartitions;
+    this.partitionPruner = partitionPruner;
     this.skipCompaction = skipCompaction;
     this.skipClustering = skipClustering;
   }
@@ -128,14 +127,12 @@ public class IncrementalInputSplits implements Serializable {
    * Returns the incremental input splits.
    *
    * @param metaClient The meta client
-   * @param hadoopConf The hadoop configuration
    * @param cdcEnabled Whether cdc is enabled
    *
    * @return The list of incremental input splits or empty if there are no new instants
    */
   public Result inputSplits(
       HoodieTableMetaClient metaClient,
-      org.apache.hadoop.conf.Configuration hadoopConf,
       boolean cdcEnabled) {
     HoodieTimeline commitTimeline = getReadTimeline(metaClient);
     if (commitTimeline.empty()) {
@@ -183,7 +180,7 @@ public class IncrementalInputSplits implements Serializable {
     //   3. the start commit is archived
     //   4. the end commit is archived
     Set<String> readPartitions;
-    final FileStatus[] fileStatuses;
+    final List<StoragePathInfo> fileInfoList;
     if (fullTableScan) {
       // scans the partitions and files directly.
       FileIndex fileIndex = getFileIndex();
@@ -192,7 +189,7 @@ public class IncrementalInputSplits implements Serializable {
         LOG.warn("No partitions found for reading in user provided path.");
         return Result.EMPTY;
       }
-      fileStatuses = fileIndex.getFilesInPartitions();
+      fileInfoList = fileIndex.getFilesInPartitions();
     } else {
       if (instants.size() == 0) {
         LOG.info("No new instant found for the table under path " + path + ", skip reading");
@@ -206,15 +203,17 @@ public class IncrementalInputSplits implements Serializable {
       // case2: normal incremental read
       String tableName = conf.getString(FlinkOptions.TABLE_NAME);
       List<HoodieCommitMetadata> metadataList = instants.stream()
-          .map(instant -> WriteProfiles.getCommitMetadata(tableName, path, instant, commitTimeline)).collect(Collectors.toList());
+          .map(instant -> WriteProfiles.getCommitMetadata(tableName, path, instant, commitTimeline))
+          .collect(Collectors.toList());
       readPartitions = getReadPartitions(metadataList);
       if (readPartitions.size() == 0) {
         LOG.warn("No partitions found for reading in user provided path.");
         return Result.EMPTY;
       }
-      FileStatus[] files = WriteProfiles.getRawWritePathsOfInstants(path, hadoopConf, metadataList, metaClient.getTableType());
-      FileSystem fs = FSUtils.getFs(path.toString(), hadoopConf);
-      if (Arrays.stream(files).anyMatch(fileStatus -> !StreamerUtil.fileExists(fs, fileStatus.getPath()))) {
+      List<StoragePathInfo> files = WriteProfiles.getFilesFromMetadata(
+          path, (org.apache.hadoop.conf.Configuration) metaClient.getStorageConf().unwrap(),
+          metadataList, metaClient.getTableType(), false);
+      if (files == null) {
         LOG.warn("Found deleted files in metadata, fall back to full table scan.");
         // fallback to full table scan
         // reading from the earliest, scans the partitions and files directly.
@@ -224,19 +223,19 @@ public class IncrementalInputSplits implements Serializable {
           LOG.warn("No partitions found for reading in user provided path.");
           return Result.EMPTY;
         }
-        fileStatuses = fileIndex.getFilesInPartitions();
+        fileInfoList = fileIndex.getFilesInPartitions();
       } else {
-        fileStatuses = files;
+        fileInfoList = files;
       }
     }
 
-    if (fileStatuses.length == 0) {
+    if (fileInfoList.size() == 0) {
       LOG.warn("No files found for reading in user provided path.");
       return Result.EMPTY;
     }
 
     List<MergeOnReadInputSplit> inputSplits = getInputSplits(metaClient, commitTimeline,
-        fileStatuses, readPartitions, endInstant, instantRange, false);
+        fileInfoList, readPartitions, endInstant, instantRange, false);
 
     return Result.instance(inputSplits, endInstant);
   }
@@ -245,16 +244,16 @@ public class IncrementalInputSplits implements Serializable {
    * Returns the incremental input splits.
    *
    * @param metaClient    The meta client
-   * @param hadoopConf    The hadoop configuration
    * @param issuedInstant The last issued instant, only valid in streaming read
+   * @param issuedOffset  The last issued offset, only valid in streaming read
    * @param cdcEnabled    Whether cdc is enabled
    *
    * @return The list of incremental input splits or empty if there are no new instants
    */
   public Result inputSplits(
       HoodieTableMetaClient metaClient,
-      @Nullable org.apache.hadoop.conf.Configuration hadoopConf,
-      String issuedInstant,
+      @Nullable String issuedInstant,
+      @Nullable String issuedOffset,
       boolean cdcEnabled) {
     metaClient.reloadActiveTimeline();
     HoodieTimeline commitTimeline = getReadTimeline(metaClient);
@@ -262,87 +261,158 @@ public class IncrementalInputSplits implements Serializable {
       LOG.warn("No splits found for the table under path " + path);
       return Result.EMPTY;
     }
+
+    // Assumes a timeline:
+    //    c1.inflight, c2(issued instant), c3, c4
+    // -> c1,          c2(issued instant), c3, c4
+    // c1, c3 and c4 are the candidate instants,
+    // we call c1 a 'hollow' instant which has lower version number but greater completion time,
+    // filtering the timeline using just c2 could cause data loss,
+    // check these hollow instants first.
+    Result hollowSplits = getHollowInputSplits(metaClient,
+        metaClient.getStorageConf().unwrapAs(org.apache.hadoop.conf.Configuration.class), issuedInstant, issuedOffset, commitTimeline, cdcEnabled);
+
     List<HoodieInstant> instants = filterInstantsWithRange(commitTimeline, issuedInstant);
     // get the latest instant that satisfies condition
-    final HoodieInstant instantToIssue = instants.size() == 0 ? null : instants.get(instants.size() - 1);
+    final String endInstant = instants.size() == 0 ? null : instants.get(instants.size() - 1).getTimestamp();
     final InstantRange instantRange;
-    if (instantToIssue != null) {
+    if (endInstant != null) {
       // when cdc is enabled, returns instant range with nullable boundary
       // to filter the reading instants on the timeline
-      instantRange = getInstantRange(issuedInstant, instantToIssue.getTimestamp(), cdcEnabled);
-    } else {
+      instantRange = getInstantRange(issuedInstant, endInstant, cdcEnabled);
+    } else if (hollowSplits.isEmpty()) {
       LOG.info("No new instant found for the table under path " + path + ", skip reading");
       return Result.EMPTY;
+    } else {
+      return hollowSplits;
     }
 
-    final Set<String> readPartitions;
-    final FileStatus[] fileStatuses;
+    // version number should be monotonically increasing
+    // fetch the instant offset by completion time
+    String offsetToIssue = instants.stream().map(HoodieInstant::getStateTransitionTime).max(String::compareTo).orElse(endInstant);
 
     if (instantRange == null) {
       // reading from the earliest, scans the partitions and files directly.
       FileIndex fileIndex = getFileIndex();
-      readPartitions = new TreeSet<>(fileIndex.getOrBuildPartitionPaths());
+
+      Set<String> readPartitions = new TreeSet<>(fileIndex.getOrBuildPartitionPaths());
       if (readPartitions.size() == 0) {
         LOG.warn("No partitions found for reading under path: " + path);
         return Result.EMPTY;
       }
-      fileStatuses = fileIndex.getFilesInPartitions();
 
-      if (fileStatuses.length == 0) {
+      List<StoragePathInfo> pathInfoList = fileIndex.getFilesInPartitions();
+      if (pathInfoList.size() == 0) {
         LOG.warn("No files found for reading under path: " + path);
         return Result.EMPTY;
       }
 
-      final String endInstant = instantToIssue.getTimestamp();
       List<MergeOnReadInputSplit> inputSplits = getInputSplits(metaClient, commitTimeline,
-          fileStatuses, readPartitions, endInstant, null, false);
+          pathInfoList, readPartitions, endInstant, null, false);
 
-      return Result.instance(inputSplits, endInstant);
+      return Result.instance(inputSplits, endInstant, offsetToIssue);
     } else {
-      // streaming read
-      if (cdcEnabled) {
-        // case1: cdc change log enabled
-        final String endInstant = instantToIssue.getTimestamp();
-        List<MergeOnReadInputSplit> inputSplits = getCdcInputSplits(metaClient, instantRange);
-        return Result.instance(inputSplits, endInstant);
-      }
-      // case2: normal streaming read
-      String tableName = conf.getString(FlinkOptions.TABLE_NAME);
-      List<HoodieCommitMetadata> activeMetadataList = instants.stream()
-          .map(instant -> WriteProfiles.getCommitMetadata(tableName, path, instant, commitTimeline)).collect(Collectors.toList());
-      List<HoodieCommitMetadata> archivedMetadataList = getArchivedMetadata(metaClient, instantRange, commitTimeline, tableName);
-      if (archivedMetadataList.size() > 0) {
-        LOG.warn("\n"
-            + "--------------------------------------------------------------------------------\n"
-            + "---------- caution: the reader has fall behind too much from the writer,\n"
-            + "---------- tweak 'read.tasks' option to add parallelism of read tasks.\n"
-            + "--------------------------------------------------------------------------------");
-      }
-      List<HoodieCommitMetadata> metadataList = archivedMetadataList.size() > 0
-          // IMPORTANT: the merged metadata list must be in ascending order by instant time
-          ? mergeList(archivedMetadataList, activeMetadataList)
-          : activeMetadataList;
-
-      readPartitions = getReadPartitions(metadataList);
-      if (readPartitions.size() == 0) {
-        LOG.warn("No partitions found for reading under path: " + path);
-        return Result.EMPTY;
-      }
-      fileStatuses = WriteProfiles.getWritePathsOfInstants(path, hadoopConf, metadataList, metaClient.getTableType());
-
-      if (fileStatuses.length == 0) {
-        LOG.warn("No files found for reading under path: " + path);
-        return Result.EMPTY;
-      }
-
-      final String endInstant = instantToIssue.getTimestamp();
-      List<MergeOnReadInputSplit> inputSplits = getInputSplits(metaClient, commitTimeline,
-          fileStatuses, readPartitions, endInstant, instantRange, skipCompaction);
-
-      return Result.instance(inputSplits, endInstant);
+      List<MergeOnReadInputSplit> inputSplits = getIncInputSplits(metaClient,
+          metaClient.getStorageConf().unwrapAs(org.apache.hadoop.conf.Configuration.class),
+          commitTimeline, instants, instantRange, endInstant, cdcEnabled);
+      return Result.instance(mergeList(hollowSplits.getInputSplits(), inputSplits), endInstant, offsetToIssue);
     }
   }
 
+  /**
+   * Returns the input splits for streaming incremental read.
+   */
+  private List<MergeOnReadInputSplit> getIncInputSplits(
+      HoodieTableMetaClient metaClient,
+      org.apache.hadoop.conf.Configuration hadoopConf,
+      HoodieTimeline commitTimeline,
+      List<HoodieInstant> instants,
+      InstantRange instantRange,
+      String endInstant,
+      boolean cdcEnabled) {
+    // streaming read
+    if (cdcEnabled) {
+      // case1: cdc change log enabled
+      return getCdcInputSplits(metaClient, instantRange);
+    }
+    // case2: normal streaming read
+    String tableName = conf.getString(FlinkOptions.TABLE_NAME);
+    List<HoodieCommitMetadata> activeMetadataList = instants.stream()
+        .map(instant -> WriteProfiles.getCommitMetadata(tableName, path, instant, commitTimeline)).collect(Collectors.toList());
+    List<HoodieCommitMetadata> archivedMetadataList = getArchivedMetadata(metaClient, instantRange, commitTimeline, tableName);
+    if (archivedMetadataList.size() > 0) {
+      LOG.warn("\n"
+          + "--------------------------------------------------------------------------------\n"
+          + "---------- caution: the reader has fall behind too much from the writer,\n"
+          + "---------- tweak 'read.tasks' option to add parallelism of read tasks.\n"
+          + "--------------------------------------------------------------------------------");
+    }
+    // IMPORTANT: the merged metadata list must be in ascending order by instant time
+    List<HoodieCommitMetadata> metadataList = mergeList(archivedMetadataList, activeMetadataList);
+
+    Set<String> readPartitions = getReadPartitions(metadataList);
+    if (readPartitions.size() == 0) {
+      LOG.warn("No partitions found for reading under path: " + path);
+      return Collections.emptyList();
+    }
+    List<StoragePathInfo> pathInfoList = WriteProfiles.getFilesFromMetadata(
+        path, hadoopConf, metadataList, metaClient.getTableType());
+
+    if (pathInfoList.size() == 0) {
+      LOG.warn("No files found for reading under path: " + path);
+      return Collections.emptyList();
+    }
+
+    return getInputSplits(metaClient, commitTimeline,
+        pathInfoList, readPartitions, endInstant, instantRange, skipCompaction);
+  }
+
+  /**
+   * Returns the input splits for 'hollow' instants.
+   */
+  private Result getHollowInputSplits(
+      HoodieTableMetaClient metaClient,
+      org.apache.hadoop.conf.Configuration hadoopConf,
+      @Nullable String issuedInstant,
+      @Nullable String issuedOffset,
+      HoodieTimeline commitTimeline,
+      boolean cdcEnabled) {
+    if (issuedInstant == null || issuedOffset == null) {
+      return Result.EMPTY;
+    }
+    // find the write commit instant that finishes later than the issued instant
+    // while with smaller txn start time.
+    List<HoodieInstant> instants = commitTimeline.getInstantsAsStream()
+        .filter(s -> HoodieTimeline.compareTimestamps(s.getTimestamp(), LESSER_THAN, issuedInstant))
+        .filter(s -> HoodieTimeline.compareTimestamps(s.getStateTransitionTime(), GREATER_THAN, issuedOffset))
+        .filter(s -> StreamerUtil.isWriteCommit(metaClient.getTableType(), s, commitTimeline)).collect(Collectors.toList());
+    if (instants.isEmpty()) {
+      return Result.EMPTY;
+    }
+    String offsetToIssue = instants.stream().map(HoodieInstant::getStateTransitionTime).max(String::compareTo).orElse(issuedOffset);
+    List<MergeOnReadInputSplit> inputSplits = instants.stream().map(instant -> {
+      String instantTs = instant.getTimestamp();
+
+      // Assumes we consume from timeline:
+      //    c0, c1.inflight, c2(issued instant), c3, c4
+      // -> c0, c1,          c2(issued instant), c3, c4
+      // c1, c3 and c4 are the candidate instants,
+
+      // c4 data file could include overlapping records from c2,
+      // use (c2, c4] instant range for c3 and c4,
+
+      // c1 data file could include overlapping records from c0,
+      // use the [c1, c1] instant range for c1.
+      InstantRange instantRange = InstantRange.builder()
+          .startInstant(instantTs)
+          .endInstant(instantTs)
+          .nullableBoundary(cdcEnabled)
+          .rangeType(InstantRange.RangeType.CLOSE_CLOSE).build();
+      return getIncInputSplits(metaClient, hadoopConf, commitTimeline, Collections.singletonList(instant), instantRange, instantTs, cdcEnabled);
+    }).flatMap(Collection::stream).collect(Collectors.toList());
+    return Result.instance(inputSplits, issuedInstant, offsetToIssue);
+  }
+
   @Nullable
   private InstantRange getInstantRange(String issuedInstant, String instantToIssue, boolean nullableBoundary) {
     if (issuedInstant != null) {
@@ -367,12 +437,12 @@ public class IncrementalInputSplits implements Serializable {
   private List<MergeOnReadInputSplit> getInputSplits(
       HoodieTableMetaClient metaClient,
       HoodieTimeline commitTimeline,
-      FileStatus[] fileStatuses,
+      List<StoragePathInfo> pathInfoList,
       Set<String> readPartitions,
       String endInstant,
       InstantRange instantRange,
       boolean skipBaseFiles) {
-    final HoodieTableFileSystemView fsView = new HoodieTableFileSystemView(metaClient, commitTimeline, fileStatuses);
+    final HoodieTableFileSystemView fsView = new HoodieTableFileSystemView(metaClient, commitTimeline, pathInfoList);
     final AtomicInteger cnt = new AtomicInteger(0);
     final String mergeType = this.conf.getString(FlinkOptions.MERGE_TYPE);
     return readPartitions.stream()
@@ -421,12 +491,12 @@ public class IncrementalInputSplits implements Serializable {
   }
 
   private FileIndex getFileIndex() {
-    FileIndex fileIndex = FileIndex.instance(new org.apache.hadoop.fs.Path(path.toUri()), conf, rowType);
-    if (this.requiredPartitions != null) {
-      // apply partition push down
-      fileIndex.setPartitionPaths(this.requiredPartitions);
-    }
-    return fileIndex;
+    return FileIndex.builder()
+        .path(new org.apache.hadoop.fs.Path(path.toUri()))
+        .conf(conf)
+        .rowType(rowType)
+        .partitionPruner(partitionPruner)
+        .build();
   }
 
   /**
@@ -437,11 +507,16 @@ public class IncrementalInputSplits implements Serializable {
    * @return the set of read partitions
    */
   private Set<String> getReadPartitions(List<HoodieCommitMetadata> metadataList) {
-    Set<String> partitions = HoodieInputFormatUtils.getWritePartitionPaths(metadataList);
+    Set<String> partitions = HoodieTableMetadataUtil.getWritePartitionPaths(metadataList);
     // apply partition push down
-    if (this.requiredPartitions != null) {
-      return partitions.stream()
-          .filter(this.requiredPartitions::contains).collect(Collectors.toSet());
+    if (this.partitionPruner != null) {
+      Set<String> selectedPartitions = this.partitionPruner.filter(partitions);
+      double total = partitions.size();
+      double selectedNum = selectedPartitions.size();
+      double percentPruned = total == 0 ? 0 : (1 - selectedNum / total) * 100;
+      LOG.info("Selected " + selectedNum + " partitions out of " + total
+          + ", pruned " + percentPruned + "% partitions.");
+      return selectedPartitions;
     }
     return partitions;
   }
@@ -476,13 +551,13 @@ public class IncrementalInputSplits implements Serializable {
 
   private HoodieTimeline getReadTimeline(HoodieTableMetaClient metaClient) {
     HoodieTimeline timeline = metaClient.getCommitsAndCompactionTimeline().filterCompletedAndCompactionInstants();
-    return filterInstantsByCondition(timeline);
+    return filterInstantsAsPerUserConfigs(timeline);
   }
 
   private HoodieTimeline getArchivedReadTimeline(HoodieTableMetaClient metaClient, String startInstant) {
     HoodieArchivedTimeline archivedTimeline = metaClient.getArchivedTimeline(startInstant, false);
     HoodieTimeline archivedCompleteTimeline = archivedTimeline.getCommitsTimeline().filterCompletedInstants();
-    return filterInstantsByCondition(archivedCompleteTimeline);
+    return filterInstantsAsPerUserConfigs(archivedCompleteTimeline);
   }
 
   /**
@@ -490,12 +565,13 @@ public class IncrementalInputSplits implements Serializable {
    *
    * @param commitTimeline The completed commits timeline
    * @param issuedInstant  The last issued instant that has already been delivered to downstream
+   *
    * @return the filtered hoodie instants
    */
   @VisibleForTesting
   public List<HoodieInstant> filterInstantsWithRange(
       HoodieTimeline commitTimeline,
-      final String issuedInstant) {
+      @Nullable final String issuedInstant) {
     HoodieTimeline completedTimeline = commitTimeline.filterCompletedInstants();
     if (issuedInstant != null) {
       // returns early for streaming mode
@@ -509,11 +585,7 @@ public class IncrementalInputSplits implements Serializable {
 
     if (OptionsResolver.hasNoSpecificReadCommits(this.conf)) {
       // by default read from the latest commit
-      List<HoodieInstant> instants = completedTimeline.getInstants();
-      if (instants.size() > 1) {
-        return Collections.singletonList(instants.get(instants.size() - 1));
-      }
-      return instants;
+      return completedTimeline.lastInstant().map(Collections::singletonList).orElseGet(Collections::emptyList);
     }
 
     if (OptionsResolver.isSpecificStartCommit(this.conf)) {
@@ -529,16 +601,16 @@ public class IncrementalInputSplits implements Serializable {
   }
 
   /**
-   * Filters out the unnecessary instants by user specified condition.
+   * Filters out the unnecessary instants as per user specified configs.
    *
    * @param timeline The timeline
    *
    * @return the filtered timeline
    */
   @VisibleForTesting
-  public HoodieTimeline filterInstantsByCondition(HoodieTimeline timeline) {
+  public HoodieTimeline filterInstantsAsPerUserConfigs(HoodieTimeline timeline) {
     final HoodieTimeline oriTimeline = timeline;
-    if (this.skipCompaction) {
+    if (OptionsResolver.isMorTable(this.conf) & this.skipCompaction) {
       // the compaction commit uses 'commit' as action which is tricky
       timeline = timeline.filter(instant -> !instant.getAction().equals(HoodieTimeline.COMMIT_ACTION));
     }
@@ -549,6 +621,12 @@ public class IncrementalInputSplits implements Serializable {
   }
 
   private static <T> List<T> mergeList(List<T> list1, List<T> list2) {
+    if (list1.isEmpty()) {
+      return list2;
+    }
+    if (list2.isEmpty()) {
+      return list1;
+    }
     List<T> merged = new ArrayList<>(list1);
     merged.addAll(list2);
     return merged;
@@ -564,6 +642,7 @@ public class IncrementalInputSplits implements Serializable {
   public static class Result {
     private final List<MergeOnReadInputSplit> inputSplits; // input splits
     private final String endInstant; // end instant to consume to
+    private final String offset;     // monotonic increasing consumption offset
 
     public static final Result EMPTY = instance(Collections.emptyList(), "");
 
@@ -579,13 +658,23 @@ public class IncrementalInputSplits implements Serializable {
       return this.endInstant;
     }
 
-    private Result(List<MergeOnReadInputSplit> inputSplits, String endInstant) {
+    @Nullable
+    public String getOffset() {
+      return offset;
+    }
+
+    private Result(List<MergeOnReadInputSplit> inputSplits, String endInstant, @Nullable String offset) {
       this.inputSplits = inputSplits;
       this.endInstant = endInstant;
+      this.offset = offset;
     }
 
     public static Result instance(List<MergeOnReadInputSplit> inputSplits, String endInstant) {
-      return new Result(inputSplits, endInstant);
+      return new Result(inputSplits, endInstant, null);
+    }
+
+    public static Result instance(List<MergeOnReadInputSplit> inputSplits, String endInstant, String offset) {
+      return new Result(inputSplits, endInstant, offset);
     }
   }
 
@@ -598,7 +687,7 @@ public class IncrementalInputSplits implements Serializable {
     private RowType rowType;
     private long maxCompactionMemoryInBytes;
     // for partition pruning
-    private Set<String> requiredPartitions;
+    private PartitionPruners.PartitionPruner partitionPruner;
     // skip compaction
     private boolean skipCompaction = false;
     // skip clustering
@@ -627,8 +716,8 @@ public class IncrementalInputSplits implements Serializable {
       return this;
     }
 
-    public Builder requiredPartitions(@Nullable Set<String> requiredPartitions) {
-      this.requiredPartitions = requiredPartitions;
+    public Builder partitionPruner(@Nullable PartitionPruners.PartitionPruner partitionPruner) {
+      this.partitionPruner = partitionPruner;
       return this;
     }
 
@@ -645,7 +734,7 @@ public class IncrementalInputSplits implements Serializable {
     public IncrementalInputSplits build() {
       return new IncrementalInputSplits(
           Objects.requireNonNull(this.conf), Objects.requireNonNull(this.path), Objects.requireNonNull(this.rowType),
-          this.maxCompactionMemoryInBytes, this.requiredPartitions, this.skipCompaction, this.skipClustering);
+          this.maxCompactionMemoryInBytes, this.partitionPruner, this.skipCompaction, this.skipClustering);
     }
   }
 }
