@@ -25,9 +25,11 @@ import org.apache.hudi.common.table.HoodieTableMetaClient;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
 import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.common.testutils.FileCreateUtils;
+import org.apache.hudi.common.testutils.HoodieTestUtils;
 import org.apache.hudi.configuration.FlinkOptions;
 import org.apache.hudi.index.HoodieIndex.IndexType;
-import org.apache.hudi.util.StreamerUtil;
+import org.apache.hudi.storage.HoodieStorage;
+import org.apache.hudi.storage.StoragePath;
 import org.apache.hudi.utils.FlinkMiniCluster;
 import org.apache.hudi.utils.TestConfigurations;
 import org.apache.hudi.utils.TestData;
@@ -36,8 +38,6 @@ import org.apache.hudi.utils.TestSQL;
 import org.apache.flink.table.api.EnvironmentSettings;
 import org.apache.flink.table.api.TableEnvironment;
 import org.apache.flink.table.api.internal.TableEnvironmentImpl;
-import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.Path;
 import org.junit.jupiter.api.extension.ExtendWith;
 import org.junit.jupiter.api.io.TempDir;
 import org.junit.jupiter.params.ParameterizedTest;
@@ -85,15 +85,15 @@ public class ITTestBucketStreamWrite {
 
     if (isCow) {
       TestData.checkWrittenData(tempFile, EXPECTED, 4);
-    } else  {
-      FileSystem fs = FSUtils.getFs(tempFile.getAbsolutePath(), new org.apache.hadoop.conf.Configuration());
-      TestData.checkWrittenDataMOR(fs, tempFile, EXPECTED, 4);
+    } else {
+      HoodieStorage storage = HoodieTestUtils.getStorage(tempFile.getAbsolutePath());
+      TestData.checkWrittenDataMOR(storage, tempFile, EXPECTED, 4);
     }
   }
 
   private static void doDeleteCommit(String tablePath, boolean isCow) throws Exception {
     // create metaClient
-    HoodieTableMetaClient metaClient = StreamerUtil.createMetaClient(tablePath, new org.apache.hadoop.conf.Configuration());
+    HoodieTableMetaClient metaClient = HoodieTestUtils.createMetaClient(tablePath);
 
     // should only contain one instant
     HoodieTimeline activeCompletedTimeline = metaClient.getActiveTimeline().filterCompletedInstants();
@@ -105,12 +105,13 @@ public class ITTestBucketStreamWrite {
     String filename = activeCompletedTimeline.getInstants().get(0).getFileName();
 
     HoodieCommitMetadata commitMetadata = HoodieCommitMetadata
-        .fromBytes(metaClient.getActiveTimeline().getInstantDetails(instant).get(), HoodieCommitMetadata.class);
+        .fromBytes(metaClient.getActiveTimeline().getInstantDetails(instant).get(),
+            HoodieCommitMetadata.class);
 
     // delete successful commit to simulate an unsuccessful write
-    FileSystem fs = metaClient.getFs();
-    Path path = new Path(metaClient.getMetaPath() + Path.SEPARATOR + filename);
-    fs.delete(path);
+    HoodieStorage storage = metaClient.getStorage();
+    StoragePath path = new StoragePath(metaClient.getMetaPath(), filename);
+    storage.deleteDirectory(path);
 
     // marker types are different for COW and MOR
     IOType ioType = isCow ? IOType.CREATE : IOType.APPEND;
@@ -120,7 +121,7 @@ public class ITTestBucketStreamWrite {
       String[] partitionFileNameSplit = relativePath.split("/");
       String fileInstant = FSUtils.getCommitTime(partitionFileNameSplit[1]);
       String partition = partitionFileNameSplit[0];
-      String writeToken = isCow ? getWriteToken(partitionFileNameSplit[1]) : FSUtils.getWriteTokenFromLogPath(new Path(relativePath));
+      String writeToken = isCow ? getWriteToken(partitionFileNameSplit[1]) : FSUtils.getWriteTokenFromLogPath(new StoragePath(relativePath));
       try {
         FileCreateUtils.createMarkerFile(tablePath, partition, commitInstant, fileInstant, fileId, ioType, writeToken);
       } catch (IOException e) {
