@@ -21,6 +21,7 @@ package org.apache.hudi.sink.cluster;
 import org.apache.hudi.avro.model.HoodieClusteringGroup;
 import org.apache.hudi.avro.model.HoodieClusteringPlan;
 import org.apache.hudi.client.HoodieFlinkWriteClient;
+import org.apache.hudi.common.model.HoodieTableType;
 import org.apache.hudi.common.model.WriteOperationType;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
 import org.apache.hudi.common.table.timeline.HoodieActiveTimeline;
@@ -47,21 +48,29 @@ import org.apache.hudi.utils.TestData;
 import org.apache.hudi.utils.TestSQL;
 
 import org.apache.avro.Schema;
+import org.apache.flink.api.common.RuntimeExecutionMode;
+import org.apache.flink.api.common.restartstrategy.RestartStrategies;
+import org.apache.flink.api.common.time.Time;
 import org.apache.flink.api.common.typeinfo.TypeInformation;
 import org.apache.flink.configuration.Configuration;
+import org.apache.flink.configuration.ExecutionOptions;
 import org.apache.flink.streaming.api.datastream.DataStream;
 import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
 import org.apache.flink.streaming.api.functions.sink.DiscardingSink;
 import org.apache.flink.table.api.DataTypes;
 import org.apache.flink.table.api.EnvironmentSettings;
+import org.apache.flink.table.api.Table;
 import org.apache.flink.table.api.TableEnvironment;
 import org.apache.flink.table.api.TableResult;
 import org.apache.flink.table.api.ValidationException;
+import org.apache.flink.table.api.bridge.java.StreamTableEnvironment;
 import org.apache.flink.table.api.config.ExecutionConfigOptions;
+import org.apache.flink.table.api.config.TableConfigOptions;
 import org.apache.flink.table.api.internal.TableEnvironmentImpl;
 import org.apache.flink.table.planner.plan.nodes.exec.utils.ExecNodeUtil;
 import org.apache.flink.table.types.DataType;
 import org.apache.flink.table.types.logical.RowType;
+import org.apache.flink.types.Row;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.extension.ExtendWith;
 import org.junit.jupiter.api.io.TempDir;
@@ -74,6 +83,7 @@ import java.util.concurrent.TimeUnit;
 import java.util.stream.Collectors;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
@@ -108,7 +118,6 @@ public class ITTestHoodieFlinkClustering {
 
     // use append mode
     options.put(FlinkOptions.OPERATION.key(), WriteOperationType.INSERT.value());
-    options.put(FlinkOptions.INSERT_CLUSTER.key(), "false");
 
     String hoodieTableDDL = TestConfigurations.getCreateHoodieTableDDL("t1", options);
     tableEnv.executeSql(hoodieTableDDL);
@@ -148,53 +157,54 @@ public class ITTestHoodieFlinkClustering {
     // To compute the clustering instant time and do clustering.
     String clusteringInstantTime = HoodieActiveTimeline.createNewInstantTime();
 
-    HoodieFlinkWriteClient writeClient = FlinkWriteClients.createWriteClient(conf);
-    HoodieFlinkTable<?> table = writeClient.getHoodieTable();
+    try (HoodieFlinkWriteClient writeClient = FlinkWriteClients.createWriteClient(conf)) {
+      HoodieFlinkTable<?> table = writeClient.getHoodieTable();
 
-    boolean scheduled = writeClient.scheduleClusteringAtInstant(clusteringInstantTime, Option.empty());
+      boolean scheduled = writeClient.scheduleClusteringAtInstant(clusteringInstantTime, Option.empty());
 
-    assertTrue(scheduled, "The clustering plan should be scheduled");
+      assertTrue(scheduled, "The clustering plan should be scheduled");
 
-    // fetch the instant based on the configured execution sequence
-    table.getMetaClient().reloadActiveTimeline();
-    HoodieTimeline timeline = table.getActiveTimeline().filterPendingReplaceTimeline()
-        .filter(instant -> instant.getState() == HoodieInstant.State.REQUESTED);
+      // fetch the instant based on the configured execution sequence
+      table.getMetaClient().reloadActiveTimeline();
+      HoodieTimeline timeline = table.getActiveTimeline().filterPendingReplaceTimeline()
+          .filter(instant -> instant.getState() == HoodieInstant.State.REQUESTED);
 
-    // generate clustering plan
-    // should support configurable commit metadata
-    Option<Pair<HoodieInstant, HoodieClusteringPlan>> clusteringPlanOption = ClusteringUtils.getClusteringPlan(
-        table.getMetaClient(), timeline.lastInstant().get());
+      // generate clustering plan
+      // should support configurable commit metadata
+      Option<Pair<HoodieInstant, HoodieClusteringPlan>> clusteringPlanOption = ClusteringUtils.getClusteringPlan(
+          table.getMetaClient(), timeline.lastInstant().get());
 
-    HoodieClusteringPlan clusteringPlan = clusteringPlanOption.get().getRight();
+      HoodieClusteringPlan clusteringPlan = clusteringPlanOption.get().getRight();
 
-    // Mark instant as clustering inflight
-    HoodieInstant instant = HoodieTimeline.getReplaceCommitRequestedInstant(clusteringInstantTime);
-    table.getActiveTimeline().transitionReplaceRequestedToInflight(instant, Option.empty());
+      // Mark instant as clustering inflight
+      HoodieInstant instant = HoodieTimeline.getReplaceCommitRequestedInstant(clusteringInstantTime);
+      table.getActiveTimeline().transitionReplaceRequestedToInflight(instant, Option.empty());
 
-    final Schema tableAvroSchema = StreamerUtil.getTableAvroSchema(table.getMetaClient(), false);
-    final DataType rowDataType = AvroSchemaConverter.convertToDataType(tableAvroSchema);
-    final RowType rowType = (RowType) rowDataType.getLogicalType();
+      final Schema tableAvroSchema = StreamerUtil.getTableAvroSchema(table.getMetaClient(), false);
+      final DataType rowDataType = AvroSchemaConverter.convertToDataType(tableAvroSchema);
+      final RowType rowType = (RowType) rowDataType.getLogicalType();
 
-    DataStream<ClusteringCommitEvent> dataStream = env.addSource(new ClusteringPlanSourceFunction(clusteringInstantTime, clusteringPlan))
-        .name("clustering_source")
-        .uid("uid_clustering_source")
-        .rebalance()
-        .transform("clustering_task",
-            TypeInformation.of(ClusteringCommitEvent.class),
-            new ClusteringOperator(conf, rowType))
-        .setParallelism(clusteringPlan.getInputGroups().size());
+      DataStream<ClusteringCommitEvent> dataStream = env.addSource(new ClusteringPlanSourceFunction(clusteringInstantTime, clusteringPlan, conf))
+          .name("clustering_source")
+          .uid("uid_clustering_source")
+          .rebalance()
+          .transform("clustering_task",
+              TypeInformation.of(ClusteringCommitEvent.class),
+              new ClusteringOperator(conf, rowType))
+          .setParallelism(clusteringPlan.getInputGroups().size());
 
-    ExecNodeUtil.setManagedMemoryWeight(dataStream.getTransformation(),
-        conf.getInteger(FlinkOptions.WRITE_SORT_MEMORY) * 1024L * 1024L);
+      ExecNodeUtil.setManagedMemoryWeight(dataStream.getTransformation(),
+          conf.getInteger(FlinkOptions.WRITE_SORT_MEMORY) * 1024L * 1024L);
 
-    dataStream
-        .addSink(new ClusteringCommitSink(conf))
-        .name("clustering_commit")
-        .uid("uid_clustering_commit")
-        .setParallelism(1);
+      dataStream
+          .addSink(new ClusteringCommitSink(conf))
+          .name("clustering_commit")
+          .uid("uid_clustering_commit")
+          .setParallelism(1);
 
-    env.execute("flink_hudi_clustering");
-    TestData.checkWrittenData(tempFile, EXPECTED, 4);
+      env.execute("flink_hudi_clustering");
+      TestData.checkWrittenData(tempFile, EXPECTED, 4);
+    }
   }
 
   @Test
@@ -209,7 +219,6 @@ public class ITTestHoodieFlinkClustering {
 
     // use append mode
     options.put(FlinkOptions.OPERATION.key(), WriteOperationType.INSERT.value());
-    options.put(FlinkOptions.INSERT_CLUSTER.key(), "false");
 
     String hoodieTableDDL = TestConfigurations.getCreateHoodieTableDDL("t1", options);
     tableEnv.executeSql(hoodieTableDDL);
@@ -246,7 +255,6 @@ public class ITTestHoodieFlinkClustering {
 
     // use append mode
     options.put(FlinkOptions.OPERATION.key(), WriteOperationType.INSERT.value());
-    options.put(FlinkOptions.INSERT_CLUSTER.key(), "false");
 
     String hoodieTableDDL = TestConfigurations.getCreateHoodieTableDDL("t1", options);
     tableEnv.executeSql(hoodieTableDDL);
@@ -285,21 +293,22 @@ public class ITTestHoodieFlinkClustering {
     // To compute the clustering instant time.
     String clusteringInstantTime = HoodieActiveTimeline.createNewInstantTime();
 
-    HoodieFlinkWriteClient writeClient = FlinkWriteClients.createWriteClient(conf);
+    try (HoodieFlinkWriteClient writeClient = FlinkWriteClients.createWriteClient(conf)) {
 
-    boolean scheduled = writeClient.scheduleClusteringAtInstant(clusteringInstantTime, Option.empty());
+      boolean scheduled = writeClient.scheduleClusteringAtInstant(clusteringInstantTime, Option.empty());
 
-    assertFalse(scheduled, "1 delta commit, the clustering plan should not be scheduled");
+      assertFalse(scheduled, "1 delta commit, the clustering plan should not be scheduled");
 
-    tableEnv.executeSql(TestSQL.INSERT_T1).await();
-    // wait for the asynchronous commit to finish
-    TimeUnit.SECONDS.sleep(3);
+      tableEnv.executeSql(TestSQL.INSERT_T1).await();
+      // wait for the asynchronous commit to finish
+      TimeUnit.SECONDS.sleep(3);
 
-    clusteringInstantTime = HoodieActiveTimeline.createNewInstantTime();
+      clusteringInstantTime = HoodieActiveTimeline.createNewInstantTime();
 
-    scheduled = writeClient.scheduleClusteringAtInstant(clusteringInstantTime, Option.empty());
+      scheduled = writeClient.scheduleClusteringAtInstant(clusteringInstantTime, Option.empty());
 
-    assertTrue(scheduled, "2 delta commits, the clustering plan should be scheduled");
+      assertTrue(scheduled, "2 delta commits, the clustering plan should be scheduled");
+    }
   }
 
   @Test
@@ -315,7 +324,6 @@ public class ITTestHoodieFlinkClustering {
 
     // use append mode
     options.put(FlinkOptions.OPERATION.key(), WriteOperationType.INSERT.value());
-    options.put(FlinkOptions.INSERT_CLUSTER.key(), "false");
 
     String hoodieTableDDL = TestConfigurations.getCreateHoodieTableDDL("t1", options);
     tableEnv.executeSql(hoodieTableDDL);
@@ -359,77 +367,78 @@ public class ITTestHoodieFlinkClustering {
     // To compute the clustering instant time and do clustering.
     String firstClusteringInstant = HoodieActiveTimeline.createNewInstantTime();
 
-    HoodieFlinkWriteClient<?> writeClient = FlinkWriteClients.createWriteClient(conf);
-    HoodieFlinkTable<?> table = writeClient.getHoodieTable();
+    try (HoodieFlinkWriteClient<?> writeClient = FlinkWriteClients.createWriteClient(conf)) {
+      HoodieFlinkTable<?> table = writeClient.getHoodieTable();
 
-    boolean scheduled = writeClient.scheduleClusteringAtInstant(firstClusteringInstant, Option.empty());
+      boolean scheduled = writeClient.scheduleClusteringAtInstant(firstClusteringInstant, Option.empty());
 
-    assertTrue(scheduled, "The clustering plan should be scheduled");
+      assertTrue(scheduled, "The clustering plan should be scheduled");
 
-    // fetch the instant based on the configured execution sequence
-    table.getMetaClient().reloadActiveTimeline();
-    HoodieTimeline timeline = table.getActiveTimeline().filterPendingReplaceTimeline()
-        .filter(i -> i.getState() == HoodieInstant.State.REQUESTED);
+      // fetch the instant based on the configured execution sequence
+      table.getMetaClient().reloadActiveTimeline();
+      HoodieTimeline timeline = table.getActiveTimeline().filterPendingReplaceTimeline()
+          .filter(i -> i.getState() == HoodieInstant.State.REQUESTED);
 
-    // generate clustering plan
-    // should support configurable commit metadata
-    Option<Pair<HoodieInstant, HoodieClusteringPlan>> clusteringPlanOption = ClusteringUtils.getClusteringPlan(
-        table.getMetaClient(), timeline.lastInstant().get());
+      // generate clustering plan
+      // should support configurable commit metadata
+      Option<Pair<HoodieInstant, HoodieClusteringPlan>> clusteringPlanOption = ClusteringUtils.getClusteringPlan(
+          table.getMetaClient(), timeline.lastInstant().get());
 
-    HoodieClusteringPlan clusteringPlan = clusteringPlanOption.get().getRight();
+      HoodieClusteringPlan clusteringPlan = clusteringPlanOption.get().getRight();
 
-    // Mark instant as clustering inflight
-    HoodieInstant instant = HoodieTimeline.getReplaceCommitRequestedInstant(firstClusteringInstant);
-    table.getActiveTimeline().transitionReplaceRequestedToInflight(instant, Option.empty());
+      // Mark instant as clustering inflight
+      HoodieInstant instant = HoodieTimeline.getReplaceCommitRequestedInstant(firstClusteringInstant);
+      table.getActiveTimeline().transitionReplaceRequestedToInflight(instant, Option.empty());
 
-    final Schema tableAvroSchema = StreamerUtil.getTableAvroSchema(table.getMetaClient(), false);
-    final DataType rowDataType = AvroSchemaConverter.convertToDataType(tableAvroSchema);
-    final RowType rowType = (RowType) rowDataType.getLogicalType();
+      final Schema tableAvroSchema = StreamerUtil.getTableAvroSchema(table.getMetaClient(), false);
+      final DataType rowDataType = AvroSchemaConverter.convertToDataType(tableAvroSchema);
+      final RowType rowType = (RowType) rowDataType.getLogicalType();
 
-    DataStream<ClusteringCommitEvent> dataStream =
-        env.addSource(new ClusteringPlanSourceFunction(firstClusteringInstant, clusteringPlan))
-            .name("clustering_source")
-            .uid("uid_clustering_source")
-            .rebalance()
-            .transform(
-                "clustering_task",
-                TypeInformation.of(ClusteringCommitEvent.class),
-                new ClusteringOperator(conf, rowType))
-            .setParallelism(clusteringPlan.getInputGroups().size());
+      DataStream<ClusteringCommitEvent> dataStream =
+          env.addSource(new ClusteringPlanSourceFunction(firstClusteringInstant, clusteringPlan, conf))
+              .name("clustering_source")
+              .uid("uid_clustering_source")
+              .rebalance()
+              .transform(
+                  "clustering_task",
+                  TypeInformation.of(ClusteringCommitEvent.class),
+                  new ClusteringOperator(conf, rowType))
+              .setParallelism(clusteringPlan.getInputGroups().size());
 
-    ExecNodeUtil.setManagedMemoryWeight(
-        dataStream.getTransformation(),
-        conf.getInteger(FlinkOptions.WRITE_SORT_MEMORY) * 1024L * 1024L);
+      ExecNodeUtil.setManagedMemoryWeight(
+          dataStream.getTransformation(),
+          conf.getInteger(FlinkOptions.WRITE_SORT_MEMORY) * 1024L * 1024L);
 
-    // keep pending clustering, not committing clustering
-    dataStream
-        .addSink(new DiscardingSink<>())
-        .name("clustering_commit")
-        .uid("uid_clustering_commit")
-        .setParallelism(1);
+      // keep pending clustering, not committing clustering
+      dataStream
+          .addSink(new DiscardingSink<>())
+          .name("discarding-sink")
+          .uid("uid_discarding-sink")
+          .setParallelism(1);
 
-    env.execute("flink_hudi_clustering");
+      env.execute("flink_hudi_clustering");
 
-    tableEnv.executeSql(TestSQL.INSERT_T1).await();
-    // wait for the asynchronous commit to finish
-    TimeUnit.SECONDS.sleep(3);
+      tableEnv.executeSql(TestSQL.INSERT_T1).await();
+      // wait for the asynchronous commit to finish
+      TimeUnit.SECONDS.sleep(3);
 
-    // archive the first commit, retain the second commit before the inflight replacecommit
-    writeClient.archive();
+      // archive the first commit, retain the second commit before the inflight replacecommit
+      writeClient.archive();
 
-    scheduled = writeClient.scheduleClusteringAtInstant(HoodieActiveTimeline.createNewInstantTime(), Option.empty());
+      scheduled = writeClient.scheduleClusteringAtInstant(HoodieActiveTimeline.createNewInstantTime(), Option.empty());
 
-    assertTrue(scheduled, "The clustering plan should be scheduled");
-    table.getMetaClient().reloadActiveTimeline();
-    timeline = table.getActiveTimeline().filterPendingReplaceTimeline()
-        .filter(i -> i.getState() == HoodieInstant.State.REQUESTED);
+      assertTrue(scheduled, "The clustering plan should be scheduled");
+      table.getMetaClient().reloadActiveTimeline();
+      timeline = table.getActiveTimeline().filterPendingReplaceTimeline()
+          .filter(i -> i.getState() == HoodieInstant.State.REQUESTED);
 
-    HoodieInstant secondClusteringInstant = timeline.lastInstant().get();
-    List<HoodieClusteringGroup> inputFileGroups = ClusteringUtils.getClusteringPlan(table.getMetaClient(), secondClusteringInstant).get().getRight().getInputGroups();
-    // clustering plan has no previous file slice generated by previous pending clustering
-    assertFalse(inputFileGroups
-        .stream().anyMatch(fg -> fg.getSlices()
-            .stream().anyMatch(s -> s.getDataFilePath().contains(firstClusteringInstant))));
+      HoodieInstant secondClusteringInstant = timeline.lastInstant().get();
+      List<HoodieClusteringGroup> inputFileGroups = ClusteringUtils.getClusteringPlan(table.getMetaClient(), secondClusteringInstant).get().getRight().getInputGroups();
+      // clustering plan has no previous file slice generated by previous pending clustering
+      assertFalse(inputFileGroups
+          .stream().anyMatch(fg -> fg.getSlices()
+              .stream().anyMatch(s -> s.getDataFilePath().contains(firstClusteringInstant))));
+    }
   }
 
   /**
@@ -555,55 +564,173 @@ public class ITTestHoodieFlinkClustering {
     // To compute the clustering instant time and do clustering.
     String clusteringInstantTime = HoodieActiveTimeline.createNewInstantTime();
 
-    HoodieFlinkWriteClient writeClient = FlinkWriteClients.createWriteClient(conf);
-    HoodieFlinkTable<?> table = writeClient.getHoodieTable();
+    try (HoodieFlinkWriteClient writeClient = FlinkWriteClients.createWriteClient(conf)) {
+      HoodieFlinkTable<?> table = writeClient.getHoodieTable();
 
-    boolean scheduled = writeClient.scheduleClusteringAtInstant(clusteringInstantTime, Option.empty());
+      boolean scheduled = writeClient.scheduleClusteringAtInstant(clusteringInstantTime, Option.empty());
 
-    assertTrue(scheduled, "The clustering plan should be scheduled");
+      assertTrue(scheduled, "The clustering plan should be scheduled");
 
-    // fetch the instant based on the configured execution sequence
-    table.getMetaClient().reloadActiveTimeline();
-    HoodieTimeline timeline = table.getActiveTimeline().filterPendingReplaceTimeline()
-        .filter(instant -> instant.getState() == HoodieInstant.State.REQUESTED);
+      // fetch the instant based on the configured execution sequence
+      table.getMetaClient().reloadActiveTimeline();
+      HoodieTimeline timeline = table.getActiveTimeline().filterPendingReplaceTimeline()
+          .filter(instant -> instant.getState() == HoodieInstant.State.REQUESTED);
 
-    // generate clustering plan
-    // should support configurable commit metadata
-    Option<Pair<HoodieInstant, HoodieClusteringPlan>> clusteringPlanOption = ClusteringUtils.getClusteringPlan(
-        table.getMetaClient(), timeline.lastInstant().get());
+      // generate clustering plan
+      // should support configurable commit metadata
+      Option<Pair<HoodieInstant, HoodieClusteringPlan>> clusteringPlanOption = ClusteringUtils.getClusteringPlan(
+          table.getMetaClient(), timeline.lastInstant().get());
 
-    HoodieClusteringPlan clusteringPlan = clusteringPlanOption.get().getRight();
+      HoodieClusteringPlan clusteringPlan = clusteringPlanOption.get().getRight();
 
-    // Mark instant as clustering inflight
-    HoodieInstant instant = HoodieTimeline.getReplaceCommitRequestedInstant(clusteringInstantTime);
-    table.getActiveTimeline().transitionReplaceRequestedToInflight(instant, Option.empty());
+      // Mark instant as clustering inflight
+      HoodieInstant instant = HoodieTimeline.getReplaceCommitRequestedInstant(clusteringInstantTime);
+      table.getActiveTimeline().transitionReplaceRequestedToInflight(instant, Option.empty());
 
-    DataStream<ClusteringCommitEvent> dataStream = env.addSource(new ClusteringPlanSourceFunction(clusteringInstantTime, clusteringPlan))
-        .name("clustering_source")
-        .uid("uid_clustering_source")
-        .rebalance()
-        .transform("clustering_task",
-            TypeInformation.of(ClusteringCommitEvent.class),
-            new ClusteringOperator(conf, rowType))
-        .setParallelism(clusteringPlan.getInputGroups().size());
+      DataStream<ClusteringCommitEvent> dataStream = env.addSource(new ClusteringPlanSourceFunction(clusteringInstantTime, clusteringPlan, conf))
+          .name("clustering_source")
+          .uid("uid_clustering_source")
+          .rebalance()
+          .transform("clustering_task",
+              TypeInformation.of(ClusteringCommitEvent.class),
+              new ClusteringOperator(conf, rowType))
+          .setParallelism(clusteringPlan.getInputGroups().size());
 
-    ExecNodeUtil.setManagedMemoryWeight(dataStream.getTransformation(),
-        conf.getInteger(FlinkOptions.WRITE_SORT_MEMORY) * 1024L * 1024L);
+      ExecNodeUtil.setManagedMemoryWeight(dataStream.getTransformation(),
+          conf.getInteger(FlinkOptions.WRITE_SORT_MEMORY) * 1024L * 1024L);
 
-    dataStream
-        .addSink(new ClusteringCommitSink(conf))
-        .name("clustering_commit")
-        .uid("uid_clustering_commit")
-        .setParallelism(1);
+      dataStream
+          .addSink(new ClusteringCommitSink(conf))
+          .name("clustering_commit")
+          .uid("uid_clustering_commit")
+          .setParallelism(1);
 
-    env.execute("flink_hudi_clustering");
+      env.execute("flink_hudi_clustering");
 
-    // test output
-    final Map<String, String> expected = new HashMap<>();
-    expected.put("par1", "[id1,par1,id1,Danny,23,1100001,par1, id2,par1,id2,Stephen,33,2100001,par1]");
-    expected.put("par2", "[id3,par2,id3,Julian,53,3100001,par2, id4,par2,id4,Fabian,31,4100001,par2]");
-    expected.put("par3", "[id5,par3,id5,Sophia,18,5100001,par3, id6,par3,id6,Emma,20,6100001,par3]");
-    expected.put("par4", "[id7,par4,id7,Bob,44,7100001,par4, id8,par4,id8,Han,56,8100001,par4]");
-    TestData.checkWrittenData(tempFile, expected, 4);
+      // test output
+      final Map<String, String> expected = new HashMap<>();
+      expected.put("par1", "[id1,par1,id1,Danny,23,1100001,par1, id2,par1,id2,Stephen,33,2100001,par1]");
+      expected.put("par2", "[id3,par2,id3,Julian,53,3100001,par2, id4,par2,id4,Fabian,31,4100001,par2]");
+      expected.put("par3", "[id5,par3,id5,Sophia,18,5100001,par3, id6,par3,id6,Emma,20,6100001,par3]");
+      expected.put("par4", "[id7,par4,id7,Bob,44,7100001,par4, id8,par4,id8,Han,56,8100001,par4]");
+      TestData.checkWrittenData(tempFile, expected, 4);
+    }
+  }
+
+  @Test
+  public void testOfflineClusterFailoverAfterCommit() throws Exception {
+    StreamTableEnvironment tableEnv = prepareEnvAndTable();
+
+    FlinkClusteringConfig cfg = new FlinkClusteringConfig();
+    cfg.path = tempFile.getAbsolutePath();
+    cfg.targetPartitions = 4;
+    Configuration conf = FlinkClusteringConfig.toFlinkConfig(cfg);
+    assertDoesNotThrow(() -> runOfflineCluster(tableEnv, conf));
+
+    Table result = tableEnv.sqlQuery("select count(*) from t1");
+    assertEquals(16L, tableEnv.toDataStream(result, Row.class).executeAndCollect(1).get(0).getField(0));
+  }
+
+  private StreamTableEnvironment prepareEnvAndTable() {
+    // Create hoodie table and insert into data.
+    Configuration conf = new org.apache.flink.configuration.Configuration();
+    conf.set(ExecutionOptions.RUNTIME_MODE, RuntimeExecutionMode.BATCH);
+    StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment(conf);
+    StreamTableEnvironment tEnv = StreamTableEnvironment.create(env);
+    tEnv.getConfig().getConfiguration().setInteger(ExecutionConfigOptions.TABLE_EXEC_RESOURCE_DEFAULT_PARALLELISM, 4);
+    tEnv.getConfig().getConfiguration().set(TableConfigOptions.TABLE_DML_SYNC, true);
+    Map<String, String> options = new HashMap<>();
+    options.put(FlinkOptions.PATH.key(), tempFile.getAbsolutePath());
+
+    // use append mode
+    options.put(FlinkOptions.OPERATION.key(), WriteOperationType.INSERT.value());
+    options.put(FlinkOptions.INSERT_CLUSTER.key(), "false");
+    options.put(FlinkOptions.TABLE_TYPE.key(), HoodieTableType.COPY_ON_WRITE.name());
+
+    String hoodieTableDDL = TestConfigurations.getCreateHoodieTableDDL("t1", options);
+    tEnv.executeSql(hoodieTableDDL);
+    tEnv.executeSql(TestSQL.INSERT_T1);
+    return tEnv;
+  }
+
+  /**
+   * schedule clustering, insert another batch, run clustering.
+   */
+  private void runOfflineCluster(TableEnvironment tableEnv, Configuration conf) throws Exception {
+    // Make configuration and setAvroSchema.
+    StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
+    env.setRestartStrategy(RestartStrategies.fixedDelayRestart(1, Time.milliseconds(1)));
+
+    // create metaClient
+    HoodieTableMetaClient metaClient = StreamerUtil.createMetaClient(conf);
+
+    // set the table name
+    conf.setString(FlinkOptions.TABLE_NAME, metaClient.getTableConfig().getTableName());
+
+    // set record key field
+    conf.setString(FlinkOptions.RECORD_KEY_FIELD, metaClient.getTableConfig().getRecordKeyFieldProp());
+    // set partition field
+    conf.setString(FlinkOptions.PARTITION_PATH_FIELD, metaClient.getTableConfig().getPartitionFieldProp());
+
+    long ckpTimeout = env.getCheckpointConfig().getCheckpointTimeout();
+    conf.setLong(FlinkOptions.WRITE_COMMIT_ACK_TIMEOUT, ckpTimeout);
+    conf.setString(FlinkOptions.PARTITION_PATH_FIELD, "partition");
+
+    // set table schema
+    CompactionUtil.setAvroSchema(conf, metaClient);
+
+    // judge whether have operation
+    // To compute the clustering instant time and do clustering.
+    String clusteringInstantTime = HoodieActiveTimeline.createNewInstantTime();
+
+    try (HoodieFlinkWriteClient writeClient = FlinkWriteClients.createWriteClient(conf)) {
+      HoodieFlinkTable<?> table = writeClient.getHoodieTable();
+
+      boolean scheduled = writeClient.scheduleClusteringAtInstant(clusteringInstantTime, Option.empty());
+
+      assertTrue(scheduled, "The clustering plan should be scheduled");
+
+      tableEnv.executeSql(TestSQL.INSERT_T1);
+
+      // fetch the instant based on the configured execution sequence
+      table.getMetaClient().reloadActiveTimeline();
+      HoodieTimeline timeline = table.getActiveTimeline().filterPendingReplaceTimeline()
+          .filter(instant -> instant.getState() == HoodieInstant.State.REQUESTED);
+
+      // generate clustering plan
+      // should support configurable commit metadata
+      Option<Pair<HoodieInstant, HoodieClusteringPlan>> clusteringPlanOption = ClusteringUtils.getClusteringPlan(
+          table.getMetaClient(), timeline.lastInstant().get());
+
+      HoodieClusteringPlan clusteringPlan = clusteringPlanOption.get().getRight();
+
+      // Mark instant as clustering inflight
+      HoodieInstant instant = HoodieTimeline.getReplaceCommitRequestedInstant(clusteringInstantTime);
+      table.getActiveTimeline().transitionReplaceRequestedToInflight(instant, Option.empty());
+
+      final Schema tableAvroSchema = StreamerUtil.getTableAvroSchema(table.getMetaClient(), false);
+      final DataType rowDataType = AvroSchemaConverter.convertToDataType(tableAvroSchema);
+      final RowType rowType = (RowType) rowDataType.getLogicalType();
+
+      DataStream<ClusteringCommitEvent> dataStream = env.addSource(new ClusteringPlanSourceFunction(clusteringInstantTime, clusteringPlan, conf))
+          .name("clustering_source")
+          .uid("uid_clustering_source")
+          .rebalance()
+          .transform("clustering_task",
+              TypeInformation.of(ClusteringCommitEvent.class),
+              new ClusteringOperator(conf, rowType))
+          .setParallelism(clusteringPlan.getInputGroups().size());
+
+      ExecNodeUtil.setManagedMemoryWeight(dataStream.getTransformation(),
+          conf.getInteger(FlinkOptions.WRITE_SORT_MEMORY) * 1024L * 1024L);
+
+      dataStream
+          .addSink(new ClusteringCommitTestSink(conf))
+          .name("clustering_commit")
+          .uid("uid_clustering_commit")
+          .setParallelism(1);
+
+      env.execute("flink_hudi_clustering");
+    }
   }
 }
