@@ -55,6 +55,10 @@ public class ClusteringFunctionWrapper {
    * Function that generates the {@code HoodieClusteringPlan}.
    */
   private ClusteringPlanOperator clusteringPlanOperator;
+  /**
+   * Output to collect the clustering plan events.
+   */
+  private CollectorOutput<ClusteringPlanEvent> planEventOutput;
   /**
    * Output to collect the clustering commit events.
    */
@@ -83,6 +87,8 @@ public class ClusteringFunctionWrapper {
 
   public void openFunction() throws Exception {
     clusteringPlanOperator = new ClusteringPlanOperator(conf);
+    planEventOutput =  new CollectorOutput<>();
+    clusteringPlanOperator.setup(streamTask, streamConfig, planEventOutput);
     clusteringPlanOperator.open();
 
     clusteringOperator = new ClusteringOperator(conf, TestConfigurations.ROW_TYPE);
