@@ -33,6 +33,7 @@ import org.apache.hudi.common.util.ClusteringUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.configuration.FlinkOptions;
 import org.apache.hudi.exception.HoodieIOException;
+import org.apache.hudi.index.HoodieIndex;
 import org.apache.hudi.table.HoodieFlinkTable;
 import org.apache.hudi.util.ClusteringUtil;
 import org.apache.hudi.util.FlinkTables;
@@ -114,6 +115,16 @@ public class TestClusteringUtil {
         .stream().map(HoodieInstant::getTimestamp).collect(Collectors.toList());
     assertThat(actualInstants, is(oriInstants));
   }
+  
+  @Test
+  void validateClusteringScheduling() throws Exception {
+    beforeEach();
+    ClusteringUtil.validateClusteringScheduling(this.conf);
+    
+    // validate bucket index
+    this.conf.setString(FlinkOptions.INDEX_TYPE, HoodieIndex.IndexType.BUCKET.name());
+    ClusteringUtil.validateClusteringScheduling(this.conf);
+  }
 
   /**
    * Generates a clustering plan on the timeline and returns its instant time.
