@@ -22,6 +22,7 @@ package org.apache.hudi.gcp.bigquery;
 import org.apache.hudi.common.util.StringUtils;
 import org.apache.hudi.common.util.VisibleForTesting;
 import org.apache.hudi.sync.common.HoodieSyncClient;
+import org.apache.hudi.sync.common.util.ManifestFileWriter;
 
 import com.google.cloud.bigquery.BigQuery;
 import com.google.cloud.bigquery.BigQueryException;
@@ -43,18 +44,21 @@ import com.google.cloud.bigquery.Table;
 import com.google.cloud.bigquery.TableId;
 import com.google.cloud.bigquery.TableInfo;
 import com.google.cloud.bigquery.ViewDefinition;
+import com.google.cloud.bigquery.StandardTableDefinition;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
-import java.util.UUID;
 import java.util.stream.Collectors;
 
+import static org.apache.hudi.gcp.bigquery.BigQuerySyncConfig.BIGQUERY_SYNC_BIG_LAKE_CONNECTION_ID;
 import static org.apache.hudi.gcp.bigquery.BigQuerySyncConfig.BIGQUERY_SYNC_DATASET_LOCATION;
 import static org.apache.hudi.gcp.bigquery.BigQuerySyncConfig.BIGQUERY_SYNC_DATASET_NAME;
 import static org.apache.hudi.gcp.bigquery.BigQuerySyncConfig.BIGQUERY_SYNC_PROJECT_ID;
+import static org.apache.hudi.gcp.bigquery.BigQuerySyncConfig.BIGQUERY_SYNC_REQUIRE_PARTITION_FILTER;
 
 public class HoodieBigQuerySyncClient extends HoodieSyncClient {
 
@@ -62,14 +66,18 @@ public class HoodieBigQuerySyncClient extends HoodieSyncClient {
 
   protected final BigQuerySyncConfig config;
   private final String projectId;
+  private final String bigLakeConnectionId;
   private final String datasetName;
+  private final boolean requirePartitionFilter;
   private transient BigQuery bigquery;
 
   public HoodieBigQuerySyncClient(final BigQuerySyncConfig config) {
     super(config);
     this.config = config;
     this.projectId = config.getString(BIGQUERY_SYNC_PROJECT_ID);
+    this.bigLakeConnectionId = config.getString(BIGQUERY_SYNC_BIG_LAKE_CONNECTION_ID);
     this.datasetName = config.getString(BIGQUERY_SYNC_DATASET_NAME);
+    this.requirePartitionFilter = config.getBoolean(BIGQUERY_SYNC_REQUIRE_PARTITION_FILTER);
     this.createBigQueryConnection();
   }
 
@@ -79,7 +87,9 @@ public class HoodieBigQuerySyncClient extends HoodieSyncClient {
     this.config = config;
     this.projectId = config.getString(BIGQUERY_SYNC_PROJECT_ID);
     this.datasetName = config.getString(BIGQUERY_SYNC_DATASET_NAME);
+    this.requirePartitionFilter = config.getBoolean(BIGQUERY_SYNC_REQUIRE_PARTITION_FILTER);
     this.bigquery = bigquery;
+    this.bigLakeConnectionId = config.getString(BIGQUERY_SYNC_BIG_LAKE_CONNECTION_ID);
   }
 
   private void createBigQueryConnection() {
@@ -95,19 +105,22 @@ public class HoodieBigQuerySyncClient extends HoodieSyncClient {
     }
   }
 
-  public void createTableUsingBqManifestFile(String tableName, String bqManifestFileUri, String sourceUriPrefix, Schema schema) {
+  public void createOrUpdateTableUsingBqManifestFile(String tableName, String bqManifestFileUri, String sourceUriPrefix, Schema schema) {
     try {
       String withClauses = String.format("( %s )", BigQuerySchemaResolver.schemaToSqlString(schema));
       String extraOptions = "enable_list_inference=true,";
       if (!StringUtils.isNullOrEmpty(sourceUriPrefix)) {
         withClauses += " WITH PARTITION COLUMNS";
-        extraOptions += String.format(" hive_partition_uri_prefix=\"%s\",", sourceUriPrefix);
+        extraOptions += String.format(" hive_partition_uri_prefix=\"%s\", require_hive_partition_filter=%s,", sourceUriPrefix, requirePartitionFilter);
+      }
+      if (!StringUtils.isNullOrEmpty(bigLakeConnectionId)) {
+        withClauses += String.format(" WITH CONNECTION `%s`", bigLakeConnectionId);
       }
 
       String query =
           String.format(
-              "CREATE EXTERNAL TABLE `%s.%s.%s` %s OPTIONS (%s "
-              + "uris=[\"%s\"], format=\"PARQUET\", file_set_spec_type=\"NEW_LINE_DELIMITED_MANIFEST\")",
+              "CREATE OR REPLACE EXTERNAL TABLE `%s.%s.%s` %s OPTIONS (%s "
+                  + "uris=[\"%s\"], format=\"PARQUET\", file_set_spec_type=\"NEW_LINE_DELIMITED_MANIFEST\")",
               projectId,
               datasetName,
               tableName,
@@ -118,7 +131,7 @@ public class HoodieBigQuerySyncClient extends HoodieSyncClient {
       QueryJobConfiguration queryConfig = QueryJobConfiguration.newBuilder(query)
           .setUseLegacySql(false)
           .build();
-      JobId jobId = JobId.of(UUID.randomUUID().toString());
+      JobId jobId = JobId.newBuilder().setProject(projectId).setRandomJob().build();
       Job queryJob = bigquery.create(JobInfo.newBuilder(queryConfig).setJobId(jobId).build());
 
       queryJob = queryJob.waitFor();
@@ -126,7 +139,7 @@ public class HoodieBigQuerySyncClient extends HoodieSyncClient {
       if (queryJob == null) {
         LOG.error("Job for table creation no longer exists");
       } else if (queryJob.getStatus().getError() != null) {
-        LOG.error("Job for table creation failed: " + queryJob.getStatus().getError().toString());
+        LOG.error("Job for table creation failed: {}", queryJob.getStatus().getError().toString());
       } else {
         LOG.info("External table created using manifest file.");
       }
@@ -171,20 +184,37 @@ public class HoodieBigQuerySyncClient extends HoodieSyncClient {
     Table existingTable = bigquery.getTable(TableId.of(projectId, datasetName, tableName));
     ExternalTableDefinition definition = existingTable.getDefinition();
     Schema remoteTableSchema = definition.getSchema();
-    // Add the partition fields into the schema to avoid conflicts while updating
-    List<Field> updatedTableFields = remoteTableSchema.getFields().stream()
+    List<Field> finalTableFields = new ArrayList<>(schema.getFields());
+    // Add the partition fields into the schema to avoid conflicts while updating. And ensure the partition fields are at the end to
+    // avoid unnecessary updates.
+    List<Field> bqPartitionFields = remoteTableSchema.getFields().stream()
         .filter(field -> partitionFields.contains(field.getName()))
         .collect(Collectors.toList());
-    updatedTableFields.addAll(schema.getFields());
-    Schema finalSchema = Schema.of(updatedTableFields);
-    if (definition.getSchema() != null && definition.getSchema().equals(finalSchema)) {
+    finalTableFields.addAll(bqPartitionFields);
+    Schema finalSchema = Schema.of(finalTableFields);
+    boolean sameSchema = definition.getSchema() != null && definition.getSchema().equals(finalSchema);
+    boolean samePartitionFilter = partitionFields.isEmpty()
+        || (requirePartitionFilter == (definition.getHivePartitioningOptions().getRequirePartitionFilter() != null && definition.getHivePartitioningOptions().getRequirePartitionFilter()));
+    if (sameSchema && samePartitionFilter) {
+      LOG.info("No table update is needed.");
       return; // No need to update schema.
     }
-    Table updatedTable = existingTable.toBuilder()
-        .setDefinition(definition.toBuilder().setSchema(finalSchema).setAutodetect(false).build())
-        .build();
-
-    bigquery.update(updatedTable);
+    if (!StringUtils.isNullOrEmpty(bigLakeConnectionId)) {
+      Table updatedTable =
+              existingTable.toBuilder().setDefinition(StandardTableDefinition.of(finalSchema)).build();
+      updatedTable.update();
+    } else {
+      ExternalTableDefinition.Builder builder = definition.toBuilder();
+      builder.setSchema(finalSchema);
+      builder.setAutodetect(false);
+      if (definition.getHivePartitioningOptions() != null) {
+        builder.setHivePartitioningOptions(definition.getHivePartitioningOptions().toBuilder().setRequirePartitionFilter(requirePartitionFilter).build());
+      }
+      Table updatedTable = existingTable.toBuilder()
+              .setDefinition(builder.build())
+              .build();
+      bigquery.update(updatedTable);
+    }
   }
 
   public void createVersionsTable(String tableName, String sourceUri, String sourceUriPrefix, List<String> partitionFields) {
@@ -265,6 +295,28 @@ public class HoodieBigQuerySyncClient extends HoodieSyncClient {
     return table != null && table.exists();
   }
 
+  /**
+   * Checks for the existence of a table that uses the manifest file approach and matches other requirements.
+   * @param tableName name of the table
+   * @return Returns true if the table does not exist or if the table does exist but does not use the manifest file. False otherwise.
+   */
+  public boolean tableNotExistsOrDoesNotMatchSpecification(String tableName) {
+    TableId tableId = TableId.of(projectId, datasetName, tableName);
+    Table table = bigquery.getTable(tableId);
+    if (table == null || !table.exists()) {
+      return true;
+    }
+    ExternalTableDefinition externalTableDefinition = table.getDefinition();
+    boolean manifestDoesNotExist =
+        externalTableDefinition.getSourceUris() == null
+            || externalTableDefinition.getSourceUris().stream().noneMatch(uri -> uri.contains(ManifestFileWriter.ABSOLUTE_PATH_MANIFEST_FOLDER_NAME));
+    if (!StringUtils.isNullOrEmpty(config.getString(BIGQUERY_SYNC_BIG_LAKE_CONNECTION_ID))) {
+      // If bigLakeConnectionId is present and connectionId is not present in table definition, we need to replace the table.
+      return manifestDoesNotExist || externalTableDefinition.getConnectionId() == null;
+    }
+    return manifestDoesNotExist;
+  }
+
   @Override
   public void close() {
     bigquery = null;
