@@ -40,8 +40,11 @@ import org.apache.hudi.hadoop.HiveHoodieTableFileIndex;
 import org.apache.hudi.hadoop.HoodieCopyOnWriteTableInputFormat;
 import org.apache.hudi.hadoop.LocatedFileStatusWithBootstrapBaseFile;
 import org.apache.hudi.hadoop.RealtimeFileStatus;
+import org.apache.hudi.hadoop.fs.HadoopFSUtils;
 import org.apache.hudi.hadoop.utils.HoodieInputFormatUtils;
 import org.apache.hudi.hadoop.utils.HoodieRealtimeInputFormatUtils;
+import org.apache.hudi.metadata.HoodieTableMetadataUtil;
+import org.apache.hudi.storage.StoragePathInfo;
 
 import org.apache.avro.Schema;
 import org.apache.hadoop.conf.Configurable;
@@ -110,6 +113,24 @@ public class HoodieMergeOnReadTableInputFormat extends HoodieCopyOnWriteTableInp
     }
   }
 
+  /**
+   * return non hoodie paths
+   * @param job
+   * @return
+   * @throws IOException
+   */
+  @Override
+  public FileStatus[] listStatusForNonHoodiePaths(JobConf job) throws IOException {
+    FileStatus[] fileStatuses = doListStatus(job);
+    List<FileStatus> result = new ArrayList<>();
+    for (FileStatus fileStatus : fileStatuses) {
+      String baseFilePath = fileStatus.getPath().toUri().toString();
+      RealtimeFileStatus realtimeFileStatus = new RealtimeFileStatus(fileStatus, baseFilePath, new ArrayList<>(), false, Option.empty());
+      result.add(realtimeFileStatus);
+    }
+    return result.toArray(new FileStatus[0]);
+  }
+
   @Override
   protected boolean checkIfValidFileSlice(FileSlice fileSlice) {
     Option<HoodieBaseFile> baseFileOpt = fileSlice.getBaseFile();
@@ -165,28 +186,35 @@ public class HoodieMergeOnReadTableInputFormat extends HoodieCopyOnWriteTableInp
           try {
             return TimelineUtils.getCommitMetadata(instant, commitsTimelineToReturn);
           } catch (IOException e) {
-            throw new HoodieException(String.format("cannot get metadata for instant: %s", instant));
+            throw new HoodieException(
+                String.format("cannot get metadata for instant: %s", instant));
           }
         }).collect(Collectors.toList());
 
     // build fileGroup from fsView
-    List<FileStatus> affectedFileStatus = Arrays.asList(HoodieInputFormatUtils
-        .listAffectedFilesForCommits(job, new Path(tableMetaClient.getBasePath()), metadataList));
+    List<StoragePathInfo> affectedPathInfoList = HoodieInputFormatUtils
+        .listAffectedFilesForCommits(job, tableMetaClient.getBasePathV2(),
+            metadataList);
     // step3
-    HoodieTableFileSystemView fsView = new HoodieTableFileSystemView(tableMetaClient, commitsTimelineToReturn, affectedFileStatus.toArray(new FileStatus[0]));
+    HoodieTableFileSystemView fsView = new HoodieTableFileSystemView(
+        tableMetaClient, commitsTimelineToReturn, affectedPathInfoList);
     // build fileGroup from fsView
     Path basePath = new Path(tableMetaClient.getBasePath());
     // filter affectedPartition by inputPaths
-    List<String> affectedPartition = HoodieInputFormatUtils.getWritePartitionPaths(metadataList).stream()
-        .filter(k -> k.isEmpty() ? inputPaths.contains(basePath) : inputPaths.contains(new Path(basePath, k))).collect(Collectors.toList());
+    List<String> affectedPartition =
+        HoodieTableMetadataUtil.getWritePartitionPaths(metadataList).stream()
+            .filter(k -> k.isEmpty() ? inputPaths.contains(basePath) :
+                inputPaths.contains(new Path(basePath, k))).collect(Collectors.toList());
     if (affectedPartition.isEmpty()) {
       return result;
     }
     List<HoodieFileGroup> fileGroups = affectedPartition.stream()
-        .flatMap(partitionPath -> fsView.getAllFileGroups(partitionPath)).collect(Collectors.toList());
+        .flatMap(partitionPath -> fsView.getAllFileGroups(partitionPath))
+        .collect(Collectors.toList());
     // step4
     setInputPaths(job, affectedPartition.stream()
-        .map(p -> p.isEmpty() ? basePath.toString() : new Path(basePath, p).toString()).collect(Collectors.joining(",")));
+        .map(p -> p.isEmpty() ? basePath.toString() : new Path(basePath, p).toString())
+        .collect(Collectors.joining(",")));
 
     // step5
     // find all file status in partitionPaths.
@@ -261,10 +289,13 @@ public class HoodieMergeOnReadTableInputFormat extends HoodieCopyOnWriteTableInp
         }
         // add file group which has only logs.
         if (f.getLatestFileSlice().isPresent() && baseFiles.isEmpty()) {
-          List<FileStatus> logFileStatus = f.getLatestFileSlice().get().getLogFiles().map(logFile -> logFile.getFileStatus()).collect(Collectors.toList());
-          if (logFileStatus.size() > 0) {
-            List<HoodieLogFile> deltaLogFiles = logFileStatus.stream().map(l -> new HoodieLogFile(l.getPath(), l.getLen())).collect(Collectors.toList());
-            RealtimeFileStatus fileStatus = new RealtimeFileStatus(logFileStatus.get(0), basePath,
+          List<StoragePathInfo> logPathInfoList = f.getLatestFileSlice().get().getLogFiles()
+              .map(logFile -> logFile.getPathInfo()).collect(Collectors.toList());
+          if (logPathInfoList.size() > 0) {
+            List<HoodieLogFile> deltaLogFiles = logPathInfoList.stream()
+                .map(l -> new HoodieLogFile(l.getPath(), l.getLength())).collect(Collectors.toList());
+            RealtimeFileStatus fileStatus = new RealtimeFileStatus(
+                HadoopFSUtils.convertToHadoopFileStatus(logPathInfoList.get(0)), basePath,
                 deltaLogFiles, true, virtualKeyInfoOpt);
             fileStatus.setMaxCommitTime(maxCommitTime);
             result.add(fileStatus);
@@ -367,7 +398,8 @@ public class HoodieMergeOnReadTableInputFormat extends HoodieCopyOnWriteTableInp
                                                                       Option<HoodieVirtualKeyInfo> virtualKeyInfoOpt) {
     List<HoodieLogFile> sortedLogFiles = logFiles.sorted(HoodieLogFile.getLogFileComparator()).collect(Collectors.toList());
     try {
-      RealtimeFileStatus rtFileStatus = new RealtimeFileStatus(latestLogFile.getFileStatus(), basePath,
+      RealtimeFileStatus rtFileStatus = new RealtimeFileStatus(
+          HadoopFSUtils.convertToHadoopFileStatus(latestLogFile.getPathInfo()), basePath,
           sortedLogFiles, false, virtualKeyInfoOpt);
 
       if (latestCompletedInstantOpt.isPresent()) {
