@@ -18,19 +18,6 @@
 
 package org.apache.hudi.hadoop.utils;
 
-import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.fs.FileStatus;
-import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.LocatedFileStatus;
-import org.apache.hadoop.fs.Path;
-import org.apache.hadoop.hive.ql.io.orc.OrcInputFormat;
-import org.apache.hadoop.hive.ql.io.orc.OrcOutputFormat;
-import org.apache.hadoop.hive.ql.io.orc.OrcSerde;
-import org.apache.hadoop.hive.ql.io.parquet.MapredParquetOutputFormat;
-import org.apache.hadoop.hive.ql.io.parquet.serde.ParquetHiveSerDe;
-import org.apache.hadoop.mapred.FileInputFormat;
-import org.apache.hadoop.mapreduce.Job;
-import org.apache.hadoop.mapreduce.JobContext;
 import org.apache.hudi.common.config.HoodieMetadataConfig;
 import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.HoodieBaseFile;
@@ -41,6 +28,7 @@ import org.apache.hudi.common.table.HoodieTableMetaClient;
 import org.apache.hudi.common.table.timeline.HoodieDefaultTimeline;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
 import org.apache.hudi.common.table.timeline.HoodieTimeline;
+import org.apache.hudi.common.table.timeline.TimelineUtils.HollowCommitHandling;
 import org.apache.hudi.common.table.view.HoodieTableFileSystemView;
 import org.apache.hudi.common.table.view.TableFileSystemView;
 import org.apache.hudi.common.util.Option;
@@ -51,13 +39,36 @@ import org.apache.hudi.hadoop.FileStatusWithBootstrapBaseFile;
 import org.apache.hudi.hadoop.HoodieHFileInputFormat;
 import org.apache.hudi.hadoop.HoodieParquetInputFormat;
 import org.apache.hudi.hadoop.LocatedFileStatusWithBootstrapBaseFile;
+import org.apache.hudi.hadoop.fs.HadoopFSUtils;
 import org.apache.hudi.hadoop.realtime.HoodieHFileRealtimeInputFormat;
 import org.apache.hudi.hadoop.realtime.HoodieParquetRealtimeInputFormat;
-import org.apache.log4j.LogManager;
-import org.apache.log4j.Logger;
+import org.apache.hudi.hadoop.realtime.HoodieRealtimeFileSplit;
+import org.apache.hudi.hadoop.realtime.HoodieRealtimePath;
+import org.apache.hudi.storage.HoodieStorage;
+import org.apache.hudi.storage.HoodieStorageUtils;
+import org.apache.hudi.storage.StoragePath;
+import org.apache.hudi.storage.StoragePathInfo;
+import org.apache.hudi.storage.hadoop.HoodieHadoopStorage;
+
+import org.apache.hadoop.conf.Configuration;
+import org.apache.hadoop.fs.FileStatus;
+import org.apache.hadoop.fs.LocatedFileStatus;
+import org.apache.hadoop.fs.Path;
+import org.apache.hadoop.hive.ql.io.orc.OrcInputFormat;
+import org.apache.hadoop.hive.ql.io.orc.OrcOutputFormat;
+import org.apache.hadoop.hive.ql.io.orc.OrcSerde;
+import org.apache.hadoop.hive.ql.io.parquet.MapredParquetOutputFormat;
+import org.apache.hadoop.hive.ql.io.parquet.serde.ParquetHiveSerDe;
+import org.apache.hadoop.mapred.FileInputFormat;
+import org.apache.hadoop.mapred.FileSplit;
+import org.apache.hadoop.mapreduce.Job;
+import org.apache.hadoop.mapreduce.JobContext;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -67,9 +78,12 @@ import java.util.Set;
 import java.util.function.Function;
 import java.util.stream.Collectors;
 
+import static org.apache.hudi.common.config.HoodieCommonConfig.INCREMENTAL_READ_HANDLE_HOLLOW_COMMIT;
 import static org.apache.hudi.common.config.HoodieMetadataConfig.DEFAULT_METADATA_ENABLE_FOR_READERS;
 import static org.apache.hudi.common.config.HoodieMetadataConfig.ENABLE;
 import static org.apache.hudi.common.table.HoodieTableMetaClient.METAFOLDER_NAME;
+import static org.apache.hudi.common.table.timeline.TimelineUtils.handleHollowCommitIfNeeded;
+import static org.apache.hudi.hadoop.fs.HadoopFSUtils.convertToStoragePath;
 
 public class HoodieInputFormatUtils {
 
@@ -79,7 +93,7 @@ public class HoodieInputFormatUtils {
   public static final int HOODIE_PARTITION_PATH_COL_POS = 3;
   public static final String HOODIE_READ_COLUMNS_PROP = "hoodie.read.columns.set";
 
-  private static final Logger LOG = LogManager.getLogger(HoodieInputFormatUtils.class);
+  private static final Logger LOG = LoggerFactory.getLogger(HoodieInputFormatUtils.class);
 
   public static FileInputFormat getInputFormat(HoodieFileFormat baseFileFormat, boolean realtime, Configuration conf) {
     switch (baseFileFormat) {
@@ -108,6 +122,16 @@ public class HoodieInputFormatUtils {
     }
   }
 
+  public static String getInputFormatClassName(HoodieFileFormat baseFileFormat, boolean realtime, boolean usePreApacheFormat) {
+    if (baseFileFormat.equals(HoodieFileFormat.PARQUET) && usePreApacheFormat) {
+      // Parquet input format had an InputFormat class visible under the old naming scheme.
+      return realtime
+          ? com.uber.hoodie.hadoop.realtime.HoodieRealtimeInputFormat.class.getName()
+          : com.uber.hoodie.hadoop.HoodieInputFormat.class.getName();
+    }
+    return getInputFormatClassName(baseFileFormat, realtime);
+  }
+
   public static String getInputFormatClassName(HoodieFileFormat baseFileFormat, boolean realtime) {
     switch (baseFileFormat) {
       case PARQUET:
@@ -132,7 +156,6 @@ public class HoodieInputFormatUtils {
   public static String getOutputFormatClassName(HoodieFileFormat baseFileFormat) {
     switch (baseFileFormat) {
       case PARQUET:
-        return MapredParquetOutputFormat.class.getName();
       case HFILE:
         return MapredParquetOutputFormat.class.getName();
       case ORC:
@@ -145,7 +168,6 @@ public class HoodieInputFormatUtils {
   public static String getSerDeClassName(HoodieFileFormat baseFileFormat) {
     switch (baseFileFormat) {
       case PARQUET:
-        return ParquetHiveSerDe.class.getName();
       case HFILE:
         return ParquetHiveSerDe.class.getName();
       case ORC:
@@ -164,7 +186,7 @@ public class HoodieInputFormatUtils {
       return getInputFormat(HoodieFileFormat.HFILE, realtime, conf);
     }
     // now we support read log file, try to find log file
-    if (FSUtils.isLogFile(new Path(path)) && realtime) {
+    if (HadoopFSUtils.isLogFile(new Path(path)) && realtime) {
       return getInputFormat(HoodieFileFormat.PARQUET, realtime, conf);
     }
     throw new HoodieIOException("Hoodie InputFormat not implemented for base file of type " + extension);
@@ -173,16 +195,17 @@ public class HoodieInputFormatUtils {
   /**
    * Filter any specific instants that we do not want to process.
    * example timeline:
-   *
+   * <p>
    * t0 -> create bucket1.parquet
    * t1 -> create and append updates bucket1.log
    * t2 -> request compaction
    * t3 -> create bucket2.parquet
-   *
+   * <p>
    * if compaction at t2 takes a long time, incremental readers on RO tables can move to t3 and would skip updates in t1
-   *
+   * <p>
    * To workaround this problem, we want to stop returning data belonging to commits > t2.
    * After compaction is complete, incremental reader would see updates in t2, t3, so on.
+   *
    * @param timeline
    * @return
    */
@@ -206,6 +229,7 @@ public class HoodieInputFormatUtils {
 
   /**
    * Extract partitions touched by the commitsToCheck.
+   *
    * @param commitsToCheck
    * @param tableMetaClient
    * @param timeline
@@ -214,9 +238,9 @@ public class HoodieInputFormatUtils {
    * @throws IOException
    */
   public static Option<String> getAffectedPartitions(List<HoodieInstant> commitsToCheck,
-                                      HoodieTableMetaClient tableMetaClient,
-                                      HoodieTimeline timeline,
-                                      List<Path> inputPaths) throws IOException {
+                                                     HoodieTableMetaClient tableMetaClient,
+                                                     HoodieTimeline timeline,
+                                                     List<Path> inputPaths) throws IOException {
     Set<String> partitionsToList = new HashSet<>();
     for (HoodieInstant commit : commitsToCheck) {
       HoodieCommitMetadata commitMetadata = HoodieCommitMetadata.fromBytes(timeline.getInstantDetails(commit).get(),
@@ -227,7 +251,7 @@ public class HoodieInputFormatUtils {
       return Option.empty();
     }
     String incrementalInputPaths = partitionsToList.stream()
-        .map(s -> StringUtils.isNullOrEmpty(s) ? tableMetaClient.getBasePath() : tableMetaClient.getBasePath() + Path.SEPARATOR + s)
+        .map(s -> StringUtils.isNullOrEmpty(s) ? tableMetaClient.getBasePath() : tableMetaClient.getBasePath() + StoragePath.SEPARATOR + s)
         .filter(s -> {
           /*
            * Ensure to return only results from the original input path that has incremental changes
@@ -260,6 +284,7 @@ public class HoodieInputFormatUtils {
 
   /**
    * Extract HoodieTimeline based on HoodieTableMetaClient.
+   *
    * @param job
    * @param tableMetaClient
    * @return
@@ -272,11 +297,19 @@ public class HoodieInputFormatUtils {
     } else {
       baseTimeline = tableMetaClient.getActiveTimeline();
     }
-    return Option.of(baseTimeline.getCommitsTimeline().filterCompletedInstants());
+    HollowCommitHandling handlingMode = HollowCommitHandling.valueOf(job.getConfiguration()
+        .get(INCREMENTAL_READ_HANDLE_HOLLOW_COMMIT.key(), INCREMENTAL_READ_HANDLE_HOLLOW_COMMIT.defaultValue()));
+    HoodieTimeline filteredTimeline = handleHollowCommitIfNeeded(
+        baseTimeline.getCommitsTimeline().filterCompletedInstants(),
+        tableMetaClient,
+        handlingMode);
+
+    return Option.of(filteredTimeline);
   }
 
   /**
    * Get commits for incremental query from Hive map reduce configuration.
+   *
    * @param job
    * @param tableName
    * @param timeline
@@ -304,6 +337,7 @@ public class HoodieInputFormatUtils {
 
   /**
    * Extract HoodieTableMetaClient by partition path.
+   *
    * @param conf       The hadoop conf
    * @param partitions The partitions
    * @return partition path to table meta client mapping
@@ -326,15 +360,17 @@ public class HoodieInputFormatUtils {
    */
   public static HoodieTableMetaClient getTableMetaClientForBasePathUnchecked(Configuration conf, Path partitionPath) throws IOException {
     Path baseDir = partitionPath;
-    FileSystem fs = partitionPath.getFileSystem(conf);
-    if (HoodiePartitionMetadata.hasPartitionMetadata(fs, partitionPath)) {
-      HoodiePartitionMetadata metadata = new HoodiePartitionMetadata(fs, partitionPath);
+    HoodieStorage storage = HoodieStorageUtils.getStorage(
+        partitionPath.toString(), HadoopFSUtils.getStorageConf(conf));
+    StoragePath partitionStoragePath = convertToStoragePath(partitionPath);
+    if (HoodiePartitionMetadata.hasPartitionMetadata(storage,  partitionStoragePath)) {
+      HoodiePartitionMetadata metadata = new HoodiePartitionMetadata(storage, partitionStoragePath);
       metadata.readFromFS();
       int levels = metadata.getPartitionDepth();
       baseDir = HoodieHiveUtils.getNthParent(partitionPath, levels);
     } else {
       for (int i = 0; i < partitionPath.depth(); i++) {
-        if (fs.exists(new Path(baseDir, METAFOLDER_NAME))) {
+        if (storage.exists(new StoragePath(convertToStoragePath(baseDir), METAFOLDER_NAME))) {
           break;
         } else if (i == partitionPath.depth() - 1) {
           throw new TableNotFoundException(partitionPath.toString());
@@ -344,24 +380,29 @@ public class HoodieInputFormatUtils {
       }
     }
     LOG.info("Reading hoodie metadata from path " + baseDir.toString());
-    return HoodieTableMetaClient.builder().setConf(fs.getConf()).setBasePath(baseDir.toString()).build();
+    return HoodieTableMetaClient.builder()
+        .setConf(storage.getConf().newInstance()).setBasePath(baseDir.toString()).build();
   }
 
   public static FileStatus getFileStatus(HoodieBaseFile baseFile) throws IOException {
+    FileStatus fileStatus = HadoopFSUtils.convertToHadoopFileStatus(baseFile.getPathInfo());
     if (baseFile.getBootstrapBaseFile().isPresent()) {
-      if (baseFile.getFileStatus() instanceof LocatedFileStatus) {
-        return new LocatedFileStatusWithBootstrapBaseFile((LocatedFileStatus)baseFile.getFileStatus(),
-            baseFile.getBootstrapBaseFile().get().getFileStatus());
+      if (fileStatus instanceof LocatedFileStatus) {
+        return new LocatedFileStatusWithBootstrapBaseFile((LocatedFileStatus) fileStatus,
+            HadoopFSUtils.convertToHadoopFileStatus(
+                baseFile.getBootstrapBaseFile().get().getPathInfo()));
       } else {
-        return new FileStatusWithBootstrapBaseFile(baseFile.getFileStatus(),
-            baseFile.getBootstrapBaseFile().get().getFileStatus());
+        return new FileStatusWithBootstrapBaseFile(fileStatus,
+            HadoopFSUtils.convertToHadoopFileStatus(
+                baseFile.getBootstrapBaseFile().get().getPathInfo()));
       }
     }
-    return baseFile.getFileStatus();
+    return fileStatus;
   }
 
   /**
    * Filter a list of FileStatus based on commitsToCheck for incremental view.
+   *
    * @param job
    * @param tableMetaClient
    * @param timeline
@@ -370,8 +411,11 @@ public class HoodieInputFormatUtils {
    * @return
    */
   public static List<FileStatus> filterIncrementalFileStatus(Job job, HoodieTableMetaClient tableMetaClient,
-      HoodieTimeline timeline, FileStatus[] fileStatuses, List<HoodieInstant> commitsToCheck) throws IOException {
-    TableFileSystemView.BaseFileOnlyView roView = new HoodieTableFileSystemView(tableMetaClient, timeline, fileStatuses);
+                                                             HoodieTimeline timeline, FileStatus[] fileStatuses, List<HoodieInstant> commitsToCheck) throws IOException {
+    TableFileSystemView.BaseFileOnlyView roView = new HoodieTableFileSystemView(tableMetaClient, timeline,
+        Arrays.stream(fileStatuses)
+            .map(HadoopFSUtils::convertToStoragePathInfo)
+            .collect(Collectors.toList()));
     List<String> commitsList = commitsToCheck.stream().map(HoodieInstant::getTimestamp).collect(Collectors.toList());
     List<HoodieBaseFile> filteredFiles = roView.getLatestBaseFilesInRange(commitsList).collect(Collectors.toList());
     List<FileStatus> returns = new ArrayList<>();
@@ -385,8 +429,9 @@ public class HoodieInputFormatUtils {
   }
 
   /**
-   * Takes in a list of filesStatus and a list of table metadatas. Groups the files status list
+   * Takes in a list of filesStatus and a list of table metadata. Groups the files status list
    * based on given table metadata.
+   *
    * @param fileStatuses
    * @param fileExtension
    * @param metaClientList
@@ -430,7 +475,7 @@ public class HoodieInputFormatUtils {
     for (Path path : snapshotPaths) {
       // Find meta client associated with the input path
       metaClientList.stream().filter(metaClient -> path.toString().contains(metaClient.getBasePath()))
-              .forEach(metaClient -> grouped.get(metaClient).add(path));
+          .forEach(metaClient -> grouped.get(metaClient).add(path));
     }
     return grouped;
   }
@@ -445,17 +490,19 @@ public class HoodieInputFormatUtils {
    * Checks the file status for a race condition which can set the file size to 0. 1. HiveInputFormat does
    * super.listStatus() and gets back a FileStatus[] 2. Then it creates the HoodieTableMetaClient for the paths listed.
    * 3. Generation of splits looks at FileStatus size to create splits, which skips this file
+   *
    * @param conf
    * @param dataFile
    * @return
    */
   private static HoodieBaseFile refreshFileStatus(Configuration conf, HoodieBaseFile dataFile) {
-    Path dataPath = dataFile.getFileStatus().getPath();
+    StoragePath dataPath = dataFile.getPathInfo().getPath();
     try {
       if (dataFile.getFileSize() == 0) {
-        FileSystem fs = dataPath.getFileSystem(conf);
+        HoodieStorage storage = HoodieStorageUtils.getStorage(dataPath, HadoopFSUtils.getStorageConf(conf));
         LOG.info("Refreshing file status " + dataFile.getPath());
-        return new HoodieBaseFile(fs.getFileStatus(dataPath), dataFile.getBootstrapBaseFile().orElse(null));
+        return new HoodieBaseFile(storage.getPathInfo(dataPath),
+            dataFile.getBootstrapBaseFile().orElse(null));
       }
       return dataFile;
     } catch (IOException e) {
@@ -471,29 +518,26 @@ public class HoodieInputFormatUtils {
    *
    * @param basePath     The table base path
    * @param metadataList The metadata list to read the data from
-   *
    * @return the affected file status array
    */
-  public static FileStatus[] listAffectedFilesForCommits(Configuration hadoopConf, Path basePath, List<HoodieCommitMetadata> metadataList) {
+  public static List<StoragePathInfo> listAffectedFilesForCommits(Configuration hadoopConf,
+                                                                  StoragePath basePath,
+                                                                  List<HoodieCommitMetadata> metadataList) {
     // TODO: Use HoodieMetaTable to extract affected file directly.
-    HashMap<String, FileStatus> fullPathToFileStatus = new HashMap<>();
+    HashMap<String, StoragePathInfo> fullPathToInfoMap = new HashMap<>();
+    HoodieStorage storage = new HoodieHadoopStorage(basePath, HadoopFSUtils.getStorageConf(hadoopConf));
     // Iterate through the given commits.
-    for (HoodieCommitMetadata metadata: metadataList) {
-      fullPathToFileStatus.putAll(metadata.getFullPathToFileStatus(hadoopConf, basePath.toString()));
+    for (HoodieCommitMetadata metadata : metadataList) {
+      fullPathToInfoMap.putAll(metadata.getFullPathToInfo(storage, basePath.toString()));
     }
-    return fullPathToFileStatus.values().toArray(new FileStatus[0]);
+    return new ArrayList<>(fullPathToInfoMap.values());
   }
 
-  /**
-   * Returns all the incremental write partition paths as a set with the given commits metadata.
-   *
-   * @param metadataList The commits metadata
-   * @return the partition path set
-   */
-  public static Set<String> getWritePartitionPaths(List<HoodieCommitMetadata> metadataList) {
-    return metadataList.stream()
-        .map(HoodieCommitMetadata::getWritePartitionPaths)
-        .flatMap(Collection::stream)
-        .collect(Collectors.toSet());
+  public static HoodieRealtimeFileSplit createRealtimeFileSplit(HoodieRealtimePath path, long start, long length, String[] hosts) {
+    try {
+      return new HoodieRealtimeFileSplit(new FileSplit(path, start, length, hosts), path);
+    } catch (IOException e) {
+      throw new HoodieIOException(String.format("Failed to create instance of %s", HoodieRealtimeFileSplit.class.getName()), e);
+    }
   }
 }
