@@ -33,6 +33,9 @@ import org.apache.hudi.common.util.CommitUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.hadoop.realtime.HoodieParquetRealtimeInputFormat;
 import org.apache.hudi.hadoop.testutils.InputFormatTestUtil;
+import org.apache.hudi.storage.HoodieStorage;
+import org.apache.hudi.storage.StorageConfiguration;
+import org.apache.hudi.storage.hadoop.HoodieHadoopStorage;
 
 import org.apache.avro.Schema;
 import org.apache.hadoop.conf.Configuration;
@@ -75,6 +78,7 @@ import static org.junit.jupiter.api.Assertions.assertTrue;
 public class TestHoodieCombineHiveInputFormat extends HoodieCommonTestHarness {
 
   private static HdfsTestService hdfsTestService;
+  private static HoodieStorage storage;
   private static FileSystem fs;
 
   @BeforeAll
@@ -82,6 +86,7 @@ public class TestHoodieCombineHiveInputFormat extends HoodieCommonTestHarness {
     // Append is not supported in LocalFileSystem. HDFS needs to be setup.
     hdfsTestService = new HdfsTestService();
     fs = hdfsTestService.start(true).getFileSystem();
+    storage = new HoodieHadoopStorage(fs);
   }
 
   @AfterAll
@@ -89,6 +94,7 @@ public class TestHoodieCombineHiveInputFormat extends HoodieCommonTestHarness {
     hdfsTestService.stop();
     if (fs != null) {
       fs.close();
+      storage.close();
     }
   }
 
@@ -107,7 +113,7 @@ public class TestHoodieCombineHiveInputFormat extends HoodieCommonTestHarness {
   @Test
   public void multiPartitionReadersRealtimeCombineHoodieInputFormat() throws Exception {
     // test for HUDI-1718
-    Configuration conf = new Configuration();
+    StorageConfiguration<Configuration> conf = HoodieTestUtils.getDefaultStorageConf();
     // initial commit
     Schema schema = HoodieAvroUtils.addMetadataFields(SchemaTestUtil.getEvolvedSchema());
     HoodieTestUtils.init(conf, tempDir.toAbsolutePath().toString(), HoodieTableType.MERGE_ON_READ);
@@ -139,8 +145,8 @@ public class TestHoodieCombineHiveInputFormat extends HoodieCommonTestHarness {
     mrwork.getMapWork().setPathToAliases(talias);
 
     Path mapWorkPath = new Path(tempDir.toAbsolutePath().toString());
-    Utilities.setMapRedWork(conf, mrwork, mapWorkPath);
-    JobConf jobConf = new JobConf(conf);
+    Utilities.setMapRedWork(conf.unwrap(), mrwork, mapWorkPath);
+    JobConf jobConf = new JobConf(conf.unwrap());
     // Add three partition path to InputPaths
     Path[] partitionDirArray = new Path[partitionDirs.size()];
     partitionDirs.stream().map(p -> new Path(p.getPath())).collect(Collectors.toList()).toArray(partitionDirArray);
@@ -190,7 +196,7 @@ public class TestHoodieCombineHiveInputFormat extends HoodieCommonTestHarness {
   @Test
   public void multiLevelPartitionReadersRealtimeCombineHoodieInputFormat() throws Exception {
     // test for HUDI-1718
-    Configuration conf = new Configuration();
+    StorageConfiguration<Configuration> conf = HoodieTestUtils.getDefaultStorageConf();
     // initial commit
     Schema schema = HoodieAvroUtils.addMetadataFields(SchemaTestUtil.getEvolvedSchema());
     HoodieTestUtils.init(conf, tempDir.toAbsolutePath().toString(), HoodieTableType.MERGE_ON_READ);
@@ -225,8 +231,8 @@ public class TestHoodieCombineHiveInputFormat extends HoodieCommonTestHarness {
     mrwork.getMapWork().setPathToAliases(talias);
 
     Path mapWorkPath = new Path(tempDir.toAbsolutePath().toString());
-    Utilities.setMapRedWork(conf, mrwork, mapWorkPath);
-    JobConf jobConf = new JobConf(conf);
+    Utilities.setMapRedWork(conf.unwrap(), mrwork, mapWorkPath);
+    JobConf jobConf = new JobConf(conf.unwrap());
     // Add the paths
     FileInputFormat.setInputPaths(jobConf, partitionDir.getPath());
     jobConf.set(HAS_MAP_WORK, "true");
@@ -262,7 +268,7 @@ public class TestHoodieCombineHiveInputFormat extends HoodieCommonTestHarness {
   @Test
   public void testMultiReaderRealtimeCombineHoodieInputFormat() throws Exception {
     // test for hudi-1722
-    Configuration conf = new Configuration();
+    StorageConfiguration<Configuration> conf = HoodieTestUtils.getDefaultStorageConf();
     // initial commit
     Schema schema = HoodieAvroUtils.addMetadataFields(SchemaTestUtil.getEvolvedSchema());
     HoodieTestUtils.init(conf, tempDir.toAbsolutePath().toString(), HoodieTableType.MERGE_ON_READ);
@@ -279,7 +285,8 @@ public class TestHoodieCombineHiveInputFormat extends HoodieCommonTestHarness {
     // insert 1000 update records to log file 2
     // now fileid0, fileid1 has no log files, fileid2 has log file
     HoodieLogFormat.Writer writer =
-        InputFormatTestUtil.writeDataBlockToLogFile(partitionDir, fs, schema, "fileid2", commitTime, newCommitTime,
+        InputFormatTestUtil.writeDataBlockToLogFile(partitionDir, storage, schema, "fileid2",
+            commitTime, newCommitTime,
             numRecords, numRecords, 0);
     writer.close();
 
@@ -298,8 +305,8 @@ public class TestHoodieCombineHiveInputFormat extends HoodieCommonTestHarness {
     mrwork.getMapWork().setPathToPartitionInfo(pt);
     mrwork.getMapWork().setPathToAliases(tableAlias);
     Path mapWorkPath = new Path(tempDir.toAbsolutePath().toString());
-    Utilities.setMapRedWork(conf, mrwork, mapWorkPath);
-    JobConf jobConf = new JobConf(conf);
+    Utilities.setMapRedWork(conf.unwrap(), mrwork, mapWorkPath);
+    JobConf jobConf = new JobConf(conf.unwrap());
     // Add the paths
     FileInputFormat.setInputPaths(jobConf, partitionDir.getPath());
     jobConf.set(HAS_MAP_WORK, "true");
@@ -332,7 +339,7 @@ public class TestHoodieCombineHiveInputFormat extends HoodieCommonTestHarness {
   @Disabled
   public void testHoodieRealtimeCombineHoodieInputFormat() throws Exception {
 
-    Configuration conf = new Configuration();
+    StorageConfiguration<Configuration> conf = HoodieTestUtils.getDefaultStorageConf();
     // initial commit
     Schema schema = HoodieAvroUtils.addMetadataFields(SchemaTestUtil.getEvolvedSchema());
     HoodieTestUtils.init(conf, tempDir.toAbsolutePath().toString(), HoodieTableType.MERGE_ON_READ);
@@ -347,17 +354,20 @@ public class TestHoodieCombineHiveInputFormat extends HoodieCommonTestHarness {
     // insert 1000 update records to log file 0
     String newCommitTime = "101";
     HoodieLogFormat.Writer writer =
-        InputFormatTestUtil.writeDataBlockToLogFile(partitionDir, fs, schema, "fileid0", commitTime, newCommitTime,
+        InputFormatTestUtil.writeDataBlockToLogFile(partitionDir, storage, schema, "fileid0",
+            commitTime, newCommitTime,
             numRecords, numRecords, 0);
     writer.close();
     // insert 1000 update records to log file 1
     writer =
-        InputFormatTestUtil.writeDataBlockToLogFile(partitionDir, fs, schema, "fileid1", commitTime, newCommitTime,
+        InputFormatTestUtil.writeDataBlockToLogFile(partitionDir, storage, schema, "fileid1",
+            commitTime, newCommitTime,
             numRecords, numRecords, 0);
     writer.close();
     // insert 1000 update records to log file 2
     writer =
-        InputFormatTestUtil.writeDataBlockToLogFile(partitionDir, fs, schema, "fileid2", commitTime, newCommitTime,
+        InputFormatTestUtil.writeDataBlockToLogFile(partitionDir, storage, schema, "fileid2",
+            commitTime, newCommitTime,
             numRecords, numRecords, 0);
     writer.close();
 
@@ -370,8 +380,8 @@ public class TestHoodieCombineHiveInputFormat extends HoodieCommonTestHarness {
     MapredWork mrwork = new MapredWork();
     mrwork.getMapWork().setPathToPartitionInfo(pt);
     Path mapWorkPath = new Path(tempDir.toAbsolutePath().toString());
-    Utilities.setMapRedWork(conf, mrwork, mapWorkPath);
-    JobConf jobConf = new JobConf(conf);
+    Utilities.setMapRedWork(conf.unwrap(), mrwork, mapWorkPath);
+    JobConf jobConf = new JobConf(conf.unwrap());
     // Add the paths
     FileInputFormat.setInputPaths(jobConf, partitionDir.getPath());
     jobConf.set(HAS_MAP_WORK, "true");
