@@ -17,11 +17,10 @@
 
 package org.apache.hudi
 
-import org.apache.hadoop.fs.{FileStatus, Path}
 import org.apache.hudi.BaseHoodieTableFileIndex.PartitionPath
 import org.apache.hudi.DataSourceReadOptions._
 import org.apache.hudi.HoodieConversionUtils.toJavaOption
-import org.apache.hudi.SparkHoodieTableFileIndex._
+import org.apache.hudi.SparkHoodieTableFileIndex.{deduceQueryType, extractEqualityPredicatesLiteralValues, generateFieldMap, haveProperPartitionValues, shouldListLazily, shouldUsePartitionPathPrefixAnalysis, shouldValidatePartitionColumns}
 import org.apache.hudi.client.common.HoodieSparkEngineContext
 import org.apache.hudi.common.config.TypedProperties
 import org.apache.hudi.common.model.HoodieRecord.HOODIE_META_COLUMNS_WITH_OPERATION
@@ -29,10 +28,14 @@ import org.apache.hudi.common.model.{FileSlice, HoodieTableQueryType}
 import org.apache.hudi.common.table.{HoodieTableMetaClient, TableSchemaResolver}
 import org.apache.hudi.common.util.ValidationUtils.checkState
 import org.apache.hudi.config.HoodieBootstrapConfig.DATA_QUERIES_ONLY
+import org.apache.hudi.hadoop.fs.HadoopFSUtils
 import org.apache.hudi.internal.schema.Types.RecordType
 import org.apache.hudi.internal.schema.utils.Conversions
 import org.apache.hudi.keygen.{StringPartitionPathFormatter, TimestampBasedAvroKeyGenerator, TimestampBasedKeyGenerator}
+import org.apache.hudi.storage.{StoragePath, StoragePathInfo}
 import org.apache.hudi.util.JFunction
+
+import org.apache.hadoop.fs.{FileStatus, Path}
 import org.apache.spark.api.java.JavaSparkContext
 import org.apache.spark.internal.Logging
 import org.apache.spark.sql.SparkSession
@@ -41,10 +44,11 @@ import org.apache.spark.sql.catalyst.util.DateTimeUtils
 import org.apache.spark.sql.catalyst.{InternalRow, expressions}
 import org.apache.spark.sql.execution.datasources.{FileStatusCache, NoopCache}
 import org.apache.spark.sql.internal.SQLConf
-import org.apache.spark.sql.types._
+import org.apache.spark.sql.types.{ByteType, DateType, IntegerType, LongType, ShortType, StringType, StructField, StructType}
 
 import java.util.Collections
 import javax.annotation.concurrent.NotThreadSafe
+
 import scala.collection.JavaConverters._
 import scala.language.implicitConversions
 import scala.util.{Success, Try}
@@ -52,19 +56,19 @@ import scala.util.{Success, Try}
 /**
  * Implementation of the [[BaseHoodieTableFileIndex]] for Spark
  *
- * @param spark spark session
- * @param metaClient Hudi table's meta-client
- * @param schemaSpec optional table's schema
- * @param configProperties unifying configuration (in the form of generic properties)
+ * @param spark                 spark session
+ * @param metaClient            Hudi table's meta-client
+ * @param schemaSpec            optional table's schema
+ * @param configProperties      unifying configuration (in the form of generic properties)
  * @param specifiedQueryInstant instant as of which table is being queried
- * @param fileStatusCache transient cache of fetched [[FileStatus]]es
+ * @param fileStatusCache       transient cache of fetched [[FileStatus]]es
  */
 @NotThreadSafe
 class SparkHoodieTableFileIndex(spark: SparkSession,
                                 metaClient: HoodieTableMetaClient,
                                 schemaSpec: Option[StructType],
                                 configProperties: TypedProperties,
-                                queryPaths: Seq[Path],
+                                queryPaths: Seq[StoragePath],
                                 specifiedQueryInstant: Option[String] = None,
                                 @transient fileStatusCache: FileStatusCache = NoopCache,
                                 beginInstantTime: Option[String] = None,
@@ -117,10 +121,10 @@ class SparkHoodieTableFileIndex(spark: SparkSession,
       val keyGeneratorClassName = tableConfig.getKeyGeneratorClassName
       if (classOf[TimestampBasedKeyGenerator].getName.equalsIgnoreCase(keyGeneratorClassName)
         || classOf[TimestampBasedAvroKeyGenerator].getName.equalsIgnoreCase(keyGeneratorClassName)) {
-        val partitionFields = partitionColumns.get().map(column => StructField(column, StringType))
+        val partitionFields: Array[StructField] = partitionColumns.get().map(column => StructField(column, StringType))
         StructType(partitionFields)
       } else {
-        val partitionFields = partitionColumns.get().filter(column => nameFieldMap.contains(column))
+        val partitionFields: Array[StructField] = partitionColumns.get().filter(column => nameFieldMap.contains(column))
           .map(column => nameFieldMap.apply(column))
 
         if (partitionFields.length != partitionColumns.get().length) {
@@ -184,7 +188,7 @@ class SparkHoodieTableFileIndex(spark: SparkSession,
     // Prune the partition path by the partition filters
     val prunedPartitions = listMatchingPartitionPaths(partitionFilters)
     getInputFileSlices(prunedPartitions: _*).asScala.map {
-      case (partition, fileSlices) => (partition.path, fileSlices.asScala)
+      case (partition, fileSlices) => (partition.path, fileSlices.asScala.toSeq)
     }.toMap
   }
 
@@ -216,14 +220,14 @@ class SparkHoodieTableFileIndex(spark: SparkSession,
     }
 
     if (partitionPruningPredicates.isEmpty) {
-      val queryPartitionPaths = getAllQueryPartitionPaths.asScala
+      val queryPartitionPaths = getAllQueryPartitionPaths.asScala.toSeq
       logInfo(s"No partition predicates provided, listing full table (${queryPartitionPaths.size} partitions)")
       queryPartitionPaths
     } else {
       // NOTE: We fallback to already cached partition-paths only in cases when we can subsequently
       //       rely on partition-pruning to eliminate not matching provided predicates (that requires
       //       partition-values to be successfully recovered from the partition-paths)
-      val partitionPaths = if (areAllPartitionPathsCached && haveProperPartitionValues(getAllQueryPartitionPaths.asScala)) {
+      val partitionPaths = if (areAllPartitionPathsCached && haveProperPartitionValues(getAllQueryPartitionPaths.asScala.toSeq)) {
         logDebug("All partition paths have already been cached, using these directly")
         getAllQueryPartitionPaths.asScala
       } else if (!shouldUsePartitionPathPrefixAnalysis(configProperties)) {
@@ -237,7 +241,7 @@ class SparkHoodieTableFileIndex(spark: SparkSession,
       //       we might not be able to properly parse partition-values from the listed partition-paths.
       //       In that case, we simply could not apply partition pruning and will have to regress to scanning
       //       the whole table
-      if (haveProperPartitionValues(partitionPaths) && partitionSchema.nonEmpty) {
+      if (haveProperPartitionValues(partitionPaths.toSeq) && partitionSchema.nonEmpty) {
         val predicate = partitionPruningPredicates.reduce(expressions.And)
         val boundPredicate = InterpretedPredicate(predicate.transform {
           case a: AttributeReference =>
@@ -247,7 +251,7 @@ class SparkHoodieTableFileIndex(spark: SparkSession,
 
         val prunedPartitionPaths = partitionPaths.filter {
           partitionPath => boundPredicate.eval(InternalRow.fromSeq(partitionPath.values))
-        }
+        }.toSeq
 
         logInfo(s"Using provided predicates to prune number of target table's partitions scanned from" +
           s" ${partitionPaths.size} to ${prunedPartitionPaths.size}")
@@ -257,7 +261,7 @@ class SparkHoodieTableFileIndex(spark: SparkSession,
         logWarning(s"Unable to apply partition pruning, due to failure to parse partition values from the" +
           s" following path(s): ${partitionPaths.find(_.values.length == 0).map(e => e.getPath)}")
 
-        partitionPaths
+        partitionPaths.toSeq
       }
     }
   }
@@ -341,16 +345,16 @@ class SparkHoodieTableFileIndex(spark: SparkSession,
           partitionColumnPredicates.flatMap {
             expr => sparkAdapter.translateFilter(expr)
           })
-        listPartitionPaths(Collections.singletonList(""), partitionTypes, convertedFilters).asScala
+        listPartitionPaths(Collections.singletonList(""), partitionTypes, convertedFilters).asScala.toSeq
       case (true, None) =>
         logDebug("Unable to compose relative partition path prefix from the predicates; falling back to fetching all partitions")
-        getAllQueryPartitionPaths.asScala
+        getAllQueryPartitionPaths.asScala.toSeq
       case (false, _) =>
         // Based on the static partition-column name-value pairs, we'll try to compose static partition-path
         // prefix to try to reduce the scope of the required file-listing
         val relativePartitionPathPrefix = composeRelativePartitionPath(staticPartitionColumnNameValuePairs)
 
-        if (!metaClient.getFs.exists(new Path(getBasePath, relativePartitionPathPrefix))) {
+        if (!metaClient.getStorage.exists(new StoragePath(getBasePath, relativePartitionPathPrefix))) {
           Seq()
         } else if (staticPartitionColumnNameValuePairs.length == partitionColumnNames.length) {
           // In case composed partition path is complete, we can return it directly avoiding extra listing operation
@@ -362,10 +366,10 @@ class SparkHoodieTableFileIndex(spark: SparkSession,
               partitionColumnPredicates.flatMap {
                 expr => sparkAdapter.translateFilter(expr)
               })
-            listPartitionPaths(Seq(relativePartitionPathPrefix).toList.asJava, partitionTypes, convertedFilters).asScala
+            listPartitionPaths(Seq(relativePartitionPathPrefix).asJava, partitionTypes, convertedFilters).asScala.toSeq
           }.getOrElse {
             log.warn("Met incompatible issue when converting to hudi data type, rollback to list by prefix directly")
-            listPartitionPaths(Seq(relativePartitionPathPrefix).toList.asJava).asScala
+            listPartitionPaths(Seq(relativePartitionPathPrefix).asJava).asScala.toSeq
           }
         }
     }
@@ -488,8 +492,15 @@ object SparkHoodieTableFileIndex extends SparkAdapterSupport {
 
   private def adapt(cache: FileStatusCache): BaseHoodieTableFileIndex.FileStatusCache = {
     new BaseHoodieTableFileIndex.FileStatusCache {
-      override def get(path: Path): org.apache.hudi.common.util.Option[Array[FileStatus]] = toJavaOption(cache.getLeafFiles(path))
-      override def put(path: Path, leafFiles: Array[FileStatus]): Unit = cache.putLeafFiles(path, leafFiles)
+      override def get(path: StoragePath): org.apache.hudi.common.util.Option[java.util.List[StoragePathInfo]] =
+        toJavaOption(cache.getLeafFiles(new Path(path.toUri)).map(opt => opt.map(
+          e => HadoopFSUtils.convertToStoragePathInfo(e)).toList.asJava
+        ))
+
+      override def put(path: StoragePath, leafFiles: java.util.List[StoragePathInfo]): Unit =
+        cache.putLeafFiles(new Path(path.toUri), leafFiles.asScala.map(e => new FileStatus(
+          e.getLength, e.isDirectory, 0, e.getBlockSize, e.getModificationTime, new Path(e.getPath.toUri))).toArray)
+
       override def invalidate(): Unit = cache.invalidateAll()
     }
   }
