@@ -28,8 +28,11 @@ import org.apache.hudi.common.table.{HoodieTableConfig, HoodieTableMetaClient}
 import org.apache.hudi.common.util.StringUtils
 import org.apache.hudi.common.util.ValidationUtils.checkArgument
 import org.apache.hudi.keygen.constant.KeyGeneratorOptions
+import org.apache.hudi.hadoop.fs.HadoopFSUtils
+import org.apache.hudi.keygen.constant.KeyGeneratorType
 import org.apache.hudi.keygen.factory.HoodieSparkKeyGeneratorFactory
 import org.apache.hudi.{AvroConversionUtils, DataSourceOptionsHelper}
+
 import org.apache.spark.internal.Logging
 import org.apache.spark.sql.avro.SchemaConverters
 import org.apache.spark.sql.catalyst.TableIdentifier
@@ -83,7 +86,7 @@ class HoodieCatalogTable(val spark: SparkSession, var table: CatalogTable) exten
    */
   lazy val metaClient: HoodieTableMetaClient = HoodieTableMetaClient.builder()
     .setBasePath(tableLocation)
-    .setConf(hadoopConf)
+    .setConf(HadoopFSUtils.getStorageConfWithCopy(hadoopConf))
     .build()
 
   /**
@@ -159,11 +162,6 @@ class HoodieCatalogTable(val spark: SparkSession, var table: CatalogTable) exten
     StructType(tableSchema.filterNot(f => partitionFields.contains(f.name)))
   }
 
-  /**
-   * The schema of data fields not including hoodie meta fields
-   */
-  lazy val dataSchemaWithoutMetaFields: StructType = removeMetaFields(dataSchema)
-
   /**
    * The schema of partition fields
    */
@@ -173,9 +171,9 @@ class HoodieCatalogTable(val spark: SparkSession, var table: CatalogTable) exten
    * All the partition paths, excludes lazily deleted partitions.
    */
   def getPartitionPaths: Seq[String] = {
-    val droppedPartitions = TimelineUtils.getDroppedPartitions(metaClient.getActiveTimeline)
+    val droppedPartitions = TimelineUtils.getDroppedPartitions(metaClient, org.apache.hudi.common.util.Option.empty(), org.apache.hudi.common.util.Option.empty())
 
-    getAllPartitionPaths(spark, table)
+    getAllPartitionPaths(spark, table, metaClient.getStorage)
       .filter(!droppedPartitions.contains(_))
   }
 
@@ -211,7 +209,7 @@ class HoodieCatalogTable(val spark: SparkSession, var table: CatalogTable) exten
         .fromProperties(properties)
         .setDatabaseName(catalogDatabaseName)
         .setTableCreateSchema(SchemaConverters.toAvroType(dataSchema, recordName = recordName).toString())
-        .initTable(hadoopConf, tableLocation)
+        .initTable(HadoopFSUtils.getStorageConfWithCopy(hadoopConf), tableLocation)
     } else {
       val (recordName, namespace) = AvroConversionUtils.getAvroRecordNameAndNamespace(table.identifier.table)
       val schema = SchemaConverters.toAvroType(dataSchema, nullable = false, recordName, namespace)
@@ -227,7 +225,7 @@ class HoodieCatalogTable(val spark: SparkSession, var table: CatalogTable) exten
         .setTableName(table.identifier.table)
         .setTableCreateSchema(schema.toString())
         .setPartitionFields(partitionColumns)
-        .initTable(hadoopConf, tableLocation)
+        .initTable(HadoopFSUtils.getStorageConfWithCopy(hadoopConf), tableLocation)
     }
   }
 
