@@ -28,6 +28,8 @@ import org.apache.hudi.common.util.ClusteringUtils;
 import org.apache.hudi.common.util.CollectionUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.collection.Pair;
+import org.apache.hudi.hadoop.fs.HadoopFSUtils;
+import org.apache.hudi.storage.HoodieStorage;
 
 import org.apache.hadoop.fs.FileSystem;
 
@@ -49,16 +51,29 @@ public class HoodieDataSourceHelpers {
     return listCommitsSince(fs, basePath, commitTimestamp).size() > 0;
   }
 
+  public static boolean hasNewCommits(HoodieStorage storage, String basePath,
+                                      String commitTimestamp) {
+    return listCommitsSince(storage, basePath, commitTimestamp).size() > 0;
+  }
+
   /**
    * Get a list of instant times that have occurred, from the given instant timestamp.
    */
   @PublicAPIMethod(maturity = ApiMaturityLevel.STABLE)
-  public static List<String> listCommitsSince(FileSystem fs, String basePath, String instantTimestamp) {
+  public static List<String> listCommitsSince(FileSystem fs, String basePath,
+                                              String instantTimestamp) {
     HoodieTimeline timeline = allCompletedCommitsCompactions(fs, basePath);
     return timeline.findInstantsAfter(instantTimestamp, Integer.MAX_VALUE).getInstantsAsStream()
         .map(HoodieInstant::getTimestamp).collect(Collectors.toList());
   }
 
+  public static List<String> listCommitsSince(HoodieStorage storage, String basePath,
+                                              String instantTimestamp) {
+    HoodieTimeline timeline = allCompletedCommitsCompactions(storage, basePath);
+    return timeline.findInstantsAfter(instantTimestamp, Integer.MAX_VALUE).getInstantsAsStream()
+        .map(HoodieInstant::getTimestamp).collect(Collectors.toList());
+  }
+
   /**
    * Returns the last successful write operation's instant time.
    */
@@ -68,13 +83,37 @@ public class HoodieDataSourceHelpers {
     return timeline.lastInstant().get().getTimestamp();
   }
 
+  public static String latestCommit(HoodieStorage storage, String basePath) {
+    HoodieTimeline timeline = allCompletedCommitsCompactions(storage, basePath);
+    return timeline.lastInstant().get().getTimestamp();
+  }
+
   /**
    * Obtain all the commits, compactions that have occurred on the timeline, whose instant times could be fed into the
    * datasource options.
    */
   @PublicAPIMethod(maturity = ApiMaturityLevel.STABLE)
   public static HoodieTimeline allCompletedCommitsCompactions(FileSystem fs, String basePath) {
-    HoodieTableMetaClient metaClient = HoodieTableMetaClient.builder().setConf(fs.getConf()).setBasePath(basePath).setLoadActiveTimelineOnLoad(true).build();
+    HoodieTableMetaClient metaClient =
+        HoodieTableMetaClient.builder()
+            .setConf(HadoopFSUtils.getStorageConfWithCopy(fs.getConf()))
+            .setBasePath(basePath)
+            .setLoadActiveTimelineOnLoad(true).build();
+    if (metaClient.getTableType().equals(HoodieTableType.MERGE_ON_READ)) {
+      return metaClient.getActiveTimeline().getTimelineOfActions(
+          CollectionUtils.createSet(HoodieActiveTimeline.COMMIT_ACTION,
+              HoodieActiveTimeline.DELTA_COMMIT_ACTION,
+              HoodieActiveTimeline.REPLACE_COMMIT_ACTION)).filterCompletedInstants();
+    } else {
+      return metaClient.getCommitTimeline().filterCompletedInstants();
+    }
+  }
+
+  public static HoodieTimeline allCompletedCommitsCompactions(HoodieStorage storage,
+                                                              String basePath) {
+    HoodieTableMetaClient metaClient = HoodieTableMetaClient.builder()
+        .setConf(storage.getConf().newInstance())
+        .setBasePath(basePath).setLoadActiveTimelineOnLoad(true).build();
     if (metaClient.getTableType().equals(HoodieTableType.MERGE_ON_READ)) {
       return metaClient.getActiveTimeline().getTimelineOfActions(
           CollectionUtils.createSet(HoodieActiveTimeline.COMMIT_ACTION,
@@ -86,11 +125,14 @@ public class HoodieDataSourceHelpers {
   }
 
   @PublicAPIMethod(maturity = ApiMaturityLevel.STABLE)
-  public static Option<HoodieClusteringPlan> getClusteringPlan(FileSystem fs, String basePath, String instantTime) {
-    HoodieTableMetaClient metaClient = HoodieTableMetaClient.builder().setConf(fs.getConf())
+  public static Option<HoodieClusteringPlan> getClusteringPlan(FileSystem fs, String basePath,
+                                                               String instantTime) {
+    HoodieTableMetaClient metaClient = HoodieTableMetaClient.builder()
+        .setConf(HadoopFSUtils.getStorageConfWithCopy(fs.getConf()))
         .setBasePath(basePath).setLoadActiveTimelineOnLoad(true).build();
     HoodieInstant hoodieInstant = HoodieTimeline.getReplaceCommitRequestedInstant(instantTime);
-    Option<Pair<HoodieInstant, HoodieClusteringPlan>> clusteringPlan = ClusteringUtils.getClusteringPlan(metaClient, hoodieInstant);
+    Option<Pair<HoodieInstant, HoodieClusteringPlan>> clusteringPlan =
+        ClusteringUtils.getClusteringPlan(metaClient, hoodieInstant);
     if (clusteringPlan.isPresent()) {
       return Option.of(clusteringPlan.get().getValue());
     } else {
