@@ -18,9 +18,12 @@
 package org.apache.spark.sql.hudi.command.procedures
 
 import org.apache.hudi.HoodieCLIUtils
+import org.apache.hudi.common.table.HoodieTableMetaClient
 import org.apache.hudi.config.{HoodieIndexConfig, HoodieWriteConfig}
 import org.apache.hudi.exception.HoodieException
+import org.apache.hudi.hadoop.fs.HadoopFSUtils
 import org.apache.hudi.index.HoodieIndex.IndexType
+
 import org.apache.spark.api.java.JavaSparkContext
 import org.apache.spark.sql.SparkSession
 import org.apache.spark.sql.catalyst.InternalRow
@@ -39,6 +42,12 @@ abstract class BaseProcedure extends Procedure {
       .build
   }
 
+  protected def createMetaClient(jsc: JavaSparkContext, basePath: String): HoodieTableMetaClient = {
+    HoodieTableMetaClient.builder
+      .setConf(HadoopFSUtils.getStorageConfWithCopy(jsc.hadoopConfiguration()))
+      .setBasePath(basePath).build
+  }
+
   protected def getParamKey(parameter: ProcedureParameter, isNamedArgs: Boolean): String = {
     if (isNamedArgs) {
       parameter.name
