@@ -79,8 +79,8 @@ class TestHoodieRecordSerialization extends SparkClientFunctionalTestHarness {
     val hoodieInternalRow = new HoodieInternalRow(new Array[UTF8String](5), unsafeRow, false)
 
     Seq(
-      (unsafeRow, rowSchema, 87),
-      (hoodieInternalRow, addMetaFields(rowSchema), 127)
+      (unsafeRow, rowSchema, 89),
+      (hoodieInternalRow, addMetaFields(rowSchema), 129)
     ) foreach { case (row, schema, expectedSize) => routine(row, schema, expectedSize) }
   }
 
@@ -105,13 +105,15 @@ class TestHoodieRecordSerialization extends SparkClientFunctionalTestHarness {
     val key = new HoodieKey("rec-key", "part-path")
 
     val legacyRecord = toLegacyAvroRecord(avroRecord, key)
+    legacyRecord.setIgnoreIndexUpdate(true)
     val avroIndexedRecord = new HoodieAvroIndexedRecord(key, avroRecord)
+    avroIndexedRecord.setIgnoreIndexUpdate(true)
 
-    val expectedLagacyRecordSize = if (HoodieSparkUtils.gteqSpark3_4) 534 else 528
+    val expectedLagacyRecordSize = if (HoodieSparkUtils.gteqSpark3_4) 536 else 530
 
     Seq(
       (legacyRecord, expectedLagacyRecordSize),
-      (avroIndexedRecord, 389)
+      (avroIndexedRecord, 391)
     ) foreach { case (record, expectedSize) => routine(record, expectedSize) }
   }
 
@@ -130,7 +132,7 @@ class TestHoodieRecordSerialization extends SparkClientFunctionalTestHarness {
     }
 
     val key = new HoodieKey("rec-key", "part-path")
-    val expectedEmptyRecordSize = if (HoodieSparkUtils.gteqSpark3_4) 30 else 27
+    val expectedEmptyRecordSize = if (HoodieSparkUtils.gteqSpark3_4) 32 else 29
     Seq(
       (new HoodieEmptyRecord[GenericRecord](key, HoodieOperation.INSERT, 1, HoodieRecordType.AVRO),
         expectedEmptyRecordSize),
