@@ -22,22 +22,24 @@ package org.apache.hudi.functional
 import org.apache.hudi.client.validator.{SqlQueryEqualityPreCommitValidator, SqlQueryInequalityPreCommitValidator}
 import org.apache.hudi.common.config.HoodieMetadataConfig
 import org.apache.hudi.common.config.TimestampKeyGeneratorConfig.{TIMESTAMP_INPUT_DATE_FORMAT, TIMESTAMP_OUTPUT_DATE_FORMAT, TIMESTAMP_TYPE_FIELD}
-import org.apache.hudi.common.fs.FSUtils
 import org.apache.hudi.common.model.WriteOperationType
-import org.apache.hudi.common.table.HoodieTableMetaClient
 import org.apache.hudi.common.table.timeline.{HoodieInstant, HoodieTimeline}
 import org.apache.hudi.common.testutils.HoodieTestDataGenerator
 import org.apache.hudi.common.testutils.RawTripTestPayload.recordsToStrings
 import org.apache.hudi.config.{HoodiePreCommitValidatorConfig, HoodieWriteConfig}
 import org.apache.hudi.exception.{HoodieUpsertException, HoodieValidationException}
 import org.apache.hudi.keygen.{NonpartitionedKeyGenerator, TimestampBasedKeyGenerator}
+import org.apache.hudi.testutils.HoodieClientTestUtils.createMetaClient
 import org.apache.hudi.testutils.SparkClientFunctionalTestHarness
 import org.apache.hudi.testutils.SparkClientFunctionalTestHarness.getSparkSqlConf
 import org.apache.hudi.{DataSourceReadOptions, DataSourceWriteOptions, HoodieDataSourceHelpers}
+import org.apache.hudi.common.fs.FSUtils
+import org.apache.hudi.hadoop.fs.HadoopFSUtils
+
 import org.apache.spark.SparkConf
-import org.apache.spark.sql._
 import org.apache.spark.sql.functions.{col, lit}
 import org.apache.spark.sql.types.StringType
+import org.apache.spark.sql.{DataFrame, SaveMode}
 import org.junit.jupiter.api.Assertions.{assertEquals, assertFalse, assertThrows, assertTrue}
 import org.junit.jupiter.api.Tag
 import org.junit.jupiter.api.function.Executable
@@ -45,7 +47,7 @@ import org.junit.jupiter.params.ParameterizedTest
 import org.junit.jupiter.params.provider.Arguments.arguments
 import org.junit.jupiter.params.provider.{Arguments, CsvSource, MethodSource, ValueSource}
 
-import scala.collection.JavaConversions._
+import scala.collection.JavaConverters._
 
 
 @Tag("functional")
@@ -92,9 +94,9 @@ class TestCOWDataSourceStorage extends SparkClientFunctionalTestHarness {
       options += TIMESTAMP_OUTPUT_DATE_FORMAT.key -> "yyyyMMdd"
     }
     val dataGen = new HoodieTestDataGenerator(0xDEED)
-    val fs = FSUtils.getFs(basePath, spark.sparkContext.hadoopConfiguration)
+    val fs = HadoopFSUtils.getFs(basePath, spark.sparkContext.hadoopConfiguration)
     // Insert Operation
-    val records0 = recordsToStrings(dataGen.generateInserts("000", 100)).toList
+    val records0 = recordsToStrings(dataGen.generateInserts("000", 100)).asScala.toList
     val inputDF0 = spark.read.json(spark.sparkContext.parallelize(records0, 2))
     inputDF0.write.format("org.apache.hudi")
       .options(options)
@@ -110,7 +112,7 @@ class TestCOWDataSourceStorage extends SparkClientFunctionalTestHarness {
       .load(basePath)
     assertEquals(100, snapshotDF1.count())
 
-    val records1 = recordsToStrings(dataGen.generateUpdates("001", 100)).toList
+    val records1 = recordsToStrings(dataGen.generateUpdates("001", 100)).asScala.toList
     val inputDF1 = spark.read.json(spark.sparkContext.parallelize(records1, 2))
     val verificationRowKey = inputDF1.limit(1).select("_row_key").first.getString(0)
     var updateDf: DataFrame = null
@@ -140,7 +142,7 @@ class TestCOWDataSourceStorage extends SparkClientFunctionalTestHarness {
     assertEquals(updatedVerificationVal, snapshotDF2.filter(col("_row_key") === verificationRowKey).select(verificationCol).first.getString(0))
 
     // Upsert Operation without Hudi metadata columns
-    val records2 = recordsToStrings(dataGen.generateUpdates("002", 100)).toList
+    val records2 = recordsToStrings(dataGen.generateUpdates("002", 100)).asScala.toList
     var inputDF2 = spark.read.json(spark.sparkContext.parallelize(records2, 2))
 
     if (isTimestampBasedKeyGen) {
@@ -201,7 +203,7 @@ class TestCOWDataSourceStorage extends SparkClientFunctionalTestHarness {
     assertEquals(0, emptyIncDF.count())
 
     // Upsert an empty dataFrame
-    val emptyRecords = recordsToStrings(dataGen.generateUpdates("003", 0)).toList
+    val emptyRecords = recordsToStrings(dataGen.generateUpdates("003", 0)).asScala.toList
     val emptyDF = spark.read.json(spark.sparkContext.parallelize(emptyRecords, 1))
     emptyDF.write.format("org.apache.hudi")
       .options(options)
@@ -246,7 +248,7 @@ class TestCOWDataSourceStorage extends SparkClientFunctionalTestHarness {
     val dataGenPartition2 = new HoodieTestDataGenerator(Array[String](HoodieTestDataGenerator.DEFAULT_SECOND_PARTITION_PATH))
 
     // do one bulk insert to all partitions
-    val records = recordsToStrings(dataGen.generateInserts("%05d".format(1), 100)).toList
+    val records = recordsToStrings(dataGen.generateInserts("%05d".format(1), 100)).asScala.toList
     val inputDF = spark.read.json(spark.sparkContext.parallelize(records, 2))
     val partition1RecordCount = inputDF.filter(row => row.getAs("partition_path")
       .equals(HoodieTestDataGenerator.DEFAULT_FIRST_PARTITION_PATH)).count()
@@ -278,8 +280,7 @@ class TestCOWDataSourceStorage extends SparkClientFunctionalTestHarness {
     }
 
     assertRecordCount(basePath, expectedRecCount + 500)
-    val metaClient = HoodieTableMetaClient.builder().setConf(spark.sparkContext.hadoopConfiguration).setBasePath(basePath)
-      .setLoadActiveTimelineOnLoad(true).build()
+    val metaClient = createMetaClient(spark, basePath)
     val commits = metaClient.getActiveTimeline.filterCompletedInstants().getInstants.toArray
       .map(instant => instant.asInstanceOf[HoodieInstant].getAction)
     // assert replace commit is archived and not part of active timeline.
@@ -316,8 +317,8 @@ class TestCOWDataSourceStorage extends SparkClientFunctionalTestHarness {
     }
 
     val dataGen = new HoodieTestDataGenerator(0xDEED)
-    val fs = FSUtils.getFs(basePath, spark.sparkContext.hadoopConfiguration)
-    val records = recordsToStrings(dataGen.generateInserts("001", 100)).toList
+    val fs = HadoopFSUtils.getFs(basePath, spark.sparkContext.hadoopConfiguration)
+    val records = recordsToStrings(dataGen.generateInserts("001", 100)).asScala.toList
 
     // First commit, new partition, no existing table schema
     // Validation should succeed
@@ -384,7 +385,7 @@ class TestCOWDataSourceStorage extends SparkClientFunctionalTestHarness {
   }
 
   def writeRecords(commitTime: Int, dataGen: HoodieTestDataGenerator, writeOperation: String, basePath: String): Unit = {
-    val records = recordsToStrings(dataGen.generateInserts("%05d".format(commitTime), 100)).toList
+    val records = recordsToStrings(dataGen.generateInserts("%05d".format(commitTime), 100)).asScala.toList
     val inputDF = spark.read.json(spark.sparkContext.parallelize(records, 2))
     inputDF.write.format("hudi")
       .options(commonOpts)
