@@ -19,16 +19,18 @@ package org.apache.hudi.functional
 
 import org.apache.hudi.common.model.HoodieFileFormat
 import org.apache.hudi.common.table.HoodieTableMetaClient
+import org.apache.hudi.common.testutils.HoodieTestUtils
 import org.apache.hudi.common.testutils.RawTripTestPayload.recordsToStrings
 import org.apache.hudi.config.HoodieWriteConfig
 import org.apache.hudi.testutils.HoodieSparkClientTestBase
 import org.apache.hudi.{DataSourceWriteOptions, HoodieDataSourceHelpers}
+
 import org.apache.spark.sql._
 import org.junit.jupiter.api.Assertions.assertEquals
 import org.junit.jupiter.api.{AfterEach, BeforeEach, Test}
 import org.slf4j.LoggerFactory
 
-import scala.collection.JavaConversions._
+import scala.collection.JavaConverters._
 
 /**
  * Tests on HoodieActionTimeLine using the real hudi table.
@@ -55,7 +57,7 @@ class TestHoodieActiveTimeline extends HoodieSparkClientTestBase {
     initSparkContexts()
     spark = sqlContext.sparkSession
     initTestDataGenerator()
-    initFileSystem()
+    initHoodieStorage()
   }
 
   @AfterEach
@@ -68,7 +70,7 @@ class TestHoodieActiveTimeline extends HoodieSparkClientTestBase {
   @Test
   def testGetLastCommitMetadataWithValidDataForCOW(): Unit = {
     // First Operation:
-    val records1 = recordsToStrings(dataGen.generateInserts("001", 100)).toList
+    val records1 = recordsToStrings(dataGen.generateInserts("001", 100)).asScala.toList
     val inputDF1 = spark.read.json(spark.sparkContext.parallelize(records1, 2))
     inputDF1.write.format("org.apache.hudi")
       .options(commonOpts)
@@ -76,14 +78,14 @@ class TestHoodieActiveTimeline extends HoodieSparkClientTestBase {
       .option(DataSourceWriteOptions.TABLE_TYPE.key, DataSourceWriteOptions.COW_TABLE_TYPE_OPT_VAL)
       .mode(SaveMode.Overwrite)
       .save(basePath)
-    val commit1Time = HoodieDataSourceHelpers.latestCommit(fs, basePath)
+    val commit1Time = HoodieDataSourceHelpers.latestCommit(storage, basePath)
     val partitionsForCommit1 = spark.read.format("org.apache.hudi").load(basePath)
       .select("_hoodie_partition_path")
       .distinct().collect()
       .map(_.get(0).toString).sorted
     assert(Array("2015/03/16", "2015/03/17", "2016/03/15").sameElements(partitionsForCommit1))
 
-    val metaClient: HoodieTableMetaClient = HoodieTableMetaClient.builder().setBasePath(basePath).setConf(hadoopConf).build()
+    val metaClient: HoodieTableMetaClient = createMetaClient(basePath)
     var activeTimeline = metaClient.getActiveTimeline
 
     // check that get the latest parquet file
@@ -103,7 +105,7 @@ class TestHoodieActiveTimeline extends HoodieSparkClientTestBase {
       .option(DataSourceWriteOptions.PARTITIONS_TO_DELETE.key, "2015/03/16")
       .mode(SaveMode.Append)
       .save(basePath)
-    val commit2Time = HoodieDataSourceHelpers.latestCommit(fs, basePath)
+    val commit2Time = HoodieDataSourceHelpers.latestCommit(storage, basePath)
     val countPartitionDropped = spark.read.format("org.apache.hudi").load(basePath)
       .where("_hoodie_partition_path = '2015/03/16'").count()
     assertEquals(countPartitionDropped, 0)
@@ -120,13 +122,13 @@ class TestHoodieActiveTimeline extends HoodieSparkClientTestBase {
 
     // Third Operation:
     // Upsert with 50 duplicate records. Produced the second log file for each parquet.
-    val records3 = recordsToStrings(dataGen.generateUniqueUpdates("003", 50)).toList
+    val records3 = recordsToStrings(dataGen.generateUniqueUpdates("003", 50)).asScala.toList
     val inputDF3: Dataset[Row] = spark.read.json(spark.sparkContext.parallelize(records3, 2))
     inputDF3.write.format("org.apache.hudi")
       .options(commonOpts)
       .mode(SaveMode.Append)
       .save(basePath)
-    val commit3Time = HoodieDataSourceHelpers.latestCommit(fs, basePath)
+    val commit3Time = HoodieDataSourceHelpers.latestCommit(storage, basePath)
 
     // check that get the latest parquet file generated by compaction
     activeTimeline = activeTimeline.reload()
@@ -142,7 +144,7 @@ class TestHoodieActiveTimeline extends HoodieSparkClientTestBase {
   @Test
   def testGetLastCommitMetadataWithValidDataForMOR(): Unit = {
     // First Operation:
-    val records1 = recordsToStrings(dataGen.generateInserts("001", 100)).toList
+    val records1 = recordsToStrings(dataGen.generateInserts("001", 100)).asScala.toList
     val inputDF1 = spark.read.json(spark.sparkContext.parallelize(records1, 2))
     inputDF1.write.format("org.apache.hudi")
       .options(commonOpts)
@@ -151,9 +153,9 @@ class TestHoodieActiveTimeline extends HoodieSparkClientTestBase {
       .option(DataSourceWriteOptions.TABLE_TYPE.key, DataSourceWriteOptions.MOR_TABLE_TYPE_OPT_VAL)
       .mode(SaveMode.Overwrite)
       .save(basePath)
-    val commit1Time = HoodieDataSourceHelpers.latestCommit(fs, basePath)
+    val commit1Time = HoodieDataSourceHelpers.latestCommit(storage, basePath)
 
-    val metaClient: HoodieTableMetaClient = HoodieTableMetaClient.builder().setBasePath(basePath).setConf(hadoopConf).build()
+    val metaClient: HoodieTableMetaClient = createMetaClient(basePath)
     var activeTimeline = metaClient.getActiveTimeline
 
     // check that get the latest parquet file
@@ -167,13 +169,13 @@ class TestHoodieActiveTimeline extends HoodieSparkClientTestBase {
 
     // Second Operation:
     // Upsert with duplicate records. Produced a log file for each parquet.
-    val records2 = recordsToStrings(dataGen.generateUniqueUpdates("002", 100)).toList
+    val records2 = recordsToStrings(dataGen.generateUniqueUpdates("002", 100)).asScala.toList
     val inputDF2: Dataset[Row] = spark.read.json(spark.sparkContext.parallelize(records2, 2))
     inputDF2.write.format("org.apache.hudi")
       .options(commonOpts)
       .mode(SaveMode.Append)
       .save(basePath)
-    val commit2Time = HoodieDataSourceHelpers.latestCommit(fs, basePath)
+    val commit2Time = HoodieDataSourceHelpers.latestCommit(storage, basePath)
 
     // check that get the latest .log file
     activeTimeline = activeTimeline.reload()
@@ -189,13 +191,13 @@ class TestHoodieActiveTimeline extends HoodieSparkClientTestBase {
     // Third Operation:
     // Upsert with 50 duplicate records. Produced the second log file for each parquet.
     // And trigger compaction.
-    val records3 = recordsToStrings(dataGen.generateUniqueUpdates("003", 50)).toList
+    val records3 = recordsToStrings(dataGen.generateUniqueUpdates("003", 50)).asScala.toList
     val inputDF3: Dataset[Row] = spark.read.json(spark.sparkContext.parallelize(records3, 2))
     inputDF3.write.format("org.apache.hudi")
       .options(commonOpts).option("hoodie.compact.inline", "true")
       .option("hoodie.compact.inline.max.delta.commits", "1")
       .mode(SaveMode.Append).save(basePath)
-    val commit3Time = HoodieDataSourceHelpers.latestCommit(fs, basePath)
+    val commit3Time = HoodieDataSourceHelpers.latestCommit(storage, basePath)
 
     // check that get the latest parquet file generated by compaction
     activeTimeline = activeTimeline.reload()
@@ -209,13 +211,13 @@ class TestHoodieActiveTimeline extends HoodieSparkClientTestBase {
 
     // Fourth Operation:
     // Upsert with 50 duplicate records.
-    val records4 = recordsToStrings(dataGen.generateUniqueUpdates("004", 50)).toList
+    val records4 = recordsToStrings(dataGen.generateUniqueUpdates("004", 50)).asScala.toList
     val inputDF4: Dataset[Row] = spark.read.json(spark.sparkContext.parallelize(records4, 2))
     inputDF4.write.format("org.apache.hudi")
       .options(commonOpts)
       .mode(SaveMode.Append)
       .save(basePath)
-    val commit4Time = HoodieDataSourceHelpers.latestCommit(fs, basePath)
+    val commit4Time = HoodieDataSourceHelpers.latestCommit(storage, basePath)
 
     activeTimeline = activeTimeline.reload()
     val ret4 = activeTimeline.getLastCommitMetadataWithValidData()
