@@ -20,12 +20,13 @@ package org.apache.hudi.functional
 
 import org.apache.hudi.HoodieFileIndex.DataSkippingFailureMode
 import org.apache.hudi.common.config.HoodieMetadataConfig
-import org.apache.hudi.common.table.HoodieTableMetaClient
 import org.apache.hudi.common.table.timeline.{HoodieInstant, HoodieTimeline}
+import org.apache.hudi.common.testutils.HoodieTestUtils
 import org.apache.hudi.common.testutils.RawTripTestPayload.recordsToStrings
 import org.apache.hudi.config.{HoodieClusteringConfig, HoodieWriteConfig}
 import org.apache.hudi.testutils.HoodieSparkClientTestBase
 import org.apache.hudi.{DataSourceReadOptions, DataSourceWriteOptions}
+
 import org.apache.spark.sql._
 import org.apache.spark.sql.types._
 import org.junit.jupiter.api.Assertions.assertEquals
@@ -34,7 +35,7 @@ import org.junit.jupiter.params.ParameterizedTest
 import org.junit.jupiter.params.provider.Arguments.arguments
 import org.junit.jupiter.params.provider.{Arguments, MethodSource}
 
-import scala.collection.JavaConversions._
+import scala.collection.JavaConverters._
 
 @Tag("functional")
 class TestLayoutOptimization extends HoodieSparkClientTestBase {
@@ -72,7 +73,7 @@ class TestLayoutOptimization extends HoodieSparkClientTestBase {
     initSparkContexts()
     spark = sqlContext.sparkSession
     initTestDataGenerator()
-    initFileSystem()
+    initHoodieStorage()
   }
 
   @AfterEach
@@ -94,7 +95,7 @@ class TestLayoutOptimization extends HoodieSparkClientTestBase {
 
     val targetRecordsCount = 10000
     // Bulk Insert Operation
-    val records = recordsToStrings(dataGen.generateInserts("001", targetRecordsCount)).toList
+    val records = recordsToStrings(dataGen.generateInserts("001", targetRecordsCount)).asScala.toList
     val writeDf: Dataset[Row] = spark.read.json(spark.sparkContext.parallelize(records, 2))
 
     // If there are any failures in the Data Skipping flow, test should fail
@@ -119,11 +120,7 @@ class TestLayoutOptimization extends HoodieSparkClientTestBase {
       .mode(SaveMode.Overwrite)
       .save(basePath)
 
-    val hudiMetaClient = HoodieTableMetaClient.builder
-      .setConf(hadoopConf)
-      .setBasePath(basePath)
-      .setLoadActiveTimelineOnLoad(true)
-      .build
+    val hudiMetaClient = createMetaClient(basePath)
 
     val lastCommit = hudiMetaClient.getActiveTimeline.getAllCommitsTimeline.lastInstant().get()
 
