@@ -17,7 +17,6 @@
 
 package org.apache.hudi.functional
 
-import org.apache.hadoop.fs.{FileSystem, Path}
 import org.apache.hudi.DataSourceWriteOptions.STREAMING_CHECKPOINT_IDENTIFIER
 import org.apache.hudi.HoodieStreamingSink.SINK_CHECKPOINT_KEY
 import org.apache.hudi.client.transaction.lock.InProcessLockProvider
@@ -26,13 +25,14 @@ import org.apache.hudi.common.model.{FileSlice, HoodieTableType, WriteConcurrenc
 import org.apache.hudi.common.table.HoodieTableMetaClient
 import org.apache.hudi.common.table.timeline.HoodieTimeline
 import org.apache.hudi.common.testutils.RawTripTestPayload.recordsToStrings
-import org.apache.hudi.common.testutils.{HoodieTestDataGenerator, HoodieTestTable}
+import org.apache.hudi.common.testutils.{HoodieTestDataGenerator, HoodieTestTable, HoodieTestUtils}
 import org.apache.hudi.common.util.{CollectionUtils, CommitUtils}
 import org.apache.hudi.config.{HoodieClusteringConfig, HoodieCompactionConfig, HoodieLockConfig, HoodieWriteConfig}
 import org.apache.hudi.exception.TableNotFoundException
-import org.apache.hudi.testutils.HoodieClientTestBase
+import org.apache.hudi.storage.{HoodieStorage, StoragePath}
+import org.apache.hudi.testutils.HoodieSparkClientTestBase
 import org.apache.hudi.{DataSourceReadOptions, DataSourceWriteOptions, HoodieDataSourceHelpers}
-import org.apache.log4j.LogManager
+
 import org.apache.spark.sql._
 import org.apache.spark.sql.streaming.{OutputMode, StreamingQuery, Trigger}
 import org.apache.spark.sql.types.StructType
@@ -40,8 +40,9 @@ import org.junit.jupiter.api.Assertions.{assertEquals, assertTrue}
 import org.junit.jupiter.api.{BeforeEach, Test}
 import org.junit.jupiter.params.ParameterizedTest
 import org.junit.jupiter.params.provider.{EnumSource, ValueSource}
+import org.slf4j.LoggerFactory
 
-import scala.collection.JavaConversions._
+import scala.collection.JavaConverters._
 import scala.concurrent.ExecutionContext.Implicits.global
 import scala.concurrent.duration.Duration
 import scala.concurrent.{Await, Future}
@@ -49,8 +50,8 @@ import scala.concurrent.{Await, Future}
 /**
  * Basic tests on the spark datasource for structured streaming sink
  */
-class TestStructuredStreaming extends HoodieClientTestBase {
-  private val log = LogManager.getLogger(getClass)
+class TestStructuredStreaming extends HoodieSparkClientTestBase {
+  private val log = LoggerFactory.getLogger(getClass)
 
   var spark: SparkSession = _
 
@@ -90,10 +91,10 @@ class TestStructuredStreaming extends HoodieClientTestBase {
   }
 
   def initStreamingSourceAndDestPath(sourceDirName: String, destDirName: String): (String, String) = {
-    fs.delete(new Path(basePath), true)
+    storage.deleteDirectory(new StoragePath(basePath))
     val sourcePath = basePath + "/" + sourceDirName
     val destPath = basePath + "/" + destDirName
-    fs.mkdirs(new Path(sourcePath))
+    storage.createDirectory(new StoragePath(sourcePath))
     (sourcePath, destPath)
   }
 
@@ -104,7 +105,7 @@ class TestStructuredStreaming extends HoodieClientTestBase {
   def getClusteringOpts(tableType: HoodieTableType, isInlineClustering: String,
                         isAsyncClustering: String, clusteringNumCommit: String,
                         fileMaxRecordNum: Int): Map[String, String] = {
-    getOptsWithTableType(tableType) + (
+    getOptsWithTableType(tableType) ++ Map(
       HoodieClusteringConfig.INLINE_CLUSTERING.key -> isInlineClustering,
       HoodieClusteringConfig.INLINE_CLUSTERING_MAX_COMMITS.key -> clusteringNumCommit,
       DataSourceWriteOptions.ASYNC_CLUSTERING_ENABLE.key -> isAsyncClustering,
@@ -114,7 +115,7 @@ class TestStructuredStreaming extends HoodieClientTestBase {
   }
 
   def getCompactionOpts(tableType: HoodieTableType, isAsyncCompaction: Boolean): Map[String, String] = {
-    getOptsWithTableType(tableType) + (
+    getOptsWithTableType(tableType) ++ Map(
       DataSourceWriteOptions.ASYNC_COMPACT_ENABLE.key -> isAsyncCompaction.toString,
       HoodieCompactionConfig.INLINE_COMPACT_NUM_DELTA_COMMITS.key -> "1"
     )
@@ -123,11 +124,11 @@ class TestStructuredStreaming extends HoodieClientTestBase {
   def structuredStreamingTestRunner(tableType: HoodieTableType, addCompactionConfigs: Boolean, isAsyncCompaction: Boolean): Unit = {
     val (sourcePath, destPath) = initStreamingSourceAndDestPath("source", "dest")
     // First chunk of data
-    val records1 = recordsToStrings(dataGen.generateInserts("000", 100)).toList
+    val records1 = recordsToStrings(dataGen.generateInserts("000", 100)).asScala.toList
     val inputDF1 = spark.read.json(spark.sparkContext.parallelize(records1, 2))
 
     // Second chunk of data
-    val records2 = recordsToStrings(dataGen.generateUpdates("001", 100)).toList
+    val records2 = recordsToStrings(dataGen.generateUpdates("001", 100)).asScala.toList
     val inputDF2 = spark.read.json(spark.sparkContext.parallelize(records2, 2))
     val uniqueKeyCnt = inputDF2.select("_row_key").distinct().count()
 
@@ -142,9 +143,9 @@ class TestStructuredStreaming extends HoodieClientTestBase {
     val f2 = Future {
       inputDF1.coalesce(1).write.mode(SaveMode.Append).json(sourcePath)
       // wait for spark streaming to process one microbatch
-      val currNumCommits = waitTillAtleastNCommits(fs, destPath, 1, 120, 5)
-      assertTrue(HoodieDataSourceHelpers.hasNewCommits(fs, destPath, "000"))
-      val commitInstantTime1 = HoodieDataSourceHelpers.latestCommit(fs, destPath)
+      val currNumCommits = waitTillAtleastNCommits(storage, destPath, 1, 120, 5)
+      assertTrue(HoodieDataSourceHelpers.hasNewCommits(storage, destPath, "000"))
+      val commitInstantTime1 = HoodieDataSourceHelpers.latestCommit(storage, destPath)
       // Read RO View
       val hoodieROViewDF1 = spark.read.format("org.apache.hudi")
         .load(destPath + "/*/*/*/*")
@@ -153,16 +154,16 @@ class TestStructuredStreaming extends HoodieClientTestBase {
       inputDF2.coalesce(1).write.mode(SaveMode.Append).json(sourcePath)
       // When the compaction configs are added, one more commit of the compaction is expected
       val numExpectedCommits = if (addCompactionConfigs) currNumCommits + 2 else currNumCommits + 1
-      waitTillAtleastNCommits(fs, destPath, numExpectedCommits, 120, 5)
+      waitTillAtleastNCommits(storage, destPath, numExpectedCommits, 120, 5)
 
       val commitInstantTime2 = if (tableType == HoodieTableType.MERGE_ON_READ) {
         // For the records that are processed by the compaction in MOR table
         // the "_hoodie_commit_time" still reflects the latest delta commit
-        latestInstant(fs, destPath, HoodieTimeline.DELTA_COMMIT_ACTION)
+        latestInstant(storage, destPath, HoodieTimeline.DELTA_COMMIT_ACTION)
       } else {
-        HoodieDataSourceHelpers.latestCommit(fs, destPath)
+        HoodieDataSourceHelpers.latestCommit(storage, destPath)
       }
-      assertEquals(numExpectedCommits, HoodieDataSourceHelpers.listCommitsSince(fs, destPath, "000").size())
+      assertEquals(numExpectedCommits, HoodieDataSourceHelpers.listCommitsSince(storage, destPath, "000").size())
       // Read RO View
       val hoodieROViewDF2 = spark.read.format("org.apache.hudi")
         .load(destPath + "/*/*/*/*")
@@ -170,7 +171,7 @@ class TestStructuredStreaming extends HoodieClientTestBase {
 
       // Read Incremental View
       // we have 2 commits, try pulling the first commit (which is not the latest)
-      val firstCommit = HoodieDataSourceHelpers.listCommitsSince(fs, destPath, "000").get(0)
+      val firstCommit = HoodieDataSourceHelpers.listCommitsSince(storage, destPath, "000").get(0)
       val hoodieIncViewDF1 = spark.read.format("org.apache.hudi")
         .option(DataSourceReadOptions.QUERY_TYPE.key, DataSourceReadOptions.QUERY_TYPE_INCREMENTAL_OPT_VAL)
         .option(DataSourceReadOptions.BEGIN_INSTANTTIME.key, "000")
@@ -207,15 +208,17 @@ class TestStructuredStreaming extends HoodieClientTestBase {
   }
 
   @throws[InterruptedException]
-  private def waitTillAtleastNCommits(fs: FileSystem, tablePath: String,
+  private def waitTillAtleastNCommits(storage: HoodieStorage, tablePath: String,
                                       numCommits: Int, timeoutSecs: Int, sleepSecsAfterEachRun: Int) = {
     val beginTime = System.currentTimeMillis
     var currTime = beginTime
     val timeoutMsecs = timeoutSecs * 1000
     var numInstants = 0
     var success = false
-    while ({!success && (currTime - beginTime) < timeoutMsecs}) try {
-      val timeline = HoodieDataSourceHelpers.allCompletedCommitsCompactions(fs, tablePath)
+    while ( {
+      !success && (currTime - beginTime) < timeoutMsecs
+    }) try {
+      val timeline = HoodieDataSourceHelpers.allCompletedCommitsCompactions(storage, tablePath)
       log.info("Timeline :" + timeline.getInstants.toArray.mkString("Array(", ", ", ")"))
       if (timeline.countInstants >= numCommits) {
         numInstants = timeline.countInstants
@@ -266,7 +269,7 @@ class TestStructuredStreaming extends HoodieClientTestBase {
       HoodieLockConfig.LOCK_PROVIDER_CLASS_NAME.key -> classOf[InProcessLockProvider].getName
     )
 
-    val records1 = recordsToStrings(dataGen.generateInsertsForPartition("000", 100, HoodieTestDataGenerator.DEFAULT_FIRST_PARTITION_PATH)).toList
+    val records1 = recordsToStrings(dataGen.generateInsertsForPartition("000", 100, HoodieTestDataGenerator.DEFAULT_FIRST_PARTITION_PATH)).asScala.toList
     val inputDF1 = spark.read.json(spark.sparkContext.parallelize(records1, 2))
     val schema = inputDF1.schema
 
@@ -284,13 +287,12 @@ class TestStructuredStreaming extends HoodieClientTestBase {
       .start(destPath)
 
     query1.processAllAvailable()
-    var metaClient = HoodieTableMetaClient.builder
-      .setConf(fs.getConf).setBasePath(destPath).setLoadActiveTimelineOnLoad(true).build
+    var metaClient = HoodieTestUtils.createMetaClient(storage, destPath)
 
     assertLatestCheckpointInfoMatched(metaClient, "streaming_identifier1", "0")
 
     // Add another identifier checkpoint info to the commit.
-    val records2 = recordsToStrings(dataGen.generateInsertsForPartition("001", 100, HoodieTestDataGenerator.DEFAULT_FIRST_PARTITION_PATH)).toList
+    val records2 = recordsToStrings(dataGen.generateInsertsForPartition("001", 100, HoodieTestDataGenerator.DEFAULT_FIRST_PARTITION_PATH)).asScala.toList
     val inputDF2 = spark.read.json(spark.sparkContext.parallelize(records2, 2))
 
     inputDF2.coalesce(1).write.mode(SaveMode.Append).json(sourcePath)
@@ -329,8 +331,8 @@ class TestStructuredStreaming extends HoodieClientTestBase {
       .start(destPath)
 
     query3.processAllAvailable()
-    metaClient = HoodieTableMetaClient.builder
-      .setConf(fs.getConf).setBasePath(destPath).setLoadActiveTimelineOnLoad(true).build
+    query3.stop()
+    metaClient = HoodieTestUtils.createMetaClient(storage, destPath)
 
     assertLatestCheckpointInfoMatched(metaClient, "streaming_identifier1", "2")
     assertLatestCheckpointInfoMatched(metaClient, "streaming_identifier2", "0")
@@ -348,7 +350,7 @@ class TestStructuredStreaming extends HoodieClientTestBase {
 
   def testStructuredStreamingInternal(operation : String = "upsert"): Unit = {
     val (sourcePath, destPath) = initStreamingSourceAndDestPath("source", "dest")
-    val records1 = recordsToStrings(dataGen.generateInsertsForPartition("000", 100, HoodieTestDataGenerator.DEFAULT_FIRST_PARTITION_PATH)).toList
+    val records1 = recordsToStrings(dataGen.generateInsertsForPartition("000", 100, HoodieTestDataGenerator.DEFAULT_FIRST_PARTITION_PATH)).asScala.toList
     val inputDF1 = spark.read.json(spark.sparkContext.parallelize(records1, 2))
     val schema = inputDF1.schema
     inputDF1.coalesce(1).write.mode(SaveMode.Append).json(sourcePath)
@@ -365,8 +367,7 @@ class TestStructuredStreaming extends HoodieClientTestBase {
       .start(destPath)
 
     query1.processAllAvailable()
-    val metaClient = HoodieTableMetaClient.builder
-      .setConf(fs.getConf).setBasePath(destPath).setLoadActiveTimelineOnLoad(true).build
+    val metaClient = HoodieTestUtils.createMetaClient(storage, destPath)
 
     assertLatestCheckpointInfoMatched(metaClient, STREAMING_CHECKPOINT_IDENTIFIER.defaultValue(), "0")
     query1.stop()
@@ -385,11 +386,11 @@ class TestStructuredStreaming extends HoodieClientTestBase {
                                                  isInlineClustering: Boolean, isAsyncClustering: Boolean,
                                                  partitionOfRecords: String, checkClusteringResult: String => Unit): Unit = {
     // First insert of data
-    val records1 = recordsToStrings(dataGen.generateInsertsForPartition("000", 100, partitionOfRecords)).toList
+    val records1 = recordsToStrings(dataGen.generateInsertsForPartition("000", 100, partitionOfRecords)).asScala.toList
     val inputDF1 = spark.read.json(spark.sparkContext.parallelize(records1, 2))
 
     // Second insert of data
-    val records2 = recordsToStrings(dataGen.generateInsertsForPartition("001", 100, partitionOfRecords)).toList
+    val records2 = recordsToStrings(dataGen.generateInsertsForPartition("001", 100, partitionOfRecords)).asScala.toList
     val inputDF2 = spark.read.json(spark.sparkContext.parallelize(records2, 2))
 
     val hudiOptions = getClusteringOpts(
@@ -400,19 +401,18 @@ class TestStructuredStreaming extends HoodieClientTestBase {
     val f2 = Future {
       inputDF1.coalesce(1).write.mode(SaveMode.Append).json(sourcePath)
       // wait for spark streaming to process one microbatch
-      var currNumCommits = waitTillAtleastNCommits(fs, destPath, 1, 120, 5)
-      assertTrue(HoodieDataSourceHelpers.hasNewCommits(fs, destPath, "000"))
+      var currNumCommits = waitTillAtleastNCommits(storage, destPath, 1, 120, 5)
+      assertTrue(HoodieDataSourceHelpers.hasNewCommits(storage, destPath, "000"))
 
       inputDF2.coalesce(1).write.mode(SaveMode.Append).json(sourcePath)
       // wait for spark streaming to process second microbatch
-      currNumCommits = waitTillAtleastNCommits(fs, destPath, currNumCommits + 1, 120, 5)
+      currNumCommits = waitTillAtleastNCommits(storage, destPath, currNumCommits + 1, 120, 5)
 
       // Wait for the clustering to finish
-      this.metaClient = HoodieTableMetaClient.builder().setConf(fs.getConf).setBasePath(destPath)
-        .setLoadActiveTimelineOnLoad(true).build()
+      this.metaClient = HoodieTestUtils.createMetaClient(storage, destPath)
       checkClusteringResult(destPath)
 
-      assertEquals(3, HoodieDataSourceHelpers.listCommitsSince(fs, destPath, "000").size())
+      assertEquals(3, HoodieDataSourceHelpers.listCommitsSince(storage, destPath, "000").size())
       // Check have at least one file group
       assertTrue(getLatestFileGroupsFileId(partitionOfRecords).size > 0)
 
@@ -422,7 +422,7 @@ class TestStructuredStreaming extends HoodieClientTestBase {
       assertEquals(200, hoodieROViewDF2.count())
       val countsPerCommit = hoodieROViewDF2.groupBy("_hoodie_commit_time").count().collect()
       assertEquals(2, countsPerCommit.length)
-      val commitInstantTime2 = latestInstant(fs, destPath, HoodieTimeline.COMMIT_ACTION)
+      val commitInstantTime2 = latestInstant(storage, destPath, HoodieTimeline.COMMIT_ACTION)
       assertEquals(commitInstantTime2, countsPerCommit.maxBy(row => row.getAs[String](0)).get(0))
 
       streamingQuery.stop()
@@ -462,13 +462,47 @@ class TestStructuredStreaming extends HoodieClientTestBase {
     if (!success) throw new IllegalStateException("Timed-out waiting for completing replace instant appear in " + tablePath)
   }
 
-  private def latestInstant(fs: FileSystem, basePath: String, instantAction: String): String = {
-    val metaClient = HoodieTableMetaClient.builder
-      .setConf(fs.getConf).setBasePath(basePath).setLoadActiveTimelineOnLoad(true).build
+  private def latestInstant(storage: HoodieStorage, basePath: String, instantAction: String): String = {
+    val metaClient = HoodieTestUtils.createMetaClient(storage, basePath)
     metaClient.getActiveTimeline
       .getTimelineOfActions(CollectionUtils.createSet(instantAction))
       .filterCompletedInstants
       .lastInstant
       .get.getTimestamp
   }
+
+  private def streamingWrite(schema: StructType, sourcePath: String, destPath: String, hudiOptions: Map[String, String], checkpoint: String): Unit = {
+    val query = spark.readStream
+      .schema(schema)
+      .json(sourcePath)
+      .writeStream
+      .format("org.apache.hudi")
+      .options(hudiOptions)
+      .trigger(Trigger.Once())
+      .option("checkpointLocation", basePath + "/checkpoint" + checkpoint)
+      .outputMode(OutputMode.Append)
+      .start(destPath)
+    query.processAllAvailable()
+    query.stop()
+  }
+
+  @Test
+  def testStructuredStreamingWithDisabledCompaction(): Unit = {
+    val (sourcePath, destPath) = initStreamingSourceAndDestPath("source", "dest")
+    // First chunk of data
+    val records1 = recordsToStrings(dataGen.generateInserts("000", 10)).asScala.toList
+    val inputDF1 = spark.read.json(spark.sparkContext.parallelize(records1, 2))
+    inputDF1.coalesce(1).write.mode(SaveMode.Append).json(sourcePath)
+    val opts = commonOpts + (DataSourceWriteOptions.TABLE_TYPE.key -> HoodieTableType.MERGE_ON_READ.name()) + (DataSourceWriteOptions.STREAMING_DISABLE_COMPACTION.key -> "true")
+    streamingWrite(inputDF1.schema, sourcePath, destPath, opts, "000")
+    for (i <- 1 to 24) {
+      val id = String.format("%03d", new Integer(i))
+      val records = recordsToStrings(dataGen.generateUpdates(id, 10)).asScala.toList
+      val inputDF = spark.read.json(spark.sparkContext.parallelize(records, 2))
+      inputDF.coalesce(1).write.mode(SaveMode.Append).json(sourcePath)
+      streamingWrite(inputDF.schema, sourcePath, destPath, opts, id)
+    }
+    val metaClient = HoodieTestUtils.createMetaClient(storage, destPath);
+    assertTrue(metaClient.getActiveTimeline.getCommitAndReplaceTimeline.empty())
+  }
 }
