@@ -18,24 +18,24 @@
 
 package org.apache.hudi.functional.cdc
 
-import org.apache.avro.generic.GenericRecord
 import org.apache.hudi.DataSourceWriteOptions
 import org.apache.hudi.DataSourceWriteOptions.{MOR_TABLE_TYPE_OPT_VAL, PARTITIONPATH_FIELD_OPT_KEY, PRECOMBINE_FIELD_OPT_KEY, RECORDKEY_FIELD_OPT_KEY}
 import org.apache.hudi.QuickstartUtils.getQuickstartWriteConfigs
 import org.apache.hudi.common.table.cdc.HoodieCDCSupplementalLoggingMode.OP_KEY_ONLY
 import org.apache.hudi.common.table.cdc.HoodieCDCUtils.schemaBySupplementalLoggingMode
 import org.apache.hudi.common.table.cdc.{HoodieCDCOperation, HoodieCDCSupplementalLoggingMode}
-import org.apache.hudi.common.table.{HoodieTableConfig, HoodieTableMetaClient, TableSchemaResolver}
+import org.apache.hudi.common.table.{HoodieTableConfig, TableSchemaResolver}
 import org.apache.hudi.common.testutils.HoodieTestDataGenerator
 import org.apache.hudi.common.testutils.RawTripTestPayload.{deleteRecordsToStrings, recordsToStrings}
 import org.apache.hudi.config.HoodieWriteConfig
-import org.apache.spark.sql.{Row, SaveMode}
+import org.apache.avro.generic.GenericRecord
 import org.apache.spark.sql.types.{StringType, StructField, StructType}
+import org.apache.spark.sql.{Row, SaveMode}
 import org.junit.jupiter.api.Assertions.{assertEquals, assertFalse, assertTrue}
 import org.junit.jupiter.params.ParameterizedTest
 import org.junit.jupiter.params.provider.{CsvSource, EnumSource}
 
-import scala.collection.JavaConversions._
+import scala.collection.JavaConverters._
 
 class TestCDCDataFrameSuite extends HoodieCDCTestBase {
 
@@ -62,17 +62,14 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     var allVisibleCDCData = spark.emptyDataFrame
 
     // Insert Operation
-    val records1 = recordsToStrings(dataGen.generateInserts("000", 100)).toList
+    val records1 = recordsToStrings(dataGen.generateInserts("000", 100)).asScala.toList
     val inputDF1 = spark.read.json(spark.sparkContext.parallelize(records1, 2))
     inputDF1.write.format("org.apache.hudi")
       .options(options)
       .mode(SaveMode.Overwrite)
       .save(basePath)
 
-    metaClient = HoodieTableMetaClient.builder()
-      .setBasePath(basePath)
-      .setConf(spark.sessionState.newHadoopConf)
-      .build()
+    metaClient = createMetaClient(spark, basePath)
 
     val schemaResolver = new TableSchemaResolver(metaClient)
     val dataSchema = schemaResolver.getTableAvroSchema(false)
@@ -88,7 +85,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
 
     // Upsert Operation
     val hoodieRecords2 = dataGen.generateUniqueUpdates("001", 50)
-    val records2 = recordsToStrings(hoodieRecords2).toList
+    val records2 = recordsToStrings(hoodieRecords2).asScala.toList
     val inputDF2 = spark.read.json(spark.sparkContext.parallelize(records2, 2))
     inputDF2.write.format("org.apache.hudi")
       .options(options)
@@ -117,7 +114,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     totalInsertedCnt += insertedCnt2
 
     // Delete Operation With Clustering Operation
-    val records3 = deleteRecordsToStrings(dataGen.generateUniqueDeletes(20)).toList
+    val records3 = deleteRecordsToStrings(dataGen.generateUniqueDeletes(20)).asScala.toList
     val inputDF3 = spark.read.json(spark.sparkContext.parallelize(records3, 2))
     inputDF3.write.format("org.apache.hudi")
       .options(options)
@@ -146,7 +143,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     assertCDCOpCnt(cdcDataFrom2To3, insertedCnt2, updatedCnt2, 20)
 
     // Insert Overwrite Operation
-    val records4 = recordsToStrings(dataGen.generateInserts("003", 50)).toList
+    val records4 = recordsToStrings(dataGen.generateInserts("003", 50)).asScala.toList
     val inputDF4 = spark.read.json(spark.sparkContext.parallelize(records4, 2))
     inputDF4.write.format("org.apache.hudi")
       .options(options)
@@ -168,14 +165,14 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     allVisibleCDCData = cdcDataFrame((commitTime1.toLong - 1).toString)
     assertCDCOpCnt(allVisibleCDCData, totalInsertedCnt, totalUpdatedCnt, totalDeletedCnt)
 
-    val records5 = recordsToStrings(dataGen.generateInserts("005", 7)).toList
+    val records5 = recordsToStrings(dataGen.generateInserts("005", 7)).asScala.toList
     val inputDF5 = spark.read.json(spark.sparkContext.parallelize(records5, 2))
     inputDF5.write.format("org.apache.hudi")
       .options(options)
       .mode(SaveMode.Append)
       .save(basePath)
 
-    val records6 = recordsToStrings(dataGen.generateInserts("006", 3)).toList
+    val records6 = recordsToStrings(dataGen.generateInserts("006", 3)).asScala.toList
     val inputDF6 = spark.read.json(spark.sparkContext.parallelize(records6, 2))
     inputDF6.write.format("org.apache.hudi")
       .options(options)
@@ -183,7 +180,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
       .save(basePath)
 
     // Upsert Operation With Clean Operation
-    val records7 = recordsToStrings(dataGen.generateUniqueUpdates("007", 30)).toList
+    val records7 = recordsToStrings(dataGen.generateUniqueUpdates("007", 30)).asScala.toList
     val inputDF7 = spark.read.json(spark.sparkContext.parallelize(records7, 2))
     inputDF7.write.format("org.apache.hudi")
       .options(options)
@@ -210,7 +207,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     assertCDCOpCnt(allVisibleCDCData, totalInsertedCnt, totalUpdatedCnt, totalDeletedCnt)
 
     // Bulk_Insert Operation With Clean Operation
-    val records8 = recordsToStrings(dataGen.generateInserts("008", 20)).toList
+    val records8 = recordsToStrings(dataGen.generateInserts("008", 20)).asScala.toList
     val inputDF8 = spark.read.json(spark.sparkContext.parallelize(records8, 2))
     inputDF8.write.format("org.apache.hudi")
       .options(options)
@@ -255,17 +252,14 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     var allVisibleCDCData = spark.emptyDataFrame
 
     // 1. Insert Operation
-    val records1 = recordsToStrings(dataGen.generateInserts("000", 100)).toList
+    val records1 = recordsToStrings(dataGen.generateInserts("000", 100)).asScala.toList
     val inputDF1 = spark.read.json(spark.sparkContext.parallelize(records1, 2))
     inputDF1.write.format("org.apache.hudi")
       .options(options)
       .mode(SaveMode.Overwrite)
       .save(basePath)
 
-    metaClient = HoodieTableMetaClient.builder()
-      .setBasePath(basePath)
-      .setConf(spark.sessionState.newHadoopConf)
-      .build()
+    metaClient = createMetaClient(spark, basePath)
 
     val schemaResolver = new TableSchemaResolver(metaClient)
     val dataSchema = schemaResolver.getTableAvroSchema(false)
@@ -280,9 +274,9 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     assertCDCOpCnt(cdcDataOnly1, 100, 0, 0)
 
     // 2. Upsert Operation
-    val records2_1 = recordsToStrings(dataGen.generateUniqueUpdates("001", 30)).toList
+    val records2_1 = recordsToStrings(dataGen.generateUniqueUpdates("001", 30)).asScala.toList
     val inputDF2_1 = spark.read.json(spark.sparkContext.parallelize(records2_1, 2))
-    val records2_2 = recordsToStrings(dataGen.generateInserts("001", 20)).toList
+    val records2_2 = recordsToStrings(dataGen.generateInserts("001", 20)).asScala.toList
     val inputDF2_2 = spark.read.json(spark.sparkContext.parallelize(records2_2, 2))
     inputDF2_1.union(inputDF2_2).write.format("org.apache.hudi")
       .options(options)
@@ -309,7 +303,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     totalInsertedCnt += insertedCnt2
 
     // 3. Delete Operation With Compaction Operation
-    val records3 = deleteRecordsToStrings(dataGen.generateUniqueDeletes(20)).toList
+    val records3 = deleteRecordsToStrings(dataGen.generateUniqueDeletes(20)).asScala.toList
     val inputDF3 = spark.read.json(spark.sparkContext.parallelize(records3, 2))
     inputDF3.write.format("org.apache.hudi")
       .options(options)
@@ -334,10 +328,11 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     assertCDCOpCnt(allVisibleCDCData, totalInsertedCnt, totalUpdatedCnt, totalDeletedCnt)
 
     // 4. Bulk_Insert Operation
-    val records4 = recordsToStrings(dataGen.generateInserts("003", 100)).toList
+    val records4 = recordsToStrings(dataGen.generateInserts("003", 100)).asScala.toList
     val inputDF4 = spark.read.json(spark.sparkContext.parallelize(records4, 2))
     inputDF4.write.format("org.apache.hudi")
       .options(options)
+      .option("hoodie.compact.inline", "false")
       .option(DataSourceWriteOptions.OPERATION.key, DataSourceWriteOptions.BULK_INSERT_OPERATION_OPT_VAL)
       .mode(SaveMode.Append)
       .save(basePath)
@@ -356,12 +351,13 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     assertCDCOpCnt(allVisibleCDCData, totalInsertedCnt, totalUpdatedCnt, totalDeletedCnt)
 
     // 5. Upsert Operation With Clustering Operation
-    val records5 = recordsToStrings(dataGen.generateUniqueUpdates("004", 60)).toList
+    val records5 = recordsToStrings(dataGen.generateUniqueUpdates("004", 60)).asScala.toList
     val inputDF5 = spark.read.json(spark.sparkContext.parallelize(records5, 2))
     inputDF5.write.format("org.apache.hudi")
       .options(options)
       .option("hoodie.clustering.inline", "true")
       .option("hoodie.clustering.inline.max.commits", "1")
+      .option("hoodie.compact.inline", "false")
       .mode(SaveMode.Append)
       .save(basePath)
     val instant5 = metaClient.reloadActiveTimeline.lastInstant().get()
@@ -386,10 +382,11 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     assertCDCOpCnt(cdcDataFrom3To4, insertedCnt4, 0, 20)
 
     // 6. Insert Overwrite Operation
-    val records6 = recordsToStrings(dataGen.generateInserts("005", 70)).toList
+    val records6 = recordsToStrings(dataGen.generateInserts("005", 70)).asScala.toList
     val inputDF6 = spark.read.json(spark.sparkContext.parallelize(records6, 2))
     inputDF6.write.format("org.apache.hudi")
       .options(options)
+      .option("hoodie.compact.inline", "false")
       .option(DataSourceWriteOptions.OPERATION.key, DataSourceWriteOptions.INSERT_OVERWRITE_TABLE_OPERATION_OPT_VAL)
       .mode(SaveMode.Append)
       .save(basePath)
@@ -408,31 +405,36 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     assertCDCOpCnt(allVisibleCDCData, totalInsertedCnt, totalUpdatedCnt, totalDeletedCnt)
 
     // 7,8. insert 10 records
-    val records7 = recordsToStrings(dataGen.generateInserts("006", 7)).toList
+    val records7 = recordsToStrings(dataGen.generateInserts("006", 7)).asScala.toList
     val inputDF7 = spark.read.json(spark.sparkContext.parallelize(records7, 2))
     inputDF7.write.format("org.apache.hudi")
       .options(options)
+      .option("hoodie.compact.inline", "false")
       .mode(SaveMode.Append)
       .save(basePath)
+    totalInsertedCnt += 7
 
-    val records8 = recordsToStrings(dataGen.generateInserts("007", 3)).toList
+    val records8 = recordsToStrings(dataGen.generateInserts("007", 3)).asScala.toList
     val inputDF8 = spark.read.json(spark.sparkContext.parallelize(records8, 2))
     inputDF8.write.format("org.apache.hudi")
       .options(options)
+      .option("hoodie.compact.inline", "false")
       .mode(SaveMode.Append)
       .save(basePath)
     val instant8 = metaClient.reloadActiveTimeline.lastInstant().get()
     val commitTime8 = instant8.getTimestamp
+    totalInsertedCnt += 3
 
     // 8. Upsert Operation With Clean Operation
-    val records9 = recordsToStrings(dataGen.generateUniqueUpdates("008", 30)).toList
-    val inputDF9 = spark.read.json(spark.sparkContext.parallelize(records9, 2))
+    val inputDF9 = inputDF6.limit(30) // 30 updates to inserts added after insert overwrite table. if not for this, updates generated from datagne,
+    // could split as inserts and updates from hudi standpoint due to insert overwrite table operation.
     inputDF9.write.format("org.apache.hudi")
       .options(options)
       .option("hoodie.clean.automatic", "true")
-      .option("hoodie.keep.min.commits", "4")
-      .option("hoodie.keep.max.commits", "5")
-      .option("hoodie.cleaner.commits.retained", "3")
+      .option("hoodie.keep.min.commits", "16")
+      .option("hoodie.keep.max.commits", "17")
+      .option("hoodie.clean.commits.retained", "15")
+      .option("hoodie.compact.inline", "false")
       .mode(SaveMode.Append)
       .save(basePath)
     val instant9 = metaClient.reloadActiveTimeline.lastInstant().get()
@@ -445,13 +447,8 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     val updatedCnt9 = 30 - insertedCnt9
     assertCDCOpCnt(cdcDataOnly9, insertedCnt9, updatedCnt9, 0)
 
-    // here cause we do the clean operation and just remain the commit6 and commit7, so we need to reset the total cnt.
-    // 70 is the number of inserted records at commit 6.
-    totalInsertedCnt = 80 + insertedCnt9
-    totalUpdatedCnt = updatedCnt9
-    totalDeletedCnt = 0
     allVisibleCDCData = cdcDataFrame((commitTime1.toLong - 1).toString)
-    assertCDCOpCnt(allVisibleCDCData, totalInsertedCnt, totalUpdatedCnt, totalDeletedCnt)
+    assertCDCOpCnt(allVisibleCDCData, totalInsertedCnt, totalUpdatedCnt + 30, totalDeletedCnt)
   }
 
   /**
@@ -478,7 +475,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     var allVisibleCDCData = spark.emptyDataFrame
 
     // Insert Operation
-    val records1 = recordsToStrings(dataGen.generateInserts("000", 100)).toList
+    val records1 = recordsToStrings(dataGen.generateInserts("000", 100)).asScala.toList
     val inputDF1 = spark.read.json(spark.sparkContext.parallelize(records1, 2))
     inputDF1.write.format("org.apache.hudi")
       .options(options)
@@ -491,10 +488,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     assert(partitionToCnt.contains(HoodieTestDataGenerator.DEFAULT_SECOND_PARTITION_PATH))
 
     // init meta client
-    metaClient = HoodieTableMetaClient.builder()
-      .setBasePath(basePath)
-      .setConf(spark.sessionState.newHadoopConf)
-      .build()
+    metaClient = createMetaClient(spark, basePath)
 
     totalInsertedCnt += 100
     val instant1 = metaClient.reloadActiveTimeline.lastInstant().get()
@@ -505,7 +499,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     assertCDCOpCnt(cdcDataOnly1, 100, 0, 0)
 
     // Insert Overwrite Partition Operation
-    val records2 = recordsToStrings(dataGen.generateInsertsForPartition("001", 30, HoodieTestDataGenerator.DEFAULT_FIRST_PARTITION_PATH)).toList
+    val records2 = recordsToStrings(dataGen.generateInsertsForPartition("001", 30, HoodieTestDataGenerator.DEFAULT_FIRST_PARTITION_PATH)).asScala.toList
     val inputDF2 = spark.read.json(spark.sparkContext.parallelize(records2, 2))
     inputDF2.write.format("org.apache.hudi")
       .options(options)
@@ -553,7 +547,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     assertCDCOpCnt(allVisibleCDCData, totalInsertedCnt, totalUpdatedCnt, totalDeletedCnt)
 
     // Upsert Operation
-    val records4 = recordsToStrings(dataGen.generateUniqueUpdates("000", 50)).toList
+    val records4 = recordsToStrings(dataGen.generateUniqueUpdates("000", 50)).asScala.toList
     val inputDF4 = spark.read.json(spark.sparkContext.parallelize(records4, 2))
     inputDF4.write.format("org.apache.hudi")
       .options(options)
@@ -595,17 +589,14 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     )
 
     // Insert Operation
-    val records1 = recordsToStrings(dataGen.generateInserts("000", 100)).toList
+    val records1 = recordsToStrings(dataGen.generateInserts("000", 100)).asScala.toList
     val inputDF1 = spark.read.json(spark.sparkContext.parallelize(records1, 2))
     inputDF1.write.format("org.apache.hudi")
       .options(options)
       .mode(SaveMode.Overwrite)
       .save(basePath)
 
-    metaClient = HoodieTableMetaClient.builder()
-      .setBasePath(basePath)
-      .setConf(spark.sessionState.newHadoopConf)
-      .build()
+    metaClient = createMetaClient(spark, basePath)
 
     val schemaResolver = new TableSchemaResolver(metaClient)
     val dataSchema = schemaResolver.getTableAvroSchema(false)
@@ -613,7 +604,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
 
     // Upsert Operation
     val hoodieRecords2 = dataGen.generateUniqueUpdates("001", 50)
-    val records2 = recordsToStrings(hoodieRecords2).toList
+    val records2 = recordsToStrings(hoodieRecords2).asScala.toList
     val inputDF2 = spark.read.json(spark.sparkContext.parallelize(records2, 2))
     inputDF2.write.format("org.apache.hudi")
       .options(options)
@@ -666,7 +657,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
       StructField("code", StringType)
     ))
 
-    val df = spark.createDataFrame(data.map(Row.fromTuple), schema)
+    val df = spark.createDataFrame(data.map(Row.fromTuple).asJava, schema)
     df.write
       .format("org.apache.hudi")
       .option("hoodie.datasource.write.operation", "upsert")
@@ -680,7 +671,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
       ("3", "D", "2023-06-14 15:47:09.953746", "B")
     )
 
-    val newDf = spark.createDataFrame(newData.map(Row.fromTuple), schema)
+    val newDf = spark.createDataFrame(newData.map(Row.fromTuple).asJava, schema)
 
     newDf.write
       .format("org.apache.hudi")
@@ -710,21 +701,18 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
     )
 
     // Insert Operation
-    val records1 = recordsToStrings(dataGen.generateInserts("000", 100)).toList
+    val records1 = recordsToStrings(dataGen.generateInserts("000", 100)).asScala.toList
     val inputDF1 = spark.read.json(spark.sparkContext.parallelize(records1, 2))
     inputDF1.write.format("org.apache.hudi")
       .options(options)
       .mode(SaveMode.Overwrite)
       .save(basePath)
 
-    metaClient = HoodieTableMetaClient.builder()
-      .setBasePath(basePath)
-      .setConf(spark.sessionState.newHadoopConf)
-      .build()
+    metaClient = createMetaClient(spark, basePath)
 
     // Upsert Operation
     val hoodieRecords2 = dataGen.generateUniqueUpdates("001", 50)
-    val records2 = recordsToStrings(hoodieRecords2).toList
+    val records2 = recordsToStrings(hoodieRecords2).asScala.toList
     val inputDF2 = spark.read.json(spark.sparkContext.parallelize(records2, 2))
     inputDF2.write.format("org.apache.hudi")
       .options(options)
@@ -737,7 +725,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
 
     // Upsert Operation
     val hoodieRecords3 = dataGen.generateUniqueUpdates("002", 50)
-    val records3 = recordsToStrings(hoodieRecords3).toList
+    val records3 = recordsToStrings(hoodieRecords3).asScala.toList
     val inputDF3 = spark.read.json(spark.sparkContext.parallelize(records3, 2))
     inputDF3.write.format("org.apache.hudi")
       .options(options)
@@ -747,7 +735,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
 
     // Upsert Operation
     val hoodieRecords4 = dataGen.generateUniqueUpdates("003", 50)
-    val records4 = recordsToStrings(hoodieRecords4).toList
+    val records4 = recordsToStrings(hoodieRecords4).asScala.toList
     val inputDF4 = spark.read.json(spark.sparkContext.parallelize(records4, 2))
     inputDF4.write.format("org.apache.hudi")
       .options(options)
@@ -809,11 +797,7 @@ class TestCDCDataFrameSuite extends HoodieCDCTestBase {
         .option("hoodie.table.cdc.supplemental.logging.mode", loggingMode.name())
         .mode(SaveMode.Append).save(basePath)
 
-      val hadoopConf = spark.sessionState.newHadoopConf()
-      val metaClient = HoodieTableMetaClient.builder()
-        .setBasePath(basePath)
-        .setConf(hadoopConf)
-        .build()
+    val metaClient = createMetaClient(spark, basePath)
       val startTimeStamp = metaClient.reloadActiveTimeline().firstInstant().get.getTimestamp
       val latestTimeStamp = metaClient.reloadActiveTimeline().lastInstant().get.getTimestamp
 
