@@ -17,24 +17,28 @@
 
 package org.apache.spark.sql.hudi.procedure
 
-import org.apache.avro.Schema
-import org.apache.hadoop.conf.Configuration
-import org.apache.hadoop.fs.{FileStatus, Path}
 import org.apache.hudi.avro.HoodieAvroUtils
-import org.apache.hudi.common.fs.FSUtils
 import org.apache.hudi.common.model.HoodieFileFormat
 import org.apache.hudi.common.table.HoodieTableMetaClient
 import org.apache.hudi.common.table.timeline.HoodieTimeline
 import org.apache.hudi.common.table.view.HoodieTableFileSystemView
 import org.apache.hudi.common.testutils.{HoodieTestDataGenerator, SchemaTestUtil}
+import org.apache.hudi.hadoop.fs.HadoopFSUtils
+import org.apache.hudi.storage.{StoragePath, StoragePathInfo}
+import org.apache.hudi.testutils.HoodieClientTestUtils.createMetaClient
 import org.apache.hudi.testutils.HoodieSparkWriteableTestTable
-import org.apache.spark.api.java.JavaSparkContext
+
+import org.apache.avro.Schema
+import org.apache.hadoop.conf.Configuration
+import org.apache.hadoop.fs.Path
 import org.junit.jupiter.api.Assertions.assertEquals
 
 import java.io.IOException
 import java.net.URL
 import java.nio.file.{Files, Paths}
-import scala.collection.JavaConverters.asScalaIteratorConverter
+import java.util.Properties
+
+import scala.collection.JavaConverters._
 
 class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
 
@@ -60,18 +64,21 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
       // create commit instant
       Files.createFile(Paths.get(tablePath, ".hoodie", "100.commit"))
 
-      val metaClient = HoodieTableMetaClient.builder
-        .setConf(new JavaSparkContext(spark.sparkContext).hadoopConfiguration())
-        .setBasePath(tablePath)
-        .build
+      val metaClient = createMetaClient(spark, tablePath)
 
       // create partition path
       val partition1 = Paths.get(tablePath, "2016/03/15").toString
       val partition2 = Paths.get(tablePath, "2015/03/16").toString
       val partition3 = Paths.get(tablePath, "2015/03/17").toString
-      assertResult(metaClient.getFs.mkdirs(new Path(partition1))) {true}
-      assertResult(metaClient.getFs.mkdirs(new Path(partition2))) {true}
-      assertResult(metaClient.getFs.mkdirs(new Path(partition3))) {true}
+      assertResult(metaClient.getStorage.createDirectory(new StoragePath(partition1))) {
+        true
+      }
+      assertResult(metaClient.getStorage.createDirectory(new StoragePath(partition2))) {
+        true
+      }
+      assertResult(metaClient.getStorage.createDirectory(new StoragePath(partition3))) {
+        true
+      }
 
       // default is dry run
       val dryResult = spark.sql(s"""call repair_add_partition_meta(table => '$tableName')""").collect()
@@ -106,6 +113,22 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
            |  preCombineField = 'ts'
            | )
        """.stripMargin)
+
+      val filePath = s"""$tablePath/.hoodie/hoodie.properties"""
+      val fs = HadoopFSUtils.getFs(filePath, new Configuration())
+      val fis = fs.open(new Path(filePath))
+      val prevProps = new Properties
+      prevProps.load(fis)
+      fis.close()
+
+      // write props to a file
+      val curPropPath = s"""${tmp.getCanonicalPath}/tmp/hoodie.properties"""
+      val path = new Path(curPropPath)
+      val out = fs.create(path)
+      prevProps.store(out, "hudi properties")
+      out.close()
+      fs.close()
+
       // create commit instant
       val newProps: URL = this.getClass.getClassLoader.getResource("table-config.properties")
 
@@ -140,6 +163,15 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
         .mkString("\n")
 
       assertEquals(expectedOutput, actual)
+
+      spark.sql(s"""call repair_overwrite_hoodie_props(table => '$tableName', new_props_file_path => '${curPropPath}')""")
+      val config = createMetaClient(spark, tablePath).getTableConfig
+      val props = config.getProps
+      assertEquals(prevProps.size(), props.size())
+      props.entrySet().asScala.foreach((entry) => {
+        val key = entry.getKey.toString
+        assertEquals(entry.getValue, prevProps.getProperty(key))
+      })
     }
   }
 
@@ -162,16 +194,13 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
            |  preCombineField = 'ts'
            | )
        """.stripMargin)
-      var metaClient = HoodieTableMetaClient.builder
-        .setConf(new JavaSparkContext(spark.sparkContext).hadoopConfiguration())
-        .setBasePath(tablePath)
-        .build
+      var metaClient = createMetaClient(spark, tablePath)
 
       // Create four requested files
       for (i <- 100 until 104) {
         val timestamp = String.valueOf(i)
         // Write corrupted requested Clean File
-        createEmptyCleanRequestedFile(tablePath, timestamp, metaClient.getHadoopConf)
+        createEmptyCleanRequestedFile(tablePath, timestamp, metaClient.getStorageConf.unwrapAs(classOf[Configuration]))
       }
 
       // reload meta client
@@ -217,10 +246,7 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
            |  type = 'cow'
            | )
        """.stripMargin)
-      var metaClient = HoodieTableMetaClient.builder
-        .setConf(new JavaSparkContext(spark.sparkContext).hadoopConfiguration())
-        .setBasePath(tablePath)
-        .build
+      var metaClient = createMetaClient(spark, tablePath)
 
       generateRecords(tablePath, bashPath, metaClient)
 
@@ -228,8 +254,8 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
       metaClient = HoodieTableMetaClient.reload(metaClient)
 
       // get fs and check number of latest files
-      val fsView = new HoodieTableFileSystemView(metaClient, metaClient.getActiveTimeline.getCommitTimeline.filterCompletedInstants,
-        metaClient.getFs.listStatus(new Path(duplicatedPartitionPath)))
+      val fsView = new HoodieTableFileSystemView(metaClient, metaClient.getActiveTimeline.getCommitAndReplaceTimeline.filterCompletedInstants,
+        metaClient.getStorage.listDirectEntries(new StoragePath(duplicatedPartitionPath)))
       val filteredStatuses = fsView.getLatestBaseFiles.iterator().asScala.map(value => value.getPath).toList
       // there should be 3 files
       assertResult(3) {
@@ -251,8 +277,8 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
       }
 
       // after deduplicate, there are 200 records
-      val fileStatus = metaClient.getFs.listStatus(new Path(repairedOutputPath))
-      files = fileStatus.map((status: FileStatus) => status.getPath.toString)
+      val fileStatus = metaClient.getStorage.listDirectEntries(new StoragePath(repairedOutputPath))
+      files = fileStatus.asScala.map((pathInfo: StoragePathInfo) => pathInfo.getPath.toString).toArray
       recordCount = getRecordCount(files)
       assertResult(200){recordCount}
     }
@@ -277,10 +303,7 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
            |  type = 'cow'
            | )
        """.stripMargin)
-      var metaClient = HoodieTableMetaClient.builder
-        .setConf(new JavaSparkContext(spark.sparkContext).hadoopConfiguration())
-        .setBasePath(tablePath)
-        .build
+      var metaClient = createMetaClient(spark, tablePath)
 
       generateRecords(tablePath, bashPath, metaClient)
 
@@ -288,8 +311,8 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
       metaClient = HoodieTableMetaClient.reload(metaClient)
 
       // get fs and check number of latest files
-      val fsView = new HoodieTableFileSystemView(metaClient, metaClient.getActiveTimeline.getCommitTimeline.filterCompletedInstants,
-        metaClient.getFs.listStatus(new Path(duplicatedPartitionPathWithUpdates)))
+      val fsView = new HoodieTableFileSystemView(metaClient, metaClient.getActiveTimeline.getCommitAndReplaceTimeline.filterCompletedInstants,
+        metaClient.getStorage.listDirectEntries(new StoragePath(duplicatedPartitionPathWithUpdates)))
       val filteredStatuses = fsView.getLatestBaseFiles.iterator().asScala.map(value => value.getPath).toList
       // there should be 2 files
       assertResult(2) {
@@ -312,8 +335,8 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
       }
 
       // after deduplicate, there are 100 records
-      val fileStatus = metaClient.getFs.listStatus(new Path(repairedOutputPath))
-      files = fileStatus.map((status: FileStatus) => status.getPath.toString)
+      val fileStatus = metaClient.getStorage.listDirectEntries(new StoragePath(repairedOutputPath))
+      files = fileStatus.asScala.map((pathInfo: StoragePathInfo) => pathInfo.getPath.toString).toArray
       recordCount = getRecordCount(files)
       assertResult(100){recordCount}
     }
@@ -338,10 +361,7 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
            |  type = 'cow'
            | )
        """.stripMargin)
-      var metaClient = HoodieTableMetaClient.builder
-        .setConf(new JavaSparkContext(spark.sparkContext).hadoopConfiguration())
-        .setBasePath(tablePath)
-        .build
+      var metaClient = createMetaClient(spark, tablePath)
 
       generateRecords(tablePath, bashPath, metaClient)
 
@@ -349,8 +369,8 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
       metaClient = HoodieTableMetaClient.reload(metaClient)
 
       // get fs and check number of latest files
-      val fsView = new HoodieTableFileSystemView(metaClient, metaClient.getActiveTimeline.getCommitTimeline.filterCompletedInstants,
-        metaClient.getFs.listStatus(new Path(duplicatedPartitionPathWithUpserts)))
+      val fsView = new HoodieTableFileSystemView(metaClient, metaClient.getActiveTimeline.getCommitAndReplaceTimeline.filterCompletedInstants,
+        metaClient.getStorage.listDirectEntries(new StoragePath(duplicatedPartitionPathWithUpserts)))
       val filteredStatuses = fsView.getLatestBaseFiles.iterator().asScala.map(value => value.getPath).toList
       // there should be 3 files
       assertResult(3) {
@@ -373,8 +393,8 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
       }
 
       // after deduplicate, there are 100 records
-      val fileStatus = metaClient.getFs.listStatus(new Path(repairedOutputPath))
-      files = fileStatus.map((status: FileStatus) => status.getPath.toString)
+      val fileStatus = metaClient.getStorage.listDirectEntries(new StoragePath(repairedOutputPath))
+      files = fileStatus.asScala.map((pathInfo: StoragePathInfo) => pathInfo.getPath.toString).toArray
       recordCount = getRecordCount(files)
       assertResult(100){recordCount}
     }
@@ -399,10 +419,7 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
            |  type = 'cow'
            | )
        """.stripMargin)
-      var metaClient = HoodieTableMetaClient.builder
-        .setConf(new JavaSparkContext(spark.sparkContext).hadoopConfiguration())
-        .setBasePath(tablePath)
-        .build
+      var metaClient = createMetaClient(spark, tablePath)
 
       generateRecords(tablePath, bashPath, metaClient)
 
@@ -410,8 +427,8 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
       metaClient = HoodieTableMetaClient.reload(metaClient)
 
       // get fs and check number of latest files
-      val fsView = new HoodieTableFileSystemView(metaClient, metaClient.getActiveTimeline.getCommitTimeline.filterCompletedInstants,
-        metaClient.getFs.listStatus(new Path(duplicatedPartitionPath)))
+      val fsView = new HoodieTableFileSystemView(metaClient, metaClient.getActiveTimeline.getCommitAndReplaceTimeline.filterCompletedInstants,
+        metaClient.getStorage.listDirectEntries(new StoragePath(duplicatedPartitionPath)))
       val filteredStatuses = fsView.getLatestBaseFiles.iterator().asScala.map(value => value.getPath).toList
       // there should be 3 files
       assertResult(3) {
@@ -434,8 +451,8 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
       }
 
       // after deduplicate, there are 200 records
-      val fileStatus = metaClient.getFs.listStatus(new Path(duplicatedPartitionPath))
-      files = fileStatus.map((status: FileStatus) => status.getPath.toString).filter(p => p.endsWith(".parquet"))
+      val fileStatus = metaClient.getStorage.listDirectEntries(new StoragePath(duplicatedPartitionPath))
+      files = fileStatus.asScala.map((pathInfo: StoragePathInfo) => pathInfo.getPath.toString).filter(p => p.endsWith(".parquet")).toArray
       recordCount = getRecordCount(files)
       assertResult(200){recordCount}
     }
@@ -527,7 +544,7 @@ class TestRepairsProcedure extends HoodieSparkProcedureTestBase {
   @throws[IOException]
   def createEmptyCleanRequestedFile(basePath: String, instantTime: String, configuration: Configuration): Unit = {
     val commitFilePath = new Path(basePath + "/" + HoodieTableMetaClient.METAFOLDER_NAME + "/" + HoodieTimeline.makeRequestedCleanerFileName(instantTime))
-    val fs = FSUtils.getFs(basePath, configuration)
+    val fs = HadoopFSUtils.getFs(basePath, configuration)
     val os = fs.create(commitFilePath, true)
     os.close()
   }
