@@ -23,84 +23,73 @@ import org.apache.avro.Schema.Type._
 import org.apache.avro.generic._
 import org.apache.avro.util.Utf8
 import org.apache.avro.{LogicalTypes, Schema, SchemaBuilder}
-import org.apache.spark.sql.avro.AvroDeserializer.{RebaseSpec, createDateRebaseFuncInRead, createTimestampRebaseFuncInRead}
-import org.apache.spark.sql.avro.AvroUtils.{toFieldDescription, toFieldStr}
+import org.apache.spark.sql.avro.AvroDeserializer.{createDateRebaseFuncInRead, createTimestampRebaseFuncInRead}
 import org.apache.spark.sql.catalyst.expressions.{SpecificInternalRow, UnsafeArrayData}
 import org.apache.spark.sql.catalyst.util.DateTimeConstants.MILLIS_PER_DAY
-import org.apache.spark.sql.catalyst.util.{ArrayBasedMapData, ArrayData, DateTimeUtils, GenericArrayData, RebaseDateTime}
-import org.apache.spark.sql.catalyst.{InternalRow, NoopFilters, StructFilters}
+import org.apache.spark.sql.catalyst.util._
+import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.execution.datasources.DataSourceUtils
+import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.internal.SQLConf.LegacyBehaviorPolicy
 import org.apache.spark.sql.types._
 import org.apache.spark.unsafe.types.UTF8String
 
 import java.math.BigDecimal
 import java.nio.ByteBuffer
-import java.util.TimeZone
 import scala.collection.JavaConverters._
 import scala.collection.mutable.ArrayBuffer
 
 /**
  * A deserializer to deserialize data in avro format to data in catalyst format.
  *
- * NOTE: This code is borrowed from Spark 3.2.1
- * This code is borrowed, so that we can better control compatibility w/in Spark minor
- * branches (3.2.x, 3.1.x, etc)
+ * NOTE: This code is borrowed from Spark 3.1.2
+ *       This code is borrowed, so that we can better control compatibility w/in Spark minor
+ *       branches (3.2.x, 3.1.x, etc)
  *
- * PLEASE REFRAIN MAKING ANY CHANGES TO THIS CODE UNLESS ABSOLUTELY NECESSARY
+ *       PLEASE REFRAIN MAKING ANY CHANGES TO THIS CODE UNLESS ABSOLUTELY NECESSARY
  */
 private[sql] class AvroDeserializer(rootAvroType: Schema,
                                     rootCatalystType: DataType,
-                                    positionalFieldMatch: Boolean,
-                                    datetimeRebaseSpec: RebaseSpec,
-                                    filters: StructFilters) {
+                                    datetimeRebaseMode: LegacyBehaviorPolicy.Value) {
 
-  def this(rootAvroType: Schema,
-           rootCatalystType: DataType,
-           datetimeRebaseMode: String) = {
+  def this(rootAvroType: Schema, rootCatalystType: DataType) = {
     this(
       rootAvroType,
       rootCatalystType,
-      positionalFieldMatch = false,
-      RebaseSpec(LegacyBehaviorPolicy.withName(datetimeRebaseMode)),
-      new NoopFilters)
+      LegacyBehaviorPolicy.withName(SQLConf.get.getConf(SQLConf.LEGACY_AVRO_REBASE_MODE_IN_READ)))
   }
 
   private lazy val decimalConversions = new DecimalConversion()
 
-  private val dateRebaseFunc = createDateRebaseFuncInRead(datetimeRebaseSpec.mode, "Avro")
+  private val dateRebaseFunc = createDateRebaseFuncInRead(
+    datetimeRebaseMode, "Avro")
 
-  private val timestampRebaseFunc = createTimestampRebaseFuncInRead(datetimeRebaseSpec, "Avro")
+  private val timestampRebaseFunc = createTimestampRebaseFuncInRead(
+    datetimeRebaseMode, "Avro")
 
-  private val converter: Any => Option[Any] = try {
-    rootCatalystType match {
-      // A shortcut for empty schema.
-      case st: StructType if st.isEmpty =>
-        (_: Any) => Some(InternalRow.empty)
+  private val converter: Any => Option[Any] = rootCatalystType match {
+    // A shortcut for empty schema.
+    case st: StructType if st.isEmpty =>
+      (data: Any) => Some(InternalRow.empty)
 
-      case st: StructType =>
-        val resultRow = new SpecificInternalRow(st.map(_.dataType))
-        val fieldUpdater = new RowUpdater(resultRow)
-        val applyFilters = filters.skipRow(resultRow, _)
-        val writer = getRecordWriter(rootAvroType, st, Nil, Nil, applyFilters)
-        (data: Any) => {
-          val record = data.asInstanceOf[GenericRecord]
-          val skipRow = writer(fieldUpdater, record)
-          if (skipRow) None else Some(resultRow)
-        }
+    case st: StructType =>
+      val resultRow = new SpecificInternalRow(st.map(_.dataType))
+      val fieldUpdater = new RowUpdater(resultRow)
+      val writer = getRecordWriter(rootAvroType, st, Nil, applyFilters = _ => false)
+      (data: Any) => {
+        val record = data.asInstanceOf[GenericRecord]
+        val skipRow = writer(fieldUpdater, record)
+        if (skipRow) None else Some(resultRow)
+      }
 
-      case _ =>
-        val tmpRow = new SpecificInternalRow(Seq(rootCatalystType))
-        val fieldUpdater = new RowUpdater(tmpRow)
-        val writer = newWriter(rootAvroType, rootCatalystType, Nil, Nil)
-        (data: Any) => {
-          writer(fieldUpdater, 0, data)
-          Some(tmpRow.get(0, rootCatalystType))
-        }
-    }
-  } catch {
-    case ise: IncompatibleSchemaException => throw new IncompatibleSchemaException(
-      s"Cannot convert Avro type $rootAvroType to SQL type ${rootCatalystType.sql}.", ise)
+    case _ =>
+      val tmpRow = new SpecificInternalRow(Seq(rootCatalystType))
+      val fieldUpdater = new RowUpdater(tmpRow)
+      val writer = newWriter(rootAvroType, rootCatalystType, Nil)
+      (data: Any) => {
+        writer(fieldUpdater, 0, data)
+        Some(tmpRow.get(0, rootCatalystType))
+      }
   }
 
   def deserialize(data: Any): Option[Any] = converter(data)
@@ -111,13 +100,7 @@ private[sql] class AvroDeserializer(rootAvroType: Schema,
    */
   private def newWriter(avroType: Schema,
                         catalystType: DataType,
-                        avroPath: Seq[String],
-                        catalystPath: Seq[String]): (CatalystDataUpdater, Int, Any) => Unit = {
-    val errorPrefix = s"Cannot convert Avro ${toFieldStr(avroPath)} to " +
-      s"SQL ${toFieldStr(catalystPath)} because "
-    val incompatibleMsg = errorPrefix +
-      s"schema is incompatible (avroType = $avroType, sqlType = ${catalystType.sql})"
-
+                        path: List[String]): (CatalystDataUpdater, Int, Any) => Unit =
     (avroType.getType, catalystType) match {
       case (NULL, NullType) => (updater, ordinal, _) =>
         updater.setNullAt(ordinal)
@@ -140,13 +123,13 @@ private[sql] class AvroDeserializer(rootAvroType: Schema,
         // (the `null` case), the value is processed as timestamp type with millisecond precision.
         case null | _: TimestampMillis => (updater, ordinal, value) =>
           val millis = value.asInstanceOf[Long]
-          val micros = DateTimeUtils.millisToMicros(millis)
+          val micros = AvroUtils.millisToMicros(millis)
           updater.setLong(ordinal, timestampRebaseFunc(micros))
         case _: TimestampMicros => (updater, ordinal, value) =>
           val micros = value.asInstanceOf[Long]
           updater.setLong(ordinal, timestampRebaseFunc(micros))
-        case other => throw new IncompatibleSchemaException(errorPrefix +
-          s"Avro logical type $other cannot be converted to SQL type ${TimestampType.sql}.")
+        case other => throw new IncompatibleSchemaException(
+          s"Cannot convert Avro logical type ${other} to Catalyst Timestamp type.")
       }
 
       // Before we upgrade Avro to 1.8 for logical type support, spark-avro converts Long to Date.
@@ -186,8 +169,7 @@ private[sql] class AvroDeserializer(rootAvroType: Schema,
             b.rewind()
             bytes
           case b: Array[Byte] => b
-          case other =>
-            throw new RuntimeException(errorPrefix + s"$other is not a valid avro binary.")
+          case other => throw new RuntimeException(s"$other is not a valid avro binary.")
         }
         updater.set(ordinal, bytes)
 
@@ -206,17 +188,14 @@ private[sql] class AvroDeserializer(rootAvroType: Schema,
       case (RECORD, st: StructType) =>
         // Avro datasource doesn't accept filters with nested attributes. See SPARK-32328.
         // We can always return `false` from `applyFilters` for nested records.
-        val writeRecord =
-          getRecordWriter(avroType, st, avroPath, catalystPath, applyFilters = _ => false)
+        val writeRecord = getRecordWriter(avroType, st, path, applyFilters = _ => false)
         (updater, ordinal, value) =>
           val row = new SpecificInternalRow(st)
           writeRecord(new RowUpdater(row), value.asInstanceOf[GenericRecord])
           updater.set(ordinal, row)
 
       case (ARRAY, ArrayType(elementType, containsNull)) =>
-        val avroElementPath = avroPath :+ "element"
-        val elementWriter = newWriter(avroType.getElementType, elementType,
-          avroElementPath, catalystPath :+ "element")
+        val elementWriter = newWriter(avroType.getElementType, elementType, path)
         (updater, ordinal, value) =>
           val collection = value.asInstanceOf[java.util.Collection[Any]]
           val result = createArrayData(elementType, collection.size())
@@ -228,8 +207,8 @@ private[sql] class AvroDeserializer(rootAvroType: Schema,
             val element = iter.next()
             if (element == null) {
               if (!containsNull) {
-                throw new RuntimeException(
-                  s"Array value at path ${toFieldStr(avroElementPath)} is not allowed to be null")
+                throw new RuntimeException(s"Array value at path ${path.mkString(".")} is not " +
+                  "allowed to be null")
               } else {
                 elementUpdater.setNullAt(i)
               }
@@ -242,10 +221,8 @@ private[sql] class AvroDeserializer(rootAvroType: Schema,
           updater.set(ordinal, result)
 
       case (MAP, MapType(keyType, valueType, valueContainsNull)) if keyType == StringType =>
-        val keyWriter = newWriter(SchemaBuilder.builder().stringType(), StringType,
-          avroPath :+ "key", catalystPath :+ "key")
-        val valueWriter = newWriter(avroType.getValueType, valueType,
-          avroPath :+ "value", catalystPath :+ "value")
+        val keyWriter = newWriter(SchemaBuilder.builder().stringType(), StringType, path)
+        val valueWriter = newWriter(avroType.getValueType, valueType, path)
         (updater, ordinal, value) =>
           val map = value.asInstanceOf[java.util.Map[AnyRef, AnyRef]]
           val keyArray = createArrayData(keyType, map.size())
@@ -260,8 +237,8 @@ private[sql] class AvroDeserializer(rootAvroType: Schema,
             keyWriter(keyUpdater, i, entry.getKey)
             if (entry.getValue == null) {
               if (!valueContainsNull) {
-                throw new RuntimeException(
-                  s"Map value at path ${toFieldStr(avroPath :+ "value")} is not allowed to be null")
+                throw new RuntimeException(s"Map value at path ${path.mkString(".")} is not " +
+                  "allowed to be null")
               } else {
                 valueUpdater.setNullAt(i)
               }
@@ -281,7 +258,7 @@ private[sql] class AvroDeserializer(rootAvroType: Schema,
         val nonNullAvroType = Schema.createUnion(nonNullTypes.asJava)
         if (nonNullTypes.nonEmpty) {
           if (nonNullTypes.length == 1) {
-            newWriter(nonNullTypes.head, catalystType, avroPath, catalystPath)
+            newWriter(nonNullTypes.head, catalystType, path)
           } else {
             nonNullTypes.map(_.getType).toSeq match {
               case Seq(a, b) if Set(a, b) == Set(INT, LONG) && catalystType == LongType =>
@@ -304,8 +281,7 @@ private[sql] class AvroDeserializer(rootAvroType: Schema,
                 catalystType match {
                   case st: StructType if st.length == nonNullTypes.size =>
                     val fieldWriters = nonNullTypes.zip(st.fields).map {
-                      case (schema, field) =>
-                        newWriter(schema, field.dataType, avroPath, catalystPath :+ field.name)
+                      case (schema, field) => newWriter(schema, field.dataType, path :+ field.name)
                     }.toArray
                     (updater, ordinal, value) => {
                       val row = new SpecificInternalRow(st)
@@ -315,17 +291,27 @@ private[sql] class AvroDeserializer(rootAvroType: Schema,
                       updater.set(ordinal, row)
                     }
 
-                  case _ => throw new IncompatibleSchemaException(incompatibleMsg)
+                  case _ =>
+                    throw new IncompatibleSchemaException(
+                      s"Cannot convert Avro to catalyst because schema at path " +
+                        s"${path.mkString(".")} is not compatible " +
+                        s"(avroType = $avroType, sqlType = $catalystType).\n" +
+                        s"Source Avro schema: $rootAvroType.\n" +
+                        s"Target Catalyst type: $rootCatalystType")
                 }
             }
           }
         } else {
-          (updater, ordinal, _) => updater.setNullAt(ordinal)
+          (updater, ordinal, value) => updater.setNullAt(ordinal)
         }
 
-      case _ => throw new IncompatibleSchemaException(incompatibleMsg)
+      case _ =>
+        throw new IncompatibleSchemaException(
+          s"Cannot convert Avro to catalyst because schema at path ${path.mkString(".")} " +
+            s"is not compatible (avroType = $avroType, sqlType = $catalystType).\n" +
+            s"Source Avro schema: $rootAvroType.\n" +
+            s"Target Catalyst type: $rootCatalystType")
     }
-  }
 
   // TODO: move the following method in Decimal object on creating Decimal from BigDecimal?
   private def createDecimal(decimal: BigDecimal, precision: Int, scale: Int): Decimal = {
@@ -339,25 +325,22 @@ private[sql] class AvroDeserializer(rootAvroType: Schema,
   }
 
   private def getRecordWriter(avroType: Schema,
-                              catalystType: StructType,
-                              avroPath: Seq[String],
-                              catalystPath: Seq[String],
+                              sqlType: StructType,
+                              path: List[String],
                               applyFilters: Int => Boolean): (CatalystDataUpdater, GenericRecord) => Boolean = {
     val validFieldIndexes = ArrayBuffer.empty[Int]
     val fieldWriters = ArrayBuffer.empty[(CatalystDataUpdater, Any) => Unit]
 
-    val avroSchemaHelper =
-      new AvroUtils.AvroSchemaHelper(avroType, avroPath, positionalFieldMatch)
-    val length = catalystType.length
+    val avroSchemaHelper = new AvroUtils.AvroSchemaHelper(avroType)
+    val length = sqlType.length
     var i = 0
     while (i < length) {
-      val catalystField = catalystType.fields(i)
-      avroSchemaHelper.getAvroField(catalystField.name, i) match {
+      val sqlField = sqlType.fields(i)
+      avroSchemaHelper.getFieldByName(sqlField.name) match {
         case Some(avroField) =>
           validFieldIndexes += avroField.pos()
 
-          val baseWriter = newWriter(avroField.schema(), catalystField.dataType,
-            avroPath :+ avroField.name, catalystPath :+ catalystField.name)
+          val baseWriter = newWriter(avroField.schema(), sqlField.dataType, path :+ sqlField.name)
           val ordinal = i
           val fieldWriter = (fieldUpdater: CatalystDataUpdater, value: Any) => {
             if (value == null) {
@@ -367,11 +350,14 @@ private[sql] class AvroDeserializer(rootAvroType: Schema,
             }
           }
           fieldWriters += fieldWriter
-        case None if !catalystField.nullable =>
-          val fieldDescription =
-            toFieldDescription(catalystPath :+ catalystField.name, i, positionalFieldMatch)
+        case None if !sqlField.nullable =>
+          val fieldStr = s"${path.mkString(".")}.${sqlField.name}"
           throw new IncompatibleSchemaException(
-            s"Cannot find non-nullable $fieldDescription in Avro schema.")
+            s"""
+               |Cannot find non-nullable field $fieldStr in Avro schema.
+               |Source Avro schema: $rootAvroType.
+               |Target Catalyst type: $rootCatalystType.
+           """.stripMargin)
         case _ => // nothing to do
       }
       i += 1
@@ -474,20 +460,14 @@ private[sql] class AvroDeserializer(rootAvroType: Schema,
 
 object AvroDeserializer {
 
-  // NOTE: Following methods have been renamed in Spark 3.2.1 [1] making [[AvroDeserializer]] implementation
+  // NOTE: Following methods have been renamed in Spark 3.1.3 [1] making [[AvroDeserializer]] implementation
   //       (which relies on it) be only compatible with the exact same version of [[DataSourceUtils]].
-  //       To make sure this implementation is compatible w/ all Spark versions w/in Spark 3.2.x branch,
-  //       we're preemptively cloned those methods to make sure Hudi is compatible w/ Spark 3.2.0 as well as
-  //       w/ Spark >= 3.2.1
+  //       To make sure this implementation is compatible w/ all Spark versions w/in Spark 3.1.x branch,
+  //       we're preemptively cloned those methods to make sure Hudi is compatible w/ Spark 3.1.2 as well as
+  //       w/ Spark >= 3.1.3
   //
   // [1] https://github.com/apache/spark/pull/34978
 
-  // Specification of rebase operation including `mode` and the time zone in which it is performed
-  case class RebaseSpec(mode: LegacyBehaviorPolicy.Value, originTimeZone: Option[String] = None) {
-    // Use the default JVM time zone for backward compatibility
-    def timeZone: String = originTimeZone.getOrElse(TimeZone.getDefault.getID)
-  }
-
   def createDateRebaseFuncInRead(rebaseMode: LegacyBehaviorPolicy.Value,
                                  format: String): Int => Int = rebaseMode match {
     case LegacyBehaviorPolicy.EXCEPTION => days: Int =>
@@ -499,15 +479,15 @@ object AvroDeserializer {
     case LegacyBehaviorPolicy.CORRECTED => identity[Int]
   }
 
-  def createTimestampRebaseFuncInRead(rebaseSpec: RebaseSpec,
-                                      format: String): Long => Long = rebaseSpec.mode match {
+  def createTimestampRebaseFuncInRead(rebaseMode: LegacyBehaviorPolicy.Value,
+                                      format: String): Long => Long = rebaseMode match {
     case LegacyBehaviorPolicy.EXCEPTION => micros: Long =>
       if (micros < RebaseDateTime.lastSwitchJulianTs) {
         throw DataSourceUtils.newRebaseExceptionInRead(format)
       }
       micros
-    case LegacyBehaviorPolicy.LEGACY => micros: Long =>
-      RebaseDateTime.rebaseJulianToGregorianMicros(TimeZone.getTimeZone(rebaseSpec.timeZone), micros)
+    case LegacyBehaviorPolicy.LEGACY => RebaseDateTime.rebaseJulianToGregorianMicros
     case LegacyBehaviorPolicy.CORRECTED => identity[Long]
   }
 }
+
