@@ -21,7 +21,7 @@ import java.nio.ByteBuffer
 import scala.collection.JavaConverters._
 import org.apache.avro.Conversions.DecimalConversion
 import org.apache.avro.LogicalTypes
-import org.apache.avro.LogicalTypes.{TimestampMicros, TimestampMillis}
+import org.apache.avro.LogicalTypes.{LocalTimestampMicros, LocalTimestampMillis, TimestampMicros, TimestampMillis}
 import org.apache.avro.Schema
 import org.apache.avro.Schema.Type
 import org.apache.avro.Schema.Type._
@@ -30,7 +30,7 @@ import org.apache.avro.generic.GenericData.Record
 import org.apache.avro.util.Utf8
 import org.apache.spark.internal.Logging
 import org.apache.spark.sql.avro.AvroSerializer.{createDateRebaseFuncInWrite, createTimestampRebaseFuncInWrite}
-import org.apache.spark.sql.avro.AvroUtils.{toFieldDescription, toFieldStr}
+import org.apache.spark.sql.avro.AvroUtils.{AvroMatchedField, toFieldStr}
 import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.expressions.{SpecializedGetters, SpecificInternalRow}
 import org.apache.spark.sql.catalyst.util.{DateTimeUtils, RebaseDateTime}
@@ -44,7 +44,7 @@ import java.util.TimeZone
 /**
  * A serializer to serialize data in catalyst format to data in avro format.
  *
- * NOTE: This code is borrowed from Spark 3.2.1
+ * NOTE: This code is borrowed from Spark 3.3.0
  *       This code is borrowed, so that we can better control compatibility w/in Spark minor
  *       branches (3.2.x, 3.1.x, etc)
  *
@@ -52,7 +52,6 @@ import java.util.TimeZone
  *       BEING EXPLICITLY ANNOTATED INLINE. PLEASE MAKE SURE TO UNDERSTAND PROPERLY ALL THE
  *       MODIFICATIONS.
  *
- *
  * PLEASE REFRAIN MAKING ANY CHANGES TO THIS CODE UNLESS ABSOLUTELY NECESSARY
  */
 private[sql] class AvroSerializer(rootCatalystType: DataType,
@@ -190,6 +189,18 @@ private[sql] class AvroSerializer(rootCatalystType: DataType,
           s"SQL type ${TimestampType.sql} cannot be converted to Avro logical type $other")
       }
 
+      case (TimestampNTZType, LONG) => avroType.getLogicalType match {
+        // To keep consistent with TimestampType, if the Avro type is Long and it is not
+        // logical type (the `null` case), output the TimestampNTZ as long value
+        // in millisecond precision.
+        case null | _: LocalTimestampMillis => (getter, ordinal) =>
+          DateTimeUtils.microsToMillis(getter.getLong(ordinal))
+        case _: LocalTimestampMicros => (getter, ordinal) =>
+          getter.getLong(ordinal)
+        case other => throw new IncompatibleSchemaException(errorPrefix +
+          s"SQL type ${TimestampNTZType.sql} cannot be converted to Avro logical type $other")
+      }
+
       case (ArrayType(et, containsNull), ARRAY) =>
         val elementConverter = newConverter(
           et, resolveNullableType(avroType.getElementType, containsNull),
@@ -253,6 +264,12 @@ private[sql] class AvroSerializer(rootCatalystType: DataType,
           }
           result
 
+      case (_: YearMonthIntervalType, INT) =>
+        (getter, ordinal) => getter.getInt(ordinal)
+
+      case (_: DayTimeIntervalType, LONG) =>
+        (getter, ordinal) => getter.getLong(ordinal)
+
       case _ =>
         throw new IncompatibleSchemaException(errorPrefix +
           s"schema is incompatible (sqlType = ${catalystType.sql}, avroType = $avroType)")
@@ -264,34 +281,19 @@ private[sql] class AvroSerializer(rootCatalystType: DataType,
                                  catalystPath: Seq[String],
                                  avroPath: Seq[String]): InternalRow => Record = {
 
-    val avroPathStr = toFieldStr(avroPath)
-    if (avroStruct.getType != RECORD) {
-      throw new IncompatibleSchemaException(s"$avroPathStr was not a RECORD")
-    }
-    val avroFields = avroStruct.getFields.asScala
-    if (avroFields.size != catalystStruct.length) {
-      throw new IncompatibleSchemaException(
-        s"Avro $avroPathStr schema length (${avroFields.size}) doesn't match " +
-          s"SQL ${toFieldStr(catalystPath)} schema length (${catalystStruct.length})")
-    }
-    val avroSchemaHelper =
-      new AvroUtils.AvroSchemaHelper(avroStruct, avroPath, positionalFieldMatch)
+    val avroSchemaHelper = new AvroUtils.AvroSchemaHelper(
+      avroStruct, catalystStruct, avroPath, catalystPath, positionalFieldMatch)
 
-    val (avroIndices: Array[Int], fieldConverters: Array[Converter]) =
-      catalystStruct.zipWithIndex.map { case (catalystField, catalystPos) =>
-        val avroField = avroSchemaHelper.getAvroField(catalystField.name, catalystPos) match {
-          case Some(f) => f
-          case None =>
-            val fieldDescription = toFieldDescription(
-              catalystPath :+ catalystField.name, catalystPos, positionalFieldMatch)
-            throw new IncompatibleSchemaException(
-              s"Cannot find $fieldDescription in Avro schema at $avroPathStr")
-        }
+    avroSchemaHelper.validateNoExtraCatalystFields(ignoreNullable = false)
+    avroSchemaHelper.validateNoExtraRequiredAvroFields()
+
+    val (avroIndices, fieldConverters) = avroSchemaHelper.matchedFields.map {
+      case AvroMatchedField(catalystField, _, avroField) =>
         val converter = newConverter(catalystField.dataType,
           resolveNullableType(avroField.schema(), catalystField.nullable),
           catalystPath :+ catalystField.name, avroPath :+ avroField.name)
         (avroField.pos(), converter)
-      }.toArray.unzip
+    }.toArray.unzip
 
     val numFields = catalystStruct.length
     row: InternalRow =>
@@ -361,6 +363,7 @@ private[sql] class AvroSerializer(rootCatalystType: DataType,
   // <<< ENDS
   ////////////////////////////////////////////////////////////////////////////////////////////
 
+
   /**
    * Resolve a possibly nullable Avro Type.
    *
