@@ -36,6 +36,7 @@ import org.apache.hudi.common.model.HoodieReplaceCommitMetadata;
 import org.apache.hudi.common.model.HoodieTableType;
 import org.apache.hudi.common.model.HoodieWriteStat;
 import org.apache.hudi.common.model.WriteOperationType;
+import org.apache.hudi.common.table.HoodieTableConfig;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
 import org.apache.hudi.common.table.log.HoodieLogFormat;
 import org.apache.hudi.common.table.log.HoodieLogFormat.Writer;
@@ -47,17 +48,20 @@ import org.apache.hudi.common.testutils.SchemaTestUtil;
 import org.apache.hudi.common.testutils.minicluster.ZookeeperTestService;
 import org.apache.hudi.common.util.FileIOUtils;
 import org.apache.hudi.common.util.Option;
+import org.apache.hudi.hadoop.fs.HadoopFSUtils;
 import org.apache.hudi.hive.HiveSyncConfig;
 import org.apache.hudi.hive.HiveSyncTool;
 import org.apache.hudi.hive.SlashEncodedDayPartitionValueExtractor;
 import org.apache.hudi.hive.ddl.HiveQueryDDLExecutor;
 import org.apache.hudi.hive.ddl.QueryBasedDDLExecutor;
 import org.apache.hudi.hive.util.IMetaStoreClientUtil;
+import org.apache.hudi.storage.HoodieStorage;
+import org.apache.hudi.storage.StoragePath;
+import org.apache.hudi.storage.hadoop.HoodieHadoopStorage;
 
 import org.apache.avro.Schema;
 import org.apache.avro.generic.IndexedRecord;
 import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.fs.FSDataOutputStream;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hive.conf.HiveConf;
@@ -74,8 +78,8 @@ import org.slf4j.LoggerFactory;
 
 import java.io.File;
 import java.io.IOException;
+import java.io.OutputStream;
 import java.net.URISyntaxException;
-import java.nio.charset.StandardCharsets;
 import java.nio.file.Files;
 import java.time.Instant;
 import java.time.ZonedDateTime;
@@ -95,6 +99,7 @@ import java.util.stream.Collectors;
 
 import static org.apache.hudi.common.table.HoodieTableMetaClient.METAFOLDER_NAME;
 import static org.apache.hudi.common.table.timeline.TimelineMetadataUtils.serializeRollbackMetadata;
+import static org.apache.hudi.common.util.StringUtils.getUTF8Bytes;
 import static org.apache.hudi.hive.HiveSyncConfigHolder.HIVE_BATCH_SYNC_PARTITION_NUM;
 import static org.apache.hudi.hive.HiveSyncConfigHolder.HIVE_PASS;
 import static org.apache.hudi.hive.HiveSyncConfigHolder.HIVE_URL;
@@ -117,6 +122,7 @@ public class HiveTestUtil {
   public static String basePath;
   public static TypedProperties hiveSyncProps;
   public static HiveTestService hiveTestService;
+  public static HoodieStorage storage;
   public static FileSystem fileSystem;
   public static QueryBasedDDLExecutor ddlExecutor;
 
@@ -128,7 +134,7 @@ public class HiveTestUtil {
   private static DateTimeFormatter dtfOut;
   private static Set<String> createdTablesSet = new HashSet<>();
 
-  public static void setUp() throws IOException, InterruptedException, HiveException, MetaException {
+  public static void setUp() throws Exception {
     configuration = new Configuration();
     if (zkServer == null) {
       zkService = new ZookeeperTestService(configuration);
@@ -156,8 +162,12 @@ public class HiveTestUtil {
 
     hiveSyncConfig = new HiveSyncConfig(hiveSyncProps, hiveTestService.getHiveConf());
     fileSystem = hiveSyncConfig.getHadoopFileSystem();
+    storage = new HoodieHadoopStorage(fileSystem);
 
     dtfOut = DateTimeFormatter.ofPattern("yyyy/MM/dd");
+    if (ddlExecutor != null) {
+      ddlExecutor.close();
+    }
     ddlExecutor = new HiveQueryDDLExecutor(hiveSyncConfig, IMetaStoreClientUtil.getMSC(hiveSyncConfig.getHiveConf()));
 
     clear();
@@ -169,7 +179,7 @@ public class HiveTestUtil {
         .setTableType(HoodieTableType.COPY_ON_WRITE)
         .setTableName(TABLE_NAME)
         .setPayloadClass(HoodieAvroPayload.class)
-        .initTable(configuration, basePath);
+        .initTable(HadoopFSUtils.getStorageConfWithCopy(configuration), basePath);
 
     for (String tableName : createdTablesSet) {
       ddlExecutor.runSQL("drop table if exists " + tableName);
@@ -182,18 +192,72 @@ public class HiveTestUtil {
     return hiveServer.getHiveConf();
   }
 
-  public static void shutdown() throws IOException {
-    if (hiveServer != null) {
-      hiveServer.stop();
+  public static void shutdown() {
+    List<String> failedReleases = new ArrayList<>();
+    try {
+      clear();
+    } catch (HiveException | MetaException | IOException he) {
+      he.printStackTrace();
+      failedReleases.add("HiveData");
     }
-    if (hiveTestService != null) {
-      hiveTestService.stop();
+
+    try {
+      if (ddlExecutor != null) {
+        ddlExecutor.close();
+        ddlExecutor = null;
+      }
+    } catch (Exception ex) {
+      ex.printStackTrace();
+      failedReleases.add("DDLExecutor");
     }
-    if (zkServer != null) {
-      zkServer.shutdown(true);
+
+    try {
+      if (hiveServer != null) {
+        hiveServer.stop();
+      }
+    } catch (Exception e) {
+      e.printStackTrace();
+      failedReleases.add("HiveServer");
     }
-    if (fileSystem != null) {
-      fileSystem.close();
+
+    try {
+      if (hiveTestService != null) {
+        hiveTestService.stop();
+      }
+    } catch (Exception e) {
+      e.printStackTrace();
+      failedReleases.add("HiveTestService");
+    }
+
+    try {
+      if (zkServer != null) {
+        zkServer.shutdown(true);
+      }
+    } catch (Exception e) {
+      e.printStackTrace();
+      failedReleases.add("ZKServer");
+    }
+
+    try {
+      if (zkService != null) {
+        zkService.stop();
+      }
+    } catch (RuntimeException re) {
+      re.printStackTrace();
+      failedReleases.add("ZKService");
+    }
+
+    try {
+      if (fileSystem != null) {
+        fileSystem.close();
+      }
+    } catch (IOException ie) {
+      ie.printStackTrace();
+      failedReleases.add("FileSystem");
+    }
+
+    if (!failedReleases.isEmpty()) {
+      LOG.error("Exception happened during releasing: " + String.join(",", failedReleases));
     }
   }
 
@@ -207,7 +271,7 @@ public class HiveTestUtil {
         .setTableType(HoodieTableType.COPY_ON_WRITE)
         .setTableName(tableName)
         .setPayloadClass(HoodieAvroPayload.class)
-        .initTable(configuration, basePath);
+        .initTable(HadoopFSUtils.getStorageConfWithCopy(configuration), basePath);
 
     boolean result = fileSystem.mkdirs(path);
     checkResult(result);
@@ -233,16 +297,16 @@ public class HiveTestUtil {
   }
 
   public static void removeCommitFromActiveTimeline(String instantTime, String actionType) {
-    List<Path> pathsToDelete = new ArrayList<>();
-    Path metaFolderPath = new Path(basePath, METAFOLDER_NAME);
+    List<StoragePath> pathsToDelete = new ArrayList<>();
+    StoragePath metaFolderPath = new StoragePath(basePath, METAFOLDER_NAME);
     String actionSuffix = "." + actionType;
-    pathsToDelete.add(new Path(metaFolderPath, instantTime + actionSuffix));
-    pathsToDelete.add(new Path(metaFolderPath, instantTime + actionSuffix + ".requested"));
-    pathsToDelete.add(new Path(metaFolderPath, instantTime + actionSuffix + ".inflight"));
+    pathsToDelete.add(new StoragePath(metaFolderPath, instantTime + actionSuffix));
+    pathsToDelete.add(new StoragePath(metaFolderPath, instantTime + actionSuffix + ".requested"));
+    pathsToDelete.add(new StoragePath(metaFolderPath, instantTime + actionSuffix + ".inflight"));
     pathsToDelete.forEach(path -> {
       try {
-        if (fileSystem.exists(path)) {
-          fileSystem.delete(path, false);
+        if (storage.exists(path)) {
+          storage.deleteFile(path);
         }
       } catch (IOException e) {
         LOG.warn("Error deleting file: ", e);
@@ -281,11 +345,11 @@ public class HiveTestUtil {
     createMetaFile(
         basePath,
         HoodieTimeline.makeRequestedRollbackFileName(instantTime),
-        "".getBytes());
+        getUTF8Bytes(""));
     createMetaFile(
         basePath,
         HoodieTimeline.makeInflightRollbackFileName(instantTime),
-        "".getBytes());
+        getUTF8Bytes(""));
     createMetaFile(
         basePath,
         HoodieTimeline.makeRollbackFileName(instantTime),
@@ -300,7 +364,7 @@ public class HiveTestUtil {
         .setTableType(HoodieTableType.COPY_ON_WRITE)
         .setTableName(TABLE_NAME)
         .setPayloadClass(HoodieAvroPayload.class)
-        .initTable(configuration, basePath);
+        .initTable(HadoopFSUtils.getStorageConfWithCopy(configuration), basePath);
 
     boolean result = fileSystem.mkdirs(path);
     checkResult(result);
@@ -313,7 +377,8 @@ public class HiveTestUtil {
     fileSystem.mkdirs(partPath);
     List<HoodieWriteStat> writeStats = new ArrayList<>();
     String fileId = UUID.randomUUID().toString();
-    Path filePath = new Path(partPath.toString() + "/" + FSUtils.makeBaseFileName(instantTime, "1-0-1", fileId));
+    Path filePath = new Path(partPath.toString() + "/"
+        + FSUtils.makeBaseFileName(instantTime, "1-0-1", fileId, HoodieTableConfig.BASE_FILE_FORMAT.defaultValue().getFileExtension()));
     Schema schema = SchemaTestUtil.getSchemaFromResource(HiveTestUtil.class, schemaFileName);
     generateParquetDataWithSchema(filePath, schema);
     HoodieWriteStat writeStat = new HoodieWriteStat();
@@ -335,7 +400,7 @@ public class HiveTestUtil {
         .setTableType(HoodieTableType.MERGE_ON_READ)
         .setTableName(TABLE_NAME)
         .setPayloadClass(HoodieAvroPayload.class)
-        .initTable(configuration, basePath);
+        .initTable(HadoopFSUtils.getStorageConfWithCopy(configuration), basePath);
 
     boolean result = fileSystem.mkdirs(path);
     checkResult(result);
@@ -401,8 +466,8 @@ public class HiveTestUtil {
     for (Entry<String, List<HoodieWriteStat>> wEntry : partitionWriteStats.entrySet()) {
       String partitionPath = wEntry.getKey();
       for (HoodieWriteStat wStat : wEntry.getValue()) {
-        Path path = new Path(wStat.getPath());
-        HoodieBaseFile dataFile = new HoodieBaseFile(fileSystem.getFileStatus(path));
+        StoragePath path = new StoragePath(wStat.getPath());
+        HoodieBaseFile dataFile = new HoodieBaseFile(storage.getPathInfo(path));
         HoodieLogFile logFile = generateLogData(path, isLogSchemaSimple);
         HoodieDeltaWriteStat writeStat = new HoodieDeltaWriteStat();
         writeStat.setFileId(dataFile.getFileId());
@@ -450,7 +515,8 @@ public class HiveTestUtil {
     for (int i = 0; i < 5; i++) {
       // Create 5 files
       String fileId = UUID.randomUUID().toString();
-      Path filePath = new Path(partPath.toString() + "/" + FSUtils.makeBaseFileName(instantTime, "1-0-1", fileId));
+      Path filePath = new Path(partPath.toString() + "/"
+          + FSUtils.makeBaseFileName(instantTime, "1-0-1", fileId, HoodieTableConfig.BASE_FILE_FORMAT.defaultValue().getFileExtension()));
       generateParquetData(filePath, isParquetSchemaSimple);
       HoodieWriteStat writeStat = new HoodieWriteStat();
       writeStat.setFileId(fileId);
@@ -505,16 +571,18 @@ public class HiveTestUtil {
     writer.close();
   }
 
-  private static HoodieLogFile generateLogData(Path parquetFilePath, boolean isLogSchemaSimple)
+  private static HoodieLogFile generateLogData(StoragePath parquetFilePath,
+                                               boolean isLogSchemaSimple)
       throws IOException, InterruptedException, URISyntaxException {
     Schema schema = getTestDataSchema(isLogSchemaSimple);
-    HoodieBaseFile dataFile = new HoodieBaseFile(fileSystem.getFileStatus(parquetFilePath));
+    HoodieBaseFile dataFile = new HoodieBaseFile(storage.getPathInfo(parquetFilePath));
     // Write a log file for this parquet file
     Writer logWriter = HoodieLogFormat.newWriterBuilder().onParentPath(parquetFilePath.getParent())
         .withFileExtension(HoodieLogFile.DELTA_EXTENSION).withFileId(dataFile.getFileId())
-        .overBaseCommit(dataFile.getCommitTime()).withFs(fileSystem).build();
+        .overBaseCommit(dataFile.getCommitTime()).withStorage(storage).build();
     List<HoodieRecord> records = (isLogSchemaSimple ? SchemaTestUtil.generateTestRecords(0, 100)
-        : SchemaTestUtil.generateEvolvedTestRecords(100, 100)).stream().map(HoodieAvroIndexedRecord::new).collect(Collectors.toList());
+        : SchemaTestUtil.generateEvolvedTestRecords(100, 100)).stream()
+        .map(HoodieAvroIndexedRecord::new).collect(Collectors.toList());
     Map<HeaderMetadataType, String> header = new HashMap<>(2);
     header.put(HoodieLogBlock.HeaderMetadataType.INSTANT_TIME, dataFile.getCommitTime());
     header.put(HoodieLogBlock.HeaderMetadataType.SCHEMA, schema.toString());
@@ -553,14 +621,14 @@ public class HiveTestUtil {
     createMetaFile(
         basePath,
         HoodieTimeline.makeCommitFileName(instantTime),
-        commitMetadata.toJsonString().getBytes(StandardCharsets.UTF_8));
+        getUTF8Bytes(commitMetadata.toJsonString()));
   }
 
   public static void createReplaceCommitFile(HoodieReplaceCommitMetadata commitMetadata, String instantTime) throws IOException {
     createMetaFile(
         basePath,
         HoodieTimeline.makeReplaceFileName(instantTime),
-        commitMetadata.toJsonString().getBytes(StandardCharsets.UTF_8));
+        getUTF8Bytes(commitMetadata.toJsonString()));
   }
 
   public static void createCommitFileWithSchema(HoodieCommitMetadata commitMetadata, String instantTime, boolean isSimpleSchema) throws IOException {
@@ -573,7 +641,7 @@ public class HiveTestUtil {
     createMetaFile(
         basePath,
         HoodieTimeline.makeCommitFileName(instantTime),
-        commitMetadata.toJsonString().getBytes(StandardCharsets.UTF_8));
+        getUTF8Bytes(commitMetadata.toJsonString()));
   }
 
   private static void createDeltaCommitFile(HoodieCommitMetadata deltaCommitMetadata, String deltaCommitTime)
@@ -581,15 +649,15 @@ public class HiveTestUtil {
     createMetaFile(
         basePath,
         HoodieTimeline.makeDeltaFileName(deltaCommitTime),
-        deltaCommitMetadata.toJsonString().getBytes(StandardCharsets.UTF_8));
+        getUTF8Bytes(deltaCommitMetadata.toJsonString()));
   }
 
   private static void createMetaFile(String basePath, String fileName, byte[] bytes)
       throws IOException {
     Path fullPath = new Path(basePath + "/" + METAFOLDER_NAME + "/" + fileName);
-    FSDataOutputStream fsout = fileSystem.create(fullPath, true);
-    fsout.write(bytes);
-    fsout.close();
+    OutputStream out = fileSystem.create(fullPath, true);
+    out.write(bytes);
+    out.close();
   }
 
   public static Set<String> getCreatedTablesSet() {
