@@ -99,11 +99,9 @@ public interface HoodieMetaSyncOperations {
   }
 
   /**
-   * Get the metadata of partitions that belong to the specified table
-   * @param tableName
-   * @return
+   * Get partitions given input list of partitions.
    */
-  default List<Partition> getPartitionsByFilter(String tableName, String filter) {
+  default List<Partition> getPartitionsFromList(String tableName, List<String> partitionList) {
     return Collections.emptyList();
   }
 
@@ -233,4 +231,11 @@ public interface HoodieMetaSyncOperations {
   default void deleteLastReplicatedTimeStamp(String tableName) {
 
   }
+
+  /**
+   * Generates a push down filter string to retrieve existing partitions
+   */
+  default String generatePushDownFilter(List<String> writtenPartitions, List<FieldSchema> partitionFields) {
+    throw new UnsupportedOperationException();
+  }
 }
