@@ -21,10 +21,8 @@ package org.apache.hudi.utilities;
 
 import org.apache.hudi.client.common.HoodieSparkEngineContext;
 import org.apache.hudi.common.config.HoodieMetadataConfig;
-import org.apache.hudi.common.config.SerializableConfiguration;
 import org.apache.hudi.common.config.TypedProperties;
 import org.apache.hudi.common.engine.HoodieLocalEngineContext;
-import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.HoodieBaseFile;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
 import org.apache.hudi.common.table.view.FileSystemViewManager;
@@ -33,7 +31,10 @@ import org.apache.hudi.common.util.Option;
 import org.apache.hudi.exception.HoodieException;
 import org.apache.hudi.exception.HoodieIOException;
 import org.apache.hudi.exception.TableNotFoundException;
+import org.apache.hudi.hadoop.fs.HadoopFSUtils;
 import org.apache.hudi.metadata.HoodieTableMetadata;
+import org.apache.hudi.storage.StorageConfiguration;
+import org.apache.hudi.storage.hadoop.HoodieHadoopStorage;
 
 import com.beust.jcommander.JCommander;
 import com.beust.jcommander.Parameter;
@@ -54,6 +55,7 @@ import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStreamReader;
 import java.io.Serializable;
+import java.nio.charset.StandardCharsets;
 import java.time.LocalDate;
 import java.time.format.DateTimeFormatter;
 import java.time.format.DateTimeFormatterBuilder;
@@ -95,6 +97,7 @@ import java.util.stream.Collectors;
  */
 public class TableSizeStats implements Serializable {
 
+  private static final long serialVersionUID = 1L;
   private static final Logger LOG = LoggerFactory.getLogger(TableSizeStats.class);
 
   // Date formatter for parsing partition dates (example: 2023/5/5/ or 2023-5-5).
@@ -273,8 +276,9 @@ public class TableSizeStats implements Serializable {
         .enable(isMetadataEnabled(basePath, jsc))
         .build();
     HoodieSparkEngineContext engineContext = new HoodieSparkEngineContext(jsc);
-    HoodieTableMetadata tableMetadata = HoodieTableMetadata.create(engineContext, metadataConfig, basePath);
-    SerializableConfiguration serializableConfiguration = new SerializableConfiguration(jsc.hadoopConfiguration());
+    StorageConfiguration<?> storageConf = HadoopFSUtils.getStorageConfWithCopy(jsc.hadoopConfiguration());
+    HoodieTableMetadata tableMetadata = HoodieTableMetadata.create(
+        engineContext, new HoodieHadoopStorage(basePath, storageConf), metadataConfig, basePath);
 
     List<String> allPartitions = tableMetadata.getAllPartitionPaths();
 
@@ -310,12 +314,12 @@ public class TableSizeStats implements Serializable {
           || (startDate != null && endDate != null && ((partitionDate.isEqual(startDate) || partitionDate.isAfter(startDate)) && partitionDate.isBefore(endDate)))) {
         HoodieTableMetaClient metaClientLocal = HoodieTableMetaClient.builder()
             .setBasePath(basePath)
-            .setConf(serializableConfiguration.get()).build();
+            .setConf(storageConf.newInstance()).build();
         HoodieMetadataConfig metadataConfig1 = HoodieMetadataConfig.newBuilder()
             .enable(false)
             .build();
         HoodieTableFileSystemView fileSystemView = FileSystemViewManager
-            .createInMemoryFileSystemView(new HoodieLocalEngineContext(serializableConfiguration.get()),
+            .createInMemoryFileSystemView(new HoodieLocalEngineContext(storageConf),
                 metaClientLocal, metadataConfig1);
         List<HoodieBaseFile> baseFiles = fileSystemView.getLatestBaseFiles(partition).collect(Collectors.toList());
 
@@ -349,7 +353,7 @@ public class TableSizeStats implements Serializable {
   private static boolean isMetadataEnabled(String basePath, JavaSparkContext jsc) {
     HoodieTableMetaClient metaClient = HoodieTableMetaClient.builder()
         .setBasePath(basePath)
-        .setConf(jsc.hadoopConfiguration()).build();
+        .setConf(HadoopFSUtils.getStorageConfWithCopy(jsc.hadoopConfiguration())).build();
 
     Set<String> partitions = metaClient.getTableConfig().getMetadataPartitions();
     return !partitions.isEmpty() && partitions.contains("files");
@@ -357,12 +361,12 @@ public class TableSizeStats implements Serializable {
 
   private static List<String> getFilePaths(String propsPath, Configuration hadoopConf) {
     List<String> filePaths = new ArrayList<>();
-    FileSystem fs = FSUtils.getFs(
+    FileSystem fs = HadoopFSUtils.getFs(
         propsPath,
         Option.ofNullable(hadoopConf).orElseGet(Configuration::new)
     );
 
-    try (BufferedReader reader = new BufferedReader(new InputStreamReader(fs.open(new Path(propsPath))))) {
+    try (BufferedReader reader = new BufferedReader(new InputStreamReader(fs.open(new Path(propsPath)), StandardCharsets.UTF_8))) {
       String line = reader.readLine();
       while (line != null) {
         filePaths.add(line);
