@@ -19,15 +19,14 @@
 package org.apache.hudi.utilities;
 
 import org.apache.hudi.common.config.TypedProperties;
+import org.apache.hudi.utilities.config.SchemaProviderPostProcessorConfig;
 import org.apache.hudi.utilities.exception.HoodieSchemaPostProcessException;
 import org.apache.hudi.utilities.schema.SchemaPostProcessor;
-import org.apache.hudi.utilities.schema.SchemaPostProcessor.Config;
 import org.apache.hudi.utilities.schema.SchemaProvider;
 import org.apache.hudi.utilities.schema.SparkAvroPostProcessor;
 import org.apache.hudi.utilities.schema.postprocessor.DeleteSupportSchemaPostProcessor;
 import org.apache.hudi.utilities.schema.postprocessor.DropColumnSchemaPostProcessor;
 import org.apache.hudi.utilities.schema.postprocessor.add.AddPrimitiveColumnSchemaPostProcessor;
-import org.apache.hudi.utilities.schema.postprocessor.add.BaseSchemaPostProcessorConfig;
 import org.apache.hudi.utilities.testutils.UtilitiesTestBase;
 import org.apache.hudi.utilities.transform.FlatteningTransformer;
 
@@ -75,7 +74,7 @@ public class TestSchemaPostProcessor extends UtilitiesTestBase {
 
   @Test
   public void testPostProcessor() throws IOException {
-    properties.put(Config.SCHEMA_POST_PROCESSOR_PROP, DummySchemaPostProcessor.class.getName());
+    properties.put(SchemaProviderPostProcessorConfig.SCHEMA_POST_PROCESSOR.key(), DummySchemaPostProcessor.class.getName());
     SchemaProvider provider =
         UtilHelpers.wrapSchemaProviderWithPostProcessor(
             UtilHelpers.createSchemaProvider(DummySchemaProvider.class.getName(), properties, jsc),
@@ -89,7 +88,7 @@ public class TestSchemaPostProcessor extends UtilitiesTestBase {
 
   @Test
   public void testSparkAvro() throws IOException {
-    properties.put(Config.SCHEMA_POST_PROCESSOR_PROP, SparkAvroPostProcessor.class.getName());
+    properties.put(SchemaProviderPostProcessorConfig.SCHEMA_POST_PROCESSOR.key(), SparkAvroPostProcessor.class.getName());
     List<String> transformerClassNames = new ArrayList<>();
     transformerClassNames.add(FlatteningTransformer.class.getName());
 
@@ -115,10 +114,10 @@ public class TestSchemaPostProcessor extends UtilitiesTestBase {
   @Test
   public void testChainedSchemaPostProcessor() {
     // DeleteSupportSchemaPostProcessor first, DummySchemaPostProcessor second
-    properties.put(Config.SCHEMA_POST_PROCESSOR_PROP,
+    properties.put(SchemaProviderPostProcessorConfig.SCHEMA_POST_PROCESSOR.key(),
         "org.apache.hudi.utilities.schema.postprocessor.DeleteSupportSchemaPostProcessor,org.apache.hudi.utilities.DummySchemaPostProcessor");
 
-    SchemaPostProcessor processor = UtilHelpers.createSchemaPostProcessor(properties.getString(Config.SCHEMA_POST_PROCESSOR_PROP), properties, jsc);
+    SchemaPostProcessor processor = UtilHelpers.createSchemaPostProcessor(properties.getString(SchemaProviderPostProcessorConfig.SCHEMA_POST_PROCESSOR.key()), properties, jsc);
     Schema schema = new Schema.Parser().parse(ORIGINAL_SCHEMA);
     Schema targetSchema = processor.processSchema(schema);
 
@@ -127,10 +126,10 @@ public class TestSchemaPostProcessor extends UtilitiesTestBase {
     assertNotNull(targetSchema.getField("testString"));
 
     // DummySchemaPostProcessor first, DeleteSupportSchemaPostProcessor second
-    properties.put(Config.SCHEMA_POST_PROCESSOR_PROP,
+    properties.put(SchemaProviderPostProcessorConfig.SCHEMA_POST_PROCESSOR.key(),
         "org.apache.hudi.utilities.DummySchemaPostProcessor,org.apache.hudi.utilities.schema.postprocessor.DeleteSupportSchemaPostProcessor");
 
-    processor = UtilHelpers.createSchemaPostProcessor(properties.getString(Config.SCHEMA_POST_PROCESSOR_PROP), properties, jsc);
+    processor = UtilHelpers.createSchemaPostProcessor(properties.getString(SchemaProviderPostProcessorConfig.SCHEMA_POST_PROCESSOR.key()), properties, jsc);
     schema = new Schema.Parser().parse(ORIGINAL_SCHEMA);
     targetSchema = processor.processSchema(schema);
 
@@ -142,7 +141,7 @@ public class TestSchemaPostProcessor extends UtilitiesTestBase {
   @Test
   public void testDeleteColumn() {
     // remove column ums_id_ from source schema
-    properties.put(DropColumnSchemaPostProcessor.Config.DELETE_COLUMN_POST_PROCESSOR_COLUMN_PROP, "rider");
+    properties.put(SchemaProviderPostProcessorConfig.DELETE_COLUMN_POST_PROCESSOR_COLUMN.key(), "rider");
     DropColumnSchemaPostProcessor processor = new DropColumnSchemaPostProcessor(properties, null);
     Schema schema = new Schema.Parser().parse(ORIGINAL_SCHEMA);
     Schema targetSchema = processor.processSchema(schema);
@@ -154,7 +153,7 @@ public class TestSchemaPostProcessor extends UtilitiesTestBase {
   @Test
   public void testDeleteColumnThrows() {
     // remove all columns from source schema
-    properties.put(DropColumnSchemaPostProcessor.Config.DELETE_COLUMN_POST_PROCESSOR_COLUMN_PROP, "timestamp,_row_key,rider,driver,fare");
+    properties.put(SchemaProviderPostProcessorConfig.DELETE_COLUMN_POST_PROCESSOR_COLUMN.key(), "timestamp,_row_key,rider,driver,fare");
     DropColumnSchemaPostProcessor processor = new DropColumnSchemaPostProcessor(properties, null);
     Schema schema = new Schema.Parser().parse(ORIGINAL_SCHEMA);
 
@@ -164,9 +163,9 @@ public class TestSchemaPostProcessor extends UtilitiesTestBase {
   @ParameterizedTest
   @MethodSource("configParams")
   public void testAddPrimitiveTypeColumn(String type) {
-    properties.put(BaseSchemaPostProcessorConfig.SCHEMA_POST_PROCESSOR_ADD_COLUMN_NAME_PROP.key(), "primitive_column");
-    properties.put(BaseSchemaPostProcessorConfig.SCHEMA_POST_PROCESSOR_ADD_COLUMN_TYPE_PROP.key(), type);
-    properties.put(BaseSchemaPostProcessorConfig.SCHEMA_POST_PROCESSOR_ADD_COLUMN_DOC_PROP.key(), "primitive column test");
+    properties.put(SchemaProviderPostProcessorConfig.SCHEMA_POST_PROCESSOR_ADD_COLUMN_NAME_PROP.key(), "primitive_column");
+    properties.put(SchemaProviderPostProcessorConfig.SCHEMA_POST_PROCESSOR_ADD_COLUMN_TYPE_PROP.key(), type);
+    properties.put(SchemaProviderPostProcessorConfig.SCHEMA_POST_PROCESSOR_ADD_COLUMN_DOC_PROP.key(), "primitive column test");
 
     AddPrimitiveColumnSchemaPostProcessor processor = new AddPrimitiveColumnSchemaPostProcessor(properties, null);
     Schema schema = new Schema.Parser().parse(ORIGINAL_SCHEMA);
@@ -180,7 +179,7 @@ public class TestSchemaPostProcessor extends UtilitiesTestBase {
     assertNotEquals(type, newColumn.schema().getType().getName());
 
     // test not nullable
-    properties.put(BaseSchemaPostProcessorConfig.SCHEMA_POST_PROCESSOR_ADD_COLUMN_NULLABLE_PROP.key(), false);
+    properties.put(SchemaProviderPostProcessorConfig.SCHEMA_POST_PROCESSOR_ADD_COLUMN_NULLABLE_PROP.key(), false);
     targetSchema = processor.processSchema(schema);
     newColumn = targetSchema.getField("primitive_column");
     assertEquals(type, newColumn.schema().getType().getName());
