@@ -31,8 +31,8 @@ import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.HttpURLConnection;
-import java.nio.charset.StandardCharsets;
 
+import static org.apache.hudi.common.util.StringUtils.getUTF8Bytes;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.mockito.ArgumentMatchers.eq;
@@ -64,16 +64,16 @@ class TestSchemaRegistryProvider {
   private static TypedProperties getProps() {
     return new TypedProperties() {
       {
-        put("hoodie.deltastreamer.schemaprovider.registry.baseUrl", "http://" + BASIC_AUTH + "@localhost");
-        put("hoodie.deltastreamer.schemaprovider.registry.urlSuffix", "-value");
-        put("hoodie.deltastreamer.schemaprovider.registry.url", "http://foo:bar@localhost");
-        put("hoodie.deltastreamer.source.kafka.topic", "foo");
+        put("hoodie.streamer.schemaprovider.registry.baseUrl", "http://" + BASIC_AUTH + "@localhost");
+        put("hoodie.streamer.schemaprovider.registry.urlSuffix", "-value");
+        put("hoodie.streamer.schemaprovider.registry.url", "http://foo:bar@localhost");
+        put("hoodie.streamer.source.kafka.topic", "foo");
       }
     };
   }
 
   private static SchemaRegistryProvider getUnderTest(TypedProperties props) throws IOException {
-    InputStream is = new ByteArrayInputStream(REGISTRY_RESPONSE.getBytes(StandardCharsets.UTF_8));
+    InputStream is = new ByteArrayInputStream(getUTF8Bytes(REGISTRY_RESPONSE));
     SchemaRegistryProvider spyUnderTest = Mockito.spy(new SchemaRegistryProvider(props, null));
     Mockito.doReturn(is).when(spyUnderTest).getStream(Mockito.any());
     return spyUnderTest;
@@ -102,8 +102,8 @@ class TestSchemaRegistryProvider {
   @Test
   public void testGetSourceSchemaShouldRequestSchemaWithoutCreds() throws IOException {
     TypedProperties props = getProps();
-    props.put("hoodie.deltastreamer.schemaprovider.registry.url", "http://localhost");
-    props.put("hoodie.deltastreamer.schemaprovider.registry.schemaconverter", DummySchemaConverter.class.getName());
+    props.put("hoodie.streamer.schemaprovider.registry.url", "http://localhost");
+    props.put("hoodie.streamer.schemaprovider.registry.schemaconverter", DummySchemaConverter.class.getName());
     SchemaRegistryProvider spyUnderTest = getUnderTest(props);
     Schema actual = spyUnderTest.getSourceSchema();
     assertNotNull(actual);
@@ -114,8 +114,8 @@ class TestSchemaRegistryProvider {
   @Test
   public void testGetTargetSchemaShouldRequestSchemaWithoutCreds() throws IOException {
     TypedProperties props = getProps();
-    props.put("hoodie.deltastreamer.schemaprovider.registry.url", "http://localhost");
-    props.put("hoodie.deltastreamer.schemaprovider.registry.schemaconverter", DummySchemaConverter.class.getName());
+    props.put("hoodie.streamer.schemaprovider.registry.url", "http://localhost");
+    props.put("hoodie.streamer.schemaprovider.registry.schemaconverter", DummySchemaConverter.class.getName());
     SchemaRegistryProvider spyUnderTest = getUnderTest(props);
     Schema actual = spyUnderTest.getTargetSchema();
     assertNotNull(actual);
@@ -133,4 +133,24 @@ class TestSchemaRegistryProvider {
           .toString();
     }
   }
+
+  // The SR is checked when cachedSchema is empty, when not empty, the cachedSchema is used.
+  @Test
+  public void testGetSourceSchemaUsesCachedSchema() throws IOException {
+    TypedProperties props = getProps();
+    SchemaRegistryProvider spyUnderTest = getUnderTest(props);
+
+    // Call when cachedSchema is empty
+    Schema actual = spyUnderTest.getSourceSchema();
+    assertNotNull(actual);
+    verify(spyUnderTest, times(1)).parseSchemaFromRegistry(Mockito.any());
+
+    assert spyUnderTest.cachedSourceSchema != null;
+
+    Schema actualTwo = spyUnderTest.getSourceSchema();
+    
+    // cachedSchema should now be set, a subsequent call should not call parseSchemaFromRegistry
+    // Assuming this verify() has the scope of the whole test? so it should still be 1 from previous call?
+    verify(spyUnderTest, times(1)).parseSchemaFromRegistry(Mockito.any());
+  }
 }
